      SUBROUTINE KORALZ(MODE,KF1,PB1,EE1,KF2,PB2,EE2,XPAR,NPAR)
C=====================================================================C
C=====================================================================C
C        KK  KK    OOO    KKKKK      KKK     LL     ZZZZZZ            C
C        KK KK   OO   OO  KK  KK    KK KKK   LL        ZZ             C
C        KKK     OO   OO  KKKK     KK   KK   LL      ZZ               C
C        KK KK   OO   OO  KK KK    KKKKKKK   LL     ZZ                C
C        KK  KK    OOO    KK  KK   KK   KK   LLLLL  ZZZZZZ            C
C=====================================================================C
C=======================   VERSION 4.04 ===============================
C======================================================================
C========================= FEBRUARY 1998  =============================
C======================================================================
C       MONTE CARLO EVENT GENERATOR FOR THE COMBINED                  C
C    TAU/MU PAIR PRODUCTION AND DECAY PROCESS AT LEP/SLC              C
C    (it contains YFS3 multiphoton generator as a part)               C
C                                                                     C
C                      Authors                                        C
C  S. JADACH     /1,2/   B.F.L. WARD   /3/  Z. WAS    /1,2/           C
C                                                                     C
C      /1/  INSTITUTE OF NUCLEAR PHYS. CRACOW                         C
C      /2/  CERN, Geneva                                              C
C      /3/  Universuty of Tennessee Knoxville, Tennessee              C
C======================================================================
C======================================================================
C== ACKNOWLEDGEMENTS : AUTHORS OF THE PROGRAM ARE INDEBTED           ==
C== TO ALL USERS OF THE PROGRAM WHO PROVIDED INVALUABLE HELP         ==
C== IN DEBUGGING THE PROGRAM, SPECIAL THANKS GO TO                   ==
C== F. BOILLOT, G. BONNEAUD, A. M. LUTZ WHO HELPED TO ORGANIZE       ==
C== THE CODE AND TO PUT IT INTO HISTORIAN.                           ==
C== THE NEUTRINO MODE OF THE PROGRAM WAS DEVELOPPED WITH THE ACTIVE  ==
C== PARTICIPATION OF P. COLAS AND L. MIRABITO FROM SACLAY            ==
C== SEE MUNICH REPORT MPI-PAE-EXP-EL-211 (1989)                      ==
C======================================================================
C
C    IMPORTANT NOTE:
C    BEFORE USING THIS PROGRAM YOU MAY CONTACT Z. WAS
C       THROUGH EARNET/BITNET, WASM at CERNVM,
C    AND, PERHAPS, YOU WILL GET A BETTER VERSION!!!
C    PLEASE ALSO REPORT ANY NOTICED ERROR OR ANY OTHER PROBLEM
C                   ***
C ---------------------------------------------------------------------
C ---------------------------------------------------------------------
C KORALZ IS THE COMMUNICATION CENTER FOR THE INPUT INFORMATION,
C*MWG*  ALL OUTPUT IS CODED IN COMMON / HEPEVT /
C*MWG*  ACCORDING TO STANDARD PDG CONVENTIONS.
C
C Calling arguments
C =================
C MODE=-1 initialization or reinitialization mode,
C         prior to generation, obligatory.
C     = 0 generation mode, m.c. event is generated.
C     = 1 postgeneration mode, printouts, optional.
C
C if MODE=-1 then all parameters are input data:
C    =======
C*MWG* KF1 = 11,-11 flavour code of first beam, KF1=11 for electron.
C*MWG* KF2 = 11,-11 flavour code of second beam, KF2=-11 for positron.
C PB1 = four momentum of the first beam.
C PB2 = four momentum of the second beam.
C EE1 = spin polarization vector for the first beam.
C EE2 = spin polarization vector for the second beam,
C       both in the corresponding beam particle rest frame
C       and in both cases third axis directed along first beam,
C       i.e. EE1(3) and -EE2(3) are helicities.
C       only EE1(3) EE2(3) can be nonzero (longitudinal polarization
C                                          only)
C other input parameters are hidden in XPAR and NPAR.
C NPAR(1)          =ISPIN              spin effects in decay
C NPAR(2)          =inran (OBSOLETE!)
C NPAR(3)          =KEYGSW             level of GSW corrections
C NPAR(4)          =KEYRAD             bremsstrahlung
C NPAR(5)          =JAK1               decay type of 1-st tau
C NPAR(6)          =JAK2               decay type of 2-nd tau
C NPAR(7)          =ITFIN              type of the final fermion
C NPAR(8)          =ITDKRC             radiation in tau decay
C NPAR(9)          =KEYWLB             type of electroweak library
C NPAR(11)         =nneut : number of neutrinos in the nunubar option
C NPAR(12)         =KEYYFS steering parameter of YFS3 see routine EXPAND
C                          and its own KEYRAD.
C  (The user has to change the width himself, this parameter has no
C   effect on the other final states.)
C XPAR( 1)=AMZ     =   mass of Z0 boson
C XPAR( 2)=AMH     =   mass of Higgs boson
C XPAR( 3)=AMTOP   =   mass of top quark
C XPAR( 4)=GV      =   W-tau coupling in tau decay
C XPAR( 5)=GA      =   W-tau coupling in tau decay
C XPAR( 6)=SWSQ    =   only for KEYGSW<2,
C XPAR( 7)=GAMMZ   =   only for KEYGSW<2, Z0 width.
C XPAR( 8)=AMNUTA  =   mass of tau neutrino in decay.
C XPAR( 9)=amneut  = neutrino mass used in nu-nubar option
C                    assumed unique (= AMNUTA by default)
C                    or quark to be generated mass (in quark mode)
C XPAR(11)=XK0     =   soft/hard cut   (single bremmstrahlung)
C XPAR(12)=VVMIN   =   minimum v for YFS2, should be 1.E-5or less,
C XPAR(13)=VVMAX   =   maximum v, (v=1-s'/s, sqrt(s')=final pair mass)
C XPAR(14)=XK0DEC  =   soft/hard cut in tau decay
C
C KEYGSW=  0 Z0, NO GAMMA, BORN APPROXIMATION
C       =  1 Z0+Gamma, Born approximation
C       =  2 Z0+Gamma, vacuum polarizations ON (no more supported
C       =  3 electroweak corrections switched ON (no more supported
C       =  4 electroweak corrections switched ON in a way
C            consistent with the higher orders of QED.
C KEYRAD   The type of the QED bremsstrahlung
C KEYRAD=  0, no bremsstrahlung
C       =  1, single bremsstrahlung
C NPAR(4)= 2, single bremsstrahlung, no QED ini-fin interference
C             note that internal variable KEYRAD is set ot 1. 
C KEYRAD= 10, single bremsstralung but exponentiaited spectrum
C       = 11, YFS2/YFS3 generation; backward compatibility option in TRALOR
C       = 12, YFS2/YFS3 generation; default
C       =111, backward compatibility option
C       =112, backward compatibility option
C ISPIN = 0,1  spin effects in decay switched OFF/ON
C INRAN = initialisation constant for rand. num. gen. rnf100, positive,
C         obsolent!!
C ITFIN =1 tau pair production,
C       =2 muon pair production.
C       =3 nunub pair production.
C       =501-506 quark pair production.
C ITDKRC=0 no rad. corr. in tau decay
C       =1 with rad. corr. in tau decay
C JAK1  = -1 no decay
C       =  0 inclusive decay
C       =  1,2 electron and muon decays
C       =  3 pi decay
C       =  4,5 rho and a_1 decays
C       =  6,7 K   and K*  decays
C       =  8 ... 
C       ...
C JAK2  = the same as for JAK1
C KEYWLB=1,2 electroweak corrections due to first or second choice
C         ======
C else if MODE=0 then all parameters are ignored
C         ======
C else if MODE=1 then
C         ======
C NPAR(10)= nevtot, no. of generated events,
C XPAR(10)= cstcm,  integrated total cross section in cm**2 units,
C XPAR(16)= CSTCM,  integrated total cross section in cm**2 units,
C XPAR(17)= CSTNB,  integrated total cross section in nb units,
C XPAR(18)= DCSNB,  error on cross section,
C XPAR(20+I)= GAMPMC(I) branching ratios,
C are output information provided for the user.
C endif
C         ======
C the user may add other internal parameters to npar and xpar
C a possible candidate is the list of the tau decay branching
C ratios GAMPRT in  / TAUBRA / GAMPRT(30),JLIST(30),NCHAN.
C at present GAMPRT is initialised in INITDK (norm. to tau>e nu nu chan
C
C =====================================================================
C
C Modified by AM. Lutz/G.Bonneaud  (Nov 1988-Jan 1989)
C to provide unique initialisations of (almost) all parameters, masses,
C and physical constants.
C
C =====================================================================
C     called by : ASKUSI,ASKUSE
C ---------------------------------------------------------
C     IMPLICIT LOGICAL(A-H,O-Z)
      COMMON / INOUT / INUT,IOUT
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
       COMMON /GSWLIB/ KEYWLB
      COMMON / GSWPRM /SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
      REAL*8           SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
C     SWSQ        = sin2 (theta Weinberg)
C     AMW,AMZ     = W & Z boson masses respectively
C     AMH         = the Higgs mass
C     AMTOP       = the top mass
C     GAMMZ       = Z0 width
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / VVREK  / VVMIN,VVMAX,VV,BETI
      REAL*8            VVMIN,VVMAX,VV,BETI
      COMMON / FINUS / CSTCM,ERREL
      REAL*8           CSTCM,ERREL
      COMMON / PARMAS / AMTAU,AMNUTA,AMEL,AMNUE,AMMU,AMNUMU
     *                 ,AMPIZ,AMPI,AMRO,GAMRO,AMA1,GAMA1
     *                 ,AMK,AMKZ,AMKST,GAMKST
C
      REAL*4            AMTAU,AMNUTA,AMEL,AMNUE,AMMU,AMNUMU
     *                 ,AMPIZ,AMPI,AMRO,GAMRO,AMA1,GAMA1
     *                 ,AMK,AMKZ,AMKST,GAMKST
      COMMON / DECPAR / GFERMI,GV,GA,CCABIB,SCABIB,GAMEL
      REAL*4            GFERMI,GV,GA,CCABIB,SCABIB,GAMEL
      COMMON / JAKI   /  JAK1,JAK2,JAKP,JAKM,KTOM
      COMMON / TAUHEL / HELT1,HELT2
      REAL*4 HELT1,HELT2
      COMMON /TAUPOS/ NP1,NP2  
      COMMON / TAUBMC / GAMPMC(30),GAMPER(30),NEVDEC(30)
      REAL*4            GAMPMC    ,GAMPER
      COMMON / TAURAD / XK0DEC,ITDKRC
      REAL*8            XK0DEC
      COMMON /NEWMOD/  AMNEUT,NNEUT
      REAL*8           AMNEUT
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
      REAL*4   PB1(4),EE1(3),PB2(4),EE2(3),XPAR(40)
      INTEGER  NPAR(40)
      REAL*4   POL1(4),POL2(4)
C
      IF(MODE.EQ.-1) THEN
C ================================================    INITIALISATIONS
C
        IEVEN = 0
C
C  DECODE INPUT PARAMETERS SET BY DATA CARDS
C  -----------------------------------------
        ISPIN = NPAR(1)
        INRAN = NPAR(2)
        KEYGSW= NPAR(3)
        IF (NPAR(4).eq.2) THEN
          KEYRAD= 1
          KBRINI= 1
          KBRFIN= 1
          KBRINT= 0 
        ELSEIF (NPAR(4).eq.3) THEN
          KEYRAD= 1
          KBRINI= 1
          KBRFIN= 0
          KBRINT= 0
        ELSEIF (NPAR(4).eq.4) THEN
          KEYRAD= 1
          KBRINI= 0
          KBRFIN= 1
          KBRINT= 0
        ELSE
          KEYRAD= NPAR(4)
          KBRINI= 1
          KBRFIN= 1
          KBRINT= 1
        ENDIF
        JAK1  = NPAR(5)
        JAK2  = NPAR(6)
        ITFIN = NPAR(7)
        ITDKRC= NPAR(8)
        KEYWLB= NPAR(9)
        KEYYFS= NPAR(12)
        AMZ   = DBLE(XPAR(1))
        AMH   = DBLE(XPAR(2))
        AMTOP = DBLE(XPAR(3))
C
C         W-BOSON COUPLINGS TO FINAL STATE TAU (IN DECAY) FROM INPUT
          GV=XPAR( 4)
          GA=XPAR( 5)
C         TAUNEUTRINO MASS FROM INPUT
          AMNUTA= XPAR(8)
C         SOFT/HARD BREMMSTRAHLUNG CUT in tau decay FROM INPUT
          XK0DEC= XPAR(14)
      IF(ITFIN.EQ.3) THEN
C       Initialization related  to nu nubar option only
        NNEUT=NPAR(11)
        AMNEUT=XPAR( 9)
        ENDIF
C
      IF(ITFIN.GT.500) THEN
C       Initialization related  to quark option only
        AMNEUT=XPAR( 9)
        ENDIF
C
        IF(KEYGSW.LE.1) THEN
C         INITIALISE SIN2W AND Z WIDTH IF NOT RECOMPUTED
          SWSQ  =DBLE(XPAR(6))
          GAMMZ =DBLE(XPAR(7))
        ENDIF
C
C       INITIALISATION OF QED RAD. CORR.
C**     SOFT/HARD BREMMSTRAHLUNG CUT
        XK0   =XPAR(11)
C**     DEFINITION OF EXTRA PARAMETERS FOR EXPAND
        VVMIN =XPAR(12)
        VVMAX =XPAR(13)
C
C PRINT INPUT PARAMETERS
C ----------------------
        WRITE(IOUT,7000) KF1,KF2,PB1(3),PB1(4),PB2(3),PB2(4),
     &  EE1(3),EE2(3)
        WRITE(IOUT,7001) ISPIN,KEYGSW,npar(4),
     &  JAK1,JAK2,ITFIN,ITDKRC,KEYWLB,NNEUT
        WRITE(IOUT,7002) AMZ,AMH,AMTOP,
     &  GV,GA,AMNUTA,XK0DEC,
     &  XK0,VVMIN,VVMAX,AMNEUT
C
C CHECK INPUT PARAMETERS
C ----------------------
C*MWG*  BEAM IDENTIFIERS KF1=11,-11 DENOTE  ELECTRON, POSITRON
        IF(IABS(KF1).NE.11.OR.IABS(KF2).NE.11)        GOTO 900
        IF(KF1*KF2.GT.0)                              GOTO 900
        SUMPT=0
        DO 20 I=1,2
        SUMPT=SUMPT+PB1(I)
  20    SUMPT=SUMPT+PB2(I)
        IF(SUMPT.GT.0.0001)                           GOTO 902
        IF (     PB1(3) .NE. ABS(PB1(4))
     &  .OR. ABS(PB2(3)).NE. ABS(PB2(4))
     &  .OR.PB1(4).NE.PB2(4).OR.PB1(3).NE.(-PB2(3)) ) GOTO 903
        IF(PB1(4).LE.0)                               GOTO 903
C**     POLARISATION VECTORS OF BEAMS
        IF (EE1(3)*EE2(3).LT.-.5D0) THEN
          WRITE(IOUT,101)
  101     FORMAT('WARNING: PROGRAM IS NOT COMPLETELY O(ALPHA) FOR
     $    STRONGLY POLARIZED E+ AND E-')
        ENDIF
        SUMPT=0
        DO 25 I=1,2
        SUMPT=SUMPT+ ABS(EE1(I))
  25    SUMPT=SUMPT+ ABS(EE2(I))
        IF(SUMPT.GT.1D-8)                             GOTO 904
        IF( ABS(EE1(3)).GT.1.OR. ABS(EE2(3)).GT.1)    GOTO 904
C
C**     SWITCHES
        IF(KEYRAD.GT.9.AND.KEYGSW.EQ.3)               GOTO 905
        IF(KEYRAD.GT.2.AND.KEYRAD.LT.10)              GOTO 905
        IF(KEYRAD.EQ.10)                              GOTO 911
        IF(KEYGSW.LT.-1.OR.KEYGSW.GT.4)               GOTO 905
        IF(KEYGSW.EQ.2 .OR.KEYGSW.EQ.3)               GOTO 905
        IF(KEYGSW.EQ.0.AND.KEYWLB.EQ.2)               GOTO 913
        IF(KEYWLB.NE.1.AND.KEYWLB.NE.2)               GOTO 905
C
        IF(ISPIN.LT.0.OR.ISPIN.GT.1)                  GOTO 906
        if(npar(15).eq.1.and.itfin.ne.3)              goto 912
        if(npar(15).eq.2.and.itfin.ne.1)              goto 912
        if(npar(15).lt.0.or.npar(15).gt.2)            goto 912
      if((npar(13).ne.0).and.(itfin.lt.501.or.itfin.gt.506)) goto 912
Change KMO start
        IF (ITFIN .LE. 500 .OR. ITFIN .GT. 506) THEN
          IF(ITFIN.EQ.0.OR.ITFIN.GT.3)                GOTO 907
        END IF
Change KMO end
C
C**     CUT OFF VALUES
        IF(KEYRAD.LE.1.AND.VVMAX.GT.1.D0)             GOTO 908
        IF(KEYRAD.LE.1.AND.XK0.LT..0005D0)            GOTO 908
      IF(ITFIN.EQ.1.AND.ITDKRC.NE.0.AND.XK0DEC.LT..00049D0)   GOTO 909
        IF(ITFIN.EQ.3.AND.KEYWLB.NE.1)                GOTO 910
        IF(ITFIN.GT.9.AND.KEYWLB.NE.1)                GOTO 910
        IF(ITFIN.GT.2.AND.KEYYFS.NE.1000001.AND.KEYRAD.GT.10) GOTO 916
        IF(ITFIN.EQ.3.AND.AMNEUT.LT.0.08.AND.VVMAX.GT.0.9999) GOTO 917
C -------------------------------------------------------------------
C INITIALISE MASSES OF STABLE PARTICLES
C
        CALL INIMAS
C
C -------------------------------------------------------------------
C INITIALISE BEAM AND FINAL STATE PARAMETERS
C
        CALL INIREA (KF1,PB1,EE1,KF2,PB2,EE2,ITFIN)
C
C -------------------------------------------------------------------
C INITIALISE PHYSICAL CONSTANTS AND RAD.CORR. PARAMETERS
C
        CALL INIPHY
C INITIALIZE PHOTOS
        CALL PHOINI
        CALL INIPHX(XK0DEC)
C
C PRINT SOME PARAMETERS SET PRIOR TO THIS POINT
        WRITE(IOUT,7005) GAMMZ,AMW,SWSQ,AMFIN
        IF(DABS(SWSQ).GT.1D0) GOTO 905
C
C -------------------------------------------------------------------
C INITIALISATION OF TAU DECAY PACKAGE TAUOLA
C
        IF(ITFIN.EQ.1) THEN
          np1=3
          np2=4
          CALL INITDK
        ENDIF
c initialization of new external matrix element corrections. 
        write(iout,7012)
        call kzphynew(XPAR,NPAR)
        write(iout,7013)
C
C -------------------------------------------------------------------
C INITIALISE EVENTx
C
        CALL EVENTZ(-1,KEYYFS)
C
C --------------------------------------------------------------------
C


      ELSEIF(MODE.EQ.0) THEN
C ================================================    LOOP OVER EVENTS
C
        IEVEN=IEVEN+1
C --------------------------------------------------------------------
C FERMION PAIR PRODUCTION PROCESS
C
        CALL EVENTZ(0,KEYYFS)
C
C --------------------------------------------------------------------
C*MWG*  FILLING HEPEVT RECORD WITH BEAMS FERMIONS AND PHOTON  (ZTOHEP)
C
        DO 125 I=1,4
        POL1(I)=0
 125    POL2(I)=0
        CALL ZTOHEP
C*MWG*
C --------------------------------------------------------------------
C SPIN ASSIGNMENTS
        IF(ISPIN.NE.0) THEN
          CALL SPIGEN(POL1,POL2)
          HELT1= POL1(3)
          HELT2=-POL2(3)
        ENDIF
C
C --------------------------------------------------------------------
C TAU DECAYS
        IF(ITFIN .EQ. 1) THEN
          KTO=1
          CALL DEXAY(KTO,POL1)
          KTO=2
          CALL DEXAY(KTO,POL2)
        IF (ITDKRC.EQ.1) THEN
         CALL PHOTOS(3)
         CALL PHOTOS(4)
        ENDIF
        ENDIF
C
C --------------------------------------------------------------------
C
      ELSEIF(MODE.EQ.1) THEN
C ================================================    FINAL STATISTICS
C
C CALCULATE TOTAL CROSS SECTION
        CALL EVENTZ(1,KEYYFS)
        CSTNB= CSTCM*1.E33
        DCSNB= ERREL*CSTNB
        NPAR(10)=IEVEN
C ZW here input parameters were overwritten 4.06.89
        XPAR(10)=CSTCM
        XPAR(16)=CSTCM
        XPAR(17)=CSTNB
        XPAR(18)=DCSNB
C
C --------------------------------------------------------------------
C CALCULATE tau PARTIAL DECAY WIDTHS
        IF(ITFIN.EQ.1) THEN
          CALL DEXAY(100,POL1)
          DO 180 I=1,19
 180      XPAR(20+I)= GAMPMC(I)
        ENDIF
C
C --------------------------------------------------------------------
C FINAL REPORT
        WRITE(IOUT,7010) IEVEN,CSTNB,DCSNB,ERREL
C
C --------------------------------------------------------------------
      ELSE
C ======================================================= ILLEGAL MODE
        GOTO 901
      ENDIF
C     =====
C
      RETURN
C
  900 WRITE(IOUT,9900)
 9900 FORMAT(' KORALZ: NONSENSE VALUE OF BEAM IDENTIFIER')
      STOP
  901 WRITE(IOUT,9901)
 9901 FORMAT(' KORALZ: NONSENSE VALUE OF MODE ')
      STOP
  902 WRITE(IOUT,9902)
 9902 FORMAT(' KORALZ: NO TRANSV. MOM. ALLOWED FOR BEAMS ')
      STOP
  903 WRITE(IOUT,9903)
 9903 FORMAT(' KORALZ: SOME WRONG BEAM MOM. COMPONENT    ',/,
     $ 'TO FLIP BEAM DIRECTIONS USE KF1, KF2: BEAM IDENTIFIERS')
      STOP
  904 WRITE(IOUT,9904)
 9904 FORMAT(' KORALZ: WRONG BEAM POLARIZATION ')
      STOP
  905 WRITE(IOUT,9905)
 9905 FORMAT(' KORALZ: BAD KEYGSW OR KEYRAD OR KEYWLB ')
      STOP
  906 WRITE(IOUT,9906)
 9906 FORMAT(' KORALZ: BAD ISPIN ')
      STOP
  907 WRITE(IOUT,9907)
 9907 FORMAT(' KORALZ: BAD ITFIN ')
      STOP
  908 WRITE(IOUT,9908)
 9908 FORMAT(' KORALZ: XK0 AND/OR VVMAX OUT OF RANGE FOR CHOSEN KEYRAD')
      STOP
  909 WRITE(IOUT,9909) XK0DEC
 9909 FORMAT(' KORALZ: XK0DEC OUT OF RANGE ',E10.5)
      STOP
  910 WRITE(IOUT,9910)
 9910 FORMAT('KORALZ: combination of ITFIN and KEYWLB not yet possibl.')
      STOP
  912 WRITE(IOUT,9912)
 9912 FORMAT('KORALZ: combin. of ITFIN and IFKALIN / IFLEPTOK wrong.')
      STOP
  913 WRITE(IOUT,9913)
 9913 FORMAT(' KORALZ: Wrong value of NPR(9)=KEYWLB ')
      STOP
  911 WRITE(IOUT,9911) KEYRAD
 9911 FORMAT('KORALZ: Obsolete value of KEYRAD=',I3, // ,' in KORALZ ',
     $'documentation Comp Phys. Comm. 1994 this explanation is missing')
      STOP
  916 WRITE(IOUT,9916)
 9916 FORMAT
     $ (' KORALZ: Wrong combination  of NPR(12)=KEYYFS, NPR(7)=ITFIN ')
      STOP
 917  WRITE(IOUT,9917)
 9917 FORMAT
     $ (' KORALZ: Classical chaos danger, unlucky combinantion of: ',
     $  ' XPR(9)=AMNEUT, NPR(7)=ITFIN, XPR(13)=VVMAX ')
      STOP
C
 7000 FORMAT(///1X,15(5HKKKKK)
     $ /,' K',     25X,'======== KORALZ VERSION 4.04 ==========',9X,1HK
     $ /,' K',     25X,'== Published in Comp Phys Comm (94) ===',9X,1HK
     $ /,' K',     25X,'=last registerd corrections Aug,6, 99=',9X,1HK
     $ /,' K',     25X,'==Authors: S. Jadach, B. Ward =========',9X,1HK
     $ /,' K',     25X,'==========      Z. Was        =========',9X,1HK
     $ /,' K',     25X,'=========== INPUT PARAMETERS ==========',9X,1HK
     $ /,' K',I20  ,5X,'KF1    =  FIRST BEAM IDENTIFIER        ',9X,1HK
     $ /,' K',I20  ,5X,'KF2    =  SECOND BEAM IDENTIFIER       ',9X,1HK
     $ /,' K',     25X,'==== FOUR MOMENTA OF THE BEAMS ========',9X,1HK
     $ /,' K',F20.9,5X,'PB1(3) =  FIRST BEAM, 3-RD COMPONENT   ',9X,1HK
     $ /,' K',F20.9,5X,'PB1(4) =              0-TH COMPONENT   ',9X,1HK
     $ /,' K',F20.9,5X,'PB2(3) = SECOND BEAM, 3-RD COMPONENT   ',9X,1HK
     $ /,' K',F20.9,5X,'PB2(4) =              0-TH COMPONENT   ',9X,1HK
     $ /,' K',     25X,'==== SPIN VECTORS OF THE BEAMS ========',9X,1HK
     $ /,' K',F20.9,5X,'E1(3)  =  FIRST BEAM, 3-RD COMPONENT   ',9X,1HK
     $ /,' K',F20.9,5X,'E2(3)  = SECOND BEAM  3-RD COMPONENT   ',9X,1HK)
 7001 FORMAT(
     $   ' K',     25X,'====== INPUT PARAMS IN NPAR ===========',9X,1HK
     $ /,' K',I20  ,5X,'ISPIN    =  SPIN EFFECTS SWITCH        ',9X,1HK
     $ /,' K',I20  ,5X,'KEYGSW   =  GSW IMPLEMENTATION LEVEL   ',9X,1HK
     $ /,' K',I20  ,5X,'KEYRAD   =  BREMSSTRAHLUNG   SWITCH    ',9X,1HK
     $ /,' K',I20  ,5X,'JAK1     =  DECAY TYPE FIRST  TAU      ',9X,1HK
     $ /,' K',I20  ,5X,'JAK2     =  DECAY TYPE SECOND TAU      ',9X,1HK
     $ /,' K',I20  ,5X,'ITFIN    =  TAU/MUON ...  PROD. SWITCH ',9X,1HK
     $ /,' K',I20  ,5X,'ITDKRC   =  SWITCH FOR RC IN TAU DECAY ',9X,1HK
     $ /,' K',I20  ,5X,'KEYWLB   =  type of electroweak library',9X,1HK
     $ /,' K',I20  ,5X,'NNEUT    =  number of neutrinos (nunu) ',9X,1HK)
 7002 FORMAT(
     $   ' K',     25X,'====== INPUT PARAMS IN XPAR ===========',9X,1HK
     $ /,' K',F20.9,5X,'AMZ  = MASS OF Z0 BOSON                ',9X,1HK
     $ /,' K',F20.9,5X,'AMH  = MASS OF HIGGS BOSON             ',9X,1HK
     $ /,' K',F20.9,5X,'AMTOP= MASS OF TOP QUARK               ',9X,1HK
     $ /,' K',F20.9,5X,'GV   = VECTOR COUPLING CONST. IN DECAY ',9X,1HK
     $ /,' K',F20.9,5X,'GA   = AXIAL  COUPLING CONST. IN DECAY ',9X,1HK
     $ /,' K',F20.9,5X,'AMNUTA= MASS OF TAU-NEUTRINO  IN DECAY ',9X,1HK
     $ /,' K',F20.9,5X,'XK0DEC= SOFT PHOTON CUTOFF IN TAU DEC. ',9X,1HK
     $ /,' K',F20.9,5X,'XK0   = SOFT PHOTON CUT OFF            ',9X,1HK
     $ /,' K',F20.9,5X,'VVMIN = MIN.VAL. OF V PARAM. IN YFS    ',9X,1HK
     $ /,' K',F20.9,5X,'VVMAX = MAX.VAL. OF V PARAM. IN YFS    ',9X,1HK
     $ /,' K',F20.9,5X,'AMNEUT= neutrino (quark) f. state mass ',9X,1HK
     $  /,1X,15(5HKKKKK)/)
 7005 FORMAT(///1X,15(5HKKKKK)
     $ /,' K',     25X,'==== OTHER PARAMETERS SET IN KORALZ====',9X,1HK
     $ /,' K',F20.9,5X,'GAMM   =  Z0 BOSON WIDTH, CALCULATED   ',9X,1HK
     $ /,' K',F20.9,5X,'AMW    =  W  BOSON MASS,  CALCULATED   ',9X,1HK
     $ /,' K',F20.9,5X,'SWSQ   =  SIN**2(THETAWEINBERG)        ',9X,1HK
     $ /,' K',F20.9,5X,'AMFIN  =  OUTGOING FERMION MASS        ',9X,1HK
     $  /,1X,15(5HKKKKK)/)
 7010 FORMAT(///1X,15(5HKKKKK)
     $ /,' K',     25X,'======== KORALZ VERSION 4.04 ==========',9X,1HK
     $ /,' K',     25X,'== Published in Comp Phys Comm (94) ===',9X,1HK
     $ /,' K',     25X,'==Authors: S. Jadach, B. Ward =========',9X,1HK
     $ /,' K',     25X,'==========      Z. Was        =========',9X,1HK
     $ /,' K',     25X,'============= FINAL REPORT ============',9X,1HK
     $ /,' K',I20  ,5X,'Number of generated events             ',9X,1HK
     $ /,' K',F20.9,5X,'Total cross sections in nanobarns      ',9X,1HK
     $ /,' K',F20.9,5X,'Absolute error                         ',9X,1HK
     $ /,' K',F20.9,5X,'Relative error                         ',9X,1HK
     $ /,1X,15(5HKKKKK)/)
 7012 FORMAT(///1X,15(5HKKKKK)
     $ /,' K',     25X,'== initializattion of new physics =====',9X,1HK
     $ /,1X,15(5HKKKKK)/)
 7013 FORMAT(///1X,15(5HKKKKK)
     $ /,' K',     25X,'=======================================',9X,1HK
     $ /,1X,15(5HKKKKK)/)
      END
      SUBROUTINE ZTOHEP
C ----------------------------------------------------------------------
C
C Koral-Z to HEPEVT
C
C this routine fills the HEPEVT common block
C with event kinematics from Koral-Z:
C *  initial beam particles (positions 1,2)
C *  primary fermions of final state (3,4)
C *  radiative photons (IF NPHOTA>0) (5...4+NPHOTA)
C
C WRITTEN BY MARTIN W. GRUENEWALD AND Z. WAS (91/02/02)
C
C called by KORAL-Z
C
C ----------------------------------------------------------------------
C
      COMMON / MOMSE4 / AQF1(4),AQF2(4),ASPHUM(4),ASPHOT(100,4),NPHOTA
      REAL*4            AQF1   ,AQF2   ,ASPHUM   ,ASPHOT
      COMMON / BEAMS / XPB1(4),XPB2(4),KFB1,KFB2
      REAL*4           XPB1   ,XPB2
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / IDFC  / IDFF
C
      REAL*4 APH(4),AM
C
C initial state (1,2)
      AM=AMIN
      CALL FILHEP(1,3,KFB1,0,0,0,0,XPB1,AM,.TRUE.)
      CALL FILHEP(2,3,KFB2,0,0,0,0,XPB2,AM,.TRUE.)
C
C primary final state fermions (3,4)
      AM=AMFIN
      CALL FILHEP(3,1, IDFF,1,2,0,0,AQF1,AM,.TRUE.)
      CALL FILHEP(4,1,-IDFF,1,2,0,0,AQF2,AM,.TRUE.)
C
C radiative photons (5...4+NPHOTA) (PDG-code for gamma is 22)
      IF(NPHOTA.EQ.0)RETURN
      IP=0
      DO I=1,NPHOTA
        DO J=1,4
          APH(J)=ASPHOT(I,J)
        END DO
        IF (APH(4).GT.0.0) THEN
          IP=IP+1
          CALL FILHEP(4+IP,1,22,1,2,0,0,APH,0.0,.TRUE.)
        END IF
      END DO
C
      RETURN
      END
      SUBROUTINE EVENTZ(MODE,KEYYFS)
C ----------------------------------------------------------------------
C STEERING ROUTINE MAKES A CHOICE BETWEEN MULTIPHOTON AND SINGLE
C (OR ZERO) PHOTON GENERATOR
C
C     called by : KORALZ
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
C
      IF     (KEYRAD.EQ.0.OR.KEYRAD.EQ.1.OR.KEYRAD.EQ.10) THEN
        CALL EVENTM(MODE)
      ELSEIF (KEYRAD.GT.10.AND.KEYRAD.LT.113) THEN
        CALL EVENTE(MODE,KEYYFS)
      ELSE
        PRINT *, 'STOP IN EVENTZ - WRONG KEYRAD'
        STOP
      ENDIF
      END
      SUBROUTINE SPIGEN(POL1,POL2)
C ----------------------------------------------------------------------
C THIS ROUTINE GENERATES TAU SPIN CONFIGURATION
C output arguments :
C POL1,POL2 = tau polarisation
C
C     called by : KORALZ
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      REAL*4 POL1(4),POL2(4)
      REAL*4 RRR
      DIMENSION SIGM(4)
      DATA  INIT /0/
C
      IF (INIT.EQ.0) THEN
        INIT=1
        DO 20 K=1,4
        POL1(K)=0.0
 20     POL2(K)=0.0
      ENDIF
C CALCULATE RELATIVE WEIGHTS OF TAU SPIN CONFIGURATIONS
      IF (KEYRAD.LT.11) THEN
        SIGM(1)=WEIGHT(1, 1D0, 1D0)
        SIGM(2)=WEIGHT(1,-1D0,-1D0)
        SIGM(3)=WEIGHT(1, 1D0,-1D0)
        SIGM(4)=WEIGHT(1,-1D0, 1D0)
      ELSE
        SIGM(1)=WAGA(1, 1D0, 1D0)
        SIGM(2)=WAGA(1,-1D0,-1D0)
        SIGM(3)=WAGA(1, 1D0,-1D0)
        SIGM(4)=WAGA(1,-1D0, 1D0)
      ENDIF
      XSUM=SIGM(1)+SIGM(2)+SIGM(3)+SIGM(4)
C CHOOSE RANDOMLY SPINS OF TAUS ACCORDING TO CROSS SECTION
      CALL RANMAR(RRR,1)
      R=RRR
      IF(    R.LT. SIGM(1)/XSUM) THEN
        T1= 1D0
        T2= 1D0
      ELSEIF(R.LT.(SIGM(1)+SIGM(2))/XSUM) THEN
        T1=-1D0
        T2=-1D0
      ELSEIF(R.LT.(SIGM(1)+SIGM(2)+SIGM(3))/XSUM) THEN
        T1= 1D0
        T2=-1D0
      ELSE
        T1=-1D0
        T2= 1D0
      ENDIF
      POL1(3)=T1
      POL2(3)=T2
      END
      FUNCTION WAGA(MODE,TA,TB)
C ----------------------------------------------------------------------
C THIS FUNCTION CALCULATES DIFFERENTIAL CROSS SECTION FOR EVERY EVENT
C in multiphoton case (see WEIGHT for single photon mode).
C  INPUT : MODE:
C                 0   FOR CALL FOR RAW CROSS SECTION
C                     NO VAC. POLARIZ & NO GSW &
C                     NO INTERFERENCE INIT - FINAL
C                 1   INITIALIZATION. CORRECT X. SECTION
C                 2   CORRECT X. SECTION
C          TA,TB: TWICE OF TAU+ TAU- HELICITIES
C
C     called by : SPIGEN
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
C
C IF MODE=0 CALCULATION OF ANGULAR VARIABLES FROM 4-MOMENTA
      IF (MODE.EQ.0) THEN
        CALL SKONTY(0,SVAR,CTHE)
      ENDIF
      WAGA=BORNS(MODE,SVAR,CTHE,TA,TB)
      if (mode.ne.0.and. TA*TA.LT.0.5D0) waga=waga*wtanom(dumm)
      END
      FUNCTION BORNY(SVAR)
C ----------------------------------------------------------------------
C THIS ROUTINE CALCULATES TOTAL BORN CROSS SECTION.
C IT EXPLOITS THE FACT THAT BORN X. SECTION = A + B*C + D*C**2
C
C     called by : BONMA0, BONMA1, EXPAND, BREMKF, VVDIS
C ----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C
      FL=BORNS(0,SVAR,-1.D0,0.D0,0.D0)
      FR=BORNS(0,SVAR, 1.D0,0.D0,0.D0)
      F0=BORNS(0,SVAR, 0.D0,0.D0,0.D0)
      BORNY=(FL+FR)/8.D0+ F0/2D0
      END
      FUNCTION BORAS(SVAR)
C ----------------------------------------------------------------------
C THIS ROUTINE CALCULATES  BORN ASYMMETRY.
C IT EXPLOITS THE FACT THAT BORN X. SECTION = A + B*C + D*C**2
C
C     called by : EVENTM
C ----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
C
      FL=BORNS(0,SVAR,-1.D0,0.D0,0.D0)
      FR=BORNS(0,SVAR, 1.D0,0.D0,0.D0)
      BORAS=FL/(FL+FR)
      END
      FUNCTION BORNV(SVAR,COSTHE)
C ----------------------------------------------------------------------
C     called by : GCRUDE, GBETA0, GBETA1, GBETA2
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      BORNV=   BORNS(0,SVAR,COSTHE,0D0,0.D0)
      END
      SUBROUTINE TRALO4(KTO,AP,BP,XMP)
C ----------------------------------------------------------------------
* REAL*4 VERSION OF TRALOR
C     Modified AM. Lutz October 1988
C  INPUT PARAMETERS :
C     KTO    : tau number
C     AP     : 4-momentum to be boosted/rotated
C  output parameters
C     BP     : 4-momentum after boost/rotation
C     XMP    : mass corresponding to 4-momentum BP
C              computed in double precision
C
C     called by : DWLUxx
C ----------------------------------------------------------------------
      REAL*4 AP(4),BP(4),XMP
      REAL*8 DP(4),DXMP

C
      DO 30 I=1,4
  30  DP(I)=AP(I)
      CALL TRALOR(KTO,DP,DP)
      DXMP =0D0
      DO 31 I=1,3
      DXMP =DXMP + DP(I)**2
  31  BP(I)=DP(I)
      BP(4)=DP(4)
      DXMP =DP(4)**2-DXMP
C ZW 21.03 TO AVOID AN OVERFLOW WHEN PHOTON MASS IS CALCULATED.
       IF(DXMP.NE.0D0) DXMP=DXMP / DSQRT(DABS(DXMP))
      XMP  =DXMP
      END
      SUBROUTINE TRALOR(KTO,QQ,PP)
C ----------------------------------------------------------------------
C THIS TRANSFORMS FROM FINAL FERMION REST FRAME TO LAB CMS
C KTO=1 FOR QP, KTO=2 FOR QM
C ADDITIONAL PARAMETER, NOW FROZEN AS DATA, >MODE< IF PUT TO ZERO
C SHORTENS THE TRANSFORMATION TREE. FOR MODE.NE.1, TRANSFORMATION
C GOES FROM THE REST FRAME OF THE FINAL STATE FERMION PAIR TO LAB.
C IT IS RECCOMENDED TO USE WHEN THE PROGRAM IS INTERFACED WITH THE
C QUARK FRAGMENTATION AND/OR Q-ONIA DECAY PROGRAMS.
C
C     called by : TRALO4, EVENTM
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / UTIL2 /  XK,C,S,CG,SG
      COMMON / UTIL3 / FIG,FI,IT,IBREM
      REAL*8 PP(4),QQ(4)
      DATA PI /3.141592653589793238462643D0/
      DATA MODE /1/
      IF(KEYRAD.LE.10) THEN
C       SINGLE PHOTON CASE
        CALL TRASNG(MODE,ENE,AMFIN,AMEL,KTO,QQ,PP)
      ELSE
C       MULTIPLE PHOTON CASE
        DO 7 K=1,4
 7      PP(K)=QQ(K)
        CALL TRAMLT(MODE,KTO,PP)
      ENDIF
C     =====
      END
      SUBROUTINE TRAMLT(MODE,KTO,VEC)
C ----------------------------------------------------------------------
C     *   MULTIPHOTON KINEMATICAL TREE
C
C     called by : TRALOR
C ----------------------------------------------------------------------
C
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT  
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      DIMENSION VEC(4),PPB(4),PMB(4),PP(4),QQ(4)
      PARAMETER(ISWITH=1000001)
C
      DO 8 K=1,4
  8   QQ(K)=VEC(K)
      IF ((KEYRAD.EQ.12)) THEN
C       relax
      ELSE
        CALL BDRESS(1,PPB,PMB,SVARX,CTHE)
        EBEAM=SQRT(SVARX)/2D0
      ENDIF
      CALL TRASNG(MODE,EBEAM,AMFIN,AMIN,KTO,QQ,PP)
      CALL TRALOZ(1,-1,2D0*ENE  ,SPHUM,PPB,PMB,PP  ,QQ  )
C
      DO 9 L=1,4
  9   VEC(L)=QQ(L)
      END
      SUBROUTINE TRALOZ(INIT,MOD,CMSENE,PHSUM,PPB,PMB,QVEC,PVEC)
C ----------------------------------------------------------------------
C CALLED IN EVENTE
C TRANSFORMS FROM CMS TO VIRTUAL Z0 (GAMMA) REST FRAME
C FORTH (MODE= 1) OR BACK (MODE=-1)
C IN FINAL   SYSTEM Z-AXIS PARALLEL TO DRESSED POSITRON BEAM
C
C     called by : PEDYVV, TRAMLT, EVENTE
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(ISWITH=1000001)
      REAL*8 PVEC(4),QVEC(4),PHSUM(4)
      REAL*8 PPB(4),PMB(4)
      COMMON / VVREK  / VVMIN,VVMAX,VV,BETI
      REAL*8            VVMIN,VVMAX,VV,BETI
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT  
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      IF (KEYRAD.EQ.12) THEN
          DO K=1,4
             PVEC(K)=QVEC(K)
          ENDDO
      ELSE
       CALL TRALOO(INIT,MOD,CMSENE,PHSUM,PPB,PMB,QVEC,PVEC)
      ENDIF
C       note that TRALOO is identical to
C       old TRALOZ backward compatible to KORALZ 3.8 may be removed
C       later after comparison tests. It cannot be used for the
C       YFS3 generating final bremsstrahlung.
      END
      SUBROUTINE TRALOO(INIT,MOD,CMSENE,PHSUM,PPB,PMB,QVEC,PVEC)
C ----------------------------------------------------------------------
C CALLED IN EVENTE
C TRANSFORMS FROM CMS TO VIRTUAL Z0 (GAMMA) REST FRAME
C FORTH (MODE= 1) OR BACK (MODE=-1)
C IN FINAL   SYSTEM Z-AXIS PARALLEL TO DRESSED POSITRON BEAM
C
C     called by : PEDYVV, TRAMLT, EVENTE
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PVEC(4),QVEC(4),PHSUM(4),QSU(4)
      REAL*8 PPB(4),PMB(4),PPBT(4),PMBT(4)
      COMMON / VVREK  / VVMIN,VVMAX,VV,BETI
      REAL*8            VVMIN,VVMAX,VV,BETI
C
C ----------------------------------------------------------------------
C INITIALIZATION
      IF (INIT.EQ.0) THEN
        NPH=0
        IF (PHSUM(4).GT.VVMIN*CMSENE/2D0) THEN
C       ======================================
C
          NPH=1
          DO 10 K=1,4
          PPBT(K)=PPB(K)
          PMBT(K)=PMB(K)
   10     PVEC(K)=QVEC(K)
          CALL GIQSU(CMSENE,PHSUM,QSU)
          QSM =SQRT(QSU(4)**2-QSU(3)**2-QSU(2)**2-QSU(1)**2)
          EXE =(QSU(4)+SQRT(QSU(3)**2+QSU(2)**2+QSU(1)**2))/QSM
          FI   =ANGFI(QSU(1),QSU(2))
          TH1  =ANGXY(QSU(3),SQRT(QSU(1)**2+QSU(2)**2))
          CALL ROTOD3(    -FI,PPBT,PPBT)
          CALL ROTOD2(   -TH1,PPBT,PPBT)
          CALL BOSTD3(1D0/EXE,PPBT,PPBT)
          TH2  =-ANGXY(PPBT(3),SQRT(PPBT(1)**2+PPBT(2)**2))
C
        ENDIF
C       =====
      ENDIF
C ----------------------------------------------------------------------
C
C TRANSFORMATION
      DO 12 K=1,4
  12  PVEC(K)=QVEC(K)
      IF (NPH.EQ.1) THEN
C     ==================
        IF(    MOD.EQ.-1) THEN
          CALL ROTOD2(    TH2,PVEC,PVEC)
          CALL BOSTD3(    EXE,PVEC,PVEC)
          CALL ROTOD2(    TH1,PVEC,PVEC)
          CALL ROTOD3(     FI,PVEC,PVEC)
        ELSEIF(MOD.EQ. 1) THEN
          CALL ROTOD3(    -FI,PVEC,PVEC)
          CALL ROTOD2(   -TH1,PVEC,PVEC)
          CALL BOSTD3(1D0/EXE,PVEC,PVEC)
          CALL ROTOD2(   -TH2,PVEC,PVEC)
        ENDIF
C
      ELSE
C     ====
C
        DO 11 K=1,4
   11   PVEC(K)=QVEC(K)
C
      ENDIF
C     =====
C
      END
      SUBROUTINE TRASNG(MODE,ENE,AMFIN,AMEL,KTO,QQ,PP)
C ----------------------------------------------------------------------
C THIS TRANSFORMS FROM FINAL FERMION REST FRAME TO LAB CMS
C (OR Z0-GAMMA REST FRAME WHEN USED FOR MULTIPHOTON GENERATION)
C KTO=1 FOR QP, KTO=2 FOR QM
C SINGLE PHOTON KINEMATICAL TREE. IT IS USED FOR THE SINGLE
C OR NO PHOTON TRANSFORMATION, EITHER IN THE SINGLE BREM. MODE
C OR FOR THE FINAL STATE BREMSSTRAHLUNG BRANCH OF THE MULTIPHOTON
C GENERATION.
C
C     CALLED BY : TRAMLT, EVENTE,TRALOR
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(ISWITH=1000001)
      COMMON / KEYYFS / KEYZET,KEYBRM,KEYFIX,KEYRED,KEYWGT  
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      REAL*8 PP(4),QQ(4)
      IF (KEYRAD.EQ.12) THEN
          CALL TRANSF(KTO,QQ,PP)
      ELSE
        CALL TRASNO(MODE,ENE,AMFIN,AMEL,KTO,QQ,PP)
      ENDIF
C       note that TRASNO is identical to old TRASNG
C       old TRASNG backward compatible to KORALZ 3.8 may be removed
C       later after comparison tests. It cannot be used for the
C       YFS3 generating final bremsstrahlung.
      END
      SUBROUTINE TRANSF(KTO,QQ,PP)
C new transformation routine to be used when YFS3 with final state
C bremsstrahlung is in use.
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PP(4),QQ(4),RR(4),ZMS(4),F1(4),F2(4),SS(4),F0(4)
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
C      COMMON / MOMINI / XF1(4),XF2(4),XPHUM(4),XPHOT(100,4),NPHOX
      DATA PI /3.141592653589793238462643D0/
      DO KK=1,4
        ZMS(KK)=QF1(KK)+QF2(KK)
         F0(KK)=0.D0
      ENDDO
      IF(KTO.EQ.1) THEN
         CALL BOSTDQ( 1,ZMS,QF1,F1)
          F0(4)=F1(4)
          F0(3)= SQRT(F1(1)**2+F1(2)**2+F1(3)**2)
            FI=ANGFI(F1(1),F1(2))
          TH=ANGXY(F1(3),SQRT(F1(1)**2+F1(2)**2))
         CALL BOSTDQ(-1, F0, QQ,SS)
          CALL ROTOD2( TH,SS,SS)
          CALL ROTOD3( FI,SS,RR)
         CALL BOSTDQ(-1,ZMS, RR,PP)
      ELSE
         CALL ROTOD1(PI,QQ,SS)
         CALL BOSTDQ( 1,ZMS,QF2,F2)
          F0(4)=F2(4)
          F0(3)=SQRT(F2(1)**2+F2(2)**2+F2(3)**2)
          FI=ANGFI( F2(1), F2(2))
          TH=ANGXY( F2(3),SQRT(F2(1)**2+F2(2)**2))
         CALL BOSTDQ(-1, F0, SS,RR)
          CALL ROTOD2( TH,RR,RR)
          CALL ROTOD3( FI,RR,RR)
         CALL BOSTDQ(-1,ZMS, RR,PP)
      ENDIF
      END
      SUBROUTINE TRASNO(MODE,ENE,AMFIN,AMEL,KTO,QQ,PP)
C ----------------------------------------------------------------------
C THIS TRANSFORMS FROM FINAL FERMION REST FRAME TO LAB CMS
C (OR Z0-GAMMA REST FRAME WHEN USED FOR MULTIPHOTON GENERATION)
C KTO=1 FOR QP, KTO=2 FOR QM
C SINGLE PHOTON KINEMATICAL TREE. IT IS USED FOR THE SINGLE
C OR NO PHOTON TRANSFORMATION, EITHER IN THE SINGLE BREM. MODE
C OR FOR THE FINAL STATE BREMSSTRAHLUNG BRANCH OF THE MULTIPHOTON
C GENERATION.
C
C     CALLED BY : TRAMLT, EVENTE,TRALOR
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / UTIL2 /  XK,C,S,CG,SG
      COMMON / UTIL3 / FIG,FI,IT,IBREM
      REAL*8 PP(4),QQ(4)
      DATA PI /3.141592653589793238462643D0/

        IF(IT.EQ.3) THEN
          BET=SQRT(1-AMFIN**2/ENE**2)
          EXF=(1+BET)/SQRT(AMFIN**2/ENE**2)
        ELSE
          BET=1-AMFIN**2/(1-XK)/ENE**2
          IF(BET.LT.0D0) BET=0D0
          BET=SQRT(BET)
          EXF=(1+BET)/SQRT(AMFIN**2/(1-XK)/ENE**2)
        ENDIF
C
        IF (MODE.EQ.1) THEN
C TRANSFORMATION FROM THE TAU REST FRAME TO THE TAU-TAU SYSTEM
          IF(KTO.EQ.1) CALL ROTOD2(0D0,QQ,PP)
          IF(KTO.EQ.2) CALL ROTOD2( PI,QQ,PP)
C
          CALL BOSTD3( EXF,PP,PP)
          IF(KTO.EQ.2) CALL ROTOD2( PI,PP,PP)
        ENDIF
C
        IF(IT.EQ.1) THEN
          TH1=ANGXY(CG,SG)
          EXE=  SQRT(1-XK)
          TH2=ANGXY(C,S)
          IF(IBREM.EQ.1) THEN
            THR =ANGXY( XK+(2-XK)*CG , 2*SQRT(1-XK)*SG )
            CALL TRALOI( FI, TH1,EXE,-THR, FIG, TH2,PP)
          ELSE
            THR =ANGXY( XK-(2-XK)*CG , 2*SQRT(1-XK)*SG )
            CALL TRALOI( FI, TH1,EXE,-PI+THR, FIG,-TH2,PP)
          ENDIF
C
        ELSEIF(IT.EQ.2) THEN
          TH=ANGXY(C,S)
          EXE=  SQRT(1-XK)
          THG=ANGXY(CG,SG)
          BET=SQRT(1-AMFIN**2/ENE**2/(1-XK))
          IF(IBREM.EQ.1) THEN
            THR=ANGXY( 2*CG*BET-XK*CG*BET-XK ,2*SQRT(1-XK)*SG*BET )
            CALL TRALOF( FI,  TH, FIG, THR,EXE,-THG,PP)
          ELSE
            THR=ANGXY(-2*CG*BET+XK*CG*BET-XK ,2*SQRT(1-XK)*SG*BET )
            CALL TRALOF( FI,  TH,-FIG,-PI+THR,EXE, THG,PP)
          ENDIF
        ELSE
          TH=ANGXY(C,S)
          CALL ROTOD1(-TH,PP,PP)
          CALL ROTOD3(-FI,PP,PP)
        ENDIF
      END
      SUBROUTINE TRALOI(FI,TH1,EX,THR,PSI,TH2,PVEC)
C ----------------------------------------------------------------------
C     INITIAL  STATE RADIATION BRANCH
C
C       CALLED BY : TRASNG
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PVEC(4)
C
      CALL ROTOD2( TH2,PVEC,PVEC)
      CALL ROTOD3( PSI,PVEC,PVEC)
      CALL ROTOD2( THR,PVEC,PVEC)
      CALL BOSTD3(  EX,PVEC,PVEC)
      CALL ROTOD2( TH1,PVEC,PVEC)
      CALL ROTOD3(  FI,PVEC,PVEC)
      END
      SUBROUTINE TRALOF(FI,TH1,FI2,THR,EX,THG,PVEC)
C ----------------------------------------------------------------------
C     FINAL  STATE RADIATION BRANCH
C
C       CALLED BY : TRASNG
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PVEC(4)
C
      CALL ROTOD2( THG,PVEC,PVEC)
      CALL BOSTD3(  EX,PVEC,PVEC)
      CALL ROTOD2( THR,PVEC,PVEC)
      CALL ROTOD3( FI2,PVEC,PVEC)
      CALL ROTOD2( TH1,PVEC,PVEC)
      CALL ROTOD3(  FI,PVEC,PVEC)
      END
      SUBROUTINE EVENTM(MODE)
C ----------------------------------------------------------------------
C SINGLE PHOTON GENERATOR
C
C     called by : EVENTZ
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / GAUSPM /SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI   ,XUPZI   ,XUPGF   ,XUPZF
     &                ,NDIAG0,NDIAGA,KEYA,KEYZ
     &                ,ITCE,JTCE,ITCF,JTCF,KOLOR
      REAL*8           SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI(2),XUPZI(2),XUPGF(2),XUPZF(2)
      COMMON / FINUS / CSTCM,ERREL
      REAL*8           CSTCM,ERREL
      COMMON / UTIL2 /  XK,C,S,CG,SG
      COMMON / UTIL3 / FIG,FI,IT,IBREM
      COMMON / UTIL4 / AQP(4),AQM(4),APH(4)
      REAL*4           AQP   ,AQM   ,APH
      COMMON / UTIL8 / QP(4),QM(4),PH(4)
      REAL*8           QP   ,QM   ,PH
      COMMON / VVREK  / VVMIN,VVMAX,VV,BETI
      REAL*8            VVMIN,VVMAX,VV,BETI
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / MOMSE4 / AQF1(4),AQF2(4),ASPHUM(4),ASPHOT(100,4),NPHOTA
      REAL*4            AQF1   ,AQF2   ,ASPHUM   ,ASPHOT
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      REAL*8 SWT(3)
      REAL*4 RRR(4)
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
      DATA CMTR /389.385D-30/
C
C GENERATION OF THE PHOTON MOMENTUM XK AND DECISION ABOUT THE
C TYPE OF THE RADIATION  IT=1,2,3 CORRESPONDS TO INITIAL STATE
C RADIATION, FINAL STATE RADIATION AND SOFT PHOTON (XK.LT.XK0)
C RADIATION
C
      IF (MODE.EQ.-1) THEN
C     INITIALIZATION IN EVENTM
C     ====================
CAM     VVMIN =1D-5
CAM     VVMAX =1D0
        DO 15 K=1,3
  15    SWT(K)=0.D0
        PI=4.D0*DATAN(1.D0)
        ALF1=ALFPI
        IF(KEYRAD.EQ.0.OR.KBRINI.EQ.0) THEN
          BETI=0D0
          DELI=0D0
        ELSE
          QE2=QE*QE
          BILG=DLOG(4D0*ENE**2/AMIN**2)
          BETI=2D0*QE2*ALF1*(BILG-1D0)
          DELI=ALF1*QE2*(9D0*BILG+2D0*PI**2-12D0)/6D0
     $        +BETI*LOG(XK0)
        ENDIF
        IF(KEYRAD.EQ.0.OR.KEYRAD.EQ.10.OR.KBRFIN.EQ.0) THEN
          FACTOR=0D0
        ELSE
          FACTOR=1D0/(1.D0+DELI)
          FACTOR=FACTOR*QF*QF
        ENDIF
C INITIALISE QED CORRECTIONS
        DUMMY=FUNSKA(-1,0.2D0)
        CALL VESK8A(-1,X,Y)
C
      ELSEIF (MODE.EQ.0) THEN
C     =======================
    1   CONTINUE
        CALL RANMAR(RRR,4)
C PHOTON ENERGY GENERATION - INITIAL BREMSSTRAHLUNG
        CALL VESK8A(0,XK,PAR2)
        IF (KEYRAD.EQ.10) XK=VV
        IF (XK.GT.XK0) THEN
          IT=1
        ELSE
          IT=3
        ENDIF
C
C PHOTON ENERGY GENERATION - FINAL BREMSSTRAHLUNG
        IF (XK.LE.XK0) THEN
 333      XK=ALTPAR(4D0*ENE**2,AMFIN,XK0,FACTOR)
          IF (XK.GT.1D0-AMFIN**2/ENE**2) GO TO 333
          IF(XK.GT.XK0) IT=2
        ENDIF
C
C GENERATION OF THE ANGULAR VARIABLES
        IF(IT.EQ.3) THEN
C         SOFT PHOTON CASE  XK.LT.XK0
          R=BORAS(4D0*ENE**2)
C         ANGULAR DISTRIBUTION GENERATION
          CALL RRR7(R,C,S)
        ELSEIF (IT.EQ.1) THEN
C         HARD PHOTON CASE - INITIAL STATE
C         GENERATION OF THE PHOTON ANGULAR  DISTRIBUTION
          CALL RRR6(XK,   AMIN**2/ENE**2,CG,SG,IBREM)
C         GENERATION OF THE FINAL FERMION DISTRIBUTION
          R=BORAS(4D0*ENE**2*(1D0-XK))
          CALL RRR7(R,C,S)
C         GENERATION OF THE ANGLE BETWEEN PHOTON-BEAM AND PHOTON-TAU PL
          RR1=RRR(1)
          FIG=-PI+2D0*PI*RR1
        ELSEIF ( IT.EQ.2) THEN
C         HARD PHOTON CASE - FINAL STATE
C         PHOTON ANGULAR COORDINATES GENERATION
          CALL RRR6(XK,   AMFIN**2/ENE**2/(1D0-XK),CG,SG,IBREM)
          R=BORAS(4D0*ENE**2)
C         FERMION ANGULAR COORDINATES
          CALL RRR7(R,C,S)
C         GENERATION OF THE ANGLE BETWEEN PHOTON-BEAM AND PHOTON-TAU PL
          RR2=RRR(2)
          FIG=-PI+2D0*PI*RR2
        ENDIF
C
C GENERATION OF THE ANGLE AROUND THE BEAM
        RR3=RRR(3)
        FI=-PI+2D0*PI*RR3
C CALCULATION OF 4 MOMENTA FROM ANGULAR VARIABLES
        QP(4)=AMFIN
        PH(4)=2.D0*ENE
        QM(4)=AMFIN
        DO 103 I=1,3
        PH(I)=.0D0
        QP(I)=.0D0
  103   QM(I)=.0D0
        CALL TRALOR(1,QP,QP)
        CALL TRALOR(2,QM,QM)
C
C INITIAL STATE BREMSSTRAHLUNG - PHOTON 4-MOMENTUM
        IF(IT.EQ.1) THEN
C CALCULATED DIRECTLY FROM ANGLES
          PH(4)=XK    *ENE
          PH(3)=XK*CG *ENE
          PH(2)=0.D0  *ENE
          PH(1)=XK*SG *ENE
          CFI=COS(FI)
          SFI=SIN(FI)
          PH1    =CFI*PH(1)-SFI*PH(2)
          PH(2)=SFI*PH(1)+CFI*PH(2)
          PH(1)=PH1
        ELSE
          DO 304 I=1,4
  304     PH(I)=PH(I)-QP(I)-QM(I)
        ENDIF
C FILLING EXTRA COMMONS
        NPHOT=1
        IF (IT.EQ.3) NPHOT=0
        DO 40 I=1,4
        DO 40 J=1,100
 40     SPHOT(J,I)=0.D0
        DO 30 I=1,4
        QF1(I)=QP(I)
        QF2(I)=QM(I)
        SPHUM(I)=PH(I)
        SPHOT(1,I)=PH(I)
        AQP(I)=QP(I)
        AQM(I)=QM(I)
 30     APH(I)=PH(I)
        NPHOTA=NPHOT
        DO 35 I=1,4
        AQF1(I)=QF1(I)
        AQF2(I)=QF2(I)
        ASPHUM(I)=SPHUM(I)
        DO 36 II=1,100
 36     ASPHOT(II,I)=SPHOT(II,I)
 35     CONTINUE
C
        WT =WEIGHT(0,0.0D0,0.D0)
        WT =WEIGHT(1,0.D0,0.D0)/WT
        IF (WT.GT.4.D0) THEN
         WRITE(6,*) 'KUKU FROM EVENTM WT=',WT,'IT=',IT,'XK=',XK
         IF(IT.NE.3) THEN
          FUNTA=FUNTIH(0,0.D0,0.D0)
          FUNTB=FUNTIH(1,0.D0,0.D0)
          WRITE(6,*) 'FUNT(0)=',FUNTA,'FUNT(1)=',FUNTB
         ENDIF
            CALL SANGLE(1,SVAR,CTHE)
          GSWF0=BORNS(0,SVAR,CTHE,0D0,0D0)
          GSWFA=BORNS(1,SVAR,CTHE,0D0,0D0)
          WRITE(6,*) 'nev=',swt(1),'svar=',svar,'cthe=',cthe
          WRITE(6,*) 'GSWF0=',GSWF0,'GSWFA=',GSWFA
C         STOP
        ENDIF
        SWT(1)=SWT(1)+1D0
        SWT(2)=SWT(2)+WT
        SWT(3)=SWT(3)+WT*WT
C IMPOSING INTERFERENCE TERM ON THE GENERATED DISTRIBUTION
        RN=RRR(4)
        IF(RN.GT.WT/4D0) GO TO 1
C
      ELSEIF (MODE.EQ. 1) THEN
C     ========================
        CALL VESK8A( 1,SIGT,ERR)
        AWT =SWT(2)/SWT(1)
        DWT =SQRT(SWT(3)/SWT(2)**2-1D0/SWT(1))
        WRITE(6,*) 'SIGT=',SIGT,'ERR=',ERR
        WRITE(6,*) 'AWT=',AWT,'+-',DWT
        CSTOT=SIGT*AWT
        NTOT=INT(SWT(1))
        SIG0=4.D0*PI/ALFINV**2/3.D0/(4D0*ENE**2)
        CSTCM=SIGT*AWT*SIG0*CMTR
        ERREL=SQRT(DWT**2+ERR**2)
      ENDIF
C     =====
      END
      FUNCTION ALTPAR(TRANSF,AMFERM,EPSIL,FACTOR)
C ----------------------------------------------------------------------
C THIS FUNCTION RETURNS PHOTON MOMENTUM (IN SQRT(S)/2 UNITS)
C FOR THE FINAL STATE BREMSSTRAHLUNG.
C HARD BREMSSTRAHLUNG PART OF THE DISTRIBUTION CORRESPONDS TO  PRECISE
C SINGLE BREMSSTRAHLUNG (ORDER ALPHA) RESULT AND NO-PHOTON PROBABILITY
C IS ADJUSTED SUCH THAT THE INTEGRAL OVER ENTIRE DISTRIBUTION IS EQUAL
C TO ONE PRECISELY (AS IN ALTARELLI-PARISI CASE).
C IN THE LEADING LOG THE DISTRIBUTION COINCIDES WITH THE ALTARELLI-
C PARISI DISTRIBUTION WHERE FERMION IS FRAGMENTING COLLINEARLY INTO A
C PHOTON AND FERMION.
C
C    INPUT:
C TRANSF   = TOTAL C.M. ENERGY SQUARED (GEV**2)
C EPSIL    = INFRARED CUTOFF PARAMETER, DIMENSIONLESS, SMALL.
C AMFERM   = FERMION MASS (GEV)
C FACTOR   = NORMALLY EQUAL 1, OTHERWISE IT MAY BE USED TO REGULATE
C            THE STRENGTH OF THE BREMSTRAHLUNG (IT MULTIPLIES ALPHA).
C    OUTPUT:
C RETURNED IS A FRACTION OF THE FERMION MOMENTUM  CARRIED BY PHOTON,
C DIMENSIONLESS, IN THE RANGE (0,1).
C
C     called by : EVENTE, EVENTM
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      REAL*4 RRN1,RRR(2)
      DATA PI /3.141592653589793238462643D0/
C
C PROBABILITIES FOR HARD AND SOFT BREMSTRAHLUNG
      BILG= LOG(TRANSF/AMFERM**2)
      PRHARD= 2D0 *ALFPI *(BILG-1D0) *( LOG(1D0/EPSIL)-.75D0 )
C NONLEADING TERM  FROM LOG(1-XK) PART OF DISTRIBUTION
      PRHARD=PRHARD + ALFPI *( -PI**2/3D0 +1.25D0)
      PRHARD=PRHARD*FACTOR
      PRSOFT=1D0-PRHARD
      IF(PRSOFT.LT.0.10D0) GOTO 600
      CALL RANMAR(RRN1,1)
      RN1=RRN1
      IF(RN1.LT.PRSOFT) THEN
C        NO PHOTON....  (IE. PHOTON BELOW EPSILON)
         XK=0D0
      ELSE
C        HARD PHOTON...  (IE. PHOTON ABOVE EPSILON)
  200    CONTINUE
         CALL RANMAR(RRR,2)
         RN2=RRR(1)
         RN3=RRR(2)
         XK=  EXP(RN2* LOG(EPSIL))
         DIST0= 1D0/XK *(BILG-1D0)
         DIST1= (1D0+(1D0-XK)**2)/2D0/XK *(BILG +LOG(1D0-XK) -1D0)
         WEIGHT=DIST1/DIST0
         IF(RN3.GT.WEIGHT) GO TO 200
      ENDIF
      ALTPAR=XK
      RETURN
  600 PRINT 1600,TRANSF,PRSOFT
 1600 FORMAT(' ++++++++++++++  STOP IN ALTPAR:',
     $     /,'             TRANSF,PRSOFT,=' ,2F20.5)
      STOP
      END
      FUNCTION FUNSKA(MODE,X)
C ----------------------------------------------------------------------
C CALLED IN VESKO2
C PROVIDES V OR K DISTRIBUTION TO BE GENERATED
C
C     called by : EVENTM, VESK8A, DESK8A
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / INOUT / INUT,IOUT
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
C
      IF(KEYRAD.EQ.0) THEN
        BLE=BONMA0(MODE,X)
        IF (X.GT.XK0) BLE=0.D0
        FUNSKA=BLE
      ELSEIF(KEYRAD.EQ.1) THEN
        IF(KBRINI.EQ.0) THEN
          BLE=BONMA0(MODE,X)
          IF (X.GT.XK0) BLE=0.D0
          FUNSKA=BLE
        ELSE
          BLE=BONMA1(MODE,X)
          IF (X.GT.XK0) BLE=BLE *(1+(1-X)**2)/2D0
          FUNSKA=BLE
        ENDIF
      ELSEIF (KEYRAD.GE.10.AND.KEYRAD.LT.113) THEN
        FUNSKA=5
      ELSE
        WRITE (IOUT,*) 'STOP IN FUNSKO'
        STOP
      ENDIF
      END
      FUNCTION BONMA0(MODE,XK)
C ----------------------------------------------------------------------
C PHOTON SPECTRUM IN CASE OF NO PHOTONS - BORN ONLY
C FAKED DISTRIBUTION
C
C     called by : FUNSKO
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
C
      IF (MODE.EQ.-1) THEN
C     ====================
C       INITIALIZATION
        CALL GIVIZO( IDE, 1,AIZOR,QE,KDUMM)
        QE2=0.D0
        SOFT=BORNY(4D0*ENE**2)
        F0=.5D0*BORNY(4D0*ENE**2)
        F1=XK0/(EXP(SOFT/F0)-1)
C FUNCTION RETURNS RATIO OF THE BORN TO INITIAL SOFT CROSS SECTION
        BONMA0=1D0
C
C RUNNING MODE
      ELSEIF (MODE.EQ.0 ) THEN
C     ==========================
        IF     (XK.LE.0D0 ) THEN
          BONMA0=0D0
        ELSEIF (XK.LE.XK0 ) THEN
C         SOFT PHOTON CASE
          BONMA0=F0/(XK+F1)
        ELSE
          BONMA0=0D0
        ENDIF
C
      ENDIF
C     =====
      END
      FUNCTION BONMA1(MODE,XK)
C ----------------------------------------------------------------------
C DISTRIBUTION OF K-VARIABLE,
C BONNEAU MARTIN FORMULA
C
C     called by : FUNSKO
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / VVREK  / VVMIN,VVMAX,VV,BETI
      REAL*8            VVMIN,VVMAX,VV,BETI
      COMMON / INOUT / INUT,IOUT
      DATA PI /3.141592653589793238462643D0/
C
      IF(XK0.GT.VVMAX) GOTO 900
      QE=1D0
      QE2=QE**2
      SVAR=4D0*ENE**2
      ALF1=ALFPI
      BILG  =DLOG(4D0*ENE**2/AMIN**2)
      BETI  =2D0*QE2*ALF1*(BILG-1D0)
      BETIR =2D0*QE2*ALF1* BILG
      DELI  =ALF1*QE2*(1.5D0*BILG+PI**2/3D0-2D0)+ BETI*LOG(XK0)
      IF(    XK.LT.0D0) THEN
        DIST=0D0
      ELSEIF(XK.LT.XK0) THEN
C DISTRIBUTION BELOW XK0 IS DESIGNED SUCH THAT THE INTEGRAL
C FROM 0 TO XK0 IS PRECISELY (1+DELI)*BORNY(SVAR)
        EPS=XK0/(EXP((1D0+DELI)/BETI)-1D0)
        DIST=BETI/(EPS+XK)*BORNY(SVAR)
      ELSEIF(XK.LT.VVMAX) THEN
C NOTE THAT 1/(1-XK) FACTOR BECAUSE BORNY IS IN R-UNITS
        SOFDIS=    BETI/XK
C    SOFDIS=    BETIR/XK
        DIST=SOFDIS*BORNY(SVAR*(1D0-XK))/(1D0-XK)
      ELSE
        DIST=0D0
      ENDIF
      VV=XK
C
      BONMA1=DIST
      RETURN
  900 WRITE(IOUT,*) '============= STOP IN BONMAR '
      END
      FUNCTION WEIGHT(MODE,TA,TB)
C ----------------------------------------------------------------------
C THIS FUNCTION CALCULATES DIFFERENTIAL CROSS SECTION FOR EVERY EVENT
C  INPUT : MODE:
C                 0   FOR CALL FOR RAW CROSS SECTION
C                     NO VAC. POLARIZ & NO GSW &
C                     NO INTERFERENCE INIT - FINAL
C                 1   INITIALIZATION. CORECT X. SECTION
C                 2   CORRECT X. SECTION
C          TA,TB: TWICE OF TAU+ TAU- HELICITIES
C
C     called by : EVENTE, EVENTM, SPIGEN, ALTPAR
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      DATA ICONT/0/
      ICONT=ICONT+1
C IF MODE=0 CALCULATION OF ANGULAR VARIABLES FROM 4-MOMENTA
      CALL PEDYPR(MODE,XK,C1,S1,C2,S2,CF,SF,CG,SG)
      IF(XK.GT.1.D-6) THEN
C        HARD PHOTON CASE
         FUNTI=FUNTIH(MODE,TA,TB)
C         IF (TA.EQ.0.AND.TB.EQ.0) THEN
C         IF (MODE.EQ.0) BUFOR=FUNTI
C         IF (MODE.EQ.1) WRITE(6,*) 'hard=',FUNTI,' ',BUFOR
C         ENDIF
      ELSE
C        SOFT PHOTON CASE
         COSTHE=C1
         FUNTI=FUNTIS(MODE,COSTHE,TA,TB)
C         IF (TA.EQ.0.AND.TB.EQ.0) THEN
C         IF (MODE.EQ.0) BUFOR=FUNTI
C         IF (MODE.EQ.1) WRITE(6,*)  'soft=',FUNTI,' ',BUFOR
C         ENDIF
      ENDIF
C
      IF (KEYGSW.GE.0) THEN
        IF (MODE.EQ.0) THEN
            CALL SANGLE(0,SVAR,CTHE)
          GSWF0=BORNS(0,SVAR,CTHE,0D0,0D0)
          GSWF1=BORNS(0,SVAR,CTHE,1D0,1D0)
          GSWF2=BORNS(0,SVAR,CTHE,-1D0,-1D0)
        ENDIF
C THIS IS A PROTECTION AGAINST ZERO DIVIDE IN NUNUBAR CASE
        IF (DABS(GSWF0).LT.1D-12) GSWF0= 1.D0
        IF (DABS(GSWF1).LT.1D-12) GSWF1= 1.D0
        IF (DABS(GSWF2).LT.1D-12) GSWF2= 1.D0
C
C
        IF (MODE.EQ.0) THEN
          GSWFA=1D0
        ELSEIF (TA*TB.LT.-.5D0) THEN
          GSWFA=1D0
        ELSEIF ( TA*TA.LT.0.5D0) THEN
          GSWFA=BORNS(MODE,SVAR,CTHE,0D0,0D0)/GSWF0
        ELSEIF ( TA.GT.0.5D0)    THEN
          GSWFA=BORNS(MODE,SVAR,CTHE,1D0,1D0)/GSWF1
        ELSEIF ( TA.LT.-0.5D0)   THEN
          GSWFA=BORNS(MODE,SVAR,CTHE,-1D0,-1D0)/GSWF2
        ENDIF
C
        FUNTI=FUNTI*GSWFA
      if (mode.ne.0.and. TA*TA.LT.0.5D0) funti=funti*wtanom(dumm)
      ENDIF
C
      WEIGHT=FUNTI
C     PRINT *, 'MODE',MODE,'TA',TA,'TB',TB,'WEIGHT=',WEIGHT
      END
      SUBROUTINE SANGLE(MODE,SVAR,COSTHE)
C ----------------------------------------------------------------------
C THIS ROUTINE CALCULATES BORN LIKE VARIABLES FROM THE 4- MOMETA
C STORED IN UTIL. FOR MODE = 0 ANGULAR VARIABLES ARE CALCULATED
C AND MEMORIZED. FOR HIGHER MODES THEY ARE SUPPLIED.
C FOR ULTRAHARD PHOTONS (EDGE OF PHASE SPACE) THIS ROUTINE MAY
C PRODUCE COSTHE.GT.1D0 .....
C
C     called by : WEIGHT
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / UTIL8 / QP(4),QM(4),PH(4)
      REAL*8           QP   ,QM   ,PH
      COMPLEX*16 CPRZ0M
      REAL*8 PP(4),PM(4),PP1(4),PM1(4),QP1(4),QM1(4),P(4),Q(4)
      REAL*8 PP2(4),PM2(4),QP2(4),QM2(4)
      DATA  ICON /0/
C
      IF (MODE.EQ.0) THEN
C       INITIALIZATION OF THE  VARIABLES
C       ================================
C       4-MOMENTA OF BEAMS
        DO 7 K=1,4
        PP(K)=0D0
    7   PM(K)=0D0
C
        PP(4)= ENE
        PM(4)= ENE
        PP(3)= ENE
        PM(3)=-ENE
C 4-MOMENTA OF 'DRESSED' FERMIONS
        DO 9 K=1,4
        PP1(K)=PP(K)
        PM1(K)=PM(K)
        QP1(K)=QP(K)
    9   QM1(K)=QM(K)
C ADDITION OF THE PHOTON PH TO FERMION OF MINIMAL MASS OF
C FERMION PHOTON STATE
        DO 8 K=1,4
        PP2(K)=PP(K)-PH(K)
        PM2(K)=PM(K)-PH(K)
        QP2(K)=QP(K)+PH(K)
    8   QM2(K)=QM(K)+PH(K)
CC
        CALL MULSK(PP2,PP2,XM1)
        CALL MULSK(PM2,PM2,XM2)
        CALL MULSK(QP2,QP2,XM3)
        CALL MULSK(QM2,QM2,XM4)
        XXKM=1D0-PH(4)/ENE
        FACINI=CDABS(CPRZ0M(1,4D00*ENE**2*XXKM))**2
        FACFIN=CDABS(CPRZ0M(1,4D00*ENE**2))**2
        XM1=XM1/FACINI
        XM2=XM2/FACINI
        XM3=XM3/FACFIN
        XM4=XM4/FACFIN
        XM=MIN(-XM1,-XM2,XM3,XM4)
C
        DO 10 K=1,4
        IF     (XM1.EQ.-XM) THEN
          PP1(K)=PP(K)-PH(K)
        ELSEIF (XM2.EQ.-XM) THEN
          PM1(K)=PM(K)-PH(K)
        ELSEIF (XM3.EQ. XM) THEN
          QP1(K)=QP(K)+PH(K)
        ELSEIF (XM4.EQ. XM) THEN
          QM1(K)=QM(K)+PH(K)
        ELSE
          PRINT *,
     &    'SOMETHING IS STRANGE IN COMPILER MODIFY LOGIC OF SANGLE'
        ENDIF
   10   CONTINUE
C CALCULATION OF THE S AND COSTHE CORRESPONDING TO THE HARD-BORN LIKE
C KERNEL OF THE INTERACTION. USED ARE DRESSED 4-MOMENTA OF FERMIONS.
        DO 15 K=1,4
        Q(K)=QP1(K)-PP1(K)
   15   P(K)=PM1(K)+PP1(K)
C
        CALL MULSK(P,P,SVARI)
        CALL MULSK(Q,Q,T)
        CALL MULSK(PP1,PP1,XM1)
        CALL MULSK(PM1,PM1,XM2)
        CALL MULSK(QP1,QP1,XM3)
        CALL MULSK(QM1,QM1,XM4)
C
        P2=(SVARI**2+(XM1-XM2)**2-2D0*SVARI*(XM1+XM2))/4D0/SVARI
        Q2=(SVARI**2+(XM3-XM4)**2-2D0*SVARI*(XM3+XM4))/4D0/SVARI
        COST=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
        IF(XM1.GE.XM2) THEN
          COST=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
        ELSE
          COST=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
        ENDIF
C
      ENDIF
C     =====
C* SETTING VALUES (FOR ALL MODES)
C
      COSTHE=COST
      SVAR=SVARI
C     CALL DUMPL9(PP1,PM1,PH)
C     CALL DUMPL9(QP1,QM1,PH)
      ICON=ICON+1
C ZW CORRECTION FOR SOME OVERFLOWS AT THE EDGE OF THE PHASE SPACE
      IF ( COSTHE.GE.1D0.OR.COSTHE.LE.-1D0 ) THEN
        COSTHE=COSTHE/COSTHE**2
        IF ( COSTHE.EQ.1D0.OR.COSTHE.EQ.-1D0 ) COSTHE=COSTHE*.9999999
        PRINT *, '==== WARNING ===== SUBROUTINE SANGLE',ICON
        PRINT *, 'YOU HAVE RUN INTO THE PROBLEM AT THE EDGE OF THE PHASE
     $ SPACE. AD HOC CORRECTION HAS BEEN DONE! '
      ENDIF
      IF ( SVAR.LE.4D0*AMFIN**2) THEN
        SVAR=16D0*AMFIN**4/SVAR
        IF ( SVAR.EQ.4D0*AMFIN**2 ) SVAR=SVAR*1.00000001D0
        PRINT *, '==== WARNING ===== SUBROUTINE SANGLE',ICON
        PRINT *, 'YOU HAVE RUN INTO THE PROBLEM AT THE EDGE OF THE
     $  PHASE SPACE. AD HOC CORRECTION HAS BEEN DONE! '
      ENDIF
      RETURN
      END
      SUBROUTINE SKONTY(MODE,SVAR,COSTHE)
C ----------------------------------------------------------------------
C THIS ROUTINE CALCULATES BORN LIKE VARIABLES FROM THE 4- MOMETA
C STORED IN MOMSET. FOR MODE = 0 ANGULAR VARIABLES ARE CALCULATED
C AND MEMORIZED. FOR HIGHER MODES THEY ARE SUPPLIED.
C FOR ULTRAHARD PHOTONS (EDGE OF PHASE SPACE) THIS ROUTINE MAY
C PRODUCE COSTHE.GT.1D0 .....
C
C     called by : WAGA, WEIGHT
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / MOMINI / XF1(4),XF2(4),XPHUM(4),XPHOT(100,4),NPHOX      
      COMMON / MOMFIN / YF1(4),YF2(4),YPHUM(4),YPHOT(100,4),NPHOY      
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      REAL*8 PP(4),PM(4),PP1(4),PM1(4),QP1(4),QM1(4),P(4),Q(4)
      REAL*8 PP2(4),PM2(4),QP2(4),QM2(4)
      REAL*8 QP(4),QM(4),PH(4)
      COMPLEX*16 CPRZ0M
      LOGICAL IFINI,BETTER
C
C     BETTER=.true. denotes mode of operation when photons are 
C     classified to be initial or final
      DATA  BETTER /.TRUE./
      DATA  ICON /0/
C

      IF (MODE.EQ.0) THEN
C      INITIALIZATION OF THE  VARIABLES
C      ================================
C
C 4-MOMENTA OF BEAMS
        DO 7 K=1,4
        PP(K)=0D0
    7   PM(K)=0D0
C
        PP(4)= ENE
        PM(4)= ENE
        PP(3)= ENE
        PM(3)=-ENE
C 4-MOMENTA OF 'DRESSED' FERMIONS
        DO 9 K=1,4
        QP(K) =QF1(K)
        QM(K) =QF2(K)
        PP1(K)=PP(K)
        PM1(K)=PM(K)
        QP1(K)=QP(K)
    9   QM1(K)=QM(K)
C
        DO 77 KPHOT=1,NPHOT
C       == == =============
C ADDITION OF THE PHOTON PH TO FERMION OF MINIMAL MASS OF
C FERMION PHOTON STATE
        DO 8 K=1,4
        IFINI=KPHOT.LE.NPHOX
        IF (IFINI) THEN
          PH (K)=XPHOT(KPHOT,K)
        ELSE
          PH (K)=YPHOT(KPHOT-NPHOX,K)
          IF(NPHOY.EQ.0) PH (K)=SPHOT(KPHOT,K)
        ENDIF
        PP2(K)=PP(K)-PH(K)
        PM2(K)=PM(K)-PH(K)
        QP2(K)=QP(K)+PH(K)
    8   QM2(K)=QM(K)+PH(K)
C
        CALL MULSK(PP2,PP2,XM1)
        CALL MULSK(PM2,PM2,XM2)
        CALL MULSK(QP2,QP2,XM3)
        CALL MULSK(QM2,QM2,XM4)
        XXKM=1D0-PH(4)/ENE
        FACINI=CDABS(CPRZ0M(1,4D00*ENE**2*XXKM))**2
        FACFIN=CDABS(CPRZ0M(1,4D00*ENE**2))**2
        XM1=XM1/FACINI
        XM2=XM2/FACINI
        XM3=XM3/FACFIN
        XM4=XM4/FACFIN
        IF (BETTER) THEN
          IF (IFINI) THEN 
           XM=MIN(-XM1,-XM2,-XM1,-XM2)
          ELSE
           XM=MIN( XM4, XM3, XM3, XM4)
          ENDIF
        ELSE
           XM=MIN(-XM1,-XM2, XM3, XM4)
        ENDIF
C
        DO 10 K=1,4
        IF     (XM1.EQ.-XM) THEN
          PP1(K)=PP1(K)-PH(K)
        ELSEIF (XM2.EQ.-XM) THEN
          PM1(K)=PM1(K)-PH(K)
        ELSEIF (XM3.EQ. XM) THEN
          QP1(K)=QP1(K)+PH(K)
        ELSEIF (XM4.EQ. XM) THEN
          QM1(K)=QM1(K)+PH(K)
        ELSE
          PRINT *,
     &    'SOMETHING IS STRANGE IN COMPILER MODIFY LOGIC OF SANGLE'
        ENDIF
   10   CONTINUE
   77   CONTINUE
C  ==   ========
C CALCULATION OF THE S AND COSTHE CORRESPONDING TO THE HARD-BORN LIKE
C KERNEL OF THE INTERACTION. USED ARE DRESSED 4-MOMENTA OF FERMIONS.
        DO 15 K=1,4
        Q(K)=QP1(K)-PP1(K)
   15   P(K)=PM1(K)+PP1(K)
        CALL MULSK(P,P,SVARI)
        CALL MULSK(Q,Q,T)
        CALL MULSK(PP1,PP1,XM1)
        CALL MULSK(PM1,PM1,XM2)
        CALL MULSK(QP1,QP1,XM3)
        CALL MULSK(QM1,QM1,XM4)
C
        P2=(SVARI**2+(XM1-XM2)**2-2D0*SVARI*(XM1+XM2))/4D0/SVARI
        Q2=(SVARI**2+(XM3-XM4)**2-2D0*SVARI*(XM3+XM4))/4D0/SVARI
        COST=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
        IF(XM1.GE.XM2) THEN
          COST=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
        ELSE
          COST=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
        ENDIF

C
      ENDIF
C     =====
C* SETTING VALUES (FOR ALL MODES)
C
      COSTHE=COST
      SVAR=SVARI
      ICON=ICON+1
C ZW CORRECTION FOR SOME OVERFLOWS AT THE EDGE OF THE PHASE SPACE
      IF ( COSTHE.GE.1D0.OR.COSTHE.LE.-1D0 ) THEN
        COSTHE=COSTHE/COSTHE**2
        IF ( COSTHE.EQ.1D0.OR.COSTHE.EQ.-1D0 ) COSTHE=COSTHE*.9999999
        PRINT *, '==== WARNING ===== SUBROUTINE SKONTY',ICON
        PRINT *, 'YOU HAVE RUN INTO THE PROBLEM AT THE EDGE OF THE PHASE
     $ SPACE. AD HOC CORRECTION HAS BEEN DONE! '
       CALL DUMPS(6)
      ENDIF
      IF ( SVAR.LE.4D0*AMFIN**2) THEN
        SVAR=16D0*AMFIN**4/SVAR
        IF ( SVAR.EQ.4D0*AMFIN**2 ) SVAR=SVAR*1.00000001D0
        PRINT *, '==== WARNING ===== SUBROUTINE SKONTY',ICON
        PRINT *, 'YOU HAVE RUN INTO THE PROBLEM AT THE EDGE OF THE
     $ SPACE. AD HOC CORRECTION HAS BEEN DONE! '
       CALL DUMPS(6)
      ENDIF
      RETURN
      END
      SUBROUTINE BDRESS(MODE,PPBT,PMBT,SVAR,COSTHE)
C ----------------------------------------------------------------------
C THIS ROUTINE CALCULATES DRESSED BEAMS
C FOR ULTRAHARD PHOTONS (EDGE OF PHASE SPACE) THIS ROUTINE MAY
C PRODUCE COSTHE.GT.1D0 .....
C IT SHOULD NOT BE CALLED WITH THE MODE 0 AFTER GENERATION OF FINAL
C STATE PHOTON
C
C     called by : PEDYVV, TRAAAA, EVENTE, WEIGHT
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / MOMSET / QF1(4),QF2(4),SPHUM(4),SPHOT(100,4),NPHOT
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      REAL*8 PP(4),PM(4),PP1(4),PM1(4),P(4),Q(4),PH(4)
      REAL*8 PP2(4),PM2(4),PPB(4),PMB(4),PPBT(4),PMBT(4)
      DATA  ICON /0/
C
      IF (MODE.EQ.0) THEN
C       INITIALIZATION OF THE  VARIABLES
C       ================================
C
C 4-MOMENTA OF BEAMS
        DO 7 K=1,4
        PP(K)=0D0
    7   PM(K)=0D0
C
        PP(4)= ENE
        PM(4)= ENE
        PP(3)= ENE
        PM(3)=-ENE
C 4-MOMENTA OF 'DRESSED' FERMIONS
        DO 9 K=1,4
        PP1(K)=PP(K)
        PM1(K)=PM(K)
    9   CONTINUE
C
        DO 33 JPHOT=1,NPHOT
C       == == =============
C
C ADDITION OF THE PHOTON PH TO FERMION OF MINIMAL MASS OF
C FERMION PHOTON STATE
        DO 8 K=1,4
        PH(K)=SPHOT(JPHOT,K)
        PP2(K)=PP(K)-PH(K)
        PM2(K)=PM(K)-PH(K)
    8   CONTINUE
C
        CALL MULSK(PP2,PP2,XM1)
        CALL MULSK(PM2,PM2,XM2)
C
        XM=MIN(-XM1,-XM2)
C
        DO 10 K=1,4
        IF     (XM1.EQ.-XM) THEN
          PP1(K)=PP1(K)-PH(K)
        ELSEIF (XM2.EQ.-XM) THEN
          PM1(K)=PM1(K)-PH(K)
        ELSE
          PRINT *,
     &    'SOMETHING IS STRANGE IN COMPILER MODIFY LOGIC OF SANGLE'
        ENDIF
   10   CONTINUE
C
   33   CONTINUE
C  ==   ========
C
C CALCULATION OF THE S AND COSTHE CORRESPONDING TO THE HARD-BORN LIKE
C KERNEL OF THE INTERACTION. USED ARE DRESSED 4-MOMENTA OF FERMIONS.
        DO 15 K=1,4
        PPB(K)=PP1(K)
        PMB(K)=PM1(K)
        Q(K)=QF1(K)-PP1(K)
   15   P(K)=PM1(K)+PP1(K)
C
        CALL MULSK(P,P,SVARI)
        CALL MULSK(Q,Q,T)
        CALL MULSK(PP1,PP1,XM1)
        CALL MULSK(PM1,PM1,XM2)
        CALL MULSK(QF1,QF1,XM3)
        CALL MULSK(QF2,QF2,XM4)
C
        P2=(SVARI**2+(XM1-XM2)**2-2D0*SVARI*(XM1+XM2))/4D0/SVARI
        Q2=(SVARI**2+(XM3-XM4)**2-2D0*SVARI*(XM3+XM4))/4D0/SVARI
        IF(XM1.GE.XM2) THEN
          COST=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
        ELSE
          COST=.5D0/SQRT(P2*Q2)*(T-XM2-XM4+2D0*SQRT((P2+XM2)*(Q2+XM4)))
          COSA=.5D0/SQRT(P2*Q2)*(T-XM1-XM3+2D0*SQRT((P2+XM1)*(Q2+XM3)))
        ENDIF
C
      ENDIF
C     =====
C* SETTING VALUES (FOR ALL MODES)
C
      COSTHE=COST
      SVAR=SVARI
      DO 16 K=1,4
      PPBT(K)=PPB(K)
      PMBT(K)=PMB(K)
   16 CONTINUE
C     CALL DUMPL9(PP1,PM1,PH)
C     CALL DUMPL9(QP1,QM1,PH)
      ICON=ICON+1
C ZW CORRECTION FOR SOME OVERFLOWS AT THE EDGE OF THE PHASE SPACE
      IF (COST.GE.1D0.OR.COST.LE.-1D0) THEN
        PRINT *, '=== WARNING === SUBR. BDRESS',ICON,'costhe=',costhe
        PRINT *, '=== WARNING === SUBR. BDRESS',ICON,'cosa=',cosa
        PRINT *, '==== WARNING ===== SUBR. BDRESS',ICON,'svar=',svar
       CALL DUMPZ8('PP1',PP1 )
       CALL DUMPZ8('PM1',PM1 )
       CALL DUMPZ8('P  ',P   )
       CALL DUMPZ8('QF1',QF1 )
       CALL DUMPZ8('QF2',QF2 )
       CALL DUMPZ8('Q  ',Q )
      WRITE(6,*) 'SVARI=',SVARI,'T=',T
      WRITE(6,*) 'XM1=',XM1,'XM2=',XM2,'XM3=',XM3,'XM4=',XM4 
      WRITE(6,*) 'P2=',P2,'Q2=',Q2
        COSTHE=COSTHE/COSTHE**2
        IF ( COSTHE.EQ.1D0.OR.COSTHE.EQ.-1D0 ) COSTHE=COSTHE*.9999999
C        PRINT *, 'YOU HAVE RUN INTO THE PROBLEM AT THE EDGE OF THE PHASE
C     $ SPACE. AD HOC CORRECTION HAS BEEN DONE! '
      ENDIF
      IF ( SVAR.LE.4D0*AMFIN**2) THEN
        PRINT *, '==== WARNING ===== SUBROUTINE BDRESS',ICON,'svar=',svar
        SVAR=16D0*AMFIN**4/SVAR
        IF ( SVAR.EQ.4D0*AMFIN**2 ) SVAR=SVAR*1.00000001D0
      ENDIF
      RETURN
      END
      FUNCTION FUNTIH(MODE,TA,TB)
C ----------------------------------------------------------------------
C                                           *
C         HARD PHOTON CASE                  *
C                                           *
C
C     called by : WEIGHT
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / GAUSPM /SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI   ,XUPZI   ,XUPGF   ,XUPZF
     &                ,NDIAG0,NDIAGA,KEYA,KEYZ
     &                ,ITCE,JTCE,ITCF,JTCF,KOLOR
      REAL*8           SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI(2),XUPZI(2),XUPGF(2),XUPZF(2)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / INSPIN / SEPS1,SEPS2
      REAL*8            SEPS1,SEPS2
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
      COMPLEX*16 CPRAM,CA0,CA1
      COMPLEX*16 CPRZ0M,CP0,CP1
      COMPLEX*16 T1(2,4,4),T2(2,4,4),ZERO
      COMPLEX*16 ONEC,IMAG,DE0,DEK,CK,CL,CN,CX,SX
      DATA INIT/0/
C
      DE0(EPS,TAU)=CA0*SVAR*QE*QF*ONEC + CP0*SVAR
     $ *XUPZI(1+INT(-EPS+1.999D0)/2)*XUPZF(1+INT(-TAU+1.999D0)/2)
      DEK(EPS,TAU)=CA1*SVAR*QE*QF*ONEC + CP1*SVAR
     $ *XUPZI(1+INT(-EPS+1.999D0)/2)*XUPZF(1+INT(-TAU+1.999D0)/2)
C
C***************************************
      IF(INIT.EQ.0) THEN
C  OFF INTERFERENCE WHEN EXPONENTIATION
        IF  (KEYRAD.GE.10) KBRINT=0
        IF  (KEYRAD.GE.10) KBRFIN=0
        ZERO=DCMPLX(0.D0,0.D0)
        IMAG=DCMPLX(0.D0,1.D0)
        ONEC=DCMPLX(1.D0,0.D0)
      ENDIF
C
C****************************************
      SVAR=4D0*ENE**2
      XMEL =AMIN/ENE
      XMFIN=AMFIN/ENE
C** CALCULATE SPIN AMPLITUDES
C** BUT RECALCULATE ONLY IF IT IS NECESSARY
      IF (MODE.LE.1) THEN
        CALL PEDYPR(1,XK,C1,S1,C2,S2,CF,SF,CG,SG)
        BB=.5D0*XK/SQRT(1D0-XK)
        GB=(1.D0-.5D0*XK)/SQRT(1D0-XK)
        HINI= QE/(XMEL**2+S1**2)
        HFIN= QF/(XMFIN**2/(1D0-XK)
     $       +ABS(1D0-XMFIN**2/(1D0-XK))*S2**2)
        POLAR1= SEPS1
        POLAR2=-SEPS2
C Z0 PROPAGATORS AND GAMMA VACUUM POLARIZATIONS
        CP0 =CPRZ0M(MODE,SVAR)
        CP1 =CPRZ0M(MODE,SVAR*(1.D0-XK))
        CA0 =CPRAM (MODE,SVAR)
        CA1 =CPRAM (MODE,SVAR*(1.D0-XK))
        DO 140 J=1,2
        DO 140 K=1,4
        DO 140 I=1,4
        T1 (J, I, K)= ZERO
  140   T2 (J, I, K)= ZERO
        DO 150 J=1,2
        GE=(3.D0-2.D0*J)
        DO 150 K=1,2
        DO 150 I=1,2
        EPS=(3.D0-2.D0*I)
        TAU=(3.D0-2.D0*K)
        CX=GB*CF*ONEC+BB*SF*GE*IMAG
        SX=BB*CF*ONEC+GB*SF*GE*IMAG
        CK=-(EPS*TAU+C1*C2)*CX+S1*S2-GE*(TAU*C1+EPS*C2)*SX
        CN=(C1+GE*EPS)*(C2-C2*GE*TAU)*ONEC
        CL=(C1-GE*EPS*C1)*(C2+GE*TAU)*ONEC
C
        T1(J,I,K)  = IMAG/2.D0/BB*(GB-BB)*S1*HINI*
     $               DEK(EPS,TAU)*CK
        T1(J,I+2,K)= 0.5D0*SQRT(1D0-XK)*XMEL*C1*HINI*
     $               (CF*ONEC+SF*GE*IMAG)*DEK(-C1*EPS,TAU)*CL
        T2(J,I,K)  = IMAG/2.D0/BB*
     $              S2*HFIN*(CF*ONEC-SF*GE*IMAG)*DE0(EPS,TAU)*CK
        T2(J,I,K+2)= 0.5D0*XMFIN/SQRT(1D0-XK)*C2*HFIN*
     $                DE0(EPS,-C2*TAU)*CN
 150    CONTINUE
      ENDIF
C
      TOTINI=0.D0
      TOTFIN=0.D0
      TOTINT=0.D0
      TOT=0.D0
C CALCULATION OF THE CROSS SECTION
      DO 50 I=1,2
      HELIC= 3-2*I
      DO 50 J=1,2
      HELIT= 3-2*J
      FACTOR=KOLOR* (1D0+HELIC*POLAR1)*(1D0-HELIC*POLAR2)
      FACTOR=FACTOR*(1D0+HELIT*TA    )*(1D0+HELIT*TB)
C
      FACTOF=KOLOR* (1D0+HELIC*POLAR1)*(1D0-HELIC*POLAR2)
      FACTOF=FACTOF*(1D0+HELIT*TA    )*(1D0-HELIT*TB)
C
      IF (MODE.EQ.0)THEN
        FACTOI=KOLOR* (1D0-HELIC*C1*POLAR1)*(1D0+HELIC*C1*POLAR2)
      ELSE
        FACTOI=KOLOR* (1D0+HELIC*POLAR1)*(1D0+HELIC*POLAR2)
      ENDIF
      FACTOI=FACTOI*(1D0+HELIT*TA    )*(1D0+HELIT*TB)
      DO 50 K=1,2
C INITIAL STATE RADIATION
      TOTINI=TOTINI+  DREAL(T1(K,I  ,J  )*DCONJG(T1(K,I  ,J  )))*FACTOR
      TOTINI=TOTINI+  DREAL(T1(K,I+2,J  )*DCONJG(T1(K,I+2,J  )))*FACTOI
C FINAL STATE RADIATION
      TOTFIN=TOTFIN+  DREAL(T2(K,I  ,J  )*DCONJG(T2(K,I  ,J  )))*FACTOR
      TOTFIN=TOTFIN+  DREAL(T2(K,I  ,J+2)*DCONJG(T2(K,I  ,J+2)))*FACTOF
C INTERFERENCE
      TOTINT=TOTINT+2*DREAL(T1(K,I  ,J  )*DCONJG(T2(K,I  ,J  )))*FACTOR
  50  CONTINUE
      TOT=TOTINI*KBRINI+TOTFIN*KBRFIN+TOTINT*KBRINT

      IF (MODE.EQ.0)
     $TOT=TOTINI*KBRINI+TOTFIN*KBRFIN
      FUNTIH=TOT*2.D0
      FUNTIN=0.0
      END
      FUNCTION FUNTIS(MODE,COSTHE,TA,TB)
C ----------------------------------------------------------------------
C                                            *
C          SOFT PHOTON CASE                  *
C
C     called by : WEIGHT
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      COMMON / QEDPRM /ALFINV,ALFPI,XK0
      REAL*8           ALFINV,ALFPI,XK0
      COMMON / GSWPRM /SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
      REAL*8           SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
C     SWSQ        = sin2 (theta Weinberg)
C     AMW,AMZ     = W & Z boson masses respectively
C     AMH         = the Higgs mass
C     AMTOP       = the top mass
C     GAMMZ       = Z0 width
      COMMON / GAUSPM /SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI   ,XUPZI   ,XUPGF   ,XUPZF
     &                ,NDIAG0,NDIAGA,KEYA,KEYZ
     &                ,ITCE,JTCE,ITCF,JTCF,KOLOR
      REAL*8           SS,POLN,T3E,QE,T3F,QF
     &                ,XUPGI(2),XUPZI(2),XUPGF(2),XUPZF(2)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMMON / INSPIN / SEPS1,SEPS2
      REAL*8            SEPS1,SEPS2
      COMMON / KBREMS / KBRINI,KBRFIN,KBRINT
      COMPLEX*16 ABORN(2,2),APHOT(2,2),AZETT(2,2),BOXGG(2,2),BOXGZ(2,2)
      COMPLEX*16 PROPA,PROPZ,CPRZ0M,ONE,XX1,XX2,ZZ1,ZZ2,COEFB,DEL2,DEL3
      COMPLEX*16 CPRAM
      DATA ONE /(1D0,0D0)/
      DATA INIT/0/
C ----------------------------------------------------------------------
C***************************************
      IF (TA*TB.LT.-0.5D0) THEN
        FUNTIS=0.D0
        RETURN
      ENDIF
C****************************************
C
      IF(INIT.EQ.0) THEN
        INIT=1
C   OFF INTERFERENCE WHEN EXPONENTIATION
        IF (KEYRAD.GE.10) KBRINT=0
        IF (KEYRAD.GE.10) KBRFIN=0
        PI=4.D0*DATAN(1.D0)
        ALF1=ALFPI
        IF(KEYRAD.EQ.0) ALF1=0D0
C CHARGES IN BREMSSTRAHLUNG VERTICES TREATED SEPARATELY
        QE2=QE**2
        QF2=QF**2
        QINFI=QE*QF
        IF(KBRINI.EQ.0) QE2=0D0
        IF(KBRFIN.EQ.0) QF2=0D0
        IF(KBRINT.EQ.0) QINFI=0D0
      ENDIF
C
      SVAR=4D0*ENE**2
      IF (MODE.EQ.0) THEN
        IF(KEYRAD.NE.0) THEN
C****************************************
C** SOFT REAL AND VIRTUAL PHOTON CORRECTIONS - INITIALIZATION
          BILG=DLOG(SVAR/AMIN**2)
          BETI=2D0*QE2*ALF1*(BILG-1D0)
          DELI=ALF1*QE2*(9D0*BILG+2D0*PI**2-12D0)/6D0
     $        +BETI*LOG(XK0)
          BILGF=DLOG(SVAR/AMFIN**2)
          BETF =2D0*QF2*ALF1*(BILGF-1D0)
          DELF =ALF1*QF2*(9D0*BILGF+2D0*PI**2-12D0)/6D0
     $        +BETF*LOG(XK0)
          DELF0=ALF1*QF2*(9D0*BILGF
     $         +2D0*PI**2-16.5D0)/6D0 +BETF*LOG(XK0)
          UP=1D0+COSTHE
          UM=1D0-COSTHE
          ALP=LOG(UP/2D0)
          ALM=LOG(UM/2D0)
          DELINT=  4D0*(ALM-ALP)*LOG(XK0)
     $           + ALM**2-ALP**2-2D0*DILOGT(UM/2D0)+2D0*DILOGT(UP/2D0)
          DELINT=       QINFI*ALF1*DELINT
C EXTRA TERMS DUE TO NARROWNES OF Z0 RESONANCE
          GM=GAMMZ*AMZ/SVAR
          ZZ=1D0-AMZ**2/SVAR
          BETINT= 2D0*ALF1*(ALM-ALP)
          DEL2=(BETI+BETINT*QINFI)*
     $         (CDLOG(DCMPLX(ZZ,GM)) -CDLOG(DCMPLX((ZZ-XK0),GM)))
          DEL3=
     $         BETI*ZZ/GM*(ATAN((XK0-ZZ)/GM)-ATAN(-ZZ/GM))
     $        -BETI*(CDLOG(DCMPLX(ZZ,GM)) -CDLOG(DCMPLX((ZZ-XK0),GM)))
C
        ELSE
          DELI=0D0
          DELF=0D0
          DELINT=0D0
          DELF0=0D0
        ENDIF
      ENDIF
      IF (MODE.LE.1) THEN
C*** PROPAGATORS AND VACUUM POLARIZATIONS
        PROPA =CPRAM (MODE,SVAR)
        PROPZ =CPRZ0M(MODE,SVAR)
C* BORN SPIN AMPLITUDES
        DO 50 I=1,2
        DO 50 J=1,2
        REGULA= (3-2*I)*(3-2*J) + COSTHE
        APHOT(I,J)=PROPA*DCMPLX(XUPGI(I)*XUPGF(J)*REGULA)
        AZETT(I,J)=PROPZ*DCMPLX(XUPZI(I)*XUPZF(J)*REGULA)
   50   ABORN(I,J)=APHOT(I,J)+AZETT(I,J)
        IF(KEYRAD.NE.0) THEN
C****************************************
C* SPIN AMPLITUDES FOR BOX GAMMA-GAMMA
          IF (MODE.EQ.0) CALL BGAM2(COSTHE,ZZ1,ZZ2)
          DO 70 I=1,2
          DO 70 J=1,2
          HELPRO= (3-2*I)*(3-2*J)
          COEFB=PROPA*DCMPLX( XUPGI(I)*XUPGF(J) *.5D0*ALF1*QINFI )
   70     BOXGG(I,J)=COEFB*(DCMPLX(HELPRO)*ZZ1 + ZZ2)
C****************************************
C* SPIN AMPLITUDES FOR BOX GAMMA-ZED
C BOX GAMMA-Z0 VERSION OF PASCHOS ET. AL.
          IF (MODE.EQ.0) CALL BZED1(SVAR,AMZ,1D0,GAMMZ,COSTHE,XX1,XX2)
          DO 75 I=1,2
          DO 75 J=1,2
          HELPRO= (3-2*I)*(3-2*J)
          COEFB= PROPZ*DCMPLX( XUPZI(I)*XUPZF(J)*ALF1*QINFI )
   75     BOXGZ(I,J)=COEFB*( DCMPLX(HELPRO)*XX1 + XX2 )
C****************************************
        ENDIF
C****************************************
      ENDIF
C
C******************
C* IN CALCULATING CROSS SECTION ONLY DIAGONAL ELEMENTS
C* OF THE SPIN DENSITY MATRICES ENTER (LONGITUD. POL. ONLY.)
C* HELICITY CONSERVATION EXPLICITLY OBEYED
      POLAR1=  (SEPS1)
      POLAR2= (-SEPS2)
      BORN=0D0
      GGBOX=0D0
      GZBOX=0D0
      SOFBRM=0D0
      DO 150 I=1,2
      HELIC= 3-2*I
      DO 150 J=1,2
      HELIT=3-2*J
      FACTOR=KOLOR*(1D0+HELIC*POLAR1)*(1D0-HELIC*POLAR2)/4D0
      FACTOR=FACTOR*(1+HELIT*TA)*(1+HELIT*TB)
      BORN=BORN+CDABS(ABORN(I,J))**2*FACTOR
      IF(KEYRAD.NE.0) THEN
        GGBOX=GGBOX+2D0* REAL( ABORN(I,J)*DCONJG(BOXGG(I,J)) )*FACTOR
        GZBOX=GZBOX+2D0* REAL( ABORN(I,J)*DCONJG(BOXGZ(I,J)) )*FACTOR
        SOFBRM=SOFBRM
     $          +2D0* REAL(ABORN(I,J)*DCONJG(AZETT(I,J)*DEL2))*FACTOR
     $              + REAL(AZETT(I,J)*DCONJG(AZETT(I,J)*DEL3))*FACTOR
      ENDIF
  150 CONTINUE
C************
      IF (KEYRAD.GE.10) SOFBRM=0D0
      FUNT=BORN*(1D0+DELI+DELF+DELINT)+SOFBRM +GGBOX + GZBOX
      IF (MODE.EQ. 0)  FUNT=BORN*(1D0+DELI+DELF0)
      IF (MODE.EQ.-1)  FUNT=BORN
      IF(TA*TA.GT.0.5D0.AND.FUNT.LE.0.D0 ) FUNT=BORN
      IF(FUNT.LT.0.D0)  FUNT=0.D0
      FUNT=FUNT *SVAR**2
      FUNTIS =FUNT
      FUNTIM=0.0
C         IF (TA.EQ.0.AND.TB.EQ.0) THEN
C         IF (MODE.EQ.0) BUFINI=BORN
C         IF (MODE.EQ.0) BUFFIN=(1D0+DELI+DELF+DELINT)
C         IF (MODE.EQ.0) BUFINT=SOFBRM +GGBOX + GZBOX
C         IF (MODE.EQ.1) WRITE(6,*) '====='
C         IF (MODE.EQ.1) WRITE(6,*) 'INI',BORN,' ',BUFINI
C         IF (MODE.EQ.1) WRITE(6,*) 'INI',(1D0+DELI+DELF+DELINT),' ',BUFFIN
C         IF (MODE.EQ.1) WRITE(6,*) 'INI',SOFBRM +GGBOX + GZBOX,' ',BUFINT
C         ENDIF
      END
      SUBROUTINE BGAM2(COSTHE,ZZ1,ZZ2)
C ----------------------------------------------------------------------
C BOX GAMMA-GAMMA AS IN MUSTRAAL AND KORAL-B
C
C     called by : FUNTIS
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMPLEX*16 ZZ1,ZZ2
      DATA PI /3.141592653589793238462643D0/
C
      C=COSTHE
      UP=1D0+COSTHE
      UM=1D0-COSTHE
      ALP=LOG(UP/2D0)
      ALM=LOG(UM/2D0)
      XZ1=-C/UP*ALM**2-C/UM*ALP**2+ALM-ALP
      XZ2=-C/UP*ALM**2+C/UM*ALP**2+ALM+ALP
      YZ1=-2D0*PI*C/UP*ALM-2D0*PI*C/UM*ALP
      YZ2=-2D0*PI*C/UP*ALM+2D0*PI*C/UM*ALP+2D0*PI
      ZZ1=DCMPLX(  XZ1,  YZ1+2D0*PI*(ALM-ALP)   )
      ZZ2=DCMPLX(  XZ2,  YZ2+2D0*PI*(ALM-ALP)*C )
      END
      SUBROUTINE BZED1(SVAR,AMZ,XLAM,GAMMZ,COSTHE,ZED3,ZED4)
C ----------------------------------------------------------------------
C GAMMA-Z0 BOX   FROM W.BROWN, R. DECKER, E. PASHOS
C PHYS. REV. LETT. 52(1984)1192
C CALCULATES XX1 FOR BOX GAMA-Z
C
C     called by : FUNTIS
C ----------------------------------------------------------------------
      IMPLICIT COMPLEX*16 (A-Z)
      REAL*8 SVAR,AMZ,XLAM,GAMMZ,COSTHE
C
      E=DCMPLX(1.D0,0.D0)
      F=DCMPLX(1.D0,-1.D-4)
      CC=DCMPLX(COSTHE)
      MZ=DCMPLX(AMZ**2/SVAR,-AMZ*GAMMZ/SVAR)
      MGAM=DCMPLX(XLAM/SVAR,0.D0)
      T=-(1-CC)/2
      U=-(1+CC)/2
      A=2*(1-CC)*FIU(MGAM,MZ,U,T)
      B=2*(1+CC)*FIU(MGAM,MZ,T,U)
      D=-CDLN(T/U,E)*CDLN(MGAM,E)
      ZED3= A-B+D
      ZED4=-A-B+CC*D
      END
      FUNCTION FIU(MG,MZ,T,U)
C ----------------------------------------------------------------------
C FORMULA  (11)  FROM W.BROWN, R. DECKER, E. PASHOS
C PHYS. REV. LETT., 52 (1984), 1192
C Z0 PROPAGATOR REMOVED
C
C     called by : BZED1
C ----------------------------------------------------------------------
      IMPLICIT COMPLEX*16 (A-Z)
C
      E=DCMPLX(-1.D0,0.D0)
      FIU=1/(MZ-1)*((CDLN(CDSQRT(T*U)/MG,E)
     $ +2*CDLN((1-1/MZ),E))*CDLN((U/T),E)+SP((1+U/MZ),E)-SP((1+T/MZ),E))
     $ +  (U-T-MZ)/U/U*
     $    (CDLN((1-1/MZ),E)*CDLN((-T),E)+SP((1+T/MZ),E)-SP((1-1/MZ),E))
     $ +  1/U*((MZ-1)*CDLN((1-1/MZ),E)+CDLN(-T/MZ,E))
      FIU=-.25D0*(MZ-1)*FIU
      END
      FUNCTION CPRZ0M(MODE,S)
C ----------------------------------------------------------------------
C THIS FUNCTION SUPPLIES TO THE PROGRAM Z0 PROPAGATOR
C IT USES Z0 VACUUM POLARIZATION MEMORIZED IN THE FUNCTION CINTZZ.
C INPUT : S (GEV**2)   PHOTON ENERGY TRANSFER.
C         MODE -INTERNAL KEY IN THE ALGORITHM,
C
C     called by : FUNTIH, FUNTIS, FANTIH,FANTIS, BORNS, SANGLE, BDRESS,
C                 SKONTY, AMPGSW
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / GSWPRM /SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
      REAL*8           SWSQ,AMW,AMZ,AMH,AMTOP,GAMMZ
C     SWSQ        = sin2 (theta Weinberg)
C     AMW,AMZ     = W & Z boson masses respectively
C     AMH         = the Higgs mass
C     AMTOP       = the top mass
C     GAMMZ       = Z0 width
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMPLEX*16 CPRZ0,SIGMA,CPRZ0M
C
          SIGMA=DCMPLX(0D0,S/AMZ*GAMMZ)
          CPRZ0=DCMPLX((S-AMZ**2),0D0)
          CPRZ0M=1/(CPRZ0+SIGMA)
      IF (KEYGSW.EQ.0) THEN
        CPRZ0M=DCMPLX(0.D0,0.D0)
      ENDIF

      END
      FUNCTION CPRAM(MODE,S)
C ----------------------------------------------------------------------
C THIS FUNCTION SUPPLIES TO THE PROGRAM PHOTON PROPAGATOR
C IT USES PHOTON VACUUM POLARIZATION MEMORIZED IN THE FUNCTION CINTAA.
C INPUT : S (GEV**2)   PHOTON ENERGY TRANSFER.
C         MODE -INTERNAL KEY IN THE ALGORITHM,
C
C     called by : FUNTIH, ....
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / KEYSET / KEYGSW,KEYRAD,KEYTAB
      COMPLEX*16 CVPAAM,CPRAM,CSVAR
C
      CSVAR=S
C      IF (KEYGSW.EQ.0) THEN
C        CPRAM=DCMPLX(0.D0,0.D0)
C      ELSE
      CVPAAM=DCMPLX(0D0,0D0)
      CPRAM=1D0/(CVPAAM+CSVAR)
C      ENDIF
      END
      SUBROUTINE PEDYPR(MODE,XXK,CC1,SS1,CC2,SS2,CCF,SSF,CCG,SSG)
C ----------------------------------------------------------------------
C THIS ROUTINE RECONSTRUCTS ANGULAR VARIABLES FROM THE 4- MOMETA
C STORED IN UTIL. FOR MODE = 0 ANGULAR VARIABLES ARE CALCULATED
C AND MEMORIZED. FOR HIGHER MODES THEY ARE SUPPLIED.
C
C     called by : WEIGHT, FUNTIH
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / UTIL8 / QP(4),QM(4),PH(4)
      REAL*8           QP   ,QM   ,PH
      COMMON / BEAMPM / ENE ,AMIN,AMFIN,IDE,IDF
      REAL*8            ENE ,AMIN,AMFIN
      REAL*8 PI1(4),PI2(4),PIX(4),PIY(4)
C
      IF (MODE.EQ.0) THEN
C     ===================
C INITIALIZATION OF THE ANGULAR VARIABLES
        XK=PH(4)/ENE
        IF (XK.GT.1.D-6) THEN
C
C HARD PHOTON CASE
          CALL MULSK(PH,PH,AMP2)
          AMP=SQRT(ABS(AMP2))
          C1=PH(3)/SQRT(PH(4)**2-AMP**2)
          S1=SQRT(1.D0-C1**2)
C CALCULATION OF C2 ---------------------------
          DO 7 KKK=1,4
          PI1(KKK)=QP(KKK)+QM(KKK)
   7      PI2(KKK)=QP(KKK)-QM(KKK)
          CALL MULSK(PH,QM,R)
          CALL MULSK(PH,QP,R1)
          CALL MULSK(PI1,PH,RV)
          CALL MULSK(PI2,PI2,RT)
          CALL MULSK(PI1,PI1,RU)
          C2=(R-R1)/SQRT(-RT)/SQRT(RV**2/RU-AMP**2)
          S2=SQRT(1.D0-C2**2)
C REDEFINITION OF XK
          XK=1D0-RU/4D0/ENE**2
C CALCULATION OF CG , SG -----------------------
          CG= PH(2)/SQRT(PH(1)**2+PH(2)**2)
          SG=-PH(1)/SQRT(PH(1)**2+PH(2)**2)
C CALCULATION OF CF ----------------------------
          PIX(1)=0.D0
          PIX(2)=0.D0
          PIX(3)=ENE
          PIX(4)=0.D0
C
          CALL MULVX(PH,QP,PI1)
          CALL MULVX(PH,PIX,PI2)
          CALL MULSK(PI1,PI2,R)
          CALL MULSK(PI1,PI1,R1)
          CALL MULSK(PI2,PI2,R2)
          CF=R/SQRT(R1*R2)
C CALCULATION OF SF ---------------------------
          CALL MULSK3(PH,QP,R)
          CALL MULSK3(PH,PIX,R1)
          CALL MULSK3(PH,PH,R3)
          DO 9 KKK=1,4
          PI1(KKK)=QP(KKK)-R/R3*PH(KKK)
   9      PI2(KKK)=PIX(KKK)-R1/R3*PH(KKK)
C
          CALL MULVX(PI1,PI2,PIY)
          CALL MULSK3(PH,PIY,R)
          CALL MULSK3(PI1,PI1,R1)
          CALL MULSK3(PI2,PI2,R2)
          SF=-R/SQRT(-R1*R2*R3)
        ELSE
C
C* SOFT PHOTON CASE
          C1=QP(3)/SQRT(1.D0-AMFIN**2/ENE**2)/ENE
          S1=SQRT(1.D0-C1**2)
          C2=1D0
          S2=0D0
          CF=1D0
          SF=0D0
C CALCULATION OF CG , SG -----------------------
          IF (QP(1)**2+QP(2)**2.GT.0D0) THEN
            CG= QP(2)/SQRT(QP(1)**2+QP(2)**2)
            SG=-QP(1)/SQRT(QP(1)**2+QP(2)**2)
          ELSE
C ZW 31.12.88 HERE WAS AN ERROR, FOR THE VERYLONG SERIES IT COULD HAPPEN
C             THAT C1=1D0. THIS WAS A PROBLEM LATER IN THE CALCULATION.
            CG=1D0
            SG=0D0
            C1=C1*0.9999999D0
          ENDIF
        ENDIF
      ENDIF
C     =====
C* SETTING VALUES (FOR ALL MODES)
C
      XXK=XK
      CC1=C1
      SS1=S1
      CC2=C2
      SS2=S2
      CCF=CF
      SSF=SF
      CCG=CG
      SSG=SG
      END
      SUBROUTINE MULVX(X,Y,R)
C ----------------------------------------------------------------------
C ROUTINE USED IN PEDYPR.
C IT CALCULATES VECTOR PRODUCT OF SPACE LIKE PARTS OF FOUR VECTORS
C
C     called by : PEDYVV,PEDYPR
C ----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(4),Y(4),R(4)
C
      R(4)=0.D0
      R(1)=X(2)*Y(3)-X(3)*Y(2)
      R(2)=X(3)*Y(1)-X(1)*Y(3)
      R(3)=X(1)*Y(2)-X(2)*Y(1)
      END
      SUBROUTINE MULSK(X,Y,R)
C ----------------------------------------------------------------------
C USED IN PEDYPR
C IT CALCULATES SCALAR PRODUCT OF FOUR VECTORS
C
C     called by : PEDYVV,PEDYPR,SANGLE,BDRESS,SKONTY
C ----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(4),Y(4)
      R=X(4)*Y(4)-X(3)*Y(3)-X(2)*Y(2)-X(1)*Y(1)
      END
      SUBROUTINE MULSK3(X,Y,R)
C ----------------------------------------------------------------------
C USED IN PEDYPR
C IT CALCULATES SCALAR PRODUCT OF SPACE LIKE PARTS OF FOUR VECTORS
C
C     called by : PEDYVV, PEDYPR
C ----------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(4),Y(4)
C
      R=-X(3)*Y(3)-X(2)*Y(2)-X(1)*Y(1)
      END
      SUBROUTINE RRR6(XK,AM2,CG,SG,IBREM)
C ----------------------------------------------------------------------
C THIS IS FUNCTIONALLY THE SAME ROUTINE RRR6 AS IN MUSTRAAL.
C IT GENERATES PHOTON ANGLE AND THE BREMSSTRAHLUNG PARAMETER IBREM.
C
C     called by : EVENTE, EVENTM
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*4  RRR(3)
C
   10 CONTINUE
      CALL RANMAR(RRR,3)
C GENERATION OF THE RAW ANGULAR PHOTON DISTRIBUTION
      RN1=RRR(1)
      CALL ANGPHO(RN1,AM2,CG,SG)
C SHAPING ---- FIRST STEP
      EPS=AM2/(1.D0+SQRT(1.D0-AM2))
      BETA=SQRT(1.D0-AM2)
      DEL1=1.D0-CG*BETA
      DEL2=1.D0+CG*BETA
      DEL12=EPS/(1.D0-XK+1.D0/(1.D0-XK))*(DEL2/DEL1+DEL1/DEL2)
      RN2=RRR(2)
      IF(RN2.GT.(1.D0-DEL12)) GO TO 10
C
C SHAPING ---- SECOND STEP
      CHI1=(1.D0-.5D0*XK*DEL1)**2
      CHI2=(1.D0-.5D0*XK*DEL2)**2
      RN3=RRR(3)
      IF(RN3.GT.(CHI1+CHI2)/2.D0) GO TO 10
C
C CHOICE OF THE KINEMATICAL MODE
      IF(RN3.GT.CHI1/2.D0) THEN
        IBREM=1
      ELSE
        IBREM=-1
      ENDIF
      END
      SUBROUTINE ANGPHO(RN1,AM2,COSTHG,SINTHG)
C ----------------------------------------------------------------------
C THIS ROUTINE GENERATES PHOTON ANGULAR DISTRIBUTION
C IN THE REST FRAME OF THE FERMION PAIR. THE DISTRIBUTION
C IS TAKEN IN THE INFRAED LIMIT.
C INPUT : AM2 = 4*MASSF**2/S WHERE MASSF IS FERMION MASS
C               AND S IS FERMION PAIR EFFECTIVE MASS.
C OUTPUT: COSTHG, SINTHG, COS AND SIN OF THE PHOTON
C         ANGLE WITH RESPECT TO FERMIONS DIRECTION
C
C     called by : RRR6
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*4 RRR
C
      BETA=SQRT(1.D0-AM2)
      EPS=AM2/(1.D0+SQRT(1.D0-AM2))
      DEL1=(2.D0-EPS)*(EPS/(2.D0-EPS))**RN1
      DEL2=2.D0-DEL1
C SYMMETRIZATION
      CALL RANMAR(RRR,1)
      RN2=RRR
      IF(RN2.LE.0.5D0) THEN
        A=DEL1
        DEL1=DEL2
        DEL2=A
      ENDIF
C CALCULATION OF SIN AND COS THETA FROM INTERNAL VARIABLES
      COSTHG=(1.D0-DEL1)/BETA
      SINTHG=SQRT(DEL1*DEL2-AM2)/BETA
      END
      SUBROUTINE RRR7(P,C,S)
C ----------------------------------------------------------------------
C THIS SUBPROGRAM GENERATES (1-C)**2 OR (1+C)**2 DISTRIBUTION
C WITH PROBABILITY  P AND  1-P
C
C     called by : EVENTM
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*4 RRR
C
 5    CONTINUE
      CALL RANMAR(RRR,1)
      X=RRR
      Z=0.D0
      IF(X.GT.P) Z=1.D0
      R=(X-Z*P)/(Z-P)
      C=(1.D0-2.D0*Z)*(1.D0-ABS(8.D0*R)**(1.D0/3.D0))
      S=SQRT(1-C*C)
      IF (S.LE.0.0) GOTO 5
      END
      SUBROUTINE GIQSU(CMSENE,PHSUM,QSU)
C ----------------------------------------------------------------------
C GIVEN CMS ENERGY (CMSENE) AND PHOTON SYSTEM TOTAL FOURMOM. (PHSUM)
C PROVIDES 4-MOMENTUM OF FERMION SYSTEM (QSU)
C
C     called by : TRALOZ
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 PHSUM(4),QSU(4)
C
      QSU(1)=       -PHSUM(1)
      QSU(2)=       -PHSUM(2)
      QSU(3)=       -PHSUM(3)
      QSU(4)=CMSENE -PHSUM(4)
*======================================================================
*==================END OF YFSGEN=======================================
*======================================================================
      END
      SUBROUTINE DUMPZ4(JK,PP)
C ----------------------------------------------------------------------
C PRINTS SINGLE FOUR MOMENTUM
C
C     called by :
C ----------------------------------------------------------------------
      COMMON / INOUT / INUT,IOUT
      REAL*4 PP(4)
C
      AMS=PP(4)**2
      DO 10 I=1,3
   10 AMS=AMS-PP(I)**2
      IF(AMS.GT.0.0) AMS=SQRT(AMS)
      WRITE (IOUT,1000)
      WRITE (IOUT,1502) JK,(PP(I),I=1,4),AMS
 1000 FORMAT(1X,'* DUMPZ4 *-------',8(10H----------))
 1502 FORMAT(10X,I6,3X,'FOURMOMENTUM  ',5(1X,F12.5))
C======================================================================
C================END OF JMCLIB=========================================
C======================================================================
      END
      SUBROUTINE DUMPZ8(CH,PP)
C ----------------------------------------------------------------------
C PRINTS SINGLE FOUR MOMENTUM
C
C     called by : PEDYVV, TRAAAA, EVENTE, ...
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / INOUT / INUT,IOUT
      REAL*8 PP(4)
      CHARACTER*4 CH
C
      AMS=PP(4)**2
      DO 10 I=1,3
   10 AMS=AMS-PP(I)**2
      IF(AMS.GT.0.0) AMS=SQRT(AMS)
      WRITE(IOUT,*) '--------------------------------------------------'
      WRITE(IOUT,1502) CH,(PP(I),I=1,4),AMS
 1502 FORMAT(1X,A4,1X,4F15.10,F10.6)
      END
      SUBROUTINE DUMPL9(QP,QM,PH)
C ----------------------------------------------------------------------
C FOR TESTS
C PRINT MOMENTA OF THE PRODUCTION PROCESS
C
C     called by : KORALZ, SANGLE, BDRESS, SKONTY
C ----------------------------------------------------------------------
      IMPLICIT REAL  (A-H,O-Z)
      DIMENSION QP(4),QM(4),PH(4)
      DIMENSION SUM(4)
      DATA ICONT/0/
C
      ICONT=ICONT+1
      IF (ICONT.GT.200) RETURN
      DO 30 I=1,4
  30  SUM(I)=QP(I)+QM(I)+PH(I)
      PRINT      1100,ICONT
      AQP=     QP(4)**2-QP(3)**2-QP(2)**2-QP(1)**2
      AQM=     QM(4)**2-QM(3)**2-QM(2)**2-QM(1)**2
      APH=     PH(4)**2-PH(3)**2-PH(2)**2-PH(1)**2
      IF(AQP.GT.0.) AQP=SQRT(AQP)
      IF(AQM.GT.0.) AQM=SQRT(AQM)
      PRINT 1501, (QP(I),I=1,4),AQP
      PRINT 1502, (QM(I),I=1,4),AQM
      PRINT 1503, (PH(I),I=1,4),APH
      PRINT 1600, ( SUM(I),I=1,4)
 1100 FORMAT(  /20X,'MOMENTA FROM UTIL9,     PRINT NO.  ',I5/
     &40X,'  P(1)',7X,'  P(2)',7X,'  P(3)',7X,'  P(4)',7X,'  MASS')
 1501 FORMAT(20X,'QP  ',9X,5(1X,F12.5))
 1502 FORMAT(20X,'QM  ',9X,5(1X,F12.5))
 1503 FORMAT(20X,'PH  ',9X,5(1X,F12.5))
 1600 FORMAT(20X,'SUM ',9X,5(1X,F12.5),/)
      END
      SUBROUTINE DUMPL8
C ----------------------------------------------------------------------
C FOR TESTS
C PRINT MOMENTA OF THE PRODUCTION PROCESS
C
C     called by :
C ----------------------------------------------------------------------
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON / UTIL8 / QP(4),QM(4),PH(4)
      REAL*8           QP   ,QM   ,PH
      DIMENSION SUM(4)
      DATA ICONT/0/
C
      ICONT=ICONT+1
      DO 30 I=1,4
  30  SUM(I)=QP(I)+QM(I)+PH(I)
      PRINT      1100,ICONT
      AQP=     QP(4)**2-QP(3)**2-QP(2)**2-QP(1)**2
      AQM=     QM(4)**2-QM(3)**2-QM(2)**2-QM(1)**2
      APH=     PH(4)**2-PH(3)**2-PH(2)**2-PH(1)**2
      IF(AQP.GT.0.) AQP=SQRT(AQP)
      IF(AQM.GT.0.) AQM=SQRT(AQM)
      PRINT 1501, (QP(I),I=1,4),AQP
      PRINT 1502, (QM(I),I=1,4),AQM
      PRINT 1503, (PH(I),I=1,4),APH
      PRINT 1600, ( SUM(I),I=1,4)
 1100 FORMAT(  /20X,'MOMENTA FROM UTIL8,     EVENT NO.  ',I5/
     &40X,'  P(1)',7X,'  P(2)',7X,'  P(3)',7X,'  P(4)',7X,'  MASS')
 1501 FORMAT(20X,'QP  ',9X,5(1X,F12.5))
 1502 FORMAT(20X,'QM  ',9X,5(1X,F12.5))
 1503 FORMAT(20X,'PH  ',9X,5(1X,F12.5))
 1600 FORMAT(20X,'SUM ',9X,5(1X,F12.5),/)
      END
      SUBROUTINE VESK8A(MODE,PAR1,PAR2)
C ----------------------------------------------------------------------
C======================================================================
C======================================================================
C===================*** EXPLIB ***=====================================
C=========GENERAL PURPOSE LIBRARY OF THE MONTE CARLO ==================
C================AND LORENTZ KINEMATICS================================
C============= BY  S. JADACH,  APRIL 1986==============================
C======================================================================
C===================== V E S K 8 A ====================================
C==================S. JADACH  SEPTEMBER 1985===========================
C======================================================================
C ONE DIMENSIONAL MONTE CARLO  SAMPLER.
C DOUBLE PRECISION  FUNCTION FUNSKO IS THE DISTRIBUTION TO BE GENERATED
C JLIM1 IS THE NUMBER OF ENTRIES IN THE EQUIDISTANT LATICE WHICH
C IS FORMED IN THE FIRST STAGE AND JLIM2 IS THE TOTAL MAXIMUM
C NUMBER OF ENTRIES IN THE LATTICE, NOTE THAT DIMENSIONS OF
C MATRICES IN /CESK8A/ SHOULD BE AT LEAST JLIM2+1 .
C FOR MILD FUNSKO JLIM2=128 IS ENOUGH.
C TO CREATE AN INDEPENDANT VERSION REPLACE /ESK8A/=>/ESK8B/.
C
C     called by : EVENTM, KARLUD
C ----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON / CESK8A / XX(1025),YY(1025),ZINT(1025),ZSUM,JMAX
      COMMON / INOUT / INUT,IOUT
      REAL*4 RRR(2)
      DATA JLIM1,JLIM2/16,257/
      DATA IWARM/0/
C
      IF(MODE.EQ.-1) THEN
C     ===================
C INITIALISATION PART, SEE VINSKO FOR MORE COMMENTS
        INIRAN=1
        IWARM=1
        WT=0.
        SWT=0.
        SSWT=0.
        NEVS=0
C INITIALISATION PART, SAMPLING DISTRIBUTION FUNSKO
C AND FILLING MATRICES XX,YY,ZINT ETC.
        JMAX=1
        XX(1)=0.
        XX(2)=1.
        YY(1)=FUNSKA(0,XX(1))
        YY(2)=FUNSKA(0,XX(2))
        IF(YY(1).LT.0.0.OR.YY(2).LT.0.0) GO TO 999
        ZINT(1)=.5D0*(YY(2)+YY(1))*(XX(2)-XX(1))
C
        JDIV=1
        DO 200 K=1,JLIM2-1
        IF(JMAX.LT.JLIM1) THEN
C...      NOTE THAT DESK8A INCREMENTS JMAX=JMAX+1 IN EVERY CALL
          CALL DESK8A(JDIV)
          JDIV=JDIV+2
          IF(JDIV.GT.JMAX) JDIV=1
        ELSE
          JDIV=1
          ZMX=ZINT(1)
          DO 180 J=1,JMAX
          IF(ZMX.LT.ZINT(J)) THEN
            ZMX=ZINT(J)
            JDIV=J
          ENDIF
  180     CONTINUE
          CALL DESK8A(JDIV)
        ENDIF
  200   CONTINUE
C
C...    FINAL ADMINISTRATION, NORMALIZING ZINT ETC.
        ZSUM1=0.
        ZSUM =0.
        DO 220 J=1,JMAX
        ZSUM1=ZSUM1+ZINT(J)
        YMAX= MAX( YY(J+1),YY(J))
        ZINT(J)=YMAX*(XX(J+1)-XX(J))
  220   ZSUM=ZSUM+ZINT(J)
        SUM=0.
        DO 240 J=1,JMAX
        SUM=SUM+ZINT(J)
  240   ZINT(J)=SUM/ZSUM
C
      ELSE IF(MODE.EQ.0) THEN
C     =======================
C GENERATION PART
        IF(IWARM.EQ.0) GOTO 901
  222 CONTINUE
        CALL RANMAR(RRR,2)
        IF(WT.GT.1.) THEN
          WT=WT-1.D0
        ELSE
          RNUMB=RRR(1)
          DO 215 J=1,JMAX
          JSTOP=J
  215     IF(ZINT(J).GT.RNUMB) GOTO 216
  216     CONTINUE
          IF(JSTOP.EQ.1) THEN
            D=RNUMB/ZINT(1)
          ELSE
            D =(RNUMB-ZINT(JSTOP-1))/(ZINT(JSTOP)-ZINT(JSTOP-1))
          ENDIF
          X=XX(JSTOP)*(1.D0 -D )+XX(JSTOP+1)*D
          FN=FUNSKA(0,X)
          IF(FN.LT.0.) GOTO 999
          YYMAX=MAX(YY(JSTOP+1),YY(JSTOP))
          WT=FN/YYMAX
          NEVS=NEVS+1
          SWT=SWT+WT
          SSWT=SSWT+WT*WT
C         CALL HFILL(40,WT)
        ENDIF
        RNUMB=RRR(2)
        IF(RNUMB.GT.WT) GOTO 222
        PAR1=  X
        PAR2=  FN
C
      ELSE IF(MODE.EQ.1) THEN
C     =======================
C FINAL STATISTICS
        CINTEG=ZSUM*SWT/FLOAT(NEVS)
        ERRINT=SQRT(SSWT/SWT**2-1.D0/FLOAT(NEVS))
        PAR1=  CINTEG
        PAR2=  ERRINT
C
      ELSE
C     ====
        GOTO  902
      ENDIF
C     =====
C
      RETURN
 901  WRITE(IOUT,9010)
 9010 FORMAT(' **** STOP IN VESK8A, LACK OF INITIALISATION')
      STOP
 902  WRITE(IOUT,9020)
 9020 FORMAT(' **** STOP IN VESK8A, WRONG MODE ')
      STOP
 999  WRITE(IOUT,9990)
 9990 FORMAT(' **** STOP IN VESK8A, NEGATIVE VALUE OF FUNSKO ')
      STOP
      END
      SUBROUTINE DESK8A(JDIV)
C ----------------------------------------------------------------------
C THIS ROUTINE BELONGS TO VESK8A PACKAGE
C IT SUDIVIDES INTO TWO EQUAL PARTS THE INTERVAL
C (XX(JDIV),XX(JDIV+1))  IN THE LATTICE
C
C     called by : VESK8A
C ----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON / INOUT / INUT,IOUT
      COMMON / CESK8A / XX(1025),YY(1025),ZINT(1025),ZSUM,JMAX
C
      XNEW=.5D0*(XX(JDIV) +XX(JDIV+1))
      DO 100 J=JMAX,JDIV,-1
      XX(J+2)  =XX(J+1)
      YY(J+2)  =YY(J+1)
  100 ZINT(J+1)=ZINT(J)
      XX(JDIV+1)= XNEW
      YY(JDIV+1)= FUNSKA(0,XNEW)
      IF(YY(JDIV+1).LT.0.) GOTO 999
      ZINT(JDIV)  =.5D0*(YY(JDIV+1)+YY(JDIV)  )*(XX(JDIV+1)-XX(JDIV)  )
      ZINT(JDIV+1)=.5D0*(YY(JDIV+2)+YY(JDIV+1))*(XX(JDIV+2)-XX(JDIV+1))
      JMAX=JMAX+1
      RETURN
  999 WRITE(IOUT,9000)
 9000 FORMAT(' **** STOP IN DESK8A, NEGATIVE VALUE OF FUNSKO ')
      STOP
      END
      FUNCTION SP(Y,E)
C ----------------------------------------------------------------------
C  SPENCE FUNCTION OF Y+I*REAL(E) WHERE E IS AN INFINITESIMAL
C
C     called by : FIU, MATELM, ...
C ----------------------------------------------------------------------
C     IMPLICIT LOGICAL (A-H,O-Z)
      IMPLICIT REAL*8  (A-H,O-Z)
      COMPLEX*16 Y,E,SP
      REAL*8 B(9),FACT
      COMPLEX*16 A,CLN,PISQ6,PROD,TERM,X,Z,ZSQ
      COMPLEX*16 CDLN
C
      B(1)=1.D0/6.D0
      B(2)=-1.D0/30.D0
      B(3)=1.D0/42.D0
      B(4)=B(2)
      B(5)=5.D0/66.D0
      B(6)=-691.D0/2730.D0
      B(7)=7.D0/6.D0
      B(8)=-3617.D0/510.D0
      B(9)=43867.D0/798.D0
      PISQ6=(1.6449340668482264D0,0.D0)
      I1=0
      I2=0
      X=Y
      A=E
      IF(X.EQ.(0.D0,0.D0))THEN
        SP=(0.D0,0.D0)
        RETURN
      ENDIF
      IF(X.EQ.(1.D0,0.D0))THEN
        SP=PISQ6
        RETURN
      ENDIF
C  IF X LIES OUTSIDE THE UNIT CIRCLE THEN EVALUATE SP(1/X)
      IF(CDABS(X).GT.1.D0)THEN
        X=1.D0/X
        A=-A
        I1=1
      ENDIF
C  IF REAL(X)>1/2 THEN EVALUATE SP(1-X)
      IF(DREAL(X).GT.0.5D0)THEN
        X=1.D0-X
        A=-A
        I2=1
      ENDIF
C  EVALUATE SERIES FOR SP(X)
      Z=-CDLN(1.D0-X,-A)
      ZSQ=Z*Z
      SP=Z-ZSQ/4.D0
      PROD=Z
      FACT=1.D0
      DO 10 J=2,18,2
      FACT=FACT*DCMPLX(DBLE((J+1)*J))
      PROD=PROD*ZSQ
      TERM=B(J/2)/FACT*PROD
      SP=SP+TERM
      IF(CDABS(TERM/SP).LT.1.D-20)GO TO 20
10    CONTINUE
C  ADD APPROPRIATE LOGS TO OBTAIN SPENCE FUNCTION OF ORIGINAL ARGUEMENT
20    IF(I2.EQ.1)THEN
        SP=-SP+PISQ6-CDLN(X,A)*CDLN(1.D0-X,-A)
        X=1.D0-X
        A=-A
      ENDIF
      IF(I1.EQ.1)THEN
        CLN=CDLN(-X,-A)
        SP=-SP-PISQ6-CLN*CLN/2.D0
      ENDIF
      RETURN
      END
      DOUBLE PRECISION FUNCTION DLI2(X)
C ----------------------------------------------------------------------
C DILOGARITHM FOR X <= 1, APPROPRIATE FOR VALUES VERY CLOSE TO 0
C
C     called by : AIMPI
C ----------------------------------------------------------------------
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMPLEX*16 SP
C
      IF (DABS(X).LE.1.D-15) THEN
        DLI2=X
      ELSE
        DLI2=DREAL(SP(DCMPLX(X),DCMPLX(.1D1)))
      ENDIF
      RETURN
      END
      FUNCTION CDLN(X,A)
C ----------------------------------------------------------------------
C  COMPLEX FUNCTIONS THAT TAKE ACCOUNT OF THE I*EPSILON PRESCRIPTION
C              TO CALCULATE ANALYTIC STRUCTURE
C
C  COMPLEX LOGARITHM OF X+I*REAL(A) WHERE A IS AN INFINITESIMAL
C
C     called by : BZED1, FIU, MATELM, ...
C ----------------------------------------------------------------------
C     IMPLICIT LOGICAL (A-H,O-Z)
      IMPLICIT REAL*8  (A-H,O-Z)
      COMPLEX*16 A,X,CDLN
      COMPLEX*16 PI
      PI=(3.141592653589793238462643D0,0.D0)
      IF(DIMAG(X).EQ.0.D0.AND.DREAL(X).LE.0.D0)THEN
        CDLN=CDLOG(-X)+(0.D0,1.D0)*PI*DSIGN(1.D0,DREAL(A))
      ELSE
        CDLN=CDLOG(X)
      END IF
      IF(DIMAG(CDLN).GT.DREAL(PI))CDLN=CDLN-(0.D0,1.D0)*PI
      IF(DIMAG(CDLN).LT.DREAL(-PI))CDLN=CDLN+(0.D0,1.D0)*PI
      RETURN
      END

!      FUNCTION WTANOM(Dumm)
!      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
!      wtanom=1d0
!      end


