      SUBROUTINE ANOMINI_L3
* ==================================================================== *   
*                                                                      *     
*     SUBR.      ANOMINI_L3                                            *     
*                                                                      *
*     Initialize various constants used by TTG.  Normally these        *
*     are set in data cards (but we don't read them here)              *
*                                                                      *
*                                                                      *  
* ==================================================================== *       
      IMPLICIT NONE
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in 
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep  
      DOUBLE PRECISION gzis,gzfs    
      
C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION 
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag
  
 

C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,             
     +       ep_tm, 
     +       em_tm, 
     +       k_tm, 
     +       ep_k, 
     +       em_k, 
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_
C--
C--  Common for things required to make the 
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C-- 
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)     
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag

                   
      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL


C--
C--  Common for name of file to write ntuples, and number of events.
C-- 
      INTEGER NEVT            ! number of events to generate
      INTEGER NXS             ! number of events for cross-section calculation
      CHARACTER*80 NTPFILE    ! ntuple file
      INTEGER IGEN            ! IGEN = 0 for weights, <>0 for MC events
      DOUBLE PRECISION ECM    ! Centre of mass energy (GeV)
      REAL EMIN,CCUT          ! Cuts on minimum photon energy and cos(theta)
      REAL CTGC,XTMX          ! Cuts on cos(photon,tau) and tau maximum x
      REAL EWMX               ! Maximum expected wt*dme2 (set iteratively)
      INTEGER MNTP            ! > 0 to make ntuples

      COMMON/RUNPARS/NEVT,    ! number of events to generate
     +               NXS,     ! number of events for cross-section calculation
     +               NTPFILE, ! ntuple output file                 
     +               IGEN,    ! IGEN = 0 for weights, <>0 for MC events 
     +               ECM,     ! Centre of mass energy (GeV) 
     +               EMIN,    ! Minimum photon energy (GeV)
     +               CCUT,    ! cut |cos(theta(photon))| < CCUT
     +               CTGC,    ! cut on cos(tau,gamma) < CTGC
     +               XTMX,    ! maximum x of tau
     +               EWMX,    ! expected max of wt*dme2
     +               MNTP     ! > 0 to make ntuples

      REAL SMT,SME,SGV,SGA,SFMT,SFET,SMZ,SGZ
      REAL SEE
      REAL SISR,SFSR,SASR,SZEX,SGEX,SZGI,SAFI,SIFI,SIAI  

      REAL SINTW2 ! sin(theta_w)**2, used only for overall normalization

      INTEGER NW
      REAL SPACE
      PARAMETER(NW=10000)    ! define space for ffread
      COMMON/CFREAD/SPACE(NW)

C--   Define the number of characters per word in the
C--   horrible FFREAD method of reading in character strings
C--   and then sending them to UCTOH.
      INTEGER       NCHARW
      PARAMETER    (NCHARW=4)
C--   Counters for positions in strings                                               
      INTEGER IPOS,JPOS 
*
      REAL RATS
C
C--   Constants of nature from KORALZ
C--
      DOUBLE PRECISION SENE,SSWSQ,SALFINV,SAMZ,SGAMMZ,SAMFIN
      COMMON /KZ_STORE/ SENE,SSWSQ,SALFINV,SAMZ,SGAMMZ,SAMFIN
C
C--   OTHER USER FLAGS AND CONSTANTS FROM KORALZ
C--
      INTEGER ISFL,IF2,IF3
      COMMON /TTG_USER/ ISFL,IF2,IF3

      INTEGER IF2_ON,IF3_ON
      COMMON/F_FLAGS/IF2_ON,IF3_ON

* 
*-----------------------------------------------------------------------
*
*
C      CALL INITH
*
C--   Simple flag (see documentation in ttg.f)
      IF (ISFL.GE.-1.AND.ISFL.LE.1) THEN
        SFL=ISFL
      ELSE
        WRITE(*,*) 'ANOMINI_L3: Invalid ISFL flag ==> STOP'
        STOP
      ENDIF
*      SFL=1            ! SFL=1 corresonds to Riemann approx. (ie. basically correct)
      
      IF (IF2.EQ.0.OR.IF2.EQ.1) THEN
        IF2_ON=IF2
        IF (IF2.EQ.1)
     &  WRITE(*,*) 'Turning on tau anomalous mag. dipole moment'
      ELSE
        WRITE(*,*) 'ANOMINI_L3: Invalid IF2 flag ==> STOP'
        STOP
      ENDIF
*
      IF (IF3.EQ.0.OR.IF3.EQ.1) THEN
        IF3_ON=IF3
        IF (IF3.EQ.1) 
     &  WRITE(*,*) 'Turning on tau electric dipole moment'
      ELSE
        WRITE(*,*) 'ANOMINI_L3: Invalid IF3 flag ==> STOP'
        STOP
      ENDIF
*
      IF (IF2_ON.EQ.0.AND.IF3_ON.EQ.0) THEN
        WRITE(*,*) 'ANOMINI_L3: Warning: you have turned off'
        WRITE(*,*) '            ALL anomalous couplings with'
        WRITE(*,*) '            IF2 and IF3 flags'
      ENDIF
*      
C--   Variables to control which processes we include
      SISR = 1.0
      SFSR = 1.0
      SASR = 1.0
      SZEX = 1.0
      SGEX = 1.0
      SZGI = 1.0
      SAFI = 1.0
      SIFI = 1.0
      SIAI = 1.0

C--   Set parameters (all double precision)
*
      SFMT = 0.0
      SFET = 0.0
      SME = .511E-3                      ! okay to hardwire, as it makes no difference                                   
      SINTW2 = SNGL(SSWSQ) 
      SGA = -0.500
      SGV = SGA*(1.0-4.0*SINTW2)
*
      RATS=SNGL(SALFINV)/128.86674175E0  ! RATS used for overall normalization.  In practice
      RATS=SQRT(RATS)                    ! we always take ratios of ME, so RATS makes no
      SGV = SGV*RATS                     ! difference, but is nice to keep it for crosschecks
      SGA = SGA*RATS                     ! with Riemann, etc.
*
* TEMPORARY check to see if KORALZ+TTG works in limit of
* no EW corrections
*      SGV = SGV/RATS
*      SGA = SGA/RATS
* END TEMPORARY check
                                      
C--   Charge of electron and tau      
      SEE = SQRT(4.0*3.1415926/137.03604)

      GV = DBLE(SGV/(2.0*SQRT(SINTW2)*SQRT(1.0-SINTW2)))
      GA = DBLE(SGA/(2.0*SQRT(SINTW2)*SQRT(1.0-SINTW2)))
      FMT = DBLE(SFMT)
      FET = DBLE(SFET)
*      MZ = DBLE(SMZ)
*      GZ = DBLE(SGZ)
      EE = DBLE(SEE)
      GE = EE
      MT = SAMFIN
      ME = DBLE(SME)
      GZ = SGAMMZ
      MZ = SAMZ
      ECM= SENE

C--   Variables to control which processes we include
      ISR = DBLE(SISR)
      FSR = DBLE(SFSR)
      ASR = DBLE(SASR)
      ZEX = DBLE(SZEX)
      GEX = DBLE(SGEX)
      ZGI = DBLE(SZGI)
      AFI = DBLE(SAFI)
      IFI = DBLE(SIFI)
      IAI = DBLE(SIAI)

C--   Centre of mass energy
      ECM = SENE

      WRITE(*,*) ''
      WRITE(*,*) 'Constants set in TTG: '
      WRITE(*,*) '--------------------- '
      WRITE(*,*) 'SINTW2 = ',SINTW2
      WRITE(*,*) 'GV     = ',SGV,GV
      WRITE(*,*) 'GA     = ',SGA,GA
      WRITE(*,*) 'MT     = ',MT
      WRITE(*,*) 'ME     = ',ME
      WRITE(*,*) 'GZ     = ',GZ
      WRITE(*,*) 'MZ     = ',MZ
      WRITE(*,*) 'ECM    = ',ECM
      WRITE(*,*) ''

      RETURN
      END


      SUBROUTINE FU_L3(IDDE,IDDF,SEP,SEM,STP,STM,SK)                                                         
* ==================================================================== *   
*                                                                      *     
*     SUBR.      FU_L3                                                 *     
*                                                                      *
*     Calls the appropriate TTG routines to compute the weight         *
*     for this MDM, EDM.      The weight is defined as the             *
*     cross section (MDM,EDM) / standard model cross section           *
*                                                                      *                             
*      Input:  SEP,SEM  =   electron 4-vectors                         *
*              STP,STM  =   tau      4-vectors                         *
*              SK       =   photon   4-vector                          *
*      Variable WTKAL(6) in common KALINOUT is filled with the         *
*      6 parameters necessary to compute weight for any MDM,EDM        *
*      such that for a given event:                                    *
*                                                                      *
*       WEIGHT = WTKAL(2) +                                            *
*                WTKAL(3)*MDM**2 + WTKAL(4)*MDM                        *
*              + WTKAL(5)*EDM**2 + WTKAL(6)*EDM                        *
*                                                                      *
*      (in fact, only 3 parameters are computed, but this              *      
*       general equation works...)                                     *
*                                                                      *  
* ==================================================================== *       
      IMPLICIT NONE
*
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in 
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep  
      DOUBLE PRECISION gzis,gzfs    
      
C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION 
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag
  
 

C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,             
     +       ep_tm, 
     +       em_tm, 
     +       k_tm, 
     +       ep_k, 
     +       em_k, 
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_
C--
C--  Common for things required to make the 
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C-- 
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)     
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag

                   
      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL


C--
C--  Common for name of file to write ntuples, and number of events.
C-- 
      INTEGER NEVT            ! number of events to generate
      INTEGER NXS             ! number of events for cross-section calculation
      CHARACTER*80 NTPFILE    ! ntuple file
      INTEGER IGEN            ! IGEN = 0 for weights, <>0 for MC events
      DOUBLE PRECISION ECM    ! Centre of mass energy (GeV)
      REAL EMIN,CCUT          ! Cuts on minimum photon energy and cos(theta)
      REAL CTGC,XTMX          ! Cuts on cos(photon,tau) and tau maximum x
      REAL EWMX               ! Maximum expected wt*dme2 (set iteratively)
      INTEGER MNTP            ! > 0 to make ntuples

      COMMON/RUNPARS/NEVT,    ! number of events to generate
     +               NXS,     ! number of events for cross-section calculation
     +               NTPFILE, ! ntuple output file                 
     +               IGEN,    ! IGEN = 0 for weights, <>0 for MC events 
     +               ECM,     ! Centre of mass energy (GeV) 
     +               EMIN,    ! Minimum photon energy (GeV)
     +               CCUT,    ! cut |cos(theta(photon))| < CCUT
     +               CTGC,    ! cut on cos(tau,gamma) < CTGC
     +               XTMX,    ! maximum x of tau
     +               EWMX,    ! expected max of wt*dme2
     +               MNTP     ! > 0 to make ntuples

      REAL SMT,SME,SGV,SGA,SFMT,SFET,SMZ,SGZ
      REAL SEE
      REAL SISR,SFSR,SASR,SZEX,SGEX,SZGI,SAFI,SIFI,SIAI  
      REAL SECM

      INTEGER NW
      REAL SPACE
      PARAMETER(NW=10000)    ! define space for ffread
      COMMON/CFREAD/SPACE(NW)

C--   Define the number of characters per word in the
C--   horrible FFREAD method of reading in character strings
C--   and then sending them to UCTOH.
      INTEGER       NCHARW
      PARAMETER    (NCHARW=4)
C--   Counters for positions in strings                                               
      INTEGER IPOS,JPOS 


      INTEGER IDDE,IDDF
      INTEGER I
*      REAL SEP(4),SEM(4),STP(4),STM(4),SK(4),ANO,WT
      DOUBLE PRECISION SEP(4),SEM(4),STP(4),STM(4),SK(4),ANO,WT,XANO
      
      DOUBLE PRECISION DME2_SM,DME2_NSM,TINY
      PARAMETER (TINY=1.0D-9)
*
* *** common to write parameters necessary to compute weight for 
* *** arbitrary MDM,EDM
*     
      DOUBLE PRECISION WTKAL(6)
      COMMON /KALINOUT/ WTKAL
*
* *** Values of mdm,edm to use in computing weighting parameters
* *** (only 3 points are necessary even though there are 6 
* *** parameters.  This is because the anomalous contribution = 0
* *** for MDM=0 or EDM=0, and because the linear (interference) 
* *** term vanishes in case of EDM.)
*
      DOUBLE PRECISION P1,P2,P3
      DOUBLE PRECISION WTP1,WTP2,WTP3
      DOUBLE PRECISION A,B,C
      PARAMETER (P1=1.0D0)               ! EDM
      PARAMETER (P2=0.1D0)               ! MDM
      PARAMETER (P3=-0.1D0)              ! MDM

*
* *** This common  (PLOTS) contains stuff to pass back to pracus.f for 
* *** making plots to check things.
*
      DOUBLE PRECISION DALTPLUS,DALTMINUS,DENO,DTHEGAM
      REAL PEGAM,ALTPLUS,ALTMINUS,THEGAM
      COMMON / PLOTS  / PEGAM,ALTPLUS,ALTMINUS,THEGAM
*
* *** Flags to choose if you want to compute weights for
* *** F_2 only, F_3 only, or both
*
      INTEGER IF2_ON,IF3_ON
      COMMON/F_FLAGS/IF2_ON,IF3_ON
*
*-----------------------------------------------------------------------
*
* *** Load quantities to pass back to pracus for filling
* *** test histograms (these go into common PLOTS)
* *** This (and PLOTS common) can be removed once tests 
* *** are finished.
*
* photon energy
*!      PEGAM=SNGL(SK(4))
* angle to tau + and tau -
*!      CALL DDOTEM(STP,SK, DALTPLUS)
*!      CALL DDOTEM(STM,SK, DALTMINUS)
*!      ALTPLUS =SNGL(DALTPLUS)
*!      ALTMINUS=SNGL(DALTMINUS)

*      DENO = DSQRT(SK(1)**2 + SK(2)**2 + SK(3)**2)
      DENO=SK(4)
      IF (DENO.GT.1.0D-8) THEN
        DTHEGAM = SK(3)/ DENO
      ELSE
        PRINT *,'----> DENO too small ',DENO
        DTHEGAM = 999.D0
      ENDIF
      THEGAM = SNGL(DTHEGAM)
*
      DO I=1,4
        EP(I) = SEP(I)
        EM(I) = SEM(I)
        TP(I) = STP(I)
        TM(I) = STM(I)
        K (I) = SK (I)
      ENDDO
*
* *** Standard model matrix element
*
      FMT = 0.0D0
      FET = 0.0D0
      CALL MAKEME2(DME2_SM)
*
* *** Anomalous matrix element computed for different
* *** sets of MDM, EDM
*
      IF (IF3_ON.EQ.1) THEN
        FMT=0.0D0
        FET =P1
        CALL MAKEME2(DME2_NSM)
        WTP1 = DME2_NSM/DME2_SM
      ENDIF
*
      IF (IF2_ON.EQ.1) THEN
        FMT=P2
        FET=0.0D0
        CALL MAKEME2(DME2_NSM)
        WTP2 = DME2_NSM/DME2_SM

        FMT=P3
        FET=0.0D0
        CALL MAKEME2(DME2_NSM)
        WTP3 = DME2_NSM/DME2_SM
      ENDIF
*            
      IF (IF3_ON.EQ.1) THEN
        C=(WTP1-1.0D0)/P1**2
      ELSE
        C=0.0D0
      ENDIF
*
      IF (IF2_ON.EQ.1) THEN
        A=(1.0D0/(P3**2-P2*P3))*(WTP3-1.0D0-P3*(WTP2-1.0D0)/P2)
        B=(WTP2-1.0D0-A*P2**2)/P2
      ELSE
        A=0.0D0
        B=0.0D0
      ENDIF
*
      WTKAL(1) = 0.0D0
      WTKAL(2) = 1.0D0
      WTKAL(3) = A
      WTKAL(4) = B
      WTKAL(5) = C
      WTKAL(6) = 0.0D0
      
***      IF (DME2_SM.GT.TINY) THEN
***        WT  = DME2_NSM/DME2_SM
***        XANO= (DME2_NSM - DME2_SM)/DME2_SM
***      ELSE 
***        PRINT *,'** Warning in FU_L3: DME2_SM=0'
***        PRINT *,'** Setting weight to 1'
***        WT = 1.D0
***      ENDIF
*
      RETURN 
      END

      SUBROUTINE KZ_STOREPARMS(ene,swsq,alfinv,amz,gammz,amfin,il,i2,i3)
* ==================================================================== *   
*                                                                      *     
*     SUBR. KZ_STOREPARMS(ene,swsq,alfinv,amz,gammz,amfin,il,i2,i3)    *     
*                                                                      *
*     Copy some constants needed for TTG initialization into           *
*     the commons KZ_STORE and TTG_USER                                *
*                                                                      *  
* ==================================================================== *      
      double precision ene,swsq,alfinv,amz,gammz,amfin
      DOUBLE PRECISION SENE,SSWSQ,SALFINV,SAMZ,SGAMMZ,SAMFIN
      COMMON /KZ_STORE/ SENE,SSWSQ,SALFINV,SAMZ,SGAMMZ,SAMFIN 

      INTEGER ISFL,IF2,IF3
      COMMON /TTG_USER/ ISFL,IF2,IF3
* 
*-----------------------------------------------------------------------
*
      isfl = il              ! Simple flag for TTG (1 = Riemann approx.)
      if2  = i2              ! Compute weights for MDM (f_2) (1=on,0=off)
      if3  = i3              ! Compute weights for EDM (f_3) (1=on,0=off)
      
      SENE    =ENE
      SSWSQ   =SWSQ
      SALFINV =ALFINV
      SAMZ    =AMZ
      SGAMMZ  =GAMMZ
      SAMFIN  =AMFIN

      RETURN
      END


*      SUBROUTINE FU_L3(IDDE,IDDF,SEP,SEM,STP,STM,SK)
*
*       DOUBLE PRECISION SEP(4),SEM(4),STP(4),STM(4),SK(4),ANO,WT,XANO
*
*
*       RETURN
*       END


