
      FUNCTION delb2f(emin,delta,am1,am2,q1,q2,r1,r2,sphum)
*     ******************************************************
!----------------------------------------------------------------------!
* Exact formula for delb2 in the compensating weight exp(delb2)
* necessary in the case of erasing soft photons below emin, 
* version for the case of finite mass.
* It is a linear combination of 4 Breal function:
* 1 exact and 1 crude, for 2 boundries.
* Remark on r1, r2:
* They are ghost-fermion momenta in the definition of the crude 
* distribution, i.e. truncated S-factor is r1*r2/(k*r1)/(k*r2) 
* as generated in the crude MC.
* In QMS frame 4-momenta r1, r2 have the same energy 
* and directions as q1, q2
* but different masses am1c, am2c and therefore longer 3-momenta.
* They are usefull because we may use the same analytical
* expression for the exact Breal and  for "crude Breal"
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
      REAL*8 q1(4),q2(4),r1(4),r2(4),sphum(4)
      REAL*8 qq(4),pp(4)
*
      amf=min(am1,am2)          ! basic mass in crude MC
      DO k=1,4
         pp(k) = q1(k)+q2(k) +sphum(k)
         qq(k) = q1(k)+q2(k)
      ENDDO
      svar = pp(4)**2-pp(3)**2 -pp(2)**2 -pp(1)**2
      sprim= qq(4)**2-qq(3)**2 -qq(2)**2 -qq(1)**2
      Eqq = 0.5d0*sqrt(sprim)
      Eq1 = (sprim +am1**2 -am2**2)/(2*sqrt(sprim))
      Eq2 = (sprim -am1**2 +am2**2)/(2*sqrt(sprim))
      qqk  = qq(4)*sphum(4)-qq(3)*sphum(3)-qq(2)*sphum(2)-qq(1)*sphum(1)
      delt1 =  delta*(1+ 2*qqk/sprim)
      eming = Eqq*delt1
      amgam =1d-60 ! dummy photon mass
********* various input parameters for Btildc and Btildf
      am1c = amf*sqrt(sprim/svar)
      am2c = amf*sqrt(sprim/svar)
      q1q2  = q1(4)*q2(4) - q1(3)*q2(3) - q1(2)*q2(2) - q1(1)*q2(1)
      r1r2  = r1(4)*r2(4) - r1(3)*r2(3) - r1(2)*r2(2) - r1(1)*r2(1)
********* CMS boundry, difference crude-exact
      dcms2c = Btildc(r1r2,  r1(4),r2(4), am1c,am2c, emin,amgam) !crude
      dcms2f = Btildf(q1q2,  q1(4),q2(4),  am1, am2, emin,amgam) !exact
********* QMS boundry, difference crude-exact
      dqms2c = Btildc(r1r2,   Eqq,Eqq, am1c,am2c, eming,amgam) !crude
      dqms2f = Btildf(q1q2,   Eq1,Eq2, am1 , am2, eming,amgam) !exact
********* QMS - CMS, result should be negative ( delta<<epsilon )
      delb2f = (dqms2c-dqms2f) - (dcms2c-dcms2f)
*****[[[[[[*********DEBUG****
c      IF((1-sprim/svar) .gt. 0.2d0) THEN
c         x = Ttildf(q1q2,   Eq1,Eq2, am1 , am2, eming,amgam)
c         WRITE(6,'(a,6f20.10)')
c     $  '==: ',dqms2c,dcms2c,dqms2f,dcms2f
c      ENDIF
*****]]]]]]*********DEBUG****
      END

      FUNCTION Btildf(p1p2,E1,E2,am1,am2,aKmax,amgam)
*     ***********************************************
!----------------------------------------------------------------------!
! Full/Complete 2-particle Btilde function.
! Equivalent of routine Btilde programmed by W. Placzek (S.J.)
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
      PARAMETER ( pi = 3.1415926535897932D0, alfinv = 137.0359895d0 )
      PARAMETER ( alfpi = 1/alfinv/pi )
!
      Btildf = alfpi*(
     $     (p1p2*Aanal( p1p2,am1,am2) -1 )*2*LOG(2*aKmax/amgam)
     $     +p1p2*A4anal(p1p2,E1,E2,am1,am2)
     $     -0.5d0*am1**2*A4snal(E1,am1)
     $     -0.5d0*am2**2*A4snal(E2,am2)
     $     )
      END

      FUNCTION Btildc(p1p2,E1,E2,am1,am2,aKmax,amgam)
*     ***********************************************
!----------------------------------------------------------------------!
! Crude/Truncated (crude MC) 2-particle Btilde equivalent S.J.
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
      PARAMETER ( pi = 3.1415926535897932D0, alfinv = 137.0359895d0 )
      PARAMETER ( alfpi = 1/alfinv/pi )
!
      Btildc = alfpi*(
     $      p1p2*Aanal( p1p2      ,am1,am2)*2*LOG(2*aKmax/amgam)
     $     +p1p2*A4anal(p1p2,E1,E2,am1,am2)
     $     )
      END

      FUNCTION Aanal(p1p2,am1,am2)
*     ****************************
!----------------------------------------------------------------------!
! Function A(p1,p2)
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
      am12 = am1*am2
      IF (p1p2-am12.lt.1d-10) THEN
         Aanal=0d0
         RETURN
      ENDIF
      xlam = SQRT( (p1p2 - am12)*(p1p2 + am12) )
      Aanal  = 1/xlam *LOG( (p1p2 + xlam)/am12 )
      END

      FUNCTION A4snal(E1,am1)
*     ***********************
      IMPLICIT REAL*8(a-h,o-z)
!----------------------------------------------------------------------!
! Function (p1*p1)*A4(p1,p1)
!----------------------------------------------------------------------!
      bet1 = SQRT(1-am1**2/E1**2)
      b1ln = 2*LOG( (1+bet1)*E1/am1 )
      A4snal = -1/am1**2/bet1 *b1ln
      END

      FUNCTION A4anal(p1p2,En1,En2,xm1,xm2)
*     *************************************
!----------------------------------------------------------------------!
! This function provides an analytical result for the integral         !
! A4(p1,p2) being a part of the YFS IR function B-tilde.               !
! Note: This is a general case without any approximation!              !
! INPUT: p1p2    - scalar product of the 4-momenta p1 and p2;          !
!        E1,E2   - particles energies;                                 !
!        xm1,xm2 - particles masses;                                   !
!----------------------------------------------------------------------!
! Written by:  Wieslaw Placzek                Knoxville, January  1996 !
! Last update: 30.01.1996                by: W.P.                      !
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
! Statement function
      etaln(x1,x2,x3,x4,z) = LOG(ABS( (z-x1)*(z-x2)/(z-x3)/(z-x4) ))
! Some auxiliary variables
      E1 = En1
      E2 = En2
      am1 = xm1
      am2 = xm2
      p1s = E1**2 - am1**2
      p2s = E2**2 - am2**2
      IF (p1s.lt.p2s) THEN
        am1 = xm2
        am2 = xm1
        E1 = En2
        E2 = En1
      ENDIF
      Ep  = E1 + E2
      Em  = E1 - E2
      sm  = am1 + am2 
      dm  = am1 - am2
      Q2  = 2*p1p2 - am1**2 - am2**2
      xl  = SQRT( (Q2 + sm**2)*(Q2 + dm**2) )
      xq  = SQRT(Q2 + Em**2)
      qp = xq + Em
      qm = xq - Em
      et0 = SQRT(E2**2 - am2**2)
      IF (p1p2.gt.E1*E2) et0 = -et0
      et1 = SQRT(E1**2 - am1**2) + xq
      y1  = 0.5*( (xq - Ep) + (sm*dm + xl)/qp )
      y2  = y1 - xl/qp
      y3  = 0.5*( (xq + Ep) + (sm*dm + xl)/qm )
      y4  = y3 - xl/qm       
! Some auxiliary functions
      IF (ABS(Em).gt.1d-10) THEN
        Eln = LOG(ABS(qm/qp))*( etaln(y1,y4,y2,y3,et1) 
     &                        - etaln(y1,y4,y2,y3,et0) )
      ELSE
        Eln = 0
      ENDIF
      Vet0 = Yijeta(y1,y4,et0) + Yijeta(y2,y1,et0)
     &     + Yijeta(y3,y2,et0) - Yijeta(y3,y4,et0)
     &     + 0.5*etaln(y1,y2,y3,y4,et0)*etaln(y2,y3,y1,y4,et0)
      Vet1 = Yijeta(y1,y4,et1) + Yijeta(y2,y1,et1)
     &     + Yijeta(y3,y2,et1) - Yijeta(y3,y4,et1)
     &     + 0.5*etaln(y1,y2,y3,y4,et1)*etaln(y2,y3,y1,y4,et1)
! Function A4(p1,p2) 
      A4anal = 1/xl*(Eln + Vet1 - Vet0 )
      END

      FUNCTION Yijeta(yi,yj,eta)
*     **************************
!----------------------------------------------------------------------!
! Some auxiliary function (combination of Logs and Dilogs) used in     !
! the function A4anal for A4(p1,p2).                                   !
!----------------------------------------------------------------------!
! Written by:  Wieslaw Placzek                Knoxville, January  1996 !
! Last update: 30.01.1996                by: W.P.                      !
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
!
      Yijeta = 2*DILOGY( (yj-yi)/(eta-yi) ) 
     &       + 0.5*LOG(ABS( (eta-yi)/(eta-yj) ))**2
      END
 
      FUNCTION Btilde(p1,p2,am1,am2,aKmax,amgam)
*     ******************************************
!----------------------------------------------------------------------!
! This function provides a value of YFS real photon IR function        !
! B-tilde for any pair of charged particles.                           !
! INPUT: p1,p2   - particles 4-momenta;                                !
!        am1,am2 - particles masses;                                   !
!        amgam   - "photon mass" (IR regulator)                        !
!        aKmax   - maximum soft photon energy [GeV]                    !
!----------------------------------------------------------------------!
! Written by:  Wieslaw Placzek                Knoxville, January  1996 !
! Last update: 30.01.1996                by: W.P.                      !
!----------------------------------------------------------------------!
      IMPLICIT REAL*8(a-h,o-z)
      PARAMETER ( pi = 3.1415926535897932D0, alfinv = 137.0359895d0 )
      PARAMETER ( alfpi = 1/alfinv/pi )
      REAL*8 p1(4),p2(4)
!
      Btilde = 0
      E1 = p1(4)
      E2 = p2(4)
      am12 = am1*am2
      p1p2 = p1(4)*p2(4) - p1(3)*p2(3) - p1(2)*p2(2) - p1(1)*p2(1)
      IF (p1p2-am12.lt.1d-10) RETURN
      xlam = SQRT( (p1p2 - am12)*(p1p2 + am12) )
! Function A(p1,p2)
      A  = 1/xlam *LOG( (p1p2 + xlam)/am12 )
      bet1 = SQRT(1-am1**2/E1**2)
      bet2 = SQRT(1-am2**2/E2**2)
      b1ln = 2*LOG( (1+bet1)*E1/am1 )
      b2ln = 2*LOG( (1+bet2)*E2/am2 )
! Function A4(p1,p2)
      A4 = A4anal(p1p2,E1,E2,am1,am2)
! B-tilde(p1,p2;aKmax,amgam)
      Btian = (p1p2*A - 1) *2*LOG(2*aKmax/amgam)
     &       + p1p2*A4 + 0.5/bet1*b1ln + 0.5/bet2*b2ln
      Btilde = alfpi*Btian
      END

      SUBROUTINE wform(cmsene,q1,q2,amf,delta,eps,dyfs)
*     *************************************************
* Unused, kept for some future tests, small mass approx.
* Yennie-Frautschi-Suura formfactors for the final state ferm. pair
* S.J. 1987
*     ***********************************
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
      PARAMETER(pi=3.1415926535897932d0,alfinv=137.03604d0)
      PARAMETER(alf1=1/alfinv/pi)
      DIMENSION q1(4),q2(4)

      svar  = cmsene**2
      ene= cmsene/2
* momenta q1,q2 should be in cms
      e1 = q1(4)
      e2 = q2(4)
      gamf2 = 2*alf1* dlog(svar /amf**2)
      delb  = gamf2*dlog(ene/sqrt(e1*e2)*eps/delta)
      ep    = e1+e2
      em    = e1-e2
      q1q2  = q1(4)*q2(4)-q1(3)*q2(3)-q1(2)*q2(2)-q1(1)*q2(1)
      dl    = sqrt( 2*q1q2 +em**2 )
      remn  = pi**2/2
     $        -0.50*dlog(e1/e2)**2
     $        -0.25*dlog((dl+em)**2/(4*e1*e2))**2
     $        -0.25*dlog((dl-em)**2/(4*e1*e2))**2
     $        - dilogy((dl+ep)/(dl+em)) -dilogy((dl-ep)/(dl-em))
     $        - dilogy((dl-ep)/(dl+em)) -dilogy((dl+ep)/(dl-em))
      dyfs  = exp( delb +alf1*remn )
      END


