      PROGRAM MAIN
C*****************
      IMPLICIT LOGICAL (A-Z)
      LOGICAL BANNER
      BANNER=.TRUE.
      IF(BANNER)WRITE(6,1000)
1000  FORMAT(
     C32X,56H********************************************************,/,
     C32X,56H********************************************************,/,
     C32X,56H***                                                  ***,/,
     C32X,56H***                 PROGRAM Z0POLE                   ***,/,
     C32X,56H***                                                  ***,/,
     C32X,56H***                     AUTHOR                       ***,/,
     C32X,56H***                 ROBIN G. STUART                  ***,/,
     C32X,56H***                      CERN                        ***,/,
     C32X,56H***                                                  ***,/,
     C32X,56H***                 QCD CORRECTIONS                  ***,/,
     C32X,56H***                 BERND A. KNIEHL                  ***,/,
     C32X,56H***                      DESY                        ***,/,
     C32X,56H***                                                  ***,/,
     C32X,56H***          LAST UPDATE : JUNE, 1992                ***,/,
     C32X,56H***                                                  ***,/,
     C32X,56H***                                                  ***,/,
     C32X,56H********************************************************,/,
     C32X,56H********************************************************,
     C///)
      CALL Z0POLE
      STOP
      END
 
      SUBROUTINE Z0POLE
C**********************
C  CALCULATES CROSS SECTIONS IN NANOBARNS FOR E+E- -> F F-BAR
C    INCLUDING HIGHER ORDER ELECTROWEAK CORRECTIONS WITHOUT QED
C
      IMPLICIT LOGICAL (A-Z)
      INTEGER NSCHEM
      REAL*8 ALPHA,EPS,MF,MHIGGS,MINT,MTOP,MZ,NBARN,NCOL,NCOLF,POLN,PI,
     .       QE,QF,ROOTS,ROOTSI,ROOTSF,ROOTSD,S,
     .       SIGBWD,SIGFWD,SIGLFT,SIGRGT,SIGMAU,
     .       SIN2W,T3E,T3F,TSECBL,TSECBR,TSECFL,TSECFR,
     .       WMASS,XSCALE,ZMASS
      COMPLEX*16 POLE
      REAL*8 DGAUSS,DSIGMA
      EXTERNAL DSIGMA
      PI=4.D0*DATAN(1.D0)
      NBARN  =  3.893796623D5
C-----------------------------------------------------------------------
C  MZ IS THE Z0 BOSON MASS IN THE ON-SHELL RENORMALIZATION SCHEME
C     MHIGGS AND  MTOP ARE INPUT VALUES OF HIGGS AND TOP QUARK MASS
C     NCOL IS THE NUMBER OF QUARK COLOURS
C-----------------------------------------------------------------------
      MZ     =  91.17D0
      MHIGGS = 100.D0
      MTOP   = 120.D0
      NCOL   =   3.D0
C-----------------------------------------------------------------------
C  INPUT PARAMETERS OF THE INITIAL AND FINAL STATES
C     T3E, QE IS THE ELECTRIC CHARGE AND ISOSPIN OF THE INCOMING FERMION
C     T3F, QF IS THE ELECTRIC CHARGE AND ISOSPIN OF THE OUTGOING FERMION
C     NCOLF IS THE NUMBER OF COLOURS IN THE FINAL STATE (NO QCD CORRS)
C     MF IS THE MASS OF THE OUTGOING FERMION
C     MINT IS THE MASS OF THE ISOSPIN PARTNER OF THE OUTGOING FERMION
C-----------------------------------------------------------------------
      T3E    = -0.5D0
      QE     = -1.D0
      T3F    = -0.5D0
      QF     = -1.D0
      NCOLF  =  1.D0
      MF     =  0.D0
      MINT   =  0.D0
C-----------------------------------------------------------------------
C  RENORMALIZATION SCHEME AND SCALE FOR THE CALCULATION
C     NSCHEM = 1, ON SHELL SCHEME AND XSCALE ARBITRARY
C     NSCHEM = 2, MS-BAR SCHEME AND XSCALE IS THE 'T HOOFT MASS
C-----------------------------------------------------------------------
      NSCHEM = 1
      XSCALE = MZ**2
C-----------------------------------------------------------------------
C  INITIALIZE STANDARD MODEL PARAMETERS
C-----------------------------------------------------------------------
      CALL GSWINT(NSCHEM,XSCALE,NCOL,MHIGGS,MTOP,MZ)
C===================================================
C-----------------------------------------------------------------------
C  EXTRACT AND PRINT THE RENORMALIZED PARAMETERS AND POLE POSITION
C-----------------------------------------------------------------------
      CALL PRMZ0P(ALPHA,SIN2W,WMASS,ZMASS,POLE)
C==============================================
      WRITE(6,1000)MZ,MHIGGS,MTOP
      IF(NSCHEM.EQ.1)THEN
          WRITE(6,1001)
      ELSE IF(NSCHEM.EQ.2)THEN
          WRITE(6,1002)DSQRT(XSCALE)
      ELSE
          WRITE(6,1003)
          STOP
          END IF
      WRITE(6,1004)1.D0/ALPHA,SIN2W,WMASS,ZMASS
      WRITE(6,1005)DREAL(POLE),-DIMAG(POLE),
     .             DSQRT(DREAL(POLE)),-DIMAG(POLE)/DSQRT(DREAL(POLE)),
     .             DREAL(CDSQRT(POLE)),-2.D0*DIMAG(CDSQRT(POLE))
C-----------------------------------------------------------------------
C  PRINT THE INITIAL AND FINAL STATE PARAMETERS
C-----------------------------------------------------------------------
      WRITE(6,1006)T3E,T3F,QE,QF,MF,MINT,NCOLF
C-----------------------------------------------------------------------
C  DEFINE THE RANGE OF STEPS IN S
C-----------------------------------------------------------------------
      ROOTSI=89.155D0
      ROOTSF=93.155D0
      ROOTSD=0.4D0
      WRITE(6,1007)
C-----------------------------------------------------------------------
C  BEGIN LOOP IN ROOT S
C-----------------------------------------------------------------------
      DO 10 ROOTS=ROOTSI,ROOTSF,ROOTSD
          S=ROOTS**2
          EPS=1.D-8
C-----------------------------------------------------------------------
C  SET UP PURE LEFT-HANDED INCOMING BEAM POLARIZATION
C  TSECFL=CROSS-SECTION FOR LEFT-HANDED INCOMING FERMIONS FORWARD
C  TSECBL=CROSS-SECTION FOR LEFT-HANDED INCOMING FERMIONS BACKWARD
C-----------------------------------------------------------------------
          POLN=1.D0
          CALL EXTINT(S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN)
C========================================================
          TSECFL=2.D0*PI*DGAUSS(DSIGMA,0.D0,1.D0,EPS)
          TSECBL=2.D0*PI*DGAUSS(DSIGMA,-1.D0,0.D0,EPS)
C-----------------------------------------------------------------------
C  SET UP PURE RIGHT-HANDED INCOMING BEAM POLARIZATION
C  TSECFR=CROSS-SECTION FOR RIGHT-HANDED INCOMING FERMIONS FORWARD
C  TSECBR=CROSS-SECTION FOR RIGHT-HANDED INCOMING FERMIONS BACKWARD
C-----------------------------------------------------------------------
          POLN=-1.D0
          CALL EXTINT(S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN)
C========================================================
          TSECFR=2.D0*PI*DGAUSS(DSIGMA,0.D0,1.D0,EPS)
          TSECBR=2.D0*PI*DGAUSS(DSIGMA,-1.D0,0.D0,EPS)
C-----------------------------------------------------------------------
C  FORM VARIOUS CROSS-SECTIONS
C-----------------------------------------------------------------------
          SIGMAU=(TSECFL+TSECBL+TSECFR+TSECBR)*NBARN/4.D0
          SIGLFT=(TSECFL+TSECBL)*NBARN
          SIGRGT=(TSECFR+TSECBR)*NBARN
          SIGFWD=(TSECFL+TSECFR)*NBARN/4.D0
          SIGBWD=(TSECBL+TSECBR)*NBARN/4.D0
10        WRITE(6,1008)ROOTS,SIGMAU,SIGLFT,SIGRGT,SIGFWD,SIGBWD
      RETURN
1000  FORMAT(1H ,5X,'INPUT PARAMETERS;    ON-SHELL SCHEME Z0 MASS  = ',
     .       F9.4,/,
     .       1H ,5X,'                     HIGGS MASS               = ',
     .       F9.4,/,
     .       1H ,5X,'                     TOP QUARK MASS           = ',
     .       F9.4,//)
1001  FORMAT(1H ,5X,'NSCHEM = 1 HAS BEEN SELECTED; THE ON-SHELL RENORMAL
     .IZATION SCHEME WILL BE USED.',//,
     .       1H ,5X,'IN THE ON-SHELL SCHEME THE RENORMALIZED PARAMETERS
     .ARE...',/)
1002  FORMAT(1H ,5X,'NSCHEM = 2 HAS BEEN SELECTED; THE MS-BAR RENORMALIZ
     .ATION SCHEME WILL BE USED.',//,
     .       1H ,5X,'                     T HOOFT MASS             = ',
     .       F9.4,//,
     .       1H ,5X,'IN THE MS-BAR SCHEME THE RENORMALIZED PARAMETERS AR
     .E...',/)
1003  FORMAT(1H ,'INVALID RENORMALIZATION SCHEME SELECTED',/,
     .       1H ,'  VALID CHOICES ARE NSCHEM = 1, ON-SHELL SCHEME',/,
     .       1H ,'                           = 2, MS-BAR SCHEME')
1004  FORMAT(1H ,5X,'                         1/ALPHA              = ',
     .       F13.8,/,
     .       1H ,5X,'                         SIN**2 W             = ',
     .       F13.8,/,
     .       1H ,5X,'                         W MASS               = ',
     .       F13.8,/,
     .       1H ,5X,'                         Z MASS               = ',
     .       F13.8,/,
     .       /)
1005  FORMAT(1H ,5X,'POSITION OF THE POLE ON THE COMPLEX S-PLANE;',//,
     .       1H ,5X,'                         S POLE       = ',
     .       F11.6,  '     - I * ',F12.6,/,
     .       1H ,5X,'                                      = ',
     .       F11.6,' **2 - I * M * ',F8.6,/,
     .       1H ,5X,'                                      = ',
     .       '(',F10.6,'     - I / 2 * ',F8.6,' )**2',//)
1006  FORMAT(1H ,5X,'EXTERNAL FERMIONS:   INITIAL  STATE',28X,
     .                                   'FINAL STATE',//,
     .       1H ,5X,'                     T3 = ',F10.6,27X,
     .                                   'T3 = ',F10.6,/,
     .       1H ,5X,'                     Q  = ',F10.6,27X,
     .                                   'Q  = ',F10.6,/,
     .       1H ,66X,'MASS = ',F8.4,/,
     .       1H ,47X,'MASS OF ISOSPIN PARTNER = ',F8.4,/,
     .       1H ,44X,'NO. OF FINAL STATE COLOURS = ',F6.2,//)
1007  FORMAT(1H ,2X,'ROOT S',5X,'SIGMA U',11X,'SIGMA L',11X,'SIGMA R',
     .        11X,'SIGMA F',11X,'SIGMA B',/)
1008  FORMAT(1H ,F8.3,5F18.12)
      END

      FUNCTION SP(Y,E)
C ----------------------------------------------------------------------
C  SPENCE FUNCTION OF Y+I*REAL(E) WHERE E IS AN INFINITESIMAL
C
C     called by : FIU, MATELM, ...
C ----------------------------------------------------------------------
C     IMPLICIT LOGICAL (A-H,O-Z)
      IMPLICIT REAL*8  (A-H,O-Z)
      COMPLEX*16 Y,E,SP
      REAL*8 B(9),FACT
      COMPLEX*16 A,CLN,PISQ6,PROD,TERM,X,Z,ZSQ
      COMPLEX*16 CDLN
C
      B(1)=1.D0/6.D0
      B(2)=-1.D0/30.D0
      B(3)=1.D0/42.D0
      B(4)=B(2)
      B(5)=5.D0/66.D0
      B(6)=-691.D0/2730.D0
      B(7)=7.D0/6.D0
      B(8)=-3617.D0/510.D0
      B(9)=43867.D0/798.D0
      PISQ6=(1.6449340668482264D0,0.D0)
      I1=0
      I2=0
      X=Y
      A=E
      IF(X.EQ.(0.D0,0.D0))THEN
        SP=(0.D0,0.D0)
        RETURN
      ENDIF
      IF(X.EQ.(1.D0,0.D0))THEN
        SP=PISQ6
        RETURN
      ENDIF
C  IF X LIES OUTSIDE THE UNIT CIRCLE THEN EVALUATE SP(1/X)
      IF(CDABS(X).GT.1.D0)THEN
        X=1.D0/X
        A=-A
        I1=1
      ENDIF
C  IF REAL(X)>1/2 THEN EVALUATE SP(1-X)
      IF(DREAL(X).GT.0.5D0)THEN
        X=1.D0-X
        A=-A
        I2=1
      ENDIF
C  EVALUATE SERIES FOR SP(X)
      Z=-CDLN(1.D0-X,-A)
      ZSQ=Z*Z
      SP=Z-ZSQ/4.D0
      PROD=Z
      FACT=1.D0
      DO 10 J=2,18,2
      FACT=FACT*DCMPLX(DBLE((J+1)*J))
      PROD=PROD*ZSQ
      TERM=B(J/2)/FACT*PROD
      SP=SP+TERM
      IF(CDABS(TERM/SP).LT.1.D-20)GO TO 20
10    CONTINUE
C  ADD APPROPRIATE LOGS TO OBTAIN SPENCE FUNCTION OF ORIGINAL ARGUEMENT
20    IF(I2.EQ.1)THEN
        SP=-SP+PISQ6-CDLN(X,A)*CDLN(1.D0-X,-A)
        X=1.D0-X
        A=-A
      ENDIF
      IF(I1.EQ.1)THEN
        CLN=CDLN(-X,-A)
        SP=-SP-PISQ6-CLN*CLN/2.D0
      ENDIF
      RETURN
      END
      FUNCTION CDLN(X,A)
C ----------------------------------------------------------------------
C  COMPLEX FUNCTIONS THAT TAKE ACCOUNT OF THE I*EPSILON PRESCRIPTION
C              TO CALCULATE ANALYTIC STRUCTURE
C
C  COMPLEX LOGARITHM OF X+I*REAL(A) WHERE A IS AN INFINITESIMAL
C
C     called by : BZED1, FIU, MATELM, ...
C ----------------------------------------------------------------------
C     IMPLICIT LOGICAL (A-H,O-Z)
      IMPLICIT REAL*8  (A-H,O-Z)
      COMPLEX*16 A,X,CDLN
      COMPLEX*16 PI
      PI=(3.141592653589793238462643D0,0.D0)
      IF(DIMAG(X).EQ.0.D0.AND.DREAL(X).LE.0.D0)THEN
        CDLN=CDLOG(-X)+(0.D0,1.D0)*PI*DSIGN(1.D0,DREAL(A))
      ELSE
        CDLN=CDLOG(X)
      END IF
      IF(DIMAG(CDLN).GT.DREAL(PI))CDLN=CDLN-(0.D0,1.D0)*PI
      IF(DIMAG(CDLN).LT.DREAL(-PI))CDLN=CDLN+(0.D0,1.D0)*PI
      RETURN
      END
