CDECK  ID>, ERRUNDER.   
      SUBROUTINE ERRUNDER(X)
C-----------------------------------------------
C--   Handle underflows...
C--   If underflow encountered, return zero.
C-----------------------------------------------
      IMPLICIT NONE
      REAL X

      PRINT*,'ERROR : REAL UNDERFLOW !!'
      X = 0.0

      RETURN
      END


CDECK  ID>, D4DOT.  
      DOUBLE PRECISION FUNCTION D4DOT(P1,P2)
C----------------------------------------------
C--   Returns the Lorentz dot product of
C--   two double precision input 4 vectors
C--   of the form (px,py,pz,E).
C--   Intermediate calculations are done in
C--   double precision.
C----------------------------------------------
      IMPLICIT NONE
      DOUBLE PRECISION P1(4),P2(4)

      D4DOT = P1(4)*P2(4)
     +       -P1(3)*P2(3)
     +       -P1(2)*P2(2)
     +       -P1(1)*P2(1)

      RETURN
      END


CDECK  ID>, E.  
      DOUBLE PRECISION FUNCTION e_(v1,v2,v3,v4)
C--------------------------------------------------------
C--   This is the Levi-Civita tensor in Minkowski space
C--   contracted with 4 4-vectors v1,v2,v3, and v4.
C--   The easiest way to calculate this, I think, is
C--   to work out the determinant of the matrix whose
C--   rows (or columns...same answer either way) are the
C--   vectors we pass in.
C---------------------------------------------------------

      IMPLICIT NONE
      DOUBLE PRECISION V1(4),V2(4),V3(4),V4(4)
      DOUBLE PRECISION V(4,4) ! this is the matrix of which we want det(V).
      INTEGER I


      DOUBLE PRECISION DETERM
      DOUBLE PRECISION DET1,DET2,DET3,DET4

      DO I = 1,4
         V(1,I) = V1(I)
         V(2,I) = V2(I)
         V(3,I) = V3(I)
         V(4,1) = V4(I)
      END DO

C--   Expand around the first row
      DET1 = V(2,2)*(V(3,3)*V(4,4)-V(3,4)*V(4,3))
     +     - V(2,3)*(V(3,2)*V(4,4)-V(3,4)*V(4,2))
     +     + V(2,4)*(V(3,2)*V(4,3)-V(3,3)*V(4,2))

      DET2 = V(2,1)*(V(3,3)*V(4,4)-V(3,4)*V(4,3))
     +     - V(2,3)*(V(3,1)*V(4,4)-V(3,4)*V(4,1))
     +     + V(2,4)*(V(3,1)*V(4,3)-V(3,3)*V(4,1))

      DET3 = V(2,1)*(V(3,2)*V(4,4)-V(3,4)*V(4,2))
     +     - V(2,2)*(V(3,1)*V(4,4)-V(3,4)*V(4,1))
     +     + V(2,4)*(V(3,1)*V(4,2)-V(3,2)*V(4,1))

      DET4 = V(2,1)*(V(3,2)*V(4,3)-V(3,3)*V(4,2))
     +     - V(2,2)*(V(3,1)*V(4,3)-V(3,3)*V(4,1))
     +     + V(2,3)*(V(3,1)*V(4,2)-V(3,2)*V(4,1))


      DETERM = V(1,1)*DET1-V(1,2)*DET2+V(1,3)*DET3-V(1,4)*DET4

      e_ = DETERM

      RETURN
      END


CDECK  ID>, MAKEPROP.   
      SUBROUTINE MAKEPROPAGS
C------------------------------------------------
C--   Subroutine to work out the propagators
C--   for Z and photon exchange.
C------------------------------------------------

      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   Define variables for the propagators
C--   We only need the real denominators as the numerators
C--   were included in the amplitude calculations
      DOUBLE PRECISION PROPIZ,PROPFZ,PROPIG,PROPFG
      DOUBLE PRECISION QZ_QZ_IS,QZ_QZ_FS

      COMMON /PROPAGS/ PROPIZ,
     +                 PROPFZ,
     +                 PROPIG,
     +                 PROPFG,
     +                 QZ_QZ_IS,
     +                 QZ_QZ_FS

C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_



C--   Work out the 4-momentum squared along the propagator for
C--   final state radiation.
C--   qzfs.qzfs ; qzfs = tm + tp + k;
C--   QZDQZ = tm_tm + tm_tp + tm_k + tp_tm + tp_tp+ tp_k + k_tm + k_tp + k_k
      QZ_QZ_FS = 2.0d0*mt**2 + 2.0D0*tm_tp + 2.0D0*k_tp + 2.0D0*k_tm
C--   New shorter calculation
      QZ_QZ_FS = 2.0D0*me**2 + 2.0D0*em_ep	

C--   Work out the 4-momentum squared along the propagator for
C--   initial state radiation.
C--   qzss.qzss ; qzis = tm + tp ;
C--   QZDQZ = tm_tm + 2*tm_tp + tp_tp
      QZ_QZ_IS = 2.0d0*mt**2 + 2.0D0*tm_tp

C--   Define the q**2 dependent z widths...these are in ME2ARGS
      GZIS = GZ*QZ_QZ_IS/(MZ**2)
      GZFS = GZ*QZ_QZ_FS/(MZ**2)

C--   Make propagators...these are in PROPAGS
      PROPIZ = 1.0D0/((QZ_QZ_IS-mz**2)**2+(mz*gzis)**2)
      PROPFZ = 1.0D0/((QZ_QZ_FS-mz**2)**2+(mz*gzfs)**2)
      PROPIG = 1.0D0/QZ_QZ_IS
      PROPFG = 1.0D0/QZ_QZ_FS

      RETURN
      END


CDECK  ID>, LOADME2A.   
      SUBROUTINE LOADME2ARGS
C-----------------------------------------------------
C--   Loads the 4-vectors for electron, positron
C--   tau+,tau-, and photon and the event weight
C--   (all double precision)
C--   from the common GENOUT which should have been
C--   filled by GENBOD.
C-----------------------------------------------------

      IMPLICIT NONE

C--
C--  Common for name of file to write ntuples, and number of events.
C--
      INTEGER NEVT            ! number of events to generate
      INTEGER NXS             ! number of events for cross-section calculation
      CHARACTER*80 NTPFILE    ! ntuple file
      INTEGER IGEN            ! IGEN = 0 for weights, <>0 for MC events
      DOUBLE PRECISION ECM    ! Centre of mass energy (GeV)
      REAL EMIN,CCUT          ! Cuts on minimum photon energy and cos(theta)
      REAL CTGC,XTMX          ! Cuts on cos(photon,tau) and tau maximum x
      REAL EWMX               ! Maximum expected wt*dme2 (set iteratively)
      INTEGER MNTP            ! > 0 to make ntuples

      COMMON/RUNPARS/NEVT,    ! number of events to generate
     +               NXS,     ! number of events for cross-section calculation
     +               NTPFILE, ! ntuple output file
     +               IGEN,    ! IGEN = 0 for weights, <>0 for MC events
     +               ECM,     ! Centre of mass energy (GeV)
     +               EMIN,    ! Minimum photon energy (GeV)
     +               CCUT,    ! cut |cos(theta(photon))| < CCUT
     +               CTGC,    ! cut on cos(tau,gamma) < CTGC
     +               XTMX,    ! maximum x of tau
     +               EWMX,    ! expected max of wt*dme2
     +               MNTP     ! > 0 to make ntuples

C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_


C--   Common Blocks for FOWL
      INTEGER NP,KGENEV
      REAL TECM,AMASS,PCM,WT
      COMMON/GENIN/NP,TECM,AMASS(18),KGENEV
      COMMON/GENOUT/PCM(5,18),WT

      EP(1) = 0.0D0
      EP(2) = 0.0D0
      EP(3) = DSQRT((ECM/2.0D0)**2-ME**2)
      EP(4) = ECM/2.0D0

      EM(1) = 0.0D0
      EM(2) = 0.0D0
      EM(3) = -DSQRT((ECM/2.0D0)**2-ME**2)
      EM(4) = ECM/2.0D0

      TP(1) = DBLE(PCM(1,1))
      TP(2) = DBLE(PCM(2,1))
      TP(3) = DBLE(PCM(3,1))
      TP(4) = DBLE(PCM(4,1))

      TM(1) = DBLE(PCM(1,2))
      TM(2) = DBLE(PCM(2,2))
      TM(3) = DBLE(PCM(3,2))
      TM(4) = DBLE(PCM(4,2))

      K(1) = DBLE(PCM(1,3))
      K(2) = DBLE(PCM(2,3))
      K(3) = DBLE(PCM(3,3))
      K(4) = DBLE(PCM(4,3))

c      PRINT*,'IN LOADME2ARGS: '
c      PRINT*,'------------------------------'
c      PRINT*,'EM = ',(EM(I),I=1,4)
c      PRINT*,'E- MASS CALCULATED = ',
c     +DSQRT(EM(4)**2-EM(3)**2-EM(2)**2-EM(1)**2)

c      PRINT*,'EP = ',(EP(I),I=1,4)
c      PRINT*,'E+ MASS CALCULATED = ',
c     +DSQRT(EP(4)**2-EP(3)**2-EP(2)**2-EP(1)**2)
c      PRINT*,'E MASS IN : ',ME


c      PRINT*,'K  = ',(K(I),I=1,4)
c      PRINT*,'PHOTON MASS**2 CALCULATED = ',
c     +(K(4)**2-K(3)**2-K(2)**2-K(1)**2)

c      PRINT*,'TM = ',(TM(I),I=1,4)
c      PRINT*,'TAU- MASS**2 CALCULATED = ',
c     +TM(4)**2-TM(3)**2-TM(2)**2-TM(1)**2

c      PRINT*,'TP = ',(TP(I),I=1,4)
c      PRINT*,'TAU+ MASS**2 CALCULATED = ',
c     +TP(4)**2-TP(3)**2-TP(2)**2-TP(1)**2
c      PRINT*,'TAU MASS**2 IN : ',MT**2

c      PRINT*,'------------------------------'

      RETURN
      END

CDECK  ID>, MAKEME2.
      SUBROUTINE MAKEME2(DME2)
C---------------------------------------------------------
C--   Subroutine to calculate the matrix-element squared
C--   given the phase space point in common ME2ARGS.
C----------------------------------------------------------
      IMPLICIT NONE

C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

C--   Define variables for the propagators
C--   We only need the real denominators as the numerators
C--   were included in the amplitude calculations
      DOUBLE PRECISION PROPIZ,PROPFZ,PROPIG,PROPFG
      DOUBLE PRECISION QZ_QZ_IS,QZ_QZ_FS

      COMMON /PROPAGS/ PROPIZ,
     +                 PROPFZ,
     +                 PROPIG,
     +                 PROPFG,
     +                 QZ_QZ_IS,
     +                 QZ_QZ_FS

C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL

      DOUBLE PRECISION DME2
      DOUBLE PRECISION D4DOT
      EXTERNAL MAKEPROPAGS

C--   Running alpha at 0 and nonzero sqrt(Q**2)
      DOUBLE PRECISION ALPHA0,ALPHAQ

C--   Work out all the dot products to
C--   use in calculating the matrix element squared.
      tm_tp = d4dot(tm,tp)
      ep_tp = d4dot(ep,tp)
      em_tp = d4dot(em,tp)
      k_tp  = d4dot(k,tp)
      ep_tm = d4dot(ep,tm)
      em_tm = d4dot(em,tm)
      k_tm  = d4dot(k,tm)
      ep_k  = d4dot(ep,k)
      em_k  = d4dot(em,k)
      em_ep = d4dot(em,ep)


C--   Call a subroutine to make the propagators
C--   This routine also makes the energy-dependent z width
      CALL MAKEPROPAGS

C--   Define the correction factor: use once for each gamma
C--   exchange amplitude
C--   Get alpha at 0
      CALL CALPHA(0.0D0,ALPHA0)

C--   Z propagators are OK
      PROPIZ = PROPIZ
      PROPFZ = PROPFZ

C--   Fix up the photon propagators
      CALL CALPHA(DSQRT(QZ_QZ_IS),ALPHAQ)
      PROPIG = PROPIG*ALPHAQ/ALPHA0
      CALL CALPHA(DSQRT(QZ_QZ_FS),ALPHAQ)
      PROPFG = PROPFG*ALPHAQ/ALPHA0

C--   Calculate the separate terms
      CALL CIZIZ
      CALL CIGIG
      CALL CFZFZ
      CALL CFGFG
      CALL CAZAZ
      CALL CAGAG
      CALL CIZIG
      CALL CIZFZ
      CALL CIZFG
      CALL CIZAZ
      CALL CIZAG
      CALL CIGFZ
      CALL CIGFG
      CALL CIGAZ
      CALL CIGAG
      CALL CFZFG
      CALL CFZAZ
      CALL CFZAG
      CALL CFGAZ
      CALL CFGAG
      CALL CAZAG

C--   Calculate imaginary bits terms
      CALL ICIZIZ
      CALL ICIGIG
      CALL ICFZFZ
      CALL ICFGFG
      CALL ICAZAZ
      CALL ICAGAG
      CALL ICIZIG
      CALL ICIZFZ
      CALL ICIZFG
      CALL ICIZAZ
      CALL ICIZAG
      CALL ICIGFZ
      CALL ICIGFG
      CALL ICIGAZ
      CALL ICIGAG
      CALL ICFZFG
      CALL ICFZAZ
      CALL ICFZAG
      CALL ICFGAZ
      CALL ICFGAG
      CALL ICAZAG




C--   Calculate the matrix-element-squared
C--   from the subpieces, propagators, and switches
C--   to turn bits on and off

C--   so far:
C--   isr for initial state radiation on and off
C--   fsr for sm final state radiation on and off
C--   asr for anomalous radiation on and off
C--   zex for z exchange on and off
C--   gex for gamma exchange on and off
C--   zgi for z-gamma exchange on and off
C--   afi : interference of anomalous and sm f.s.r.
C--   ifi : interference of isr with SM fsr
C--   iai : interference of isr with anomalous SM fsr


C--    Unless we selected one of the special, simple options,
C--    just go ahead according to the rest of the interference flags

C-------------------------------------------------------------------------------
C--    Simple flag SFL = -1 removes all the contributions due to the SM
C-------------------------------------------------------------------------------

       IF(SFL.EQ.-1) THEN

       DME2 =
     +        me2azaz*propfz**2*asr*zex
     +       +me2agag*propfg**2*asr*gex
     +       +me2izaz*propiz*propfz*isr*asr*zex*iai
     +       +me2izag*propiz*propfg*isr*asr*zex*gex*zgi*iai
     +       +me2igaz*propig*propfz*isr*asr*gex*zex*zgi*iai
     +       +me2igag*propig*propfg*isr*asr*gex*iai
     +       +me2fzaz*propfz*propfz*fsr*asr*zex*afi
     +       +me2fzag*propfz*propfg*fsr*asr*zex*gex*zgi*afi
     +       +me2fgaz*propfg*propfz*fsr*asr*gex*zex*zgi*afi
     +       +me2fgag*propfg*propfg*fsr*asr*gex*afi
     +       +me2azag*propfz*propfg*asr*zex*gex*zgi

C-------------------------------------------------------------------------------
C--    Simple flag SFL = 0 is a complete calculation
C-------------------------------------------------------------------------------
       ELSE IF(SFL.EQ.0) THEN

       DME2 =
     +        me2iziz*propiz**2*isr*zex
     +       +me2igig*propig**2*isr*gex
     +       +me2fzfz*propfz**2*fsr*zex
     +       +me2fgfg*propfg**2*fsr*gex
     +       +me2azaz*propfz**2*asr*zex
     +       +me2agag*propfg**2*asr*gex
     +       +me2izig*propiz*propig*isr*zex*gex*zgi
     +       +me2izfz*propiz*propfz*isr*fsr*zex*ifi
     +       +me2izfg*propiz*propfg*isr*fsr*zex*gex*zgi*ifi
     +       +me2izaz*propiz*propfz*isr*asr*zex*iai
     +       +me2izag*propiz*propfg*isr*asr*zex*gex*zgi*iai
     +       +me2igfz*propig*propfz*isr*fsr*gex*zex*zgi*ifi
     +       +me2igfg*propig*propfg*isr*fsr*gex*ifi
     +       +me2igaz*propig*propfz*isr*asr*gex*zex*zgi*iai
     +       +me2igag*propig*propfg*isr*asr*gex*iai
     +       +me2fzfg*propfz*propfg*fsr*zex*gex*zgi
     +       +me2fzaz*propfz*propfz*fsr*asr*zex*afi
     +       +me2fzag*propfz*propfg*fsr*asr*zex*gex*zgi*afi
     +       +me2fgaz*propfg*propfz*fsr*asr*gex*zex*zgi*afi
     +       +me2fgag*propfg*propfg*fsr*asr*gex*afi
     +       +me2azag*propfz*propfg*asr*zex*gex*zgi

C-------------------------------------------------------------------------------
C--    Simple flag 1 gives the approximation of Riemann's paper
C--    with no gamma exchange and no initial state radiation
C--    but SM, interference between SM and anomalous terms, and terms
C--    quadratic in the anomalous terms.
C--    This can be used with any (possibly zero) value of anomalous couplings
C-------------------------------------------------------------------------------
       ELSE IF(SFL.EQ.1) THEN

       DME2 =
     +        me2fzfz*propfz**2          ! |SM FSR|**2 with Z exch only
     +       +me2azaz*propfz**2          ! |anomalous FSR|**2 with Z exch only
     +       +me2fzaz*propfz*propfz      ! SM/anomlous interf. with Z exch only
C-------------------------------------------------------------------------------
C--    Simple flag 2 is as above, but with no SM (i.e. the ``anomalous photons''
C-------------------------------------------------------------------------------
       ELSE IF(SFL.EQ.2) THEN

       DME2 =
     +        me2azaz*propfz**2          ! |anomalous FSR|**2 with Z exch only
     +       +me2fzaz*propfz*propfz      ! SM/anomlous interf. with Z exch only
C-------------------------------------------------------------------------------
C--    Simple flag 3 is as above, but with NO interference
C-------------------------------------------------------------------------------
       ELSE IF(SFL.EQ.3) THEN

       DME2 =
     +        me2azaz*propfz**2          ! |anomalous FSR|**2 with Z exch only
C-------------------------------------------------------------------------------
C--    Simple flag 4 is as above, but with ONLY the interference term
C-------------------------------------------------------------------------------
       ELSE IF(SFL.EQ.4) THEN

       DME2 =
     +        me2fzaz*propfz*propfz      ! SM/anomlous interf. with Z exch only
C-------------------------------------------------------------------------------
       ENDIF



         RETURN
         END

CDECK  ID>, CALPHA. 

      SUBROUTINE CALPHA (MASS,ALPHA)
************************************************************************
*                                                                      *
*     QED effective coupling constant                                  *
*                                                                      *
*     Interpolate in 1./alpha using Marciano and Sirlin values         *
*     which include effects of some radiative corrections in a way     *
*     which is consistent with explicit radiative corrections.         *
*                                                                      *
*     N.B.   alpha(0) from PDG                                         *
*            linear interpolation not rigourous (but is sufficient)    *
*                                                                      *
*     25 Aug 97: points added at 140,180, and 200 Gev.  These were     *
*                'determined' by extrapolating from the existing       *
*                points and are probably largely bogus, but it         *
*                shouldn't matter...  This was done so I can run       *
*                jobs up to 200 GeV to check anomalous xsection.       *
*                                                                      *
************************************************************************
*
      IMPLICIT NONE
      INTEGER    NAINV
      PARAMETER (NAINV=12)
      REAL*8     AINV (NAINV)         ! 1. / alpha_QED
      REAL*8     MINV (NAINV)         ! masses for AINV value
      DOUBLE PRECISION MASS,ALPHA     ! input variables
      INTEGER I                       ! counter
*
      DATA MINV/0.0D0,   0.105D0, 0.938D0,  1.25D0,   1.78D0,   4.5D0
     &,                  40.0D0,    82.2D0,   93.2D0, 140.0D0
     &,                  180.0D0, 201.0D0  /
      DATA AINV/137.035D0, 136.0D0, 133.93D0, 133.69D0, 133.29D0, 132.04D0
     &,                  128.95D0, 127.73D0, 127.66D0, 127.2D0, 126.8D0
     &,                  126.7D0 /

C--  Routine from Lucas Taylor, modified J. Swain
* ----------------------------------------------------------------------
*
      ALPHA = -99.0D0
      IF (MASS.LT.MINV(1) .OR. MASS.GT.MINV(NAINV)) THEN
          WRITE (*,*) 'CALPHA : MASS OUT OF RANGE => STOP'
          STOP
      ELSE
        DO I=1,NAINV-1
          IF (MASS.GE.MINV(I) .AND. MASS.LT.MINV(I+1)) THEN
            ALPHA = AINV(I) + ( (AINV(I+1)-AINV(I))   /
     &                          (MINV(I+1)-MINV(I)) ) *
     &                          (MASS     -MINV(I))
            ALPHA = 1.0D0 / ALPHA
          ENDIF
        ENDDO
      ENDIF
*
      IF (ALPHA .LT. 0.) THEN
        WRITE (*,*) 'CALPHA : ALPHA OUT OF RANGE => STOP'
        WRITE (*,*) '(END CONDITIONS: MASS = ',MASS,')'
        STOP
      ENDIF
*
      RETURN
      END


CDECK  ID>, CTHETA. 
      REAL*4 FUNCTION CTHETA(VECT)
C-------------------------------------------------
C--   Takes a 5-vector (p,e,m) and gives the theta
C--   of the vector
C--------------------------------------------------

      IMPLICIT NONE

      REAL*4 VECT(5)
      REAL*4 VLENGTH

      IF(VLENGTH(VECT).EQ.0.0) THEN
        CTHETA = -10.
        PRINT*,'ERROR IN CALCULATING CTHETA: DIVIDE BY ZERO'
        PRINT*,'RETURNING -10.'
      ELSE
        CTHETA = VECT(3)/VLENGTH(VECT)
      ENDIF

      RETURN
      END

CDECK  ID>, VLENGTH.
      REAL*4 FUNCTION VLENGTH(VECT)
C--------------------------------------------------
C--   Takes a 5-vector (p,e,m) and gives the length
C--   of the 3-vector part
C--------------------------------------------------

      IMPLICIT NONE
      REAL*4 VECT(5)

      VLENGTH = SQRT(VECT(1)**2+VECT(2)**2+VECT(3)**2)

      RETURN
      END

CDECK  ID>, PHI.
      REAL*4 FUNCTION PHI(VECT)
C------------------------------------------------
C--   Takes a 5-vector (p,e,m) and gives the phi
C--   of the vector
C------------------------------------------------

      IMPLICIT NONE
      REAL*4 VECT(5)
      REAL*4 PI
      PARAMETER(PI=3.1415926)

      PHI = ATAN2(VECT(2),VECT(1))
      IF(PHI.LT.0.0) PHI = PHI + 2.0*PI

      RETURN
      END

CDECK  ID>, CTHETA2.
      REAL*4 FUNCTION CTHETA2(A,B)
C-------------------------------------------------
C--   Takes 2 5-vectorS (p,e,m) and gives the cos
C--   of the angle between them
C-------------------------------------------------

      IMPLICIT NONE
      REAL*4 A(5),B(5)
      REAL*4 VLENGTH

      CTHETA2 = A(1)*B(1)+A(2)*B(2)+A(3)*B(3)
      CTHETA2 = CTHETA2/(VLENGTH(A)*VLENGTH(B))

      RETURN
      END



CDECK  ID>, AMINV.  
      REAL*4 FUNCTION AMINV(A,B)
C----------------------------------------
C--   Gives the invariant mass of A and B
C----------------------------------------

      IMPLICIT NONE
      REAL*4 A(5),B(5)
      REAL*4 AMINV2

      AMINV2 = (A(4)+B(4))**2-
     >         (A(3)+B(3))**2-
     >         (A(2)+B(2))**2-
     >         (A(1)+B(1))**2

      AMINV=SQRT(AMAX1(0.,AMINV2))
      RETURN
      END



CDECK  ID>, ISNZERO.
      LOGICAL FUNCTION ISNZERO(DOUBLE_IN)
C------------------------------------------------
C--   Logical function which returns .true. if
C--   its double precision argument is nonzero
C--   within a tolerance given by the parameter
C--   epsilon.
C------------------------------------------------
      IMPLICIT NONE
      DOUBLE PRECISION DOUBLE_IN
      DOUBLE PRECISION TOLERANCE
      PARAMETER (TOLERANCE=0.0D0)

      IF(DABS(DOUBLE_IN).LE.TOLERANCE) THEN
         ISNZERO = .FALSE.
      ELSE
         ISNZERO = .TRUE.
      ENDIF
      RETURN
      END



CDECK  ID>, TTGSUBS.
      SUBROUTINE CIZIZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2iziz =
     +  + 128*mt**6*ge**4*gv**4*ep_k*em_k**(-1)
     +  + 128*mt**6*ge**4*gv**4*ep_k**(-1)*em_k
     +  + 256*mt**6*ge**4*gv**4*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 256*mt**6*ge**4*gv**4*ep_k**(-1)*em_ep
     +  - 256*mt**6*ge**4*gv**4*em_k**(-1)*em_ep
     +  - 128*mt**6*ge**4*ga**4*ep_k*em_k**(-1)
     +  - 128*mt**6*ge**4*ga**4*ep_k**(-1)*em_k
     +  - 256*mt**6*ge**4*ga**4*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 256*mt**6*ge**4*ga**4*ep_k**(-1)*em_ep
     +  + 256*mt**6*ge**4*ga**4*em_k**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*mz**2*ep_k*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*mz**2*ep_k**(-1)*em_k
     +  - 256*mt**4*ge**4*gv**4*mz**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 256*mt**4*ge**4*gv**4*mz**2*ep_k**(-1)*em_ep
     +  + 256*mt**4*ge**4*gv**4*mz**2*em_k**(-1)*em_ep

      me2iziz = me2iziz + 256*mt**4*ge**4*gv**4*tm_tp*ep_k*em_k**(-1)
     +  + 256*mt**4*ge**4*gv**4*tm_tp*ep_k**(-1)*em_k
     +  + 512*mt**4*ge**4*gv**4*tm_tp*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 512*mt**4*ge**4*gv**4*tm_tp*ep_k**(-1)*em_ep
     +  - 512*mt**4*ge**4*gv**4*tm_tp*em_k**(-1)*em_ep
     +  + 256*mt**4*ge**4*gv**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*mt**4*ge**4*gv**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*ep_tp*em_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*ep_tp*em_tm*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*gv**4*ep_tp*k_tm*em_k**(-1)
     +  + 256*mt**4*ge**4*gv**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*em_tp*ep_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*em_tp*ep_tm*em_k**(-1)
     +  + 256*mt**4*ge**4*gv**4*em_tp*em_tm*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep

      me2iziz = me2iziz + 128*mt**4*ge**4*gv**4*em_tp*k_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*gv**4*k_tp*ep_tm*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*gv**4*k_tp*em_tm*ep_k**(-1)
     +  + 512*mt**4*ge**4*gv**2*ga**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 512*mt**4*ge**4*gv**2*ga**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 256*mt**4*ge**4*gv**2*ga**2*ep_tp*em_tm*ep_k**(-1)
     +  - 256*mt**4*ge**4*gv**2*ga**2*ep_tp*em_tm*em_k**(-1)
     +  - 256*mt**4*ge**4*gv**2*ga**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**4*ge**4*gv**2*ga**2*ep_tp*k_tm*em_k**(-1)
     +  + 512*mt**4*ge**4*gv**2*ga**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 256*mt**4*ge**4*gv**2*ga**2*em_tp*ep_tm*ep_k**(-1)

      me2iziz = me2iziz - 256*mt**4*ge**4*gv**2*ga**2*em_tp*ep_tm*
     + em_k**(-1)
     +  + 512*mt**4*ge**4*gv**2*ga**2*em_tp*em_tm*em_k**(-1)
     +  - 256*mt**4*ge**4*gv**2*ga**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**4*ge**4*gv**2*ga**2*em_tp*k_tm*ep_k**(-1)
     +  - 256*mt**4*ge**4*gv**2*ga**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**4*ge**4*gv**2*ga**2*k_tp*ep_tm*em_k**(-1)
     +  - 256*mt**4*ge**4*gv**2*ga**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**4*ge**4*gv**2*ga**2*k_tp*em_tm*ep_k**(-1)
     +  + 128*mt**4*ge**4*ga**4*mz**2*ep_k*em_k**(-1)
     +  + 128*mt**4*ge**4*ga**4*mz**2*ep_k**(-1)*em_k
     +  + 256*mt**4*ge**4*ga**4*mz**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 256*mt**4*ge**4*ga**4*mz**2*ep_k**(-1)*em_ep

      me2iziz = me2iziz - 256*mt**4*ge**4*ga**4*mz**2*em_k**(-1)*em_ep
     +  - 256*mt**4*ge**4*ga**4*tm_tp*ep_k*em_k**(-1)
     +  - 256*mt**4*ge**4*ga**4*tm_tp*ep_k**(-1)*em_k
     +  - 512*mt**4*ge**4*ga**4*tm_tp*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 512*mt**4*ge**4*ga**4*tm_tp*ep_k**(-1)*em_ep
     +  + 512*mt**4*ge**4*ga**4*tm_tp*em_k**(-1)*em_ep
     +  + 256*mt**4*ge**4*ga**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*mt**4*ge**4*ga**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**4*ge**4*ga**4*ep_tp*em_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*ep_tp*em_tm*em_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*ep_tp*k_tm*em_k**(-1)
     +  + 256*mt**4*ge**4*ga**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**4*ge**4*ga**4*em_tp*ep_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*em_tp*ep_tm*em_k**(-1)
     +  + 256*mt**4*ge**4*ga**4*em_tp*em_tm*em_k**(-1)

      me2iziz = me2iziz - 128*mt**4*ge**4*ga**4*em_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*em_tp*k_tm*ep_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*k_tp*ep_tm*em_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*k_tp*em_tm*ep_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**4*ep_k*em_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**4*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*mz**4*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 64*mt**2*ge**4*gv**4*mz**4*ep_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**4*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis**2*ep_k*em_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis**2*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*mz**2*gzis**2*ep_k**(-1)*em_k**(-1)*
     + em_ep**2

      me2iziz = me2iziz - 64*mt**2*ge**4*gv**4*mz**2*gzis**2*ep_k**(-1)
     + *em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*gzis**2*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_k*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_k**(-1)*em_k
     +  - 256*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_k**(-1)*em_k**(-1)*
     + em_ep**2
     +  + 256*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**4*mz**2*tm_tp*em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**4*mz**2*ep_tp*ep_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz - 128*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tm*
     + em_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*mz**2*em_tp*em_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*em_tp*k_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*k_tp*ep_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*k_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz + 128*mt**2*ge**4*gv**4*tm_tp**2*ep_k*
     + em_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*tm_tp**2*ep_k**(-1)*em_k
     +  + 256*mt**2*ge**4*gv**4*tm_tp**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*tm_tp**2*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**4*tm_tp**2*em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**4*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  + 512*mt**2*ge**4*gv**4*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tm*em_k**(-1)
     +  + 512*mt**2*ge**4*gv**4*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep

      me2iziz = me2iziz - 256*mt**2*ge**4*gv**4*tm_tp*em_tp*ep_tm*
     + ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  + 512*mt**2*ge**4*gv**4*tm_tp*em_tp*em_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*gv**4*tm_tp*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*k_tp*em_tm*ep_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_tm*ep_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep

      me2iziz = me2iziz + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm
     + *ep_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm*em_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*em_tm*em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*k_tm*ep_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep

      me2iziz = me2iziz - 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*
     + em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*em_tm*ep_k**(-1)
     +  + 1024*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  + 1024*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tm*em_k**(-1)
     +  + 1024*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*ep_k**(-1)

      me2iziz = me2iziz - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm
     + *em_k**(-1)
     +  + 1024*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*em_tm*em_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  - 512*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp*em_tm*ep_k**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_k*em_k**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*mz**4*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 64*mt**2*ge**4*ga**4*mz**4*ep_k**(-1)*em_ep

      me2iziz = me2iziz + 64*mt**2*ge**4*ga**4*mz**4*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis**2*ep_k*em_k**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis**2*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*gzis**2*ep_k**(-1)*em_k**(-1)*
     + em_ep**2
     +  + 64*mt**2*ge**4*ga**4*mz**2*gzis**2*ep_k**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*gzis**2*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_k*em_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_k**(-1)*em_k
     +  + 256*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_k**(-1)*em_k**(-1)*
     + em_ep**2
     +  - 256*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*ga**4*mz**2*tm_tp*em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*ga**4*mz**2*ep_tp*ep_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep

      me2iziz = me2iziz + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*
     + ep_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*mz**2*em_tp*em_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*em_tp*k_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz - 128*mt**2*ge**4*ga**4*mz**2*k_tp*ep_tm*
     + em_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*k_tp*em_tm*ep_k**(-1)
     +  - 128*mt**2*ge**4*ga**4*tm_tp**2*ep_k*em_k**(-1)
     +  - 128*mt**2*ge**4*ga**4*tm_tp**2*ep_k**(-1)*em_k
     +  - 256*mt**2*ge**4*ga**4*tm_tp**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*tm_tp**2*ep_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp**2*em_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*ga**4*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  + 512*mt**2*ge**4*ga**4*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 256*mt**2*ge**4*ga**4*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz + 256*mt**2*ge**4*ga**4*tm_tp*ep_tp*k_tm*
     + em_k**(-1)
     +  + 512*mt**2*ge**4*ga**4*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 256*mt**2*ge**4*ga**4*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  + 512*mt**2*ge**4*ga**4*tm_tp*em_tp*em_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  - 256*mt**2*ge**4*ga**4*tm_tp*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp*k_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz + 64*ge**4*gv**4*mz**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*gv**4*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*mz**4*ep_tp*k_tm*em_k**(-1)
     +  + 64*ge**4*gv**4*mz**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*gv**4*mz**4*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*gv**4*mz**4*em_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*mz**4*em_tp*k_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*mz**4*k_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*mz**4*k_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz + 64*ge**4*gv**4*mz**2*gzis**2*ep_tp*ep_tm*
     + ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*ge**4*gv**4*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*gv**4*mz**2*gzis**2*ep_tp*k_tm*em_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*ge**4*gv**4*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis**2*em_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep

      me2iziz = me2iziz + 32*ge**4*gv**4*mz**2*gzis**2*em_tp*k_tm*
     + ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*gv**4*mz**2*gzis**2*k_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*gv**4*mz**2*gzis**2*k_tp*em_tm*ep_k**(-1)
     +  - 256*ge**4*gv**4*mz**2*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  - 256*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tm*em_k**(-1)

      me2iziz = me2iziz - 256*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*
     + ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  - 256*ge**4*gv**4*mz**2*tm_tp*em_tp*em_tm*em_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)
     +  + 256*ge**4*gv**4*tm_tp**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*ge**4*gv**4*tm_tp**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz - 128*ge**4*gv**4*tm_tp**2*ep_tp*em_tm*
     + ep_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*ep_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp**2*ep_tp*k_tm*em_k**(-1)
     +  + 256*ge**4*gv**4*tm_tp**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*tm_tp**2*em_tp*ep_tm*ep_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*em_tp*ep_tm*em_k**(-1)
     +  + 256*ge**4*gv**4*tm_tp**2*em_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp**2*em_tp*k_tm*ep_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz + 128*ge**4*gv**4*tm_tp**2*k_tp*ep_tm*
     + em_k**(-1)
     +  - 128*ge**4*gv**4*tm_tp**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp**2*k_tp*em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tm*em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*ep_k**(-1)

      me2iziz = me2iziz - 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*
     + em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*em_tp*em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**4*k_tp*ep_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**4*k_tp*em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*
     + em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis**2*ep_tp*k_tm*em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*ep_tm*em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis**2*em_tp*k_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*k_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis**2*k_tp*ep_tm*em_k**(-1)

      me2iziz = me2iziz - 64*ge**4*gv**2*ga**2*mz**2*gzis**2*k_tp*em_tm
     + *ep_k**(-1)*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis**2*k_tp*em_tm*ep_k**(-1)
     +  - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tm*em_k**(-1)
     +  - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*em_tm*em_k**(-1)

      me2iziz = me2iziz + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tm*
     + ep_k**(-1)*em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)
     +  + 512*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 512*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*em_tm*ep_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*em_tm*em_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep

      me2iziz = me2iziz + 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*k_tm*
     + em_k**(-1)
     +  + 512*ge**4*gv**2*ga**2*tm_tp**2*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*em_tp*ep_tm*ep_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*em_tp*ep_tm*em_k**(-1)
     +  + 512*ge**4*gv**2*ga**2*tm_tp**2*em_tp*em_tm*em_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*em_tp*k_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp**2*em_tp*k_tm*ep_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*k_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp**2*k_tp*ep_tm*em_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*k_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp**2*k_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz + 64*ge**4*ga**4*mz**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*mz**4*ep_tp*k_tm*em_k**(-1)
     +  + 64*ge**4*ga**4*mz**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*mz**4*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*ga**4*mz**4*em_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*mz**4*em_tp*k_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*mz**4*k_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*mz**4*k_tp*em_tm*ep_k**(-1)

      me2iziz = me2iziz + 64*ge**4*ga**4*mz**2*gzis**2*ep_tp*ep_tm*
     + ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*mz**2*gzis**2*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*ga**4*mz**2*gzis**2*ep_tp*k_tm*em_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*mz**2*gzis**2*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis**2*em_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep

      me2iziz = me2iziz + 32*ge**4*ga**4*mz**2*gzis**2*em_tp*k_tm*
     + ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*ga**4*mz**2*gzis**2*k_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 32*ge**4*ga**4*mz**2*gzis**2*k_tp*em_tm*ep_k**(-1)
     +  - 256*ge**4*ga**4*mz**2*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  - 256*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tm*em_k**(-1)

      me2iziz = me2iziz - 256*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*
     + ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  - 256*ge**4*ga**4*mz**2*tm_tp*em_tp*em_tm*em_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*k_tp*em_tm*ep_k**(-1)
     +  + 256*ge**4*ga**4*tm_tp**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*ge**4*ga**4*tm_tp**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz - 128*ge**4*ga**4*tm_tp**2*ep_tp*em_tm*
     + ep_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*ep_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp**2*ep_tp*k_tm*em_k**(-1)
     +  + 256*ge**4*ga**4*tm_tp**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*tm_tp**2*em_tp*ep_tm*ep_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*em_tp*ep_tm*em_k**(-1)
     +  + 256*ge**4*ga**4*tm_tp**2*em_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp**2*em_tp*k_tm*ep_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep

      me2iziz = me2iziz + 128*ge**4*ga**4*tm_tp**2*k_tp*ep_tm*
     + em_k**(-1)
     +  - 128*ge**4*ga**4*tm_tp**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp**2*k_tp*em_tm*ep_k**(-1)


      RETURN
      END

      SUBROUTINE CIGIG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2igig =
     +  + 32*mt**2*ge**4*ep_k*em_k**(-1)
     +  + 32*mt**2*ge**4*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 64*mt**2*ge**4*ep_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*em_k**(-1)*em_ep
     +  + 64*ge**4*ep_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*ep_tp*k_tm*em_k**(-1)
     +  + 64*ge**4*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 32*ge**4*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*em_tp*em_tm*em_k**(-1)

      me2igig = me2igig - 32*ge**4*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*
     + em_ep
     +  + 32*ge**4*em_tp*k_tm*ep_k**(-1)
     +  - 32*ge**4*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*k_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 32*ge**4*k_tp*em_tm*ep_k**(-1)


      RETURN
      END

      SUBROUTINE CFZFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fzfz =
     +  - 32*mt**4*ge**4*gv**4*mz**4*k_tp**(-2)*em_ep
     +  - 32*mt**4*ge**4*gv**4*mz**4*k_tm**(-2)*em_ep
     +  - 32*mt**4*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-2)*em_ep
     +  - 32*mt**4*ge**4*gv**4*mz**2*gzfs**2*k_tm**(-2)*em_ep
     +  + 128*mt**4*ge**4*gv**4*mz**2*k_tp**(-2)*em_ep**2
     +  + 128*mt**4*ge**4*gv**4*mz**2*k_tm**(-2)*em_ep**2
     +  - 128*mt**4*ge**4*gv**4*k_tp**(-2)*em_ep**3
     +  - 128*mt**4*ge**4*gv**4*k_tm**(-2)*em_ep**3
     +  + 32*mt**4*ge**4*ga**4*mz**4*k_tp**(-2)*em_ep
     +  + 32*mt**4*ge**4*ga**4*mz**4*k_tm**(-2)*em_ep
     +  + 32*mt**4*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-2)*em_ep
     +  + 32*mt**4*ge**4*ga**4*mz**2*gzfs**2*k_tm**(-2)*em_ep
     +  - 128*mt**4*ge**4*ga**4*mz**2*k_tp**(-2)*em_ep**2
     +  - 128*mt**4*ge**4*ga**4*mz**2*k_tm**(-2)*em_ep**2
     +  + 128*mt**4*ge**4*ga**4*k_tp**(-2)*em_ep**3

      me2fzfz = me2fzfz + 128*mt**4*ge**4*ga**4*k_tm**(-2)*em_ep**3
     +  + 64*mt**2*ge**4*gv**4*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tp**(-2)*em_tm
     +  - 32*mt**2*ge**4*gv**4*mz**4*ep_tp*em_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tm**(-2)*em_k
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tp*k_tp**(-2)*ep_tm
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tp*ep_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tp*k_tm**(-2)*ep_k
     +  - 64*mt**2*ge**4*gv**4*mz**4*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  - 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-2)*ep_tm*em_k
     +  - 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-2)*em_tm*ep_k
     +  + 64*mt**2*ge**4*gv**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*ep_tp*k_tp**(-2)*em_tm
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*ep_tp*em_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*ep_tp*k_tm**(-2)*em_k

      me2fzfz = me2fzfz - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*em_tp*
     + k_tp**(-2)*ep_tm
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*em_tp*ep_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*em_tp*k_tm**(-2)*ep_k
     +  - 64*mt**2*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-2)*ep_tm*em_k
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-2)*em_tm*ep_k
     +  - 256*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*
     + em_ep**2
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-2)*em_tm*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*k_tm**(-2)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tm**(-2)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-2)*ep_tm*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*k_tm**(-2)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*k_tm**(-2)*ep_k*em_ep

      me2fzfz = me2fzfz + 256*mt**2*ge**4*gv**4*mz**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*k_tp**(-2)*ep_tm*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*k_tp**(-2)*em_tm*ep_k*em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep**3
     +  - 128*mt**2*ge**4*gv**4*ep_tp*k_tp**(-2)*em_tm*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*ep_tp*em_tm*k_tm**(-2)*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*ep_tp*k_tm**(-2)*em_k*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*em_tp*k_tp**(-2)*ep_tm*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*em_tp*ep_tm*k_tm**(-2)*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*em_tp*k_tm**(-2)*ep_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2
     +  - 128*mt**2*ge**4*gv**4*k_tp**(-2)*ep_tm*em_k*em_ep**2
     +  - 128*mt**2*ge**4*gv**4*k_tp**(-2)*em_tm*ep_k*em_ep**2
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tp**(-2)*em_tm

      me2fzfz = me2fzfz - 64*mt**2*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*
     + k_tm**(-2)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tm**(-2)*em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*em_tp*k_tp**(-2)*ep_tm
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*k_tm**(-2)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*em_tp*k_tm**(-2)*ep_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*k_tp**(-2)*ep_tm*em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*k_tp**(-2)*em_tm*ep_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*k_tp**(-2)*
     + em_tm
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*em_tm*
     + k_tm**(-2)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*k_tm**(-2)*
     + em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*em_tp*k_tp**(-2)*
     + ep_tm

      me2fzfz = me2fzfz - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*
     + em_tp*ep_tm*k_tm**(-2)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*em_tp*k_tm**(-2)*
     + ep_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*k_tp**(-2)*ep_tm*
     + em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzfs**2*k_tp**(-2)*em_tm*
     + ep_k
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tp**(-2)*em_tm*
     + em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*k_tm**(-2)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm**(-2)*em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*k_tp**(-2)*ep_tm*
     + em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*k_tm**(-2)*
     + em_ep

      me2fzfz = me2fzfz + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*
     + k_tm**(-2)*ep_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp**(-2)*ep_tm*em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp**(-2)*em_tm*ep_k*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*k_tp**(-2)*em_tm*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tm*k_tm**(-2)*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*k_tm**(-2)*em_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*k_tp**(-2)*ep_tm*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm*k_tm**(-2)*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*k_tm**(-2)*ep_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*k_tp**(-2)*ep_tm*em_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*k_tp**(-2)*em_tm*ep_k*em_ep**2
     +  - 64*mt**2*ge**4*ga**4*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tp**(-2)*em_tm
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_tp*em_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tm**(-2)*em_k

      me2fzfz = me2fzfz - 32*mt**2*ge**4*ga**4*mz**4*em_tp*k_tp**(-2)*
     + ep_tm
     +  - 32*mt**2*ge**4*ga**4*mz**4*em_tp*ep_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*ga**4*mz**4*em_tp*k_tm**(-2)*ep_k
     +  + 64*mt**2*ge**4*ga**4*mz**4*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-2)*ep_tm*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-2)*em_tm*ep_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*ep_tp*k_tp**(-2)*em_tm
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*ep_tp*em_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*ep_tp*k_tm**(-2)*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*em_tp*k_tp**(-2)*ep_tm
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*em_tp*ep_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*em_tp*k_tm**(-2)*ep_k
     +  + 64*mt**2*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k

      me2fzfz = me2fzfz - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-2)
     + *ep_tm*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-2)*em_tm*ep_k
     +  + 256*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*
     + em_ep**2
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-2)*em_tm*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*k_tm**(-2)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tm**(-2)*em_k*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-2)*ep_tm*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*k_tm**(-2)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*k_tm**(-2)*ep_k*em_ep
     +  - 256*mt**2*ge**4*ga**4*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*k_tp**(-2)*ep_tm*em_k*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*k_tp**(-2)*em_tm*ep_k*em_ep
     +  - 256*mt**2*ge**4*ga**4*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep**3

      me2fzfz = me2fzfz - 128*mt**2*ge**4*ga**4*ep_tp*k_tp**(-2)*em_tm*
     + em_ep**2
     +  - 128*mt**2*ge**4*ga**4*ep_tp*em_tm*k_tm**(-2)*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*ep_tp*k_tm**(-2)*em_k*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*em_tp*k_tp**(-2)*ep_tm*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*em_tp*ep_tm*k_tm**(-2)*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*em_tp*k_tm**(-2)*ep_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2
     +  - 128*mt**2*ge**4*ga**4*k_tp**(-2)*ep_tm*em_k*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*k_tp**(-2)*em_tm*ep_k*em_ep**2
     +  + 64*ge**4*gv**4*mz**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**4*mz**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**4*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**4*mz**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k

      me2fzfz = me2fzfz + 32*ge**4*gv**4*mz**4*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**4*mz**4*ep_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  + 32*ge**4*gv**4*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*gv**4*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**4*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*gv**4*mz**4*k_tp**(-1)*em_tm*ep_k
     +  - 64*ge**4*gv**4*mz**4*ep_tm*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm**(-1)*em_k

      me2fzfz = me2fzfz + 64*ge**4*gv**4*mz**2*gzfs**2*tm_tp*em_tp*
     + k_tp**(-1)*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**4*mz**2*gzfs**2*ep_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_tm
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k

      me2fzfz = me2fzfz + 32*ge**4*gv**4*mz**2*gzfs**2*k_tp**(-1)*em_tm
     + *ep_k
     +  - 64*ge**4*gv**4*mz**2*gzfs**2*ep_tm*em_tm*k_tm**(-1)
     +  - 256*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     + *em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     + *em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  + 256*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*em_ep

      me2fzfz = me2fzfz - 128*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*em_tm*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**4*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 128*ge**4*gv**4*mz**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**4*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 128*ge**4*gv**4*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 128*ge**4*gv**4*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 256*ge**4*gv**4*mz**2*ep_tm*em_tm*k_tm**(-1)*em_ep
     +  + 256*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)*
     + em_ep**2
     +  + 128*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 256*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*
     + em_ep**2

      me2fzfz = me2fzfz + 128*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*gv**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 128*ge**4*gv**4*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k*
     + em_ep**2
     +  - 256*ge**4*gv**4*ep_tp*em_tp*k_tp**(-1)*em_ep**2
     +  + 128*ge**4*gv**4*ep_tp*k_tp**(-1)*em_tm*em_ep**2
     +  + 128*ge**4*gv**4*ep_tp*em_tm*k_tm**(-1)*em_ep**2
     +  + 128*ge**4*gv**4*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*gv**4*em_tp*k_tp**(-1)*ep_tm*em_ep**2
     +  + 128*ge**4*gv**4*em_tp*ep_tm*k_tm**(-1)*em_ep**2
     +  + 128*ge**4*gv**4*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*gv**4*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  + 128*ge**4*gv**4*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  - 256*ge**4*gv**4*ep_tm*em_tm*k_tm**(-1)*em_ep**2

      me2fzfz = me2fzfz + 128*ge**4*gv**2*ga**2*mz**4*tm_tp*ep_tp*
     + k_tp**(-1)*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*
     + em_k
     +  + 128*ge**4*gv**2*ga**2*mz**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*
     + ep_k
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*
     + em_k
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*
     + ep_k
     +  - 128*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tm**(-1)*em_k

      me2fzfz = me2fzfz + 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tp**(-1)*
     + ep_tm
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 64*ge**4*gv**2*ga**2*mz**4*k_tp**(-1)*em_tm*ep_k
     +  - 128*ge**4*gv**2*ga**2*mz**4*ep_tm*em_tm*k_tm**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm**(-1)*em_k
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)*em_k

      me2fzfz = me2fzfz + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*tm_tp*
     + k_tp**(-1)*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*em_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_tm
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*em_tp*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzfs**2*ep_tm*em_tm*k_tm**(-1)
     +  - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)
     + *em_k*em_ep

      me2fzfz = me2fzfz - 512*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*
     + k_tp**(-1)*ep_tm*k_tm**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)
     + *ep_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     + *em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     + *ep_k*em_ep
     +  + 512*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep

      me2fzfz = me2fzfz - 256*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*em_tm*
     + ep_k*em_ep
     +  + 512*ge**4*gv**2*ga**2*mz**2*ep_tm*em_tm*k_tm**(-1)*em_ep
     +  + 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     + *em_ep**2
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 512*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     + *em_ep**2
     +  + 256*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_ep**2
     +  + 256*ge**4*gv**2*ga**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 256*ge**4*gv**2*ga**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k*
     + em_ep**2
     +  - 512*ge**4*gv**2*ga**2*ep_tp*em_tp*k_tp**(-1)*em_ep**2

      me2fzfz = me2fzfz + 256*ge**4*gv**2*ga**2*ep_tp*k_tp**(-1)*em_tm*
     + em_ep**2
     +  + 256*ge**4*gv**2*ga**2*ep_tp*em_tm*k_tm**(-1)*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*em_tp*k_tp**(-1)*ep_tm*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*em_tp*ep_tm*k_tm**(-1)*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  - 512*ge**4*gv**2*ga**2*ep_tm*em_tm*k_tm**(-1)*em_ep**2
     +  + 64*ge**4*ga**4*mz**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k
     +  + 64*ge**4*ga**4*mz**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**4*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*mz**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**4*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k

      me2fzfz = me2fzfz - 64*ge**4*ga**4*mz**4*ep_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  + 32*ge**4*ga**4*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*ga**4*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*ga**4*mz**4*k_tp**(-1)*em_tm*ep_k
     +  - 64*ge**4*ga**4*mz**4*ep_tm*em_tm*k_tm**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm**(-1)*em_k
     +  + 64*ge**4*ga**4*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)

      me2fzfz = me2fzfz + 32*ge**4*ga**4*mz**2*gzfs**2*tm_tp*em_tp*
     + k_tp**(-1)*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k
     +  - 64*ge**4*ga**4*mz**2*gzfs**2*ep_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_tm
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*ga**4*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  - 64*ge**4*ga**4*mz**2*gzfs**2*ep_tm*em_tm*k_tm**(-1)

      me2fzfz = me2fzfz - 256*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     + *em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  + 256*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  - 128*ge**4*ga**4*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep

      me2fzfz = me2fzfz - 128*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*ep_tm*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 128*ge**4*ga**4*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 128*ge**4*ga**4*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 256*ge**4*ga**4*mz**2*ep_tm*em_tm*k_tm**(-1)*em_ep
     +  + 256*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)*
     + em_ep**2
     +  + 128*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 256*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*
     + em_ep**2
     +  + 128*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_ep**2
     +  + 128*ge**4*ga**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2

      me2fzfz = me2fzfz + 128*ge**4*ga**4*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k*em_ep**2
     +  - 256*ge**4*ga**4*ep_tp*em_tp*k_tp**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*ep_tp*k_tp**(-1)*em_tm*em_ep**2
     +  + 128*ge**4*ga**4*ep_tp*em_tm*k_tm**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*ga**4*em_tp*k_tp**(-1)*ep_tm*em_ep**2
     +  + 128*ge**4*ga**4*em_tp*ep_tm*k_tm**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*ga**4*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  + 128*ge**4*ga**4*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  - 256*ge**4*ga**4*ep_tm*em_tm*k_tm**(-1)*em_ep**2


      RETURN
      END

      SUBROUTINE CFGFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fgfg =
     +  - 32*mt**4*ge**4*k_tp**(-2)*em_ep
     +  - 32*mt**4*ge**4*k_tm**(-2)*em_ep
     +  + 64*mt**2*ge**4*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep
     +  - 32*mt**2*ge**4*ep_tp*k_tp**(-2)*em_tm
     +  - 32*mt**2*ge**4*ep_tp*em_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*ep_tp*k_tm**(-2)*em_k
     +  - 32*mt**2*ge**4*em_tp*k_tp**(-2)*ep_tm
     +  - 32*mt**2*ge**4*em_tp*ep_tm*k_tm**(-2)
     +  - 32*mt**2*ge**4*em_tp*k_tm**(-2)*ep_k
     +  - 64*mt**2*ge**4*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  - 32*mt**2*ge**4*k_tp**(-2)*ep_tm*em_k
     +  - 32*mt**2*ge**4*k_tp**(-2)*em_tm*ep_k
     +  + 64*ge**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     +  + 32*ge**4*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k
     +  + 64*ge**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)

      me2fgfg = me2fgfg + 32*ge**4*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*
     + ep_k
     +  + 32*ge**4*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k
     +  - 64*ge**4*ep_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*ep_tp*k_tp**(-1)*em_tm
     +  + 32*ge**4*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*k_tp**(-1)*em_tm*ep_k
     +  - 64*ge**4*ep_tm*em_tm*k_tm**(-1)


      RETURN
      END

      SUBROUTINE CAZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2azaz =
     +  - 128*mt**2*ge**4*gv**4*Fet**2*mz**4*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*ep_tp*k_tp**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*em_tp*k_tp**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*k_tp*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**4*k_tp**(-1)*ep_tm*em_k
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**4*k_tp**(-1)*em_tm*ep_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*k_tp**(-1)*k_tm*em_ep
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 128*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*
     + em_k

      me2azaz = me2azaz - 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*
     + ep_tp*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*em_tp*k_tp**(-1)*
     + ep_k
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*em_tp*k_tm**(-1)*
     + ep_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*k_tp*k_tm**(-1)*
     + em_ep
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*ep_tm*
     + em_k
     +  - 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*em_tm*
     + ep_k
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*k_tm*
     + em_ep
     +  + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*ep_tm*k_tm**(-1)*
     + em_k

      me2azaz = me2azaz + 64*mt**2*ge**4*gv**4*Fet**2*mz**2*gzfs**2*
     + em_tm*k_tm**(-1)*ep_k
     +  + 512*mt**2*ge**4*gv**4*Fet**2*mz**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*Fet**2*mz**2*ep_tp*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*em_tp*k_tp**(-1)*ep_k*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*Fet**2*mz**2*em_tp*k_tm**(-1)*ep_k*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*k_tp*k_tm**(-1)*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*mz**2*k_tp**(-1)*ep_tm*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*Fet**2*mz**2*k_tp**(-1)*em_tm*ep_k*
     + em_ep

      me2azaz = me2azaz - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*k_tp**(-1)
     + *k_tm*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*Fet**2*mz**2*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  - 512*mt**2*ge**4*gv**4*Fet**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*Fet**2*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*Fet**2*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*k_tp*k_tm**(-1)*em_ep**3
     +  - 256*mt**2*ge**4*gv**4*Fet**2*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*Fet**2*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*k_tp**(-1)*k_tm*em_ep**3

      me2azaz = me2azaz + 256*mt**2*ge**4*gv**4*Fet**2*ep_tm*k_tm**(-1)
     + *em_k*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*Fet**2*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*ep_tp*k_tp**(-1)*
     + em_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*em_tp*k_tp**(-1)*
     + ep_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*ep_tm*k_tm**(-1)*
     + em_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*em_tm*k_tm**(-1)*
     + ep_k
     +  - 256*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**4*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*tm_tp*
     + k_tp**(-1)*k_tm**(-1)*ep_k*em_k

      me2azaz = me2azaz + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*
     + gzfs**2*ep_tp*k_tp**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*em_tp*
     + k_tp**(-1)*ep_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*ep_tm*
     + k_tm**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*em_tm*
     + k_tm**(-1)*ep_k
     +  - 256*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*em_ep
     +  + 1024*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*ep_tp*k_tp**(-1)*
     + em_k*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*em_tp*k_tp**(-1)*
     + ep_k*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*ep_tm*k_tm**(-1)*
     + em_k*em_ep

      me2azaz = me2azaz - 512*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*
     + em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 1024*mt**2*ge**4*gv**2*ga**2*Fet**2*mz**2*em_ep**2
     +  - 1024*mt**2*ge**4*gv**2*ga**2*Fet**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep**2
     +  + 512*mt**2*ge**4*gv**2*ga**2*Fet**2*ep_tp*k_tp**(-1)*em_k*
     + em_ep**2
     +  + 512*mt**2*ge**4*gv**2*ga**2*Fet**2*em_tp*k_tp**(-1)*ep_k*
     + em_ep**2
     +  + 512*mt**2*ge**4*gv**2*ga**2*Fet**2*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2
     +  + 512*mt**2*ge**4*gv**2*ga**2*Fet**2*em_tm*k_tm**(-1)*ep_k*
     + em_ep**2
     +  - 1024*mt**2*ge**4*gv**2*ga**2*Fet**2*em_ep**3
     +  - 128*mt**2*ge**4*ga**4*Fet**2*mz**4*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k

      me2azaz = me2azaz + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*ep_tp*
     + k_tp**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*em_tp*k_tm**(-1)*ep_k
     +  - 64*mt**2*ge**4*ga**4*Fet**2*mz**4*k_tp*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*k_tp**(-1)*em_tm*ep_k
     +  - 64*mt**2*ge**4*ga**4*Fet**2*mz**4*k_tp**(-1)*k_tm*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**4*em_ep
     +  - 128*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*
     + em_k

      me2azaz = me2azaz + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*
     + ep_tp*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*em_tp*k_tp**(-1)*
     + ep_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*em_tp*k_tm**(-1)*
     + ep_k
     +  - 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*k_tp*k_tm**(-1)*
     + em_ep
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*ep_tm*
     + em_k
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*em_tm*
     + ep_k
     +  - 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*k_tp**(-1)*k_tm*
     + em_ep
     +  + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*ep_tm*k_tm**(-1)*
     + em_k

      me2azaz = me2azaz + 64*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*
     + em_tm*k_tm**(-1)*ep_k
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*gzfs**2*em_ep
     +  + 512*mt**2*ge**4*ga**4*Fet**2*mz**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*ep_tp*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*em_tp*k_tp**(-1)*ep_k*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*em_tp*k_tm**(-1)*ep_k*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*Fet**2*mz**2*k_tp*k_tm**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*k_tp**(-1)*ep_tm*em_k*
     + em_ep

      me2azaz = me2azaz - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*k_tp**(-1)
     + *em_tm*ep_k*em_ep
     +  + 256*mt**2*ge**4*ga**4*Fet**2*mz**2*k_tp**(-1)*k_tm*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*Fet**2*mz**2*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  + 1024*mt**2*ge**4*ga**4*Fet**2*mz**2*em_ep**2
     +  - 512*mt**2*ge**4*ga**4*Fet**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*Fet**2*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*Fet**2*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*Fet**2*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*Fet**2*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*Fet**2*k_tp*k_tm**(-1)*em_ep**3
     +  + 256*mt**2*ge**4*ga**4*Fet**2*k_tp**(-1)*ep_tm*em_k*em_ep**2

      me2azaz = me2azaz + 256*mt**2*ge**4*ga**4*Fet**2*k_tp**(-1)*em_tm
     + *ep_k*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*Fet**2*k_tp**(-1)*k_tm*em_ep**3
     +  + 256*mt**2*ge**4*ga**4*Fet**2*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*Fet**2*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 1024*mt**2*ge**4*ga**4*Fet**2*em_ep**3
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp*ep_tp*k_tp**(-1)*
     + em_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp*em_tp*k_tp**(-1)*
     + ep_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp*ep_tm*k_tm**(-1)*
     + em_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp*em_tm*k_tm**(-1)*
     + ep_k

      me2azaz = me2azaz - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*tm_tp*
     + em_ep
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*ep_tp*em_tp*k_tp**(-1)*
     + k_tm
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*ep_tp*em_tm
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*em_tp*ep_tm
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**4*k_tp*ep_tm*em_tm*
     + k_tm**(-1)
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp**2*
     + k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp*ep_tp*
     + k_tp**(-1)*em_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp*em_tp*
     + k_tp**(-1)*ep_k
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp*ep_tm*
     + k_tm**(-1)*em_k

      me2azaz = me2azaz + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*
     + tm_tp*em_tm*k_tm**(-1)*ep_k
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp*em_ep
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*ep_tp*em_tp*
     + k_tp**(-1)*k_tm
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*ep_tp*em_tm
     +  + 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*em_tp*ep_tm
     +  - 32*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*k_tp*ep_tm*em_tm
     + *k_tm**(-1)
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_k*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_k*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*
     + em_k*em_ep

      me2azaz = me2azaz - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp*
     + em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*tm_tp*em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + k_tm*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*ep_tp*em_tm*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*em_tp*ep_tm*em_ep
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_ep
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp*ep_tp*k_tp**(-1)*em_k*
     + em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2

      me2azaz = me2azaz + 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp*em_tm*
     + k_tm**(-1)*ep_k*em_ep**2
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*tm_tp*em_ep**3
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*ep_tp*em_tm*em_ep**2
     +  + 128*mt**(-2)*ge**4*gv**4*Fmt**2*em_tp*ep_tm*em_ep**2
     +  - 128*mt**(-2)*ge**4*gv**4*Fmt**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_ep**2
     +  + 64*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**4*ep_tp*em_tm
     +  + 64*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**4*em_tp*ep_tm
     +  + 64*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*ep_tp*
     + em_tm
     +  + 64*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*em_tp*
     + ep_tm
     +  - 256*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**2*ep_tp*em_tm*em_ep

      me2azaz = me2azaz - 256*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*mz**2*
     + em_tp*ep_tm*em_ep
     +  + 256*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*ep_tp*em_tm*em_ep**2
     +  + 256*mt**(-2)*ge**4*gv**2*ga**2*Fmt**2*em_tp*ep_tm*em_ep**2
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp*ep_tp*k_tp**(-1)*
     + em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp*em_tp*k_tp**(-1)*
     + ep_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp*ep_tm*k_tm**(-1)*
     + em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp*em_tm*k_tm**(-1)*
     + ep_k
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*tm_tp*em_ep
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*ep_tp*em_tp*k_tp**(-1)*
     + k_tm

      me2azaz = me2azaz + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*ep_tp*
     + em_tm
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*em_tp*ep_tm
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**4*k_tp*ep_tm*em_tm*
     + k_tm**(-1)
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp**2*
     + k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*ep_tp*
     + k_tp**(-1)*em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*em_tp*
     + k_tp**(-1)*ep_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*ep_tm*
     + k_tm**(-1)*em_k
     +  - 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*em_tm*
     + k_tm**(-1)*ep_k
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*em_ep

      me2azaz = me2azaz + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*
     + ep_tp*em_tp*k_tp**(-1)*k_tm
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*ep_tp*em_tm
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*em_tp*ep_tm
     +  + 32*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*k_tp*ep_tm*em_tm
     + *k_tm**(-1)
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_k*em_ep
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_k*em_ep
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*
     + em_k*em_ep
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp*em_tm*k_tm**(-1)*
     + ep_k*em_ep

      me2azaz = me2azaz - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*tm_tp*
     + em_ep**2
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + k_tm*em_ep
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*ep_tp*em_tm*em_ep
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*em_tp*ep_tm*em_ep
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_ep
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep**2
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp*ep_tp*k_tp**(-1)*em_k*
     + em_ep**2
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_ep**2
     +  - 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp*ep_tm*k_tm**(-1)*em_k*
     + em_ep**2

      me2azaz = me2azaz - 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp*em_tm*
     + k_tm**(-1)*ep_k*em_ep**2
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*tm_tp*em_ep**3
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_ep**2
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*ep_tp*em_tm*em_ep**2
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*em_tp*ep_tm*em_ep**2
     +  + 128*mt**(-2)*ge**4*ga**4*Fmt**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_ep**2
     +  - 32*ge**4*gv**4*Fmt**2*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*ep_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*em_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*k_tp*k_tm**(-1)*em_ep

      me2azaz = me2azaz - 16*ge**4*gv**4*Fmt**2*mz**4*k_tp**(-1)*ep_tm*
     + em_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**4*k_tp**(-1)*em_tm*ep_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*k_tp**(-1)*k_tm*em_ep
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 32*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*k_tp*k_tm**(-1)*em_ep
     +  - 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  - 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*k_tm*em_ep

      me2azaz = me2azaz + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*ep_tm*
     + k_tm**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt**2*mz**2*gzfs**2*em_tm*k_tm**(-1)*ep_k
     +  + 128*ge**4*gv**4*Fmt**2*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*ep_tp*k_tp**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**4*Fmt**2*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**4*Fmt**2*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*k_tp*k_tm**(-1)*em_ep**2
     +  + 64*ge**4*gv**4*Fmt**2*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  + 64*ge**4*gv**4*Fmt**2*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*k_tp**(-1)*k_tm*em_ep**2
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**4*Fmt**2*mz**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  - 128*ge**4*gv**4*Fmt**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2

      me2azaz = me2azaz + 64*ge**4*gv**4*Fmt**2*ep_tp*k_tp**(-1)*em_k*
     + em_ep**2
     +  - 64*ge**4*gv**4*Fmt**2*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 64*ge**4*gv**4*Fmt**2*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  - 64*ge**4*gv**4*Fmt**2*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 64*ge**4*gv**4*Fmt**2*k_tp*k_tm**(-1)*em_ep**3
     +  - 64*ge**4*gv**4*Fmt**2*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  - 64*ge**4*gv**4*Fmt**2*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  + 64*ge**4*gv**4*Fmt**2*k_tp**(-1)*k_tm*em_ep**3
     +  + 64*ge**4*gv**4*Fmt**2*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 64*ge**4*gv**4*Fmt**2*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 128*ge**4*gv**4*Fet**2*mz**4*tm_tp**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  + 128*ge**4*gv**4*Fet**2*mz**4*tm_tp*ep_tp*k_tp**(-1)*em_k
     +  + 128*ge**4*gv**4*Fet**2*mz**4*tm_tp*em_tp*k_tp**(-1)*ep_k
     +  + 128*ge**4*gv**4*Fet**2*mz**4*tm_tp*ep_tm*k_tm**(-1)*em_k

      me2azaz = me2azaz + 128*ge**4*gv**4*Fet**2*mz**4*tm_tp*em_tm*
     + k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**4*Fet**2*mz**4*tm_tp*em_ep
     +  - 128*ge**4*gv**4*Fet**2*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm
     +  + 128*ge**4*gv**4*Fet**2*mz**4*ep_tp*em_tm
     +  + 128*ge**4*gv**4*Fet**2*mz**4*em_tp*ep_tm
     +  - 128*ge**4*gv**4*Fet**2*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)
     +  - 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_k
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_k
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*ep_tm*k_tm**(-1)*
     + em_k
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*em_tm*k_tm**(-1)*
     + ep_k

      me2azaz = me2azaz - 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*tm_tp*
     + em_ep
     +  - 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*ep_tp*em_tp*k_tp**(-1)*
     + k_tm
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*ep_tp*em_tm
     +  + 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*em_tp*ep_tm
     +  - 128*ge**4*gv**4*Fet**2*mz**2*gzfs**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)
     +  + 512*ge**4*gv**4*Fet**2*mz**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k*em_ep
     +  - 512*ge**4*gv**4*Fet**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  - 512*ge**4*gv**4*Fet**2*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_ep
     +  - 512*ge**4*gv**4*Fet**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_k*
     + em_ep

      me2azaz = me2azaz - 512*ge**4*gv**4*Fet**2*mz**2*tm_tp*em_tm*
     + k_tm**(-1)*ep_k*em_ep
     +  + 512*ge**4*gv**4*Fet**2*mz**2*tm_tp*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_ep
     +  - 512*ge**4*gv**4*Fet**2*mz**2*ep_tp*em_tm*em_ep
     +  - 512*ge**4*gv**4*Fet**2*mz**2*em_tp*ep_tm*em_ep
     +  + 512*ge**4*gv**4*Fet**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_ep
     +  - 512*ge**4*gv**4*Fet**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*tm_tp*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*tm_tp*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*tm_tp*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*tm_tp*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 512*ge**4*gv**4*Fet**2*tm_tp*em_ep**3

      me2azaz = me2azaz - 512*ge**4*gv**4*Fet**2*ep_tp*em_tp*k_tp**(-1)
     + *k_tm*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*ep_tp*em_tm*em_ep**2
     +  + 512*ge**4*gv**4*Fet**2*em_tp*ep_tm*em_ep**2
     +  - 512*ge**4*gv**4*Fet**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_ep**2
     +  - 64*ge**4*gv**2*ga**2*Fmt**2*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)
     + *ep_k*em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**4*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**2*ga**2*Fmt**2*mz**4*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*
     + em_k

      me2azaz = me2azaz + 32*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*
     + em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*ep_tm*k_tm**(-1)*
     + em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*em_tm*k_tm**(-1)*
     + ep_k
     +  - 64*ge**4*gv**2*ga**2*Fmt**2*mz**2*gzfs**2*em_ep
     +  + 256*ge**4*gv**2*ga**2*Fmt**2*mz**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt**2*mz**2*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt**2*mz**2*em_tp*k_tp**(-1)*ep_k*
     + em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt**2*mz**2*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt**2*mz**2*em_tm*k_tm**(-1)*ep_k*
     + em_ep

      me2azaz = me2azaz + 256*ge**4*gv**2*ga**2*Fmt**2*mz**2*em_ep**2
     +  - 256*ge**4*gv**2*ga**2*Fmt**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt**2*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt**2*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt**2*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt**2*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 256*ge**4*gv**2*ga**2*Fmt**2*em_ep**3
     +  + 256*ge**4*gv**2*ga**2*Fet**2*mz**4*ep_tp*em_tm
     +  + 256*ge**4*gv**2*ga**2*Fet**2*mz**4*em_tp*ep_tm
     +  + 256*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*ep_tp*em_tm
     +  + 256*ge**4*gv**2*ga**2*Fet**2*mz**2*gzfs**2*em_tp*ep_tm
     +  - 1024*ge**4*gv**2*ga**2*Fet**2*mz**2*ep_tp*em_tm*em_ep
     +  - 1024*ge**4*gv**2*ga**2*Fet**2*mz**2*em_tp*ep_tm*em_ep
     +  + 1024*ge**4*gv**2*ga**2*Fet**2*ep_tp*em_tm*em_ep**2
     +  + 1024*ge**4*gv**2*ga**2*Fet**2*em_tp*ep_tm*em_ep**2

      me2azaz = me2azaz - 32*ge**4*ga**4*Fmt**2*mz**4*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*ep_tp*k_tp**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*em_tp*k_tm**(-1)*ep_k
     +  - 16*ge**4*ga**4*Fmt**2*mz**4*k_tp*k_tm**(-1)*em_ep
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*k_tp**(-1)*em_tm*ep_k
     +  - 16*ge**4*ga**4*Fmt**2*mz**4*k_tp**(-1)*k_tm*em_ep
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 64*ge**4*ga**4*Fmt**2*mz**4*em_ep
     +  - 32*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_k

      me2azaz = me2azaz + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*ep_tp*
     + k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  - 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*k_tp*k_tm**(-1)*em_ep
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  - 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*k_tp**(-1)*k_tm*em_ep
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*ep_tm*k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*em_tm*k_tm**(-1)*ep_k
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*gzfs**2*em_ep
     +  + 128*ge**4*ga**4*Fmt**2*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*ep_tp*k_tp**(-1)*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*em_tp*k_tp**(-1)*ep_k*em_ep

      me2azaz = me2azaz - 64*ge**4*ga**4*Fmt**2*mz**2*em_tp*k_tm**(-1)*
     + ep_k*em_ep
     +  + 64*ge**4*ga**4*Fmt**2*mz**2*k_tp*k_tm**(-1)*em_ep**2
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 64*ge**4*ga**4*Fmt**2*mz**2*k_tp**(-1)*k_tm*em_ep**2
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt**2*mz**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 256*ge**4*ga**4*Fmt**2*mz**2*em_ep**2
     +  - 128*ge**4*ga**4*Fmt**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  - 64*ge**4*ga**4*Fmt**2*k_tp*k_tm**(-1)*em_ep**3

      me2azaz = me2azaz + 64*ge**4*ga**4*Fmt**2*k_tp**(-1)*ep_tm*em_k*
     + em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  - 64*ge**4*ga**4*Fmt**2*k_tp**(-1)*k_tm*em_ep**3
     +  + 64*ge**4*ga**4*Fmt**2*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 64*ge**4*ga**4*Fmt**2*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 256*ge**4*ga**4*Fmt**2*em_ep**3
     +  + 128*ge**4*ga**4*Fet**2*mz**4*tm_tp**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**4*tm_tp*ep_tp*k_tp**(-1)*em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**4*tm_tp*em_tp*k_tp**(-1)*ep_k
     +  - 128*ge**4*ga**4*Fet**2*mz**4*tm_tp*ep_tm*k_tm**(-1)*em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**4*tm_tp*em_tm*k_tm**(-1)*ep_k
     +  + 128*ge**4*ga**4*Fet**2*mz**4*tm_tp*em_ep
     +  + 128*ge**4*ga**4*Fet**2*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm
     +  + 128*ge**4*ga**4*Fet**2*mz**4*ep_tp*em_tm

      me2azaz = me2azaz + 128*ge**4*ga**4*Fet**2*mz**4*em_tp*ep_tm
     +  + 128*ge**4*ga**4*Fet**2*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)
     +  + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_k
     +  - 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*ep_tm*k_tm**(-1)*
     + em_k
     +  - 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*em_tm*k_tm**(-1)*
     + ep_k
     +  + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*tm_tp*em_ep
     +  + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*ep_tp*em_tp*k_tp**(-1)*
     + k_tm
     +  + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*ep_tp*em_tm

      me2azaz = me2azaz + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*em_tp*
     + ep_tm
     +  + 128*ge**4*ga**4*Fet**2*mz**2*gzfs**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)
     +  - 512*ge**4*ga**4*Fet**2*mz**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k*em_ep
     +  + 512*ge**4*ga**4*Fet**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  + 512*ge**4*ga**4*Fet**2*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_ep
     +  + 512*ge**4*ga**4*Fet**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_k*
     + em_ep
     +  + 512*ge**4*ga**4*Fet**2*mz**2*tm_tp*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  - 512*ge**4*ga**4*Fet**2*mz**2*tm_tp*em_ep**2
     +  - 512*ge**4*ga**4*Fet**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_ep

      me2azaz = me2azaz - 512*ge**4*ga**4*Fet**2*mz**2*ep_tp*em_tm*
     + em_ep
     +  - 512*ge**4*ga**4*Fet**2*mz**2*em_tp*ep_tm*em_ep
     +  - 512*ge**4*ga**4*Fet**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_ep
     +  + 512*ge**4*ga**4*Fet**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep**2
     +  - 512*ge**4*ga**4*Fet**2*tm_tp*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  - 512*ge**4*ga**4*Fet**2*tm_tp*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  - 512*ge**4*ga**4*Fet**2*tm_tp*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  - 512*ge**4*ga**4*Fet**2*tm_tp*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  + 512*ge**4*ga**4*Fet**2*tm_tp*em_ep**3
     +  + 512*ge**4*ga**4*Fet**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_ep**2
     +  + 512*ge**4*ga**4*Fet**2*ep_tp*em_tm*em_ep**2
     +  + 512*ge**4*ga**4*Fet**2*em_tp*ep_tm*em_ep**2
     +  + 512*ge**4*ga**4*Fet**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_ep**2


      RETURN
      END

      SUBROUTINE CAGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2agag =
     +  - 128*mt**2*ge**4*Fet**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 64*mt**2*ge**4*Fet**2*ep_tp*k_tp**(-1)*em_k
     +  - 64*mt**2*ge**4*Fet**2*ep_tp*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*Fet**2*em_tp*k_tp**(-1)*ep_k
     +  - 64*mt**2*ge**4*Fet**2*em_tp*k_tm**(-1)*ep_k
     +  + 64*mt**2*ge**4*Fet**2*k_tp*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*Fet**2*k_tp**(-1)*ep_tm*em_k
     +  - 64*mt**2*ge**4*Fet**2*k_tp**(-1)*em_tm*ep_k
     +  + 64*mt**2*ge**4*Fet**2*k_tp**(-1)*k_tm*em_ep
     +  + 64*mt**2*ge**4*Fet**2*ep_tm*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*Fet**2*em_tm*k_tm**(-1)*ep_k
     +  - 32*mt**(-2)*ge**4*Fmt**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  + 32*mt**(-2)*ge**4*Fmt**2*tm_tp*ep_tp*k_tp**(-1)*em_k
     +  + 32*mt**(-2)*ge**4*Fmt**2*tm_tp*em_tp*k_tp**(-1)*ep_k

      me2agag = me2agag + 32*mt**(-2)*ge**4*Fmt**2*tm_tp*ep_tm*
     + k_tm**(-1)*em_k
     +  + 32*mt**(-2)*ge**4*Fmt**2*tm_tp*em_tm*k_tm**(-1)*ep_k
     +  - 32*mt**(-2)*ge**4*Fmt**2*tm_tp*em_ep
     +  - 32*mt**(-2)*ge**4*Fmt**2*ep_tp*em_tp*k_tp**(-1)*k_tm
     +  + 32*mt**(-2)*ge**4*Fmt**2*ep_tp*em_tm
     +  + 32*mt**(-2)*ge**4*Fmt**2*em_tp*ep_tm
     +  - 32*mt**(-2)*ge**4*Fmt**2*k_tp*ep_tm*em_tm*k_tm**(-1)
     +  - 32*ge**4*Fmt**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 16*ge**4*Fmt**2*ep_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*Fmt**2*ep_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*Fmt**2*em_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*Fmt**2*em_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*Fmt**2*k_tp*k_tm**(-1)*em_ep
     +  - 16*ge**4*Fmt**2*k_tp**(-1)*ep_tm*em_k
     +  - 16*ge**4*Fmt**2*k_tp**(-1)*em_tm*ep_k

      me2agag = me2agag + 16*ge**4*Fmt**2*k_tp**(-1)*k_tm*em_ep
     +  + 16*ge**4*Fmt**2*ep_tm*k_tm**(-1)*em_k
     +  + 16*ge**4*Fmt**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*Fet**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 128*ge**4*Fet**2*tm_tp*ep_tp*k_tp**(-1)*em_k
     +  + 128*ge**4*Fet**2*tm_tp*em_tp*k_tp**(-1)*ep_k
     +  + 128*ge**4*Fet**2*tm_tp*ep_tm*k_tm**(-1)*em_k
     +  + 128*ge**4*Fet**2*tm_tp*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*Fet**2*tm_tp*em_ep
     +  - 128*ge**4*Fet**2*ep_tp*em_tp*k_tp**(-1)*k_tm
     +  + 128*ge**4*Fet**2*ep_tp*em_tm
     +  + 128*ge**4*Fet**2*em_tp*ep_tm
     +  - 128*ge**4*Fet**2*k_tp*ep_tm*em_tm*k_tm**(-1)


      RETURN
      END

      SUBROUTINE CIZIG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2izig =
     +  + 128*mt**4*ge**4*gv**2*ep_k*em_k**(-1)
     +  + 128*mt**4*ge**4*gv**2*ep_k**(-1)*em_k
     +  + 256*mt**4*ge**4*gv**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 256*mt**4*ge**4*gv**2*ep_k**(-1)*em_ep
     +  - 256*mt**4*ge**4*gv**2*em_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*mz**2*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*mz**2*ep_k**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**2*mz**2*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  + 128*mt**2*ge**4*gv**2*mz**2*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*mz**2*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*tm_tp*ep_k*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*tm_tp*ep_k**(-1)*em_k
     +  + 256*mt**2*ge**4*gv**2*tm_tp*ep_k**(-1)*em_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*tm_tp*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*tm_tp*em_k**(-1)*em_ep

      me2izig = me2izig + 256*mt**2*ge**4*gv**2*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*ep_tp*em_tm*ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ep_tp*em_tm*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ep_tp*k_tm*em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tp*ep_tm*ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*em_tp*ep_tm*em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*em_tp*em_tm*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*em_tp*k_tm*ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*k_tp*ep_tm*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*k_tp*em_tm*ep_k**(-1)

      me2izig = me2izig - 128*ge**4*gv**2*mz**2*ep_tp*ep_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*mz**2*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*ep_tp*em_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*ep_tp*k_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*mz**2*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*mz**2*em_tp*em_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*em_tp*k_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*k_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*k_tp*em_tm*ep_k**(-1)

      me2izig = me2izig + 256*ge**4*gv**2*tm_tp*ep_tp*ep_tm*ep_k**(-1)
     +  + 256*ge**4*gv**2*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*ep_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*ep_tp*k_tm*em_k**(-1)
     +  + 256*ge**4*gv**2*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  + 256*ge**4*gv**2*tm_tp*em_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*em_tp*k_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*em_tp*k_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*k_tp*ep_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*k_tp*ep_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*k_tp*em_tm*ep_k**(-1)*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*k_tp*em_tm*ep_k**(-1)


      RETURN
      END

      SUBROUTINE CIZFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2izfz =
     +  + 64*mt**4*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 128*mt**4*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)
     +  - 64*mt**4*ge**4*gv**4*mz**2*ep_tp*k_tm**(-1)
     +  - 64*mt**4*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 128*mt**4*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)
     +  + 64*mt**4*ge**4*gv**4*mz**2*em_tp*k_tm**(-1)
     +  + 64*mt**4*ge**4*gv**4*mz**2*k_tp**(-1)*ep_tm
     +  - 64*mt**4*ge**4*gv**4*mz**2*k_tp**(-1)*em_tm
     +  - 64*mt**4*ge**4*gv**4*mz**2*k_tp**(-1)*ep_k
     +  + 64*mt**4*ge**4*gv**4*mz**2*k_tp**(-1)*em_k
     +  - 64*mt**4*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 128*mt**4*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)

      me2izfz = me2izfz + 64*mt**4*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)
     +  + 64*mt**4*ge**4*gv**4*mz**2*k_tm**(-1)*ep_k
     +  - 64*mt**4*ge**4*gv**4*mz**2*k_tm**(-1)*em_k
     +  - 128*mt**4*ge**4*gv**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 256*mt**4*ge**4*gv**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 128*mt**4*ge**4*gv**4*ep_tp*k_tp**(-1)*em_ep
     +  + 128*mt**4*ge**4*gv**4*ep_tp*k_tm**(-1)*em_ep
     +  + 128*mt**4*ge**4*gv**4*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 256*mt**4*ge**4*gv**4*em_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 128*mt**4*ge**4*gv**4*em_tp*k_tp**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*em_tp*k_tm**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*k_tp**(-1)*ep_tm*em_ep
     +  + 128*mt**4*ge**4*gv**4*k_tp**(-1)*em_tm*em_ep

      me2izfz = me2izfz + 128*mt**4*ge**4*gv**4*k_tp**(-1)*ep_k*em_ep
     +  - 128*mt**4*ge**4*gv**4*k_tp**(-1)*em_k*em_ep
     +  + 128*mt**4*ge**4*gv**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 256*mt**4*ge**4*gv**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  + 128*mt**4*ge**4*gv**4*ep_tm*k_tm**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 256*mt**4*ge**4*gv**4*em_tm*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  - 128*mt**4*ge**4*gv**4*em_tm*k_tm**(-1)*em_ep
     +  - 128*mt**4*ge**4*gv**4*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt**4*ge**4*gv**4*k_tm**(-1)*em_k*em_ep
     +  - 128*mt**4*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm**(-1)
     +  + 128*mt**4*ge**4*gv**2*ga**2*mz**2*em_tp*k_tm**(-1)
     +  + 128*mt**4*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*ep_tm
     +  - 128*mt**4*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*em_tm
     +  + 256*mt**4*ge**4*gv**2*ga**2*ep_tp*k_tm**(-1)*em_ep
     +  - 256*mt**4*ge**4*gv**2*ga**2*em_tp*k_tm**(-1)*em_ep

      me2izfz = me2izfz - 256*mt**4*ge**4*gv**2*ga**2*k_tp**(-1)*ep_tm*
     + em_ep
     +  + 256*mt**4*ge**4*gv**2*ga**2*k_tp**(-1)*em_tm*em_ep
     +  - 64*mt**4*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 128*mt**4*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)
     +  - 64*mt**4*ge**4*ga**4*mz**2*ep_tp*k_tm**(-1)
     +  + 64*mt**4*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 128*mt**4*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)
     +  + 64*mt**4*ge**4*ga**4*mz**2*em_tp*k_tm**(-1)
     +  + 64*mt**4*ge**4*ga**4*mz**2*k_tp**(-1)*ep_tm
     +  - 64*mt**4*ge**4*ga**4*mz**2*k_tp**(-1)*em_tm
     +  + 64*mt**4*ge**4*ga**4*mz**2*k_tp**(-1)*ep_k
     +  - 64*mt**4*ge**4*ga**4*mz**2*k_tp**(-1)*em_k
     +  + 64*mt**4*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k

      me2izfz = me2izfz - 128*mt**4*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)
     +  - 64*mt**4*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 128*mt**4*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)
     +  - 64*mt**4*ge**4*ga**4*mz**2*k_tm**(-1)*ep_k
     +  + 64*mt**4*ge**4*ga**4*mz**2*k_tm**(-1)*em_k
     +  + 128*mt**4*ge**4*ga**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 256*mt**4*ge**4*ga**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 128*mt**4*ge**4*ga**4*ep_tp*k_tp**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*ep_tp*k_tm**(-1)*em_ep
     +  - 128*mt**4*ge**4*ga**4*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 256*mt**4*ge**4*ga**4*em_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 128*mt**4*ge**4*ga**4*em_tp*k_tp**(-1)*em_ep
     +  - 128*mt**4*ge**4*ga**4*em_tp*k_tm**(-1)*em_ep

      me2izfz = me2izfz - 128*mt**4*ge**4*ga**4*k_tp**(-1)*ep_tm*em_ep
     +  + 128*mt**4*ge**4*ga**4*k_tp**(-1)*em_tm*em_ep
     +  - 128*mt**4*ge**4*ga**4*k_tp**(-1)*ep_k*em_ep
     +  + 128*mt**4*ge**4*ga**4*k_tp**(-1)*em_k*em_ep
     +  - 128*mt**4*ge**4*ga**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 256*mt**4*ge**4*ga**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 128*mt**4*ge**4*ga**4*ep_tm*k_tm**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 256*mt**4*ge**4*ga**4*em_tm*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 128*mt**4*ge**4*ga**4*em_tm*k_tm**(-1)*em_ep
     +  + 128*mt**4*ge**4*ga**4*k_tm**(-1)*ep_k*em_ep
     +  - 128*mt**4*ge**4*ga**4*k_tm**(-1)*em_k*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**4*ep_tp*k_tm**(-1)

      me2izfz = me2izfz + 32*mt**2*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-1)*ep_tm
     +  + 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-1)*em_tm
     +  + 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-1)*ep_k
     +  - 32*mt**2*ge**4*gv**4*mz**4*k_tp**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**4*mz**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**4*mz**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**4*mz**4*ep_tm*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**4*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**4*em_tm*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**4*k_tm**(-1)*ep_k

      me2izfz = me2izfz + 32*mt**2*ge**4*gv**4*mz**4*k_tm**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*k_tp**(-1)*ep_tm
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*k_tp**(-1)*em_tm
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*k_tp**(-1)*ep_k

      me2izfz = me2izfz - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*
     + k_tp**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*em_tm*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*k_tm**(-1)*ep_k
     +  + 32*mt**2*ge**4*gv**4*mz**2*gzis*gzfs*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_k

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*ep_tm
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*em_tm
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*ep_k
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tp**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k

      me2izfz = me2izfz + 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*ep_tm*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 128*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*tm_tp*em_tm*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**4*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**4*mz**2*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*
     + em_k
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm**2*
     + k_tm**(-1)*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tp*em_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tp*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tp
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*ep_tm
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*em_k**(-1)*
     + em_ep**2

      me2izfz = me2izfz - 64*mt**2*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**4*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tp*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tp*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*em_tp
     +  - 64*mt**2*ge**4*gv**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)

      me2izfz = me2izfz + 64*mt**2*ge**4*gv**4*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**4*mz**2*k_tp**(-1)*ep_tm*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*k_tp**(-1)*em_tm*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*k_tp**(-1)*ep_k*em_ep
     +  + 64*mt**2*ge**4*gv**4*mz**2*k_tp**(-1)*em_k*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 64*mt**2*ge**4*gv**4*mz**2*ep_tm*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**4*mz**2*ep_tm*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*ep_tm
     +  + 64*mt**2*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep**2

      me2izfz = me2izfz + 64*mt**2*ge**4*gv**4*mz**2*em_tm*k_tm**(-1)*
     + em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*em_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*mz**2*em_tm
     +  + 64*mt**2*ge**4*gv**4*mz**2*k_tm**(-1)*ep_k*em_ep
     +  - 64*mt**2*ge**4*gv**4*mz**2*k_tm**(-1)*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 256*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*ep_tp*k_tm**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)*
     + em_ep**2

      me2izfz = me2izfz + 256*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*em_tp*k_tm**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*k_tp**(-1)*em_tm*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*k_tp**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 256*mt**2*ge**4*gv**4*tm_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*tm_tp*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep**2

      me2izfz = me2izfz - 256*mt**2*ge**4*gv**4*tm_tp*em_tm*k_tm**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 256*mt**2*ge**4*gv**4*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_ep
     +  - 256*mt**2*ge**4*gv**4*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*gv**4*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep

      me2izfz = me2izfz + 128*mt**2*ge**4*gv**4*ep_tp*k_tp**(-1)*em_tm*
     + em_ep
     +  - 256*mt**2*ge**4*gv**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*gv**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*gv**4*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**4*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*ep_tp*ep_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*ep_tp*em_ep
     +  - 256*mt**2*ge**4*gv**4*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*em_tp*k_tp**(-1)*ep_tm*em_ep

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**4*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**4*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*gv**4*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*gv**4*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**4*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*em_tp*em_k**(-1)*em_ep**2
     +  + 256*mt**2*ge**4*gv**4*em_tp*em_ep
     +  + 128*mt**2*ge**4*gv**4*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*gv**4*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**4*ep_tm*ep_k**(-1)*
     + em_ep**2
     +  + 256*mt**2*ge**4*gv**4*ep_tm*em_ep
     +  + 128*mt**2*ge**4*gv**4*em_tm*em_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*gv**4*em_tm*em_ep
     +  + 64*mt**2*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*em_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**4*k_tp**(-1)*ep_tm
     +  + 64*mt**2*ge**4*gv**2*ga**2*mz**4*k_tp**(-1)*em_tm
     +  + 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*k_tp**(-1)*ep_tm
     +  + 64*mt**2*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*k_tp**(-1)*em_tm
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tp**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tm**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tp**(-1)

      me2izfz = me2izfz + 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp
     + *k_tm**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp**(-1)*ep_tm
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp**(-1)*em_tm
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tm*k_tm**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tm*k_tm**(-1)
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm
     + *ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm
     + *em_k**(-1)

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tm**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tp
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*
     + k_tp**(-1)*ep_tm
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k
     + *em_k**(-1)
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k
     + *em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*k_tm**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp*em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tp

      me2izfz = me2izfz - 128*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*
     + em_tm*k_tm**(-1)*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)
     + *em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*ep_tm*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*k_tp**(-1)*em_tm*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tm*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*mz**2*ep_tm
     +  - 128*mt**2*ge**4*gv**2*ga**2*mz**2*em_tm*em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*mz**2*em_tm
     +  - 256*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tm**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tm**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*tm_tp*k_tp**(-1)*em_tm*em_ep

      me2izfz = me2izfz + 256*mt**2*ge**4*gv**2*ga**2*tm_tp*ep_tm*
     + k_tm**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*tm_tp*em_tm*k_tm**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*ep_tp*k_tp**(-1)*em_tm*em_ep

      me2izfz = me2izfz - 512*mt**2*ge**4*gv**2*ga**2*ep_tp*ep_tm*em_tm
     + *k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)
     + *em_k*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*ep_tp*ep_k**(-1)*em_ep**2
     +  - 512*mt**2*ge**4*gv**2*ga**2*ep_tp*em_ep
     +  - 512*mt**2*ge**4*gv**2*ga**2*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)*em_ep

      me2izfz = me2izfz - 512*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm**2*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 512*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*em_tp*em_k**(-1)*em_ep**2
     +  + 512*mt**2*ge**4*gv**2*ga**2*em_tp*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*gv**2*ga**2*ep_tm*ep_k**(-1)*em_ep**2

      me2izfz = me2izfz + 512*mt**2*ge**4*gv**2*ga**2*ep_tm*em_ep
     +  + 256*mt**2*ge**4*gv**2*ga**2*em_tm*em_k**(-1)*em_ep**2
     +  - 512*mt**2*ge**4*gv**2*ga**2*em_tm*em_ep
     +  + 32*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**4*ep_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**4*em_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-1)*ep_tm
     +  + 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-1)*em_tm
     +  - 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-1)*ep_k
     +  + 32*mt**2*ge**4*ga**4*mz**4*k_tp**(-1)*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k

      me2izfz = me2izfz + 64*mt**2*ge**4*ga**4*mz**4*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**4*ep_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**4*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**4*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*ga**4*mz**4*em_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**4*k_tm**(-1)*ep_k
     +  - 32*mt**2*ge**4*ga**4*mz**4*k_tm**(-1)*em_k
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)

      me2izfz = me2izfz + 64*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tp**(-1)*ep_tm
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tp**(-1)*em_tm
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tp**(-1)*ep_k
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tp**(-1)*em_k
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*ep_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep

      me2izfz = me2izfz + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*em_tm*
     + k_tm**(-1)
     +  + 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tm**(-1)*ep_k
     +  - 32*mt**2*ge**4*ga**4*mz**2*gzis*gzfs*k_tm**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_k
     +  + 128*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)
     + *em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tm**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 128*mt**2*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)
     + *em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tm**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*ep_tm
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*em_tm

      me2izfz = me2izfz + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*
     + ep_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tp**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  - 128*mt**2*ge**4*ga**4*mz**2*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 128*mt**2*ge**4*ga**4*mz**2*tm_tp*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  + 64*mt**2*ge**4*ga**4*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)

      me2izfz = me2izfz + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*
     + em_k
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)

      me2izfz = me2izfz + 64*mt**2*ge**4*ga**4*mz**2*ep_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tp*em_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*ep_tp*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*ep_tp
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*ep_tm
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*
     + em_ep

      me2izfz = me2izfz - 128*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 128*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*em_tp*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*em_tp
     +  - 64*mt**2*ge**4*ga**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)

      me2izfz = me2izfz + 64*mt**2*ge**4*ga**4*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*mz**2*k_tp**(-1)*ep_tm*em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*k_tp**(-1)*em_tm*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*k_tp**(-1)*ep_k*em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*k_tp**(-1)*em_k*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tm*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*ep_tm*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*ga**4*mz**2*ep_tm
     +  - 64*mt**2*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep**2

      me2izfz = me2izfz - 64*mt**2*ge**4*ga**4*mz**2*em_tm*k_tm**(-1)*
     + em_ep
     +  - 64*mt**2*ge**4*ga**4*mz**2*em_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*mz**2*em_tm
     +  - 64*mt**2*ge**4*ga**4*mz**2*k_tm**(-1)*ep_k*em_ep
     +  + 64*mt**2*ge**4*ga**4*mz**2*k_tm**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 128*mt**2*ge**4*ga**4*tm_tp*ep_tp*k_tm**(-1)*em_ep
     +  - 128*mt**2*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)*
     + em_ep**2
     +  - 128*mt**2*ge**4*ga**4*tm_tp*em_tp*k_tm**(-1)*em_ep

      me2izfz = me2izfz - 128*mt**2*ge**4*ga**4*tm_tp*k_tp**(-1)*ep_tm*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*tm_tp*k_tp**(-1)*em_tm*em_ep
     +  - 128*mt**2*ge**4*ga**4*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 128*mt**2*ge**4*ga**4*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*mt**2*ge**4*ga**4*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 128*mt**2*ge**4*ga**4*tm_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*tm_tp*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep**2
     +  + 128*mt**2*ge**4*ga**4*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  - 128*mt**2*ge**4*ga**4*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 256*mt**2*ge**4*ga**4*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)*
     + em_ep

      me2izfz = me2izfz + 256*mt**2*ge**4*ga**4*ep_tp*em_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep
     +  - 256*mt**2*ge**4*ga**4*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*ga**4*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  - 256*mt**2*ge**4*ga**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*ga**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)*
     + em_ep

      me2izfz = me2izfz + 128*mt**2*ge**4*ga**4*ep_tp*em_tm*k_tm**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*ga**4*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*ep_tp*ep_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*ep_tp*em_ep
     +  - 256*mt**2*ge**4*ga**4*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 128*mt**2*ge**4*ga**4*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)*
     + em_ep
     +  - 256*mt**2*ge**4*ga**4*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 256*mt**2*ge**4*ga**4*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  - 128*mt**2*ge**4*ga**4*em_tp*ep_tm*k_tm**(-1)*em_ep

      me2izfz = me2izfz + 128*mt**2*ge**4*ga**4*em_tp*ep_tm*ep_k**(-1)*
     + em_ep
     +  + 128*mt**2*ge**4*ga**4*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*em_tp*em_k**(-1)*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*em_tp*em_ep
     +  + 128*mt**2*ge**4*ga**4*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  - 128*mt**2*ge**4*ga**4*ep_tm*ep_k**(-1)*em_ep**2
     +  + 256*mt**2*ge**4*ga**4*ep_tm*em_ep
     +  + 128*mt**2*ge**4*ga**4*em_tm*em_k**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*ga**4*em_tm*em_ep
     +  - 32*ge**4*gv**4*mz**4*tm_tp*ep_tp*k_tp**(-1)

      me2izfz = me2izfz + 32*ge**4*gv**4*mz**4*tm_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*gv**4*mz**4*tm_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*gv**4*mz**4*tm_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**4*mz**4*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**4*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 64*ge**4*gv**4*mz**4*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  + 32*ge**4*gv**4*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  + 32*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*gv**4*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*gv**4*mz**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 64*ge**4*gv**4*mz**4*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*gv**4*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tp*em_tm*em_k**(-1)

      me2izfz = me2izfz + 32*ge**4*gv**4*mz**4*ep_tp*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**4*ep_tp
     +  - 64*ge**4*gv**4*mz**4*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)*ep_tm
     +  - 32*ge**4*gv**4*mz**4*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 64*ge**4*gv**4*mz**4*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**4*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**4*em_tp*ep_tm*ep_k**(-1)
     +  + 32*ge**4*gv**4*mz**4*em_tp*ep_tm*em_k**(-1)
     +  + 32*ge**4*gv**4*mz**4*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*em_tp*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**4*em_tp
     +  + 32*ge**4*gv**4*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**4*ep_tm*ep_k**(-1)*em_ep

      me2izfz = me2izfz + 64*ge**4*gv**4*mz**4*ep_tm
     +  + 32*ge**4*gv**4*mz**4*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**4*em_tm
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*tm_tp*ep_tp*k_tp**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*tm_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*tm_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*tm_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*em_tm*em_k**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tp
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)

      me2izfz = me2izfz - 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_tm
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*ep_tm*ep_k**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*ep_tm*em_k**(-1)
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*em_tp
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)

      me2izfz = me2izfz - 32*ge**4*gv**4*mz**2*gzis*gzfs*k_tp*ep_tm*
     + em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*gv**4*mz**2*gzis*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**2*gzis*gzfs*ep_tm
     +  + 32*ge**4*gv**4*mz**2*gzis*gzfs*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*gzis*gzfs*em_tm
     +  + 64*ge**4*gv**4*mz**2*tm_tp**2*ep_tp*k_tp**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp**2*em_tp*k_tp**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp**2*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp**2*em_tm*k_tm**(-1)
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*
     + em_k
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)

      me2izfz = me2izfz + 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*em_tm*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tp*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*ep_tp
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm
     +  + 64*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 128*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)

      me2izfz = me2izfz - 64*ge**4*gv**4*mz**2*tm_tp*em_tp*ep_tm*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*em_tp*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*em_tp
     +  - 64*ge**4*gv**4*mz**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*gv**4*mz**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*gv**4*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**2*tm_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**4*mz**2*tm_tp*ep_tm
     +  + 64*ge**4*gv**4*mz**2*tm_tp*em_tm*k_tm**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*tm_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*mz**2*tm_tp*em_tm

      me2izfz = me2izfz - 128*ge**4*gv**4*mz**2*ep_tp**2*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_ep
     +  + 128*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  + 64*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  - 64*ge**4*gv**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  - 64*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep
     +  - 64*ge**4*gv**4*mz**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  + 128*ge**4*gv**4*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  + 64*ge**4*gv**4*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep

      me2izfz = me2izfz - 128*ge**4*gv**4*mz**2*ep_tp*em_tm**2*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**2*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**4*mz**2*ep_tp*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*ep_tp*ep_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**4*mz**2*ep_tp*em_ep
     +  + 128*ge**4*gv**4*mz**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 64*ge**4*gv**4*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*mz**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep

      me2izfz = me2izfz + 64*ge**4*gv**4*mz**2*em_tp*ep_tm*k_tm**(-1)*
     + em_ep
     +  - 64*ge**4*gv**4*mz**2*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*em_tp*ep_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**4*mz**2*em_tp*em_k**(-1)*em_ep**2
     +  - 128*ge**4*gv**4*mz**2*em_tp*em_ep
     +  - 64*ge**4*gv**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*gv**4*mz**2*ep_tm*ep_k**(-1)*em_ep**2
     +  - 128*ge**4*gv**4*mz**2*ep_tm*em_ep
     +  - 64*ge**4*gv**4*mz**2*em_tm*em_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**4*mz**2*em_tm*em_ep

      me2izfz = me2izfz - 128*ge**4*gv**4*tm_tp**2*ep_tp*k_tp**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp**2*em_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*gv**4*tm_tp**2*ep_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**4*tm_tp**2*em_tm*k_tm**(-1)*em_ep
     +  + 256*ge**4*gv**4*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)*
     + em_ep
     +  + 256*ge**4*gv**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_ep
     +  - 256*ge**4*gv**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  - 128*ge**4*gv**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep

      me2izfz = me2izfz + 128*ge**4*gv**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*
     + em_ep
     +  - 256*ge**4*gv**4*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  - 128*ge**4*gv**4*tm_tp*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*ge**4*gv**4*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*gv**4*tm_tp*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**4*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**4*tm_tp*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*tm_tp*ep_tp*ep_k**(-1)*em_ep**2
     +  - 256*ge**4*gv**4*tm_tp*ep_tp*em_ep
     +  - 256*ge**4*gv**4*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*em_ep

      me2izfz = me2izfz - 128*ge**4*gv**4*tm_tp*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)*em_ep
     +  - 256*ge**4*gv**4*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 256*ge**4*gv**4*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  - 128*ge**4*gv**4*tm_tp*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**4*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*tm_tp*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**4*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*tm_tp*em_tp*em_k**(-1)*em_ep**2
     +  + 256*ge**4*gv**4*tm_tp*em_tp*em_ep
     +  + 128*ge**4*gv**4*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*ge**4*gv**4*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep

      me2izfz = me2izfz - 128*ge**4*gv**4*tm_tp*ep_tm*ep_k**(-1)*
     + em_ep**2
     +  + 256*ge**4*gv**4*tm_tp*ep_tm*em_ep
     +  + 128*ge**4*gv**4*tm_tp*em_tm*em_k**(-1)*em_ep**2
     +  - 256*ge**4*gv**4*tm_tp*em_tm*em_ep
     +  - 64*ge**4*gv**2*ga**2*mz**4*tm_tp*ep_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*em_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*tm_tp*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*tm_tp*em_tm*k_tm**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)

      me2izfz = me2izfz + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tp*
     + k_tp**(-1)*k_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*
     + em_k
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  - 128*ge**4*gv**2*ga**2*mz**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tp*em_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*ep_tp*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**4*ep_tp

      me2izfz = me2izfz - 128*ge**4*gv**2*ga**2*mz**4*em_tp**2*
     + k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tp**(-1)*ep_tm
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*ep_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*em_tp*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**4*em_tp

      me2izfz = me2izfz + 64*ge**4*gv**2*ga**2*mz**4*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**4*ep_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**4*ep_tm
     +  + 64*ge**4*gv**2*ga**2*mz**4*em_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**4*em_tm
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*tm_tp*ep_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*tm_tp*em_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*tm_tp*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*tm_tp*em_tm*k_tm**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp**2*k_tp**(-1)*
     + em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)

      me2izfz = me2izfz - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*
     + em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*
     + k_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tm**2*
     + k_tm**(-1)*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tm*k_tm**(-1)

      me2izfz = me2izfz - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*
     + em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*em_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tp
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp**2*k_tp**(-1)*
     + ep_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*ep_tm**2*
     + k_tm**(-1)*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*ep_tm*ep_k**(-1)

      me2izfz = me2izfz + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*
     + ep_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tp
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*ep_tm
     +  + 64*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*gzis*gzfs*em_tm
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp**2*ep_tp*k_tp**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp**2*em_tp*k_tp**(-1)

      me2izfz = me2izfz - 128*ge**4*gv**2*ga**2*mz**2*tm_tp**2*ep_tm*
     + k_tm**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp**2*em_tm*k_tm**(-1)
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm
     + *ep_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm
     + *em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm

      me2izfz = me2izfz + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*k_tm**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp*ep_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tp
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm

      me2izfz = me2izfz + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*em_tm*
     + k_tm**(-1)*em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*k_tm**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*ep_tm*em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k
     + *em_k**(-1)
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp*em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tp
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)
     + *ep_k**(-1)

      me2izfz = me2izfz + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*k_tp*ep_tm*
     + em_tm*k_tm**(-1)*em_k**(-1)
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*tm_tp*ep_tm
     +  + 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tm*em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*tm_tp*em_tm
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_ep

      me2izfz = me2izfz - 128*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tp*
     + k_tp**(-1)*k_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_k*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)
     + *em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*ep_tp*em_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*ep_tp*ep_k**(-1)*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*mz**2*ep_tp*em_ep

      me2izfz = me2izfz + 256*ge**4*gv**2*ga**2*mz**2*em_tp**2*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*em_tp*ep_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*em_tp*em_k**(-1)*em_ep**2
     +  - 256*ge**4*gv**2*ga**2*mz**2*em_tp*em_ep

      me2izfz = me2izfz - 128*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*mz**2*ep_tm*ep_k**(-1)*em_ep**2
     +  - 256*ge**4*gv**2*ga**2*mz**2*ep_tm*em_ep
     +  - 128*ge**4*gv**2*ga**2*mz**2*em_tm*em_k**(-1)*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*mz**2*em_tm*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tp*k_tp**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp**2*em_tp*k_tp**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp**2*ep_tm*k_tm**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp**2*em_tm*k_tm**(-1)*em_ep
     +  + 512*ge**4*gv**2*ga**2*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  + 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep

      me2izfz = me2izfz - 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_k*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  - 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)
     + *em_k*em_ep
     +  + 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*k_tm**(-1)*em_ep

      me2izfz = me2izfz - 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*
     + ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*ep_tp*ep_k**(-1)*em_ep**2
     +  - 512*ge**4*gv**2*ga**2*tm_tp*ep_tp*em_ep
     +  - 512*ge**4*gv**2*ga**2*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 512*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 512*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_ep

      me2izfz = me2izfz + 256*ge**4*gv**2*ga**2*tm_tp*em_tp*ep_tm*
     + em_k**(-1)*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*em_tp*em_k**(-1)*em_ep**2
     +  + 512*ge**4*gv**2*ga**2*tm_tp*em_tp*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 256*ge**4*gv**2*ga**2*tm_tp*ep_tm*ep_k**(-1)*em_ep**2
     +  + 512*ge**4*gv**2*ga**2*tm_tp*ep_tm*em_ep
     +  + 256*ge**4*gv**2*ga**2*tm_tp*em_tm*em_k**(-1)*em_ep**2
     +  - 512*ge**4*gv**2*ga**2*tm_tp*em_tm*em_ep
     +  - 32*ge**4*ga**4*mz**4*tm_tp*ep_tp*k_tp**(-1)
     +  + 32*ge**4*ga**4*mz**4*tm_tp*em_tp*k_tp**(-1)

      me2izfz = me2izfz + 32*ge**4*ga**4*mz**4*tm_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*ga**4*mz**4*tm_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*ga**4*mz**4*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**4*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 64*ge**4*ga**4*mz**4*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**4*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*ga**4*mz**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 64*ge**4*ga**4*mz**4*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*em_tm*k_tm**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tp*em_tm*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*ep_tp*ep_k**(-1)*em_ep

      me2izfz = me2izfz - 64*ge**4*ga**4*mz**4*ep_tp
     +  - 64*ge**4*ga**4*mz**4*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)*ep_tm
     +  - 32*ge**4*ga**4*mz**4*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 64*ge**4*ga**4*mz**4*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**4*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**4*em_tp*ep_tm*ep_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*em_tp*ep_tm*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**4*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*em_tp*em_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**4*em_tp
     +  + 32*ge**4*ga**4*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**4*ep_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**4*ep_tm

      me2izfz = me2izfz + 32*ge**4*ga**4*mz**4*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**4*em_tm
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*tm_tp*ep_tp*k_tp**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*tm_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*tm_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*tm_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*em_tm*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tp
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)

      me2izfz = me2izfz - 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_tm
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*ep_tm*ep_k**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*ep_tm*em_k**(-1)
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*em_tp
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)

      me2izfz = me2izfz - 32*ge**4*ga**4*mz**2*gzis*gzfs*k_tp*ep_tm*
     + em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*ga**4*mz**2*gzis*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**2*gzis*gzfs*ep_tm
     +  + 32*ge**4*ga**4*mz**2*gzis*gzfs*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*gzis*gzfs*em_tm
     +  + 64*ge**4*ga**4*mz**2*tm_tp**2*ep_tp*k_tp**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp**2*em_tp*k_tp**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp**2*ep_tm*k_tm**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp**2*em_tm*k_tm**(-1)
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)

      me2izfz = me2izfz + 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*
     + em_k
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*ep_k**(-1)

      me2izfz = me2izfz + 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*em_tm*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tp*ep_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*ep_tp
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm
     +  + 64*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*
     + ep_k**(-1)
     +  - 128*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*k_tm**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*ep_k**(-1)

      me2izfz = me2izfz - 64*ge**4*ga**4*mz**2*tm_tp*em_tp*ep_tm*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*em_tp*em_k**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*em_tp
     +  - 64*ge**4*ga**4*mz**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + ep_k**(-1)
     +  + 64*ge**4*ga**4*mz**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**2*tm_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*tm_tp*ep_tm
     +  + 64*ge**4*ga**4*mz**2*tm_tp*em_tm*k_tm**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*tm_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*mz**2*tm_tp*em_tm

      me2izfz = me2izfz - 128*ge**4*ga**4*mz**2*ep_tp**2*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_ep
     +  + 128*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  + 64*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  - 64*ge**4*ga**4*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  - 64*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep
     +  - 64*ge**4*ga**4*mz**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  + 128*ge**4*ga**4*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  + 64*ge**4*ga**4*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep

      me2izfz = me2izfz - 128*ge**4*ga**4*mz**2*ep_tp*em_tm**2*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**2*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*mz**2*ep_tp*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*ep_tp*ep_k**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*mz**2*ep_tp*em_ep
     +  + 128*ge**4*ga**4*mz**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 64*ge**4*ga**4*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*mz**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep

      me2izfz = me2izfz + 64*ge**4*ga**4*mz**2*em_tp*ep_tm*k_tm**(-1)*
     + em_ep
     +  - 64*ge**4*ga**4*mz**2*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*em_tp*ep_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*ga**4*mz**2*em_tp*em_k**(-1)*em_ep**2
     +  - 128*ge**4*ga**4*mz**2*em_tp*em_ep
     +  - 64*ge**4*ga**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 64*ge**4*ga**4*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  + 64*ge**4*ga**4*mz**2*ep_tm*ep_k**(-1)*em_ep**2
     +  - 128*ge**4*ga**4*mz**2*ep_tm*em_ep
     +  - 64*ge**4*ga**4*mz**2*em_tm*em_k**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*mz**2*em_tm*em_ep

      me2izfz = me2izfz - 128*ge**4*ga**4*tm_tp**2*ep_tp*k_tp**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp**2*em_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*ga**4*tm_tp**2*ep_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*ga**4*tm_tp**2*em_tm*k_tm**(-1)*em_ep
     +  + 256*ge**4*ga**4*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)*
     + em_ep
     +  + 256*ge**4*ga**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     + *em_ep
     +  - 256*ge**4*ga**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     + *em_ep
     +  - 128*ge**4*ga**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*
     + em_ep

      me2izfz = me2izfz + 128*ge**4*ga**4*tm_tp*ep_tp*k_tp**(-1)*em_tm*
     + em_ep
     +  - 256*ge**4*ga**4*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     + *em_ep
     +  - 128*ge**4*ga**4*tm_tp*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*
     + em_ep
     +  + 256*ge**4*ga**4*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)*
     + em_ep
     +  + 128*ge**4*ga**4*tm_tp*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 128*ge**4*ga**4*tm_tp*ep_tp*em_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*ga**4*tm_tp*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*tm_tp*ep_tp*ep_k**(-1)*em_ep**2
     +  - 256*ge**4*ga**4*tm_tp*ep_tp*em_ep
     +  - 256*ge**4*ga**4*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*ep_tm*em_ep

      me2izfz = me2izfz - 128*ge**4*ga**4*tm_tp*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)*em_ep
     +  - 256*ge**4*ga**4*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  + 256*ge**4*ga**4*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     + *em_ep
     +  - 128*ge**4*ga**4*tm_tp*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*ga**4*tm_tp*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*tm_tp*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*tm_tp*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*tm_tp*em_tp*em_k**(-1)*em_ep**2
     +  + 256*ge**4*ga**4*tm_tp*em_tp*em_ep
     +  + 128*ge**4*ga**4*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*
     + em_ep
     +  - 128*ge**4*ga**4*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*
     + em_ep

      me2izfz = me2izfz - 128*ge**4*ga**4*tm_tp*ep_tm*ep_k**(-1)*
     + em_ep**2
     +  + 256*ge**4*ga**4*tm_tp*ep_tm*em_ep
     +  + 128*ge**4*ga**4*tm_tp*em_tm*em_k**(-1)*em_ep**2
     +  - 256*ge**4*ga**4*tm_tp*em_tm*em_ep


      RETURN
      END

      SUBROUTINE CIZFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2izfg =
     +  - 64*mt**4*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 128*mt**4*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*gv**2*ep_tp*k_tp**(-1)
     +  + 64*mt**4*ge**4*gv**2*ep_tp*k_tm**(-1)
     +  + 64*mt**4*ge**4*gv**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 128*mt**4*ge**4*gv**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*gv**2*em_tp*k_tp**(-1)
     +  - 64*mt**4*ge**4*gv**2*em_tp*k_tm**(-1)
     +  - 64*mt**4*ge**4*gv**2*k_tp**(-1)*ep_tm
     +  + 64*mt**4*ge**4*gv**2*k_tp**(-1)*em_tm
     +  + 64*mt**4*ge**4*gv**2*k_tp**(-1)*ep_k
     +  - 64*mt**4*ge**4*gv**2*k_tp**(-1)*em_k
     +  + 64*mt**4*ge**4*gv**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 128*mt**4*ge**4*gv**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 64*mt**4*ge**4*gv**2*ep_tm*k_tm**(-1)

      me2izfg = me2izfg - 64*mt**4*ge**4*gv**2*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 128*mt**4*ge**4*gv**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*mt**4*ge**4*gv**2*em_tm*k_tm**(-1)
     +  - 64*mt**4*ge**4*gv**2*k_tm**(-1)*ep_k
     +  + 64*mt**4*ge**4*gv**2*k_tm**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*ep_tm
     +  - 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*em_tm

      me2izfg = me2izfg - 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*ep_k
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tm**(-1)*ep_k
     +  - 32*mt**2*ge**4*gv**2*mz**2*k_tm**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)
     +  + 64*mt**2*ge**4*gv**2*tm_tp*ep_tp*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)*em_ep

      me2izfg = me2izfg + 128*mt**2*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)
     +  - 64*mt**2*ge**4*gv**2*tm_tp*em_tp*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*tm_tp*k_tp**(-1)*ep_tm
     +  + 64*mt**2*ge**4*gv**2*tm_tp*k_tp**(-1)*em_tm
     +  + 64*mt**2*ge**4*gv**2*tm_tp*k_tp**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**2*tm_tp*k_tp**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**2*tm_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*tm_tp*ep_tm*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*tm_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*tm_tp*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*tm_tp*em_tm*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*tm_tp*k_tm**(-1)*ep_k
     +  + 64*mt**2*ge**4*gv**2*tm_tp*k_tm**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)

      me2izfg = me2izfg - 128*mt**2*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*ep_tp*k_tp**(-1)*em_tm
     +  - 128*mt**2*ge**4*gv**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*ep_tp*em_tm*k_tm**(-1)
     +  - 64*mt**2*ge**4*gv**2*ep_tp*em_tm*ep_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*ep_tp*em_tm*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*ep_tp*ep_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*ep_tp
     +  - 128*mt**2*ge**4*gv**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*em_tp*k_tp**(-1)*ep_tm

      me2izfg = me2izfg - 64*mt**2*ge**4*gv**2*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 128*mt**2*ge**4*gv**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  + 128*mt**2*ge**4*gv**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*em_tp*ep_tm*k_tm**(-1)
     +  + 64*mt**2*ge**4*gv**2*em_tp*ep_tm*ep_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*em_tp*ep_tm*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*em_tp*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*em_tp
     +  + 64*mt**2*ge**4*gv**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*ep_tm*ep_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ep_tm
     +  + 64*mt**2*ge**4*gv**2*em_tm*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tm

      me2izfg = me2izfg + 32*ge**4*gv**2*mz**2*tm_tp*ep_tp*k_tp**(-1)
     +  - 32*ge**4*gv**2*mz**2*tm_tp*em_tp*k_tp**(-1)
     +  - 32*ge**4*gv**2*mz**2*tm_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**2*mz**2*tm_tp*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*gv**2*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 64*ge**4*gv**2*mz**2*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*em_tm*ep_k**(-1)

      me2izfg = me2izfg + 32*ge**4*gv**2*mz**2*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*ep_tp
     +  + 64*ge**4*gv**2*mz**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*ep_tm
     +  + 32*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*em_tp
     +  - 32*ge**4*gv**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)

      me2izfg = me2izfg + 32*ge**4*gv**2*mz**2*ep_tm*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*ep_tm
     +  - 32*ge**4*gv**2*mz**2*em_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*em_tm
     +  - 64*ge**4*gv**2*tm_tp**2*ep_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*tm_tp**2*em_tp*k_tp**(-1)
     +  + 64*ge**4*gv**2*tm_tp**2*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*tm_tp**2*em_tm*k_tm**(-1)
     +  + 128*ge**4*gv**2*tm_tp*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  + 128*ge**4*gv**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*tm_tp*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 64*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*em_tm
     +  - 128*ge**4*gv**2*tm_tp*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)

      me2izfg = me2izfg - 64*ge**4*gv**2*tm_tp*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 128*ge**4*gv**2*tm_tp*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  + 64*ge**4*gv**2*tm_tp*ep_tp*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*tm_tp*ep_tp*em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*ep_tp*em_tm*em_k**(-1)
     +  + 64*ge**4*gv**2*tm_tp*ep_tp*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*tm_tp*ep_tp
     +  - 128*ge**4*gv**2*tm_tp*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*ep_tm
     +  - 64*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 128*ge**4*gv**2*tm_tp*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  + 128*ge**4*gv**2*tm_tp*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*em_tp*ep_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*tm_tp*em_tp*ep_tm*ep_k**(-1)
     +  + 64*ge**4*gv**2*tm_tp*em_tp*ep_tm*em_k**(-1)

      me2izfg = me2izfg + 64*ge**4*gv**2*tm_tp*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*em_tp*em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*em_tp
     +  + 64*ge**4*gv**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 64*ge**4*gv**2*tm_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*ep_tm
     +  + 64*ge**4*gv**2*tm_tp*em_tm*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*tm_tp*em_tm


      RETURN
      END

      SUBROUTINE CIZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2izaz =
     +  - 32*mt**4*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*ep_k
     +  + 32*mt**4*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*em_k
     +  + 32*mt**4*ge**4*gv**4*Fmt*mz**2*k_tm**(-1)*ep_k
     +  - 32*mt**4*ge**4*gv**4*Fmt*mz**2*k_tm**(-1)*em_k
     +  + 64*mt**4*ge**4*gv**4*Fmt*k_tp**(-1)*ep_k*em_ep
     +  - 64*mt**4*ge**4*gv**4*Fmt*k_tp**(-1)*em_k*em_ep
     +  - 64*mt**4*ge**4*gv**4*Fmt*k_tm**(-1)*ep_k*em_ep
     +  + 64*mt**4*ge**4*gv**4*Fmt*k_tm**(-1)*em_k*em_ep
     +  + 32*mt**4*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*ep_k
     +  - 32*mt**4*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*em_k
     +  - 32*mt**4*ge**4*ga**4*Fmt*mz**2*k_tm**(-1)*ep_k
     +  + 32*mt**4*ge**4*ga**4*Fmt*mz**2*k_tm**(-1)*em_k
     +  - 64*mt**4*ge**4*ga**4*Fmt*k_tp**(-1)*ep_k*em_ep
     +  + 64*mt**4*ge**4*ga**4*Fmt*k_tp**(-1)*em_k*em_ep
     +  + 64*mt**4*ge**4*ga**4*Fmt*k_tm**(-1)*ep_k*em_ep

      me2izaz = me2izaz - 64*mt**4*ge**4*ga**4*Fmt*k_tm**(-1)*em_k*
     + em_ep
     +  + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*em_k
     +  + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tm**(-1)*em_k
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 384*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tp

      me2izaz = me2izaz + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_tm
     +  + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 384*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tp
     +  + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tp*ep_k**(-1)*em_ep

      me2izaz = me2izaz + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tp*
     + em_k**(-1)*em_ep
     +  + 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  - 384*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*ep_tm
     +  - 256*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tm*em_k**(-1)*em_ep
     +  + 384*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*em_tm
     +  - 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tm*ep_k**(-1)*em_ep
     +  + 128*mt**3*ge**4*gv**3*ga*Fet*mz*gzfs*k_tm*em_k**(-1)*em_ep
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp**(-1)*em_k
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tm**(-1)*em_k
     +  - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*
     + ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 384*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tp
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 384*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tp
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tp*
     + em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tp*ep_k**(-1)*em_ep
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tp*em_k**(-1)*em_ep
     +  + 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  - 384*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*ep_tm
     +  - 256*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tm*em_k**(-1)*em_ep
     +  + 384*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*em_tm
     +  - 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tm*ep_k**(-1)*em_ep
     +  + 128*mt**3*ge**4*gv*ga**3*Fet*mz*gzfs*k_tm*em_k**(-1)*em_ep
     +  + 16*mt**2*ge**4*gv**4*Fmt*mz**4*k_tp**(-1)*ep_k
     +  - 16*mt**2*ge**4*gv**4*Fmt*mz**4*k_tp**(-1)*em_k
     +  - 16*mt**2*ge**4*gv**4*Fmt*mz**4*k_tm**(-1)*ep_k
     +  + 16*mt**2*ge**4*gv**4*Fmt*mz**4*k_tm**(-1)*em_k
     +  + 16*mt**2*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*k_tp**(-1)*ep_k
     +  - 16*mt**2*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*k_tp**(-1)*em_k

      me2izaz = me2izaz - 16*mt**2*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k
     +  + 16*mt**2*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*k_tm**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k
     +  + 64*mt**2*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**4*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**4*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*em_tp*ep_k*em_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)

      me2izaz = me2izaz - 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*
     + ep_k*em_ep
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*em_k*em_ep
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*ep_tm*ep_k**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**4*Fmt*mz**2*em_tm*ep_k*em_k**(-1)
     +  + 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tm**(-1)*ep_k*em_ep
     +  - 32*mt**2*ge**4*gv**4*Fmt*mz**2*k_tm**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*Fmt*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  - 128*mt**2*ge**4*gv**4*Fmt*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**4*Fmt*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt**2*ge**4*gv**4*Fmt*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 64*mt**2*ge**4*gv**4*Fmt*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k*em_ep
     +  + 64*mt**2*ge**4*gv**4*Fmt*ep_tp*ep_k**(-1)*em_k*em_ep
     +  - 64*mt**2*ge**4*gv**4*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz - 64*mt**2*ge**4*gv**4*Fmt*em_tp*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**4*Fmt*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 64*mt**2*ge**4*gv**4*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**4*Fmt*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*mt**2*ge**4*gv**4*Fmt*em_tm*ep_k*em_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*ep_k**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*em_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tm
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*em_tm
     +  + 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tm*ep_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*ga**2*Fmt*mz**2*k_tm*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz - 128*mt**2*ge**4*gv**2*ga**2*Fmt*ep_tp*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*em_tp*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tp*em_k**(-1)*em_ep**2
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*ep_tm*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*em_tm*em_ep
     +  - 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tm*ep_k**(-1)*em_ep**2
     +  + 128*mt**2*ge**4*gv**2*ga**2*Fmt*k_tm*em_k**(-1)*em_ep**2
     +  - 16*mt**2*ge**4*ga**4*Fmt*mz**4*k_tp**(-1)*ep_k
     +  + 16*mt**2*ge**4*ga**4*Fmt*mz**4*k_tp**(-1)*em_k

      me2izaz = me2izaz + 16*mt**2*ge**4*ga**4*Fmt*mz**4*k_tm**(-1)*
     + ep_k
     +  - 16*mt**2*ge**4*ga**4*Fmt*mz**4*k_tm**(-1)*em_k
     +  - 16*mt**2*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp**(-1)*ep_k
     +  + 16*mt**2*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp**(-1)*em_k
     +  + 16*mt**2*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tm**(-1)*ep_k
     +  - 16*mt**2*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tm**(-1)*em_k
     +  - 32*mt**2*ge**4*ga**4*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**2*ep_tp
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 32*mt**2*ge**4*ga**4*Fmt*mz**2*em_tp*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*Fmt*mz**2*em_tp
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp*ep_k**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*ep_k*em_ep
     +  - 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*em_k*em_ep
     +  - 32*mt**2*ge**4*ga**4*Fmt*mz**2*ep_tm*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ga**4*Fmt*mz**2*ep_tm
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*em_tm*ep_k*em_k**(-1)
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**2*em_tm
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**2*k_tm*ep_k**(-1)*em_ep
     +  - 64*mt**2*ge**4*ga**4*Fmt*mz**2*k_tm*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tm**(-1)*ep_k*em_ep
     +  + 32*mt**2*ge**4*ga**4*Fmt*mz**2*k_tm**(-1)*em_k*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fmt*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k*em_ep

      me2izaz = me2izaz - 64*mt**2*ge**4*ga**4*Fmt*ep_tp*ep_k**(-1)*
     + em_k*em_ep
     +  - 128*mt**2*ge**4*ga**4*Fmt*ep_tp*em_ep
     +  - 64*mt**2*ge**4*ga**4*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fmt*em_tp*ep_k*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*Fmt*em_tp*em_ep
     +  - 64*mt**2*ge**4*ga**4*Fmt*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 64*mt**2*ge**4*ga**4*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*ga**4*Fmt*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*mt**2*ge**4*ga**4*Fmt*k_tp*em_k**(-1)*em_ep**2
     +  + 64*mt**2*ge**4*ga**4*Fmt*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*ga**4*Fmt*ep_tm*em_ep
     +  - 64*mt**2*ge**4*ga**4*Fmt*em_tm*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz - 128*mt**2*ge**4*ga**4*Fmt*em_tm*em_ep
     +  - 128*mt**2*ge**4*ga**4*Fmt*k_tm*ep_k**(-1)*em_ep**2
     +  + 128*mt**2*ge**4*ga**4*Fmt*k_tm*em_k**(-1)*em_ep**2
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*tm_tp*k_tp**(-1)*ep_k
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*tm_tp*k_tp**(-1)*em_k
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*tm_tp*k_tm**(-1)*ep_k
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*tm_tp*k_tm**(-1)*em_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*em_tm
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*em_tm*k_tm**(-1)
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tp
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*k_tp**(-1)*ep_tm
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*ep_tm*k_tm**(-1)
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp*em_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tp
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tp*ep_k**(-1)*em_ep

      me2izaz = me2izaz + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tp*
     + em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tm*ep_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*ep_tm
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tm*em_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*em_tm
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tm*ep_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzis*k_tm*em_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*tm_tp*k_tp**(-1)*em_k
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*tm_tp*k_tm**(-1)*em_k
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*
     + ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tp
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp*em_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tp
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tp*
     + em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tp*ep_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tp*em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*ep_tm
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tm*em_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*em_tm
     +  + 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tm*ep_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv**3*ga*Fet*mz**3*gzfs*k_tm*em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*em_tm*em_ep
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*em_tm*k_tm**(-1)*
     + em_ep
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*ep_k**(-1)*em_ep**2
     +  + 384*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tp*em_ep
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*k_tp**(-1)*ep_tm*
     + em_ep
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*k_tp**(-1)*em_tm*ep_k
     + *em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*ep_tm
     + *k_tm**(-1)*em_ep
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*em_tm*k_tm**(-1)*ep_k
     + *em_k**(-1)*em_ep
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*em_k**(-1)*em_ep**2
     +  - 384*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tp*em_ep
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tp*em_k**(-1)*em_ep**2
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tm*ep_k**(-1)*em_ep**2
     +  + 384*mt*ge**4*gv**3*ga*Fet*mz*gzis*ep_tm*em_ep
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tm*em_k**(-1)*em_ep**2
     +  - 384*mt*ge**4*gv**3*ga*Fet*mz*gzis*em_tm*em_ep

      me2izaz = me2izaz + 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tm*
     + ep_k**(-1)*em_ep**2
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzis*k_tm*em_k**(-1)*em_ep**2
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp**2*k_tp**(-1)*ep_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp**2*k_tp**(-1)*em_k
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp**2*k_tm**(-1)*ep_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp**2*k_tm**(-1)*em_k
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + em_tm
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*k_tm
     + *ep_k**(-1)*em_k
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp*em_tm*
     + k_tm**(-1)

      me2izaz = me2izaz + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp
     + *ep_k**(-1)*em_ep
     +  - 384*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tp
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + ep_tm
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*k_tm
     + *ep_k*em_k**(-1)
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*ep_tm*
     + k_tm**(-1)
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp*em_k**(-1)*
     + em_ep
     +  + 384*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tp

      me2izaz = me2izaz + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp*
     + ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp*em_tm*k_tm**(-1)
     + *ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tp*em_k**(-1)*em_ep
     +  + 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tm*ep_k**(-1)*
     + em_ep
     +  - 384*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*ep_tm
     +  - 256*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tm*em_k**(-1)*
     + em_ep
     +  + 384*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*em_tm
     +  - 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tm*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv**3*ga*Fet*mz*gzfs*tm_tp*k_tm*em_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*tm_tp*k_tp**(-1)*ep_k
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*tm_tp*k_tp**(-1)*em_k

      me2izaz = me2izaz + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*tm_tp*
     + k_tm**(-1)*ep_k
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*tm_tp*k_tm**(-1)*em_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*em_tm
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*em_tm*k_tm**(-1)
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp*ep_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tp
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*k_tp**(-1)*ep_tm
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*ep_tm*k_tm**(-1)
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp*em_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tp
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tp*ep_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tp*em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tm*ep_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*ep_tm
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tm*em_k**(-1)*em_ep

      me2izaz = me2izaz + 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*em_tm
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tm*ep_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzis*k_tm*em_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*tm_tp*k_tp**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*tm_tp*k_tm**(-1)*em_k
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp*ep_k**(-1)*em_ep

      me2izaz = me2izaz + 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tp
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp*em_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tp
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tp*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tp*
     + em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tm*ep_k**(-1)*em_ep
     +  + 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*ep_tm
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tm*em_k**(-1)*em_ep
     +  - 192*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*em_tm
     +  + 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tm*ep_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv*ga**3*Fet*mz**3*gzfs*k_tm*em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*k_tp**(-1)*em_tm*
     + em_ep

      me2izaz = me2izaz - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*em_tm*k_tm**(-1)*
     + em_ep
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*ep_k**(-1)*em_ep**2
     +  + 384*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tp*em_ep
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*k_tp**(-1)*ep_tm*
     + em_ep
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*k_tp**(-1)*em_tm*ep_k
     + *em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*ep_tm*k_tm**(-1)*
     + em_ep

      me2izaz = me2izaz - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*em_tm
     + *k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*em_k**(-1)*em_ep**2
     +  - 384*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tp*em_ep
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tp*em_k**(-1)*em_ep**2
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tm*ep_k**(-1)*em_ep**2
     +  + 384*mt*ge**4*gv*ga**3*Fet*mz*gzis*ep_tm*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tm*em_k**(-1)*em_ep**2
     +  - 384*mt*ge**4*gv*ga**3*Fet*mz*gzis*em_tm*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tm*ep_k**(-1)*em_ep**2
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzis*k_tm*em_k**(-1)*em_ep**2

      me2izaz = me2izaz + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp**2*
     + k_tp**(-1)*ep_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp**2*k_tp**(-1)*em_k
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp**2*k_tm**(-1)*ep_k
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp**2*k_tm**(-1)*em_k
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + em_tm
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*k_tm
     + *ep_k**(-1)*em_k
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*em_tm*
     + k_tm**(-1)
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp*ep_k**(-1)*
     + em_ep

      me2izaz = me2izaz - 384*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tp
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + ep_tm
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*k_tm
     + *ep_k*em_k**(-1)
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*ep_tm*
     + k_tm**(-1)
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp*em_k**(-1)*
     + em_ep
     +  + 384*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tp
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp*ep_tm*k_tm**(-1)
     + *ep_k**(-1)*em_k

      me2izaz = me2izaz - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp*
     + em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tp*em_k**(-1)*em_ep
     +  + 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tm*ep_k**(-1)*
     + em_ep
     +  - 384*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*ep_tm
     +  - 256*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tm*em_k**(-1)*
     + em_ep
     +  + 384*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*em_tm
     +  - 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tm*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga**3*Fet*mz*gzfs*tm_tp*k_tm*em_k**(-1)*em_ep
     +  + 16*ge**4*gv**4*Fmt*mz**4*tm_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*gv**4*Fmt*mz**4*tm_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt*mz**4*tm_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*gv**4*Fmt*mz**4*tm_tp*k_tm**(-1)*em_k

      me2izaz = me2izaz + 16*ge**4*gv**4*Fmt*mz**4*ep_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt*mz**4*ep_tp*ep_k**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt*mz**4*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**4*em_tp*ep_k*em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**4*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  + 16*ge**4*gv**4*Fmt*mz**4*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**4*ep_tm*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt*mz**4*em_tm*ep_k*em_k**(-1)
     +  + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*em_k

      me2izaz = me2izaz + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*ep_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*ep_tp*ep_k**(-1)*em_k
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*k_tm*ep_k
     + *em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*em_tp*ep_k*em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*k_tp*em_tm*k_tm**(-1)*ep_k
     + *em_k**(-1)
     +  - 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*ep_tm*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**4*Fmt*mz**2*gzis*gzfs*em_tm*ep_k*em_k**(-1)
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp**2*k_tp**(-1)*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp**2*k_tp**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp**2*k_tm**(-1)*ep_k
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp**2*k_tm**(-1)*em_k

      me2izaz = me2izaz - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*ep_tp*ep_k**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*em_tp*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*ep_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*em_tm*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  - 32*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k*em_ep

      me2izaz = me2izaz - 32*ge**4*gv**4*Fmt*mz**2*ep_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k*em_ep
     +  - 32*ge**4*gv**4*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*em_tp*ep_k*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  - 32*ge**4*gv**4*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*ge**4*gv**4*Fmt*mz**2*ep_tm*ep_k**(-1)*em_k*em_ep
     +  - 32*ge**4*gv**4*Fmt*mz**2*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**4*Fmt*tm_tp**2*k_tp**(-1)*ep_k*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp**2*k_tp**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp**2*k_tm**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**4*Fmt*tm_tp**2*k_tm**(-1)*em_k*em_ep

      me2izaz = me2izaz + 64*ge**4*gv**4*Fmt*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**4*Fmt*tm_tp*ep_tp*ep_k**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp*em_tp*ep_k*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 64*ge**4*gv**4*Fmt*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**4*Fmt*tm_tp*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**4*Fmt*tm_tp*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*tm_tp*k_tp**(-1)*ep_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*tm_tp*k_tp**(-1)*em_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*tm_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*tm_tp*k_tm**(-1)*em_k

      me2izaz = me2izaz + 32*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tp
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*em_tp
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp*ep_k**(-1)*em_ep
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp*em_k**(-1)*em_ep
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tm
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*em_tm
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tm*ep_k**(-1)*em_ep
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**4*k_tm*em_k**(-1)*em_ep

      me2izaz = me2izaz + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*
     + tm_tp*k_tp**(-1)*ep_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*tm_tp*k_tp**(-1)*
     + em_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*
     + ep_k
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*
     + em_k
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*ep_tp
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*em_tp
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k

      me2izaz = me2izaz + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tp
     + *em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tp*ep_k**(-1)*
     + em_ep
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tp*em_k**(-1)*
     + em_ep
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*ep_tm
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*em_tm
     +  - 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tm*ep_k**(-1)*
     + em_ep
     +  + 32*ge**4*gv**2*ga**2*Fmt*mz**2*gzis*gzfs*k_tm*em_k**(-1)*
     + em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp**2*k_tp**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp**2*k_tp**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp**2*k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp**2*k_tm**(-1)*em_k

      me2izaz = me2izaz - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*ep_tp
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*em_tp
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*ep_tm
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*em_tm

      me2izaz = me2izaz + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm*
     + ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp*
     + em_k**(-1)*em_ep**2
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tm*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*em_tm*em_ep
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tm*ep_k**(-1)*em_ep**2
     +  - 64*ge**4*gv**2*ga**2*Fmt*mz**2*k_tm*em_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp**2*k_tp**(-1)*ep_k*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp**2*k_tp**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp**2*k_tm**(-1)*ep_k*em_ep
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp**2*k_tm**(-1)*em_k*em_ep
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*ep_tp*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*em_tp*em_ep

      me2izaz = me2izaz - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp*em_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*ep_tm*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*em_tm*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tm*ep_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tm*em_k**(-1)*em_ep**2
     +  + 16*ge**4*ga**4*Fmt*mz**4*tm_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*ga**4*Fmt*mz**4*tm_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*ga**4*Fmt*mz**4*tm_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*ga**4*Fmt*mz**4*tm_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt*mz**4*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k

      me2izaz = me2izaz - 16*ge**4*ga**4*Fmt*mz**4*ep_tp*ep_k**(-1)*
     + em_k
     +  - 32*ge**4*ga**4*Fmt*mz**4*ep_tp
     +  - 16*ge**4*ga**4*Fmt*mz**4*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 16*ge**4*ga**4*Fmt*mz**4*em_tp*ep_k*em_k**(-1)
     +  + 32*ge**4*ga**4*Fmt*mz**4*em_tp
     +  - 16*ge**4*ga**4*Fmt*mz**4*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  + 16*ge**4*ga**4*Fmt*mz**4*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 32*ge**4*ga**4*Fmt*mz**4*k_tp*ep_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**4*k_tp*em_k**(-1)*em_ep
     +  + 16*ge**4*ga**4*Fmt*mz**4*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*ep_tm
     +  - 16*ge**4*ga**4*Fmt*mz**4*em_tm*ep_k*em_k**(-1)

      me2izaz = me2izaz - 32*ge**4*ga**4*Fmt*mz**4*em_tm
     +  - 32*ge**4*ga**4*Fmt*mz**4*k_tm*ep_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**4*k_tm*em_k**(-1)*em_ep
     +  + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*tm_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*ep_tp*ep_k**(-1)*em_k
     +  - 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*ep_tp
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*em_tp*k_tp**(-1)*k_tm*ep_k
     + *em_k**(-1)
     +  + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*em_tp*ep_k*em_k**(-1)
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*em_tp
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp*em_tm
     + *k_tm**(-1)*ep_k*em_k**(-1)
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp*ep_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tp*em_k**(-1)*em_ep
     +  + 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*ep_tm*ep_k**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*ep_tm
     +  - 16*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*em_tm*ep_k*em_k**(-1)
     +  - 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*em_tm
     +  - 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tm*ep_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzis*gzfs*k_tm*em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp**2*k_tp**(-1)*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp**2*k_tp**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp**2*k_tm**(-1)*ep_k
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp**2*k_tm**(-1)*em_k
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*ep_tp*
     + ep_k**(-1)*em_k
     +  + 64*ge**4*ga**4*Fmt*mz**2*tm_tp*ep_tp
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*em_tp*ep_k*em_k**(-1)
     +  - 64*ge**4*ga**4*Fmt*mz**2*tm_tp*em_tp
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp*ep_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp*em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*ep_tm*ep_k**(-1)*em_k

      me2izaz = me2izaz - 64*ge**4*ga**4*Fmt*mz**2*tm_tp*ep_tm
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*em_tm*ep_k*em_k**(-1)
     +  + 64*ge**4*ga**4*Fmt*mz**2*tm_tp*em_tm
     +  + 64*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tm*ep_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tm*em_k**(-1)*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k*em_ep
     +  + 64*ge**4*ga**4*Fmt*mz**2*ep_tp*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 32*ge**4*ga**4*Fmt*mz**2*em_tp*ep_k*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*Fmt*mz**2*em_tp*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep

      me2izaz = me2izaz - 32*ge**4*ga**4*Fmt*mz**2*k_tp*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 64*ge**4*ga**4*Fmt*mz**2*k_tp*ep_k**(-1)*em_ep**2
     +  + 64*ge**4*ga**4*Fmt*mz**2*k_tp*em_k**(-1)*em_ep**2
     +  - 32*ge**4*ga**4*Fmt*mz**2*ep_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt*mz**2*ep_tm*em_ep
     +  + 32*ge**4*ga**4*Fmt*mz**2*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*Fmt*mz**2*em_tm*em_ep
     +  + 64*ge**4*ga**4*Fmt*mz**2*k_tm*ep_k**(-1)*em_ep**2
     +  - 64*ge**4*ga**4*Fmt*mz**2*k_tm*em_k**(-1)*em_ep**2
     +  + 64*ge**4*ga**4*Fmt*tm_tp**2*k_tp**(-1)*ep_k*em_ep
     +  - 64*ge**4*ga**4*Fmt*tm_tp**2*k_tp**(-1)*em_k*em_ep
     +  - 64*ge**4*ga**4*Fmt*tm_tp**2*k_tm**(-1)*ep_k*em_ep
     +  + 64*ge**4*ga**4*Fmt*tm_tp**2*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*ga**4*Fmt*tm_tp*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k*em_ep

      me2izaz = me2izaz - 64*ge**4*ga**4*Fmt*tm_tp*ep_tp*ep_k**(-1)*
     + em_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*tm_tp*ep_tp*em_ep
     +  - 64*ge**4*ga**4*Fmt*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 64*ge**4*ga**4*Fmt*tm_tp*em_tp*ep_k*em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*Fmt*tm_tp*em_tp*em_ep
     +  - 64*ge**4*ga**4*Fmt*tm_tp*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 64*ge**4*ga**4*Fmt*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 128*ge**4*ga**4*Fmt*tm_tp*k_tp*ep_k**(-1)*em_ep**2
     +  - 128*ge**4*ga**4*Fmt*tm_tp*k_tp*em_k**(-1)*em_ep**2
     +  + 64*ge**4*ga**4*Fmt*tm_tp*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 128*ge**4*ga**4*Fmt*tm_tp*ep_tm*em_ep
     +  - 64*ge**4*ga**4*Fmt*tm_tp*em_tm*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz - 128*ge**4*ga**4*Fmt*tm_tp*em_tm*em_ep
     +  - 128*ge**4*ga**4*Fmt*tm_tp*k_tm*ep_k**(-1)*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*tm_tp*k_tm*em_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_tm*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**2*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tp**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*ep_tp*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*k_tp*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)
     + *ep_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_tm*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**2*em_k**(-1)*em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  + 176*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)
     + *ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_ep**3
     +  + 176*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_k**(-1)*em_ep**3
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_ep**2

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tm**(-1)*
     + em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*k_tm**(-1)*
     + em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-2)*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*ep_k**(-1)*
     + em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-2)*em_k**(-1)*
     + em_ep**2
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*ep_k
     + *em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp
     + *k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp*k_tm**(-1)

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*ep_k
     + *em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + k_tp**(-1)*em_tm
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*k_tm*ep_k
     + *em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*k_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_k**2*
     + em_k**(-1)
     +  - 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 88*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*ep_k**(-1)
     + *em_k**2
     +  - 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*ep_k**2*
     + em_k**(-1)
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*ep_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k**2

      me2izaz = me2izaz - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*k_tm**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)*ep_tm*ep_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)*em_tm*em_k**(-1)

      me2izaz = me2izaz + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**2*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tp**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**2*
     + ep_k**(-1)

      me2izaz = me2izaz - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**2*em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_tp
     + *k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_tp
     + *k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_tp
     + *k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + ep_tp*k_tp**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_tp
     + *k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*em_tp
     + *k_tp**(-1)*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*em_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*em_tp
     + *k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + em_tp*k_tp**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*em_tp
     + *k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*k_tp
     + *k_tm**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_tm*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*k_tm*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  + 352*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_k*em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  + 352*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  - 512*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tp**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_tm
     + *k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*em_tm
     + *k_tm**(-1)*ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*k_tm**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tm**(-1)*ep_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tm**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tm**(-1)*em_k*em_ep
     +  + 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + k_tm**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*ep_k*
     + em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-2)*
     + ep_k**(-1)*em_ep**2

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-2)*em_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tp*
     + k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp*
     + k_tm**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_tm
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_tm
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + k_tm
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 192*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  - 176*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 192*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 176*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_k
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k**2*em_k**(-1)

      me2izaz = me2izaz - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + em_k
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*ep_k**(-1)*
     + em_k

      me2izaz = me2izaz + 544*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*ep_k**(-1)*em_ep
     +  + 544*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*em_k**(-1)*
     + em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_tm*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**2*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tp**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*ep_k**(-1)
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**2*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*em_tp*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep**2

      me2izaz = me2izaz + 160*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*k_tp**(-1)*ep_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_tm*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*k_tp**(-1)*k_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep**2
     +  + 176*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)
     + *ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_ep**3
     +  + 176*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_k*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*k_tp**(-1)*em_k**(-1)*em_ep**3
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tp**(-1)
     + *em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*k_tm**(-1)*
     + em_k*em_ep

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-2)*k_tm**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_k**(-1)*
     + em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*ep_k**(-1)*
     + em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-2)*em_k**(-1)*
     + em_ep**2
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*ep_k
     + *em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*ep_k
     + *em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp
     + *k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp*k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*ep_k
     + *em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*ep_k
     + *em_k**(-1)

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*em_tm
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*k_tm*ep_k
     + *em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*k_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_k**2*
     + em_k**(-1)
     +  - 96*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz - 88*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + k_tp**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*ep_k**(-1)
     + *em_k**2
     +  - 96*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*ep_k**2*
     + em_k**(-1)

      me2izaz = me2izaz - 128*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*ep_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*k_tm**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_k**(-1)*em_k
     +  + 528*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*ep_k**(-1)*em_ep
     +  + 528*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*ep_tp*k_tp**(-1)*
     + ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*em_tp*k_tp**(-1)*
     + em_k**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*
     + k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**4*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + gzis*gzfs*ep_tp*k_tp**(-1)*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + gzis*gzfs*k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + tm_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tp**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + k_tp*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp*k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*em_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*k_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_k**2
     + *em_k**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz + 44*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + k_tp**(-1)*ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 44*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*em_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*ep_k**2
     + *em_k**(-1)

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*k_tm**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**2*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2

      me2izaz = me2izaz + 160*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*k_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  + 176*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3

      me2izaz = me2izaz + 176*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k**2*em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-2)*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-2)*tm_tp*
     + em_k**(-1)*em_ep**2
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep

      me2izaz = me2izaz + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*em_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + k_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  + 44*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 44*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*k_tp**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k**(-1)*em_k**2

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*
     + gzfs*k_tm**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*k_tm**(-1)*
     + em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_k*
     + em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_k**(-1)*
     + em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*ep_k**(-1)*
     + em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*gzis*gzfs*em_k**(-1)*
     + em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tp*k_tp**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*
     + em_tp*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp*k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_tm

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*em_tm
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*k_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_k**2
     + *em_k**(-1)

      me2izaz = me2izaz - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*
     + k_tm**(-1)*ep_k**2*em_k**(-1)
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*ep_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*k_tm**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*tm_tp*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*k_tp**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*
     + k_tp**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp*k_tm**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + k_tp**(-1)*ep_tm*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep**2
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*k_tm*em_ep

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k**2*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_ep**3
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_k**(-1)*
     + em_ep**3
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tp**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tm**(-1)*ep_k**2*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tm**(-1)*ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k**2*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tm**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*k_tm**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*em_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*em_tp*
     + k_tp**(-1)*em_k**(-1)

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**4*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tp**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**4*ep_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**4*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + ep_tp*k_tp**(-1)*ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + em_tp*k_tp**(-1)*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*gzis*gzfs*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + ep_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*gzis*gzfs*
     + em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*ep_tp
     + *k_tp**(-1)*ep_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*em_tp
     + *k_tp**(-1)*em_k**(-1)

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*tm_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tp**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)

      me2izaz = me2izaz + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*tm_tp*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + ep_k**(-1)
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*tm_tp*
     + em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tp*
     + k_tp**(-1)

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*ep_tp*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tp**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*k_tp*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp*
     + k_tm**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_tm
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + k_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**2*em_k**(-1)

      me2izaz = me2izaz + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 88*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 88*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_k
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*em_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)*
     + ep_k**2*em_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)
     + *ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)*
     + ep_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)
     + *ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)*
     + em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*k_tm**(-1)*
     + em_ep

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**2*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_k**(-1)*
     + em_k
     +  - 272*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*ep_k**(-1)
     + *em_ep
     +  - 272*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**2*em_k**(-1)
     + *em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2

      me2izaz = me2izaz + 320*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tp*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tp**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tp*k_tm**(-1)*ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*k_tp*k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp*k_tm**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  + 320*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  + 352*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  + 352*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**3

      me2izaz = me2izaz - 512*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_tm*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_k*em_ep
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_ep**2

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz**(-2)*tm_tp*ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-2)*tm_tp*
     + em_k**(-1)*em_ep**2
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep

      me2izaz = me2izaz + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*ep_tp*k_tp**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*k_tp*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_tm

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 80*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_tm
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*k_tm

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 88*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 88*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*k_tp**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*ep_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tm**(-1)*ep_k**2*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tm**(-1)*ep_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tm**(-1)*ep_k**(-1)*em_k**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tm**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + k_tm**(-1)*em_ep

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + gzis*gzfs*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*gzis*gzfs*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep

      me2izaz = me2izaz - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*ep_tp*k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*k_tp*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp*
     + k_tm**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *ep_tm

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *em_tm*em_k**(-1)*em_ep
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *em_tm
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + k_tm

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 192*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *ep_k*em_k**(-1)*em_ep
     +  - 176*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *ep_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 192*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *ep_k**(-1)*em_k*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 176*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)
     + *em_k
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)
     + *ep_k*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)*
     + ep_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)
     + *ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*k_tm**(-1)*
     + em_k

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + tm_tp*k_tm**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_k**(-1)*
     + em_k
     +  + 544*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*ep_k**(-1)
     + *em_ep
     +  + 544*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*tm_tp*em_k**(-1)
     + *em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tp*k_tp**(-1)
     + *em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tp*k_tp**(-1)
     + *em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + em_tp*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp*k_tm**(-1)*
     + em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + k_tp**(-1)*ep_tm*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep**2
     +  - 160*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_tm
     + *em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + k_tp**(-1)*k_tm*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep**2
     +  - 176*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*ep_k*
     + em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_ep**3
     +  - 176*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*em_k*
     + em_ep

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  + 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tp**(-1)*
     + em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*ep_k*
     + em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*k_tm**(-1)*em_k*
     + em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + k_tm**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_k*em_k**(-1)*
     + em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_k**(-1)*em_k*
     + em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*ep_k**(-1)*
     + em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*em_k**(-1)*
     + em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*ep_tp*k_tp**(-1)*
     + ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*em_tp*k_tp**(-1)*
     + em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)*ep_tm*
     + ep_k**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*ep_k**(-1)

      me2izaz = me2izaz + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**4*
     + em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + gzis*gzfs*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + ep_k**(-1)
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*gzis*gzfs*
     + em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*em_tp*
     + k_tp**(-1)*em_k**(-1)

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + tm_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tp**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k

      me2izaz = me2izaz - 256*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + tm_tp*ep_k**(-1)
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*tm_tp*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp*k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*em_tm

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*k_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_k**2
     + *em_k**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 44*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 44*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + k_tp**(-1)*em_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*ep_k**2
     + *em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*
     + k_tm**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*k_tm**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_k**(-1)*em_k
     +  - 264*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*ep_k**(-1)*em_ep
     +  - 264*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**2*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*ep_tp*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2

      me2izaz = me2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*k_tp*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  + 160*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep

      me2izaz = me2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  + 176*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  + 176*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_ep**2

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*ep_k*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-2)*tm_tp*ep_k**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-2)*tm_tp*
     + em_k**(-1)*em_ep**2
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_tp*
     + k_tp**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*ep_tp*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*k_tp*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp*
     + k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)

      me2izaz = me2izaz + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + k_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**2*em_k**(-1)

      me2izaz = me2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 44*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 44*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tp**(-1)*
     + em_ep

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tm**(-1)*
     + em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*k_tm**(-1)*
     + em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_k*
     + em_k**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*
     + gzfs*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*ep_k**(-1)*
     + em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*gzis*gzfs*em_k**(-1)*
     + em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tp*k_tm**(-1)*
     + ep_k*em_k**(-1)

      me2izaz = me2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*
     + em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tp*k_tp**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp*k_tm**(-1)

      me2izaz = me2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*
     + k_tp**(-1)*em_tm
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*k_tm*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*k_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_k**2
     + *em_k**(-1)
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*ep_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k**2

      me2izaz = me2izaz - 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*k_tm**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_k**(-1)*em_k
     +  + 528*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*ep_k**(-1)*em_ep
     +  + 528*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*tm_tp*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2

      me2izaz = me2izaz - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*
     + k_tp**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tp**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep**2

      me2izaz = me2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp*k_tm**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*ep_k*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep**2
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_tm*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*
     + em_k**(-1)*em_ep**2
     +  - 80*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*k_tm*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_k**2*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep**2
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k**2*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_ep**3
     +  - 88*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_k**(-1)*
     + em_ep**3
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tp**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tm**(-1)*ep_k**2*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tm**(-1)*ep_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k**2*em_ep

      me2izaz = me2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + k_tm**(-1)*em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*k_tm**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*em_k**(-1)*em_ep**2


      RETURN
      END

      SUBROUTINE CIZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2izag =
     +  + 32*mt**4*ge**4*gv**2*Fmt*k_tp**(-1)*ep_k
     +  - 32*mt**4*ge**4*gv**2*Fmt*k_tp**(-1)*em_k
     +  - 32*mt**4*ge**4*gv**2*Fmt*k_tm**(-1)*ep_k
     +  + 32*mt**4*ge**4*gv**2*Fmt*k_tm**(-1)*em_k
     +  - 32*mt**3*ge**4*gv*ga*Fet*mz*gzis*k_tp**(-1)*ep_k
     +  + 32*mt**3*ge**4*gv*ga*Fet*mz*gzis*k_tp**(-1)*em_k
     +  - 32*mt**3*ge**4*gv*ga*Fet*mz*gzis*k_tm**(-1)*ep_k
     +  + 32*mt**3*ge**4*gv*ga*Fet*mz*gzis*k_tm**(-1)*em_k
     +  - 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*ep_k
     +  + 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*em_k
     +  + 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tm**(-1)*ep_k
     +  - 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tm**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*ep_k
     +  - 64*mt**2*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*Fmt*tm_tp*k_tm**(-1)*ep_k

      me2izag = me2izag + 64*mt**2*ge**4*gv**2*Fmt*tm_tp*k_tm**(-1)*
     + em_k
     +  + 32*mt**2*ge**4*gv**2*Fmt*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k
     +  + 32*mt**2*ge**4*gv**2*Fmt*ep_tp*ep_k**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**2*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 32*mt**2*ge**4*gv**2*Fmt*em_tp*ep_k*em_k**(-1)
     +  - 32*mt**2*ge**4*gv**2*Fmt*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  + 32*mt**2*ge**4*gv**2*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*mt**2*ge**4*gv**2*Fmt*ep_tm*ep_k**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**2*Fmt*em_tm*ep_k*em_k**(-1)
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzis*tm_tp*k_tp**(-1)*ep_k
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzis*tm_tp*k_tp**(-1)*em_k

      me2izag = me2izag - 32*mt*ge**4*gv*ga*Fet*mz*gzis*tm_tp*
     + k_tm**(-1)*ep_k
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzis*tm_tp*k_tm**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*k_tp**(-1)*em_tm
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*em_tm*k_tm**(-1)
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tp
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*k_tp**(-1)*ep_tm
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)

      me2izag = me2izag + 32*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*ep_tm*k_tm**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*ep_k*em_k**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp*em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv*ga*Fet*mz*gzis*em_tp
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzis*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzis*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tm*ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tm*ep_k**(-1)*em_ep
     +  + 128*mt*ge**4*gv*ga*Fet*mz*gzis*ep_tm
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzis*em_tm*ep_k*em_k**(-1)

      me2izag = me2izag + 64*mt*ge**4*gv*ga*Fet*mz*gzis*em_tm*
     + em_k**(-1)*em_ep
     +  - 128*mt*ge**4*gv*ga*Fet*mz*gzis*em_tm
     +  - 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 16*ge**4*gv**2*Fmt*mz**2*em_tp*ep_k*em_k**(-1)
     +  + 16*ge**4*gv**2*Fmt*mz**2*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)

      me2izag = me2izag + 16*ge**4*gv**2*Fmt*mz**2*ep_tm*ep_k**(-1)*
     + em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*em_tm*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*Fmt*tm_tp**2*k_tp**(-1)*ep_k
     +  - 32*ge**4*gv**2*Fmt*tm_tp**2*k_tp**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*tm_tp**2*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**2*Fmt*tm_tp**2*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**2*Fmt*tm_tp*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k
     +  + 32*ge**4*gv**2*Fmt*tm_tp*ep_tp*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*tm_tp*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 32*ge**4*gv**2*Fmt*tm_tp*em_tp*ep_k*em_k**(-1)
     +  - 32*ge**4*gv**2*Fmt*tm_tp*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*
     + em_k
     +  + 32*ge**4*gv**2*Fmt*tm_tp*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)

      me2izag = me2izag - 32*ge**4*gv**2*Fmt*tm_tp*ep_tm*ep_k**(-1)*
     + em_k
     +  + 32*ge**4*gv**2*Fmt*tm_tp*em_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep

      me2izag = me2izag - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*ep_tp*k_tm**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep

      me2izag = me2izag - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*em_tp*k_tm**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm

      me2izag = me2izag - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + k_tm
     +  - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**2*em_k**(-1)

      me2izag = me2izag + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k
     +  - 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 40*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_ep

      me2izag = me2izag + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)

      me2izag = me2izag + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*k_tm**(-1)*ep_k**2*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k*em_k**(-1)*em_ep
     +  - 56*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 56*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + em_k**(-1)*em_ep**2

      me2izag = me2izag + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + mz**(-2)*k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_k*
     + em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_k**(-1)*
     + em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*ep_k**(-1)*
     + em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*mz**(-2)*em_k**(-1)*
     + em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)

      me2izag = me2izag - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tp**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*Fet*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)

      me2izag = me2izag + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*
     + em_tp*k_tp**(-1)*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tm**(-1)*ep_k*
     + em_k**(-1)

      me2izag = me2izag + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep

      me2izag = me2izag - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*tm_tp*ep_tp*k_tm**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tp**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep

      me2izag = me2izag - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*tm_tp*em_tp*k_tm**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*k_tp*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_tm

      me2izag = me2izag - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_tm
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*k_tm
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**2*em_k**(-1)

      me2izag = me2izag + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*tm_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tp**(-1)*em_ep

      me2izag = me2izag + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)
     + *tm_tp*ep_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_tm*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*em_tm*
     + k_tm**(-1)

      me2izag = me2izag + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)
     + *tm_tp*k_tm**(-1)*ep_k**2*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2

      me2izag = me2izag + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*tm_tp*k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*ep_k*
     + em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*tm_tp*
     + em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*ep_tp*k_tp**(-1)*
     + ep_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*em_tp*k_tp**(-1)*
     + em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)

      me2izag = me2izag - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tp**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*tm_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)

      me2izag = me2izag + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k

      me2izag = me2izag + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp*k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*ep_k*
     + em_k**(-1)

      me2izag = me2izag + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k

      me2izag = me2izag + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tp**(-1)*k_tm
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**2*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 20*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k**2
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 20*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_k**(-1)*
     + em_ep**2

      me2izag = me2izag + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tp**(-1)*em_ep
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)

      me2izag = me2izag - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tm**(-1)*ep_k**2*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  + 28*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k**2
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  + 28*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_k**(-1)*
     + em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_k*em_k**(-1)

      me2izag = me2izag - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_k**(-1)*em_ep


      RETURN
      END

      SUBROUTINE CIGFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2igfz =
     +  + 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 64*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*em_tp*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*ep_tm
     +  - 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*em_tm
     +  - 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*ep_k
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*em_k
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*gv**2*mz**2*ep_tm*k_tm**(-1)

      me2igfz = me2igfz + 32*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)*
     + ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*gv**2*mz**2*em_tm*k_tm**(-1)
     +  + 32*mt**2*ge**4*gv**2*mz**2*k_tm**(-1)*ep_k
     +  - 32*mt**2*ge**4*gv**2*mz**2*k_tm**(-1)*em_k
     +  - 64*mt**2*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 128*mt**2*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  - 64*mt**2*ge**4*gv**2*ep_tp*k_tp**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**2*ep_tp*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**2*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 64*mt**2*ge**4*gv**2*em_tp*k_tp**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*em_tp*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*k_tp**(-1)*ep_tm*em_ep
     +  + 64*mt**2*ge**4*gv**2*k_tp**(-1)*em_tm*em_ep

      me2igfz = me2igfz + 64*mt**2*ge**4*gv**2*k_tp**(-1)*ep_k*em_ep
     +  - 64*mt**2*ge**4*gv**2*k_tp**(-1)*em_k*em_ep
     +  + 64*mt**2*ge**4*gv**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  + 64*mt**2*ge**4*gv**2*ep_tm*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*em_tm*k_tm**(-1)*em_k**(-1)*em_ep**2
     +  - 64*mt**2*ge**4*gv**2*em_tm*k_tm**(-1)*em_ep
     +  - 64*mt**2*ge**4*gv**2*k_tm**(-1)*ep_k*em_ep
     +  + 64*mt**2*ge**4*gv**2*k_tm**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**2*mz**2*tm_tp*ep_tp*k_tp**(-1)
     +  - 32*ge**4*gv**2*mz**2*tm_tp*em_tp*k_tp**(-1)
     +  - 32*ge**4*gv**2*mz**2*tm_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*gv**2*mz**2*tm_tp*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)

      me2igfz = me2igfz + 64*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + em_tm*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 32*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  + 64*ge**4*gv**2*mz**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 64*ge**4*gv**2*mz**2*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*em_tm*ep_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tp*em_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*ep_tp*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*ep_tp
     +  + 64*ge**4*gv**2*mz**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*ep_tm

      me2igfz = me2igfz + 32*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*em_tm*
     + ep_k*em_k**(-1)
     +  + 64*ge**4*gv**2*mz**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  - 64*ge**4*gv**2*mz**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*ep_tm*ep_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*ep_tm*em_k**(-1)
     +  - 32*ge**4*gv**2*mz**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*em_tp*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*em_tp
     +  - 32*ge**4*gv**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*gv**2*mz**2*ep_tm*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*mz**2*ep_tm
     +  - 32*ge**4*gv**2*mz**2*em_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*mz**2*em_tm

      me2igfz = me2igfz - 64*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*em_ep
     +  + 64*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*em_ep
     +  + 64*ge**4*gv**2*tm_tp*ep_tm*k_tm**(-1)*em_ep
     +  - 64*ge**4*gv**2*tm_tp*em_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  - 128*ge**4*gv**2*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 128*ge**4*gv**2*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  - 64*ge**4*gv**2*ep_tp*em_tm*ep_k**(-1)*em_ep

      me2igfz = me2igfz - 64*ge**4*gv**2*ep_tp*em_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*ep_tp*ep_k**(-1)*em_ep**2
     +  - 128*ge**4*gv**2*ep_tp*em_ep
     +  - 128*ge**4*gv**2*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  - 64*ge**4*gv**2*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep
     +  - 128*ge**4*gv**2*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 128*ge**4*gv**2*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 64*ge**4*gv**2*em_tp*ep_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*em_tp*ep_tm*em_k**(-1)*em_ep
     +  + 64*ge**4*gv**2*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*em_tp*em_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**2*em_tp*em_ep
     +  + 64*ge**4*gv**2*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  - 64*ge**4*gv**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)*em_ep

      me2igfz = me2igfz - 64*ge**4*gv**2*ep_tm*ep_k**(-1)*em_ep**2
     +  + 128*ge**4*gv**2*ep_tm*em_ep
     +  + 64*ge**4*gv**2*em_tm*em_k**(-1)*em_ep**2
     +  - 128*ge**4*gv**2*em_tm*em_ep


      RETURN
      END

      SUBROUTINE CIGFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2igfg =
     +  - 32*mt**2*ge**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 64*mt**2*ge**4*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*ep_tp*k_tp**(-1)
     +  + 32*mt**2*ge**4*ep_tp*k_tm**(-1)
     +  + 32*mt**2*ge**4*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 64*mt**2*ge**4*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*em_tp*k_tp**(-1)
     +  - 32*mt**2*ge**4*em_tp*k_tm**(-1)
     +  - 32*mt**2*ge**4*k_tp**(-1)*ep_tm
     +  + 32*mt**2*ge**4*k_tp**(-1)*em_tm
     +  + 32*mt**2*ge**4*k_tp**(-1)*ep_k
     +  - 32*mt**2*ge**4*k_tp**(-1)*em_k
     +  + 32*mt**2*ge**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  - 64*mt**2*ge**4*ep_tm*k_tm**(-1)*ep_k**(-1)*em_ep
     +  + 32*mt**2*ge**4*ep_tm*k_tm**(-1)

      me2igfg = me2igfg - 32*mt**2*ge**4*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 64*mt**2*ge**4*em_tm*k_tm**(-1)*em_k**(-1)*em_ep
     +  - 32*mt**2*ge**4*em_tm*k_tm**(-1)
     +  - 32*mt**2*ge**4*k_tm**(-1)*ep_k
     +  + 32*mt**2*ge**4*k_tm**(-1)*em_k
     +  - 32*ge**4*tm_tp*ep_tp*k_tp**(-1)
     +  + 32*ge**4*tm_tp*em_tp*k_tp**(-1)
     +  + 32*ge**4*tm_tp*ep_tm*k_tm**(-1)
     +  - 32*ge**4*tm_tp*em_tm*k_tm**(-1)
     +  + 64*ge**4*ep_tp**2*k_tp**(-1)*em_tm*ep_k**(-1)
     +  + 64*ge**4*ep_tp*em_tp*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 64*ge**4*ep_tp*em_tp*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*ge**4*ep_tp*em_tp*k_tp**(-1)*k_tm*ep_k**(-1)
     +  + 32*ge**4*ep_tp*em_tp*k_tp**(-1)*k_tm*em_k**(-1)
     +  + 32*ge**4*ep_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k

      me2igfg = me2igfg + 32*ge**4*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*ep_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*ep_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 64*ge**4*ep_tp*em_tm**2*k_tm**(-1)*em_k**(-1)
     +  + 32*ge**4*ep_tp*em_tm*k_tm**(-1)
     +  - 32*ge**4*ep_tp*em_tm*ep_k**(-1)
     +  - 32*ge**4*ep_tp*em_tm*em_k**(-1)
     +  + 32*ge**4*ep_tp*ep_k**(-1)*em_ep
     +  - 64*ge**4*ep_tp
     +  - 64*ge**4*em_tp**2*k_tp**(-1)*ep_tm*em_k**(-1)
     +  - 32*ge**4*em_tp*k_tp**(-1)*ep_tm
     +  - 32*ge**4*em_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 64*ge**4*em_tp*ep_tm**2*k_tm**(-1)*ep_k**(-1)
     +  + 64*ge**4*em_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*em_tp*ep_tm*k_tm**(-1)
     +  + 32*ge**4*em_tp*ep_tm*ep_k**(-1)

      me2igfg = me2igfg + 32*ge**4*em_tp*ep_tm*em_k**(-1)
     +  + 32*ge**4*em_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*ge**4*em_tp*em_k**(-1)*em_ep
     +  + 64*ge**4*em_tp
     +  + 32*ge**4*k_tp*ep_tm*em_tm*k_tm**(-1)*ep_k**(-1)
     +  - 32*ge**4*k_tp*ep_tm*em_tm*k_tm**(-1)*em_k**(-1)
     +  - 32*ge**4*ep_tm*ep_k**(-1)*em_ep
     +  + 64*ge**4*ep_tm
     +  + 32*ge**4*em_tm*em_k**(-1)*em_ep
     +  - 64*ge**4*em_tm


      RETURN
      END

      SUBROUTINE CIGAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2igaz =
     +  - 32*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*ep_k
     +  + 32*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*em_k
     +  - 32*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tm**(-1)*ep_k
     +  + 32*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tm**(-1)*em_k
     +  - 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*ep_k
     +  + 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*em_k
     +  + 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tm**(-1)*ep_k
     +  - 16*mt**2*ge**4*gv**2*Fmt*mz**2*k_tm**(-1)*em_k
     +  + 32*mt**2*ge**4*gv**2*Fmt*k_tp**(-1)*ep_k*em_ep
     +  - 32*mt**2*ge**4*gv**2*Fmt*k_tp**(-1)*em_k*em_ep
     +  - 32*mt**2*ge**4*gv**2*Fmt*k_tm**(-1)*ep_k*em_ep
     +  + 32*mt**2*ge**4*gv**2*Fmt*k_tm**(-1)*em_k*em_ep
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*ep_k
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*em_k
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tm**(-1)*ep_k

      me2igaz = me2igaz - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*
     + k_tm**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*k_tm*
     + ep_k**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*ep_k**(-1)*em_ep
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)

      me2igaz = me2igaz - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*em_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp*ep_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp*em_k**(-1)*em_ep
     +  + 32*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tm*ep_k**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tm*ep_k**(-1)*em_ep

      me2igaz = me2igaz - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tm
     +  - 32*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tm*ep_k*em_k**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tm*em_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tm
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tm*ep_k**(-1)*em_ep
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tm*em_k**(-1)*em_ep
     +  - 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*ep_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*em_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tm**(-1)*ep_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tm**(-1)*em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*
     + em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*ep_tp*ep_k**(-1)*em_k
     +  + 16*ge**4*gv**2*Fmt*mz**2*em_tp*k_tp**(-1)*k_tm*ep_k*
     + em_k**(-1)
     +  + 16*ge**4*gv**2*Fmt*mz**2*em_tp*ep_k*em_k**(-1)

      me2igaz = me2igaz + 16*ge**4*gv**2*Fmt*mz**2*k_tp*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*k_tp*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 16*ge**4*gv**2*Fmt*mz**2*ep_tm*ep_k**(-1)*em_k
     +  - 16*ge**4*gv**2*Fmt*mz**2*em_tm*ep_k*em_k**(-1)
     +  + 32*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*ep_k*em_ep
     +  - 32*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*em_k*em_ep
     +  - 32*ge**4*gv**2*Fmt*tm_tp*k_tm**(-1)*ep_k*em_ep
     +  + 32*ge**4*gv**2*Fmt*tm_tp*k_tm**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**2*Fmt*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*em_k*
     + em_ep
     +  + 32*ge**4*gv**2*Fmt*ep_tp*ep_k**(-1)*em_k*em_ep
     +  - 32*ge**4*gv**2*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*em_k**(-1)*
     + em_ep
     +  - 32*ge**4*gv**2*Fmt*em_tp*ep_k*em_k**(-1)*em_ep

      me2igaz = me2igaz - 32*ge**4*gv**2*Fmt*k_tp*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**2*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)*
     + em_ep
     +  - 32*ge**4*gv**2*Fmt*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*ge**4*gv**2*Fmt*em_tm*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*ep_tp*k_tp**(-1)*
     + ep_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*em_tp*k_tp**(-1)*
     + em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*ep_tm*
     + ep_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*em_tm*
     + em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*ep_k*
     + em_k**(-1)

      me2igaz = me2igaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tp**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**2*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2

      me2igaz = me2igaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tp**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep

      me2igaz = me2igaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*em_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tp**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tp*
     + k_tm**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)*em_ep

      me2igaz = me2igaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*ep_k**(-1)*em_ep**2
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_tm*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_tm*em_k**(-1)*em_ep**2

      me2igaz = me2igaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*k_tp**(-1)*em_tm*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**2*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k*em_k**(-1)*em_ep**2
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep**2
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_k*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tp**(-1)*
     + em_ep**2

      me2igaz = me2igaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)
     + *ep_tm*k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*ep_tm*
     + k_tm**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*em_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*em_tm*
     + k_tm**(-1)*em_ep

      me2igaz = me2igaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)
     + *k_tm**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k*em_k**(-1)*em_ep**2
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k**2*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_k*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + ep_k**(-1)*em_ep**3
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-2)*k_tm**(-1)*
     + em_k**(-1)*em_ep**3

      me2igaz = me2igaz - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-2)*k_tm**(-1)*em_ep**2
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tp**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*k_tm**(-1)*
     + ep_k**(-1)*em_ep

      me2igaz = me2igaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tp*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*ep_k*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*
     + ep_k**(-1)*em_ep
     +  - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tp**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*k_tm**(-1)*
     + em_k**(-1)*em_ep

      me2igaz = me2igaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tp*
     + k_tm**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*ep_k*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_k
     +  - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*
     + ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm*
     + em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*ep_k*
     + em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm*
     + ep_k**(-1)*em_ep

      me2igaz = me2igaz - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**2*
     + em_k**(-1)
     +  - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k**2
     +  - 40*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tp**(-1)*em_k**(-1)*
     + em_ep**2

      me2igaz = me2igaz - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tp**(-1)*em_ep
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)*
     + ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*ep_tm*k_tm**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*ep_k*
     + em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*
     + ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)*
     + em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*em_tm*k_tm**(-1)

      me2igaz = me2igaz - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + k_tm**(-1)*ep_k**2*em_k**(-1)
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k*
     + em_k**(-1)*em_ep
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k**2
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_k*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*ep_k**(-1)*
     + em_ep**2
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_k**(-1)*
     + em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*k_tm**(-1)*em_ep


      RETURN
      END

      SUBROUTINE CIGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2igag =
     +  + 16*mt**2*ge**4*Fmt*k_tp**(-1)*ep_k
     +  - 16*mt**2*ge**4*Fmt*k_tp**(-1)*em_k
     +  - 16*mt**2*ge**4*Fmt*k_tm**(-1)*ep_k
     +  + 16*mt**2*ge**4*Fmt*k_tm**(-1)*em_k
     +  + 16*ge**4*Fmt*tm_tp*k_tp**(-1)*ep_k
     +  - 16*ge**4*Fmt*tm_tp*k_tp**(-1)*em_k
     +  - 16*ge**4*Fmt*tm_tp*k_tm**(-1)*ep_k
     +  + 16*ge**4*Fmt*tm_tp*k_tm**(-1)*em_k
     +  + 16*ge**4*Fmt*ep_tp*k_tp**(-1)*k_tm*ep_k**(-1)*em_k
     +  + 16*ge**4*Fmt*ep_tp*ep_k**(-1)*em_k
     +  - 16*ge**4*Fmt*em_tp*k_tp**(-1)*k_tm*ep_k*em_k**(-1)
     +  - 16*ge**4*Fmt*em_tp*ep_k*em_k**(-1)
     +  - 16*ge**4*Fmt*k_tp*ep_tm*k_tm**(-1)*ep_k**(-1)*em_k
     +  + 16*ge**4*Fmt*k_tp*em_tm*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 16*ge**4*Fmt*ep_tm*ep_k**(-1)*em_k

      me2igag = me2igag + 16*ge**4*Fmt*em_tm*ep_k*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*ep_tp*k_tp**(-1)*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*em_tp*k_tp**(-1)*em_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*ep_tm*ep_k**(-1)
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*em_tm*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)*em_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tp**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tm**(-1)*ep_k*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*Fet*k_tm**(-1)*ep_k**(-1)*em_k


      RETURN
      END

      SUBROUTINE CFZFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fzfg =
     +  + 64*mt**4*ge**4*gv**2*mz**2*k_tp**(-2)*em_ep
     +  + 64*mt**4*ge**4*gv**2*mz**2*k_tm**(-2)*em_ep
     +  - 128*mt**4*ge**4*gv**2*k_tp**(-2)*em_ep**2
     +  - 128*mt**4*ge**4*gv**2*k_tm**(-2)*em_ep**2
     +  - 128*mt**2*ge**4*gv**2*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep
     +  + 64*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tp**(-2)*em_tm
     +  + 64*mt**2*ge**4*gv**2*mz**2*ep_tp*em_tm*k_tm**(-2)
     +  + 64*mt**2*ge**4*gv**2*mz**2*ep_tp*k_tm**(-2)*em_k
     +  + 64*mt**2*ge**4*gv**2*mz**2*em_tp*k_tp**(-2)*ep_tm
     +  + 64*mt**2*ge**4*gv**2*mz**2*em_tp*ep_tm*k_tm**(-2)
     +  + 64*mt**2*ge**4*gv**2*mz**2*em_tp*k_tm**(-2)*ep_k
     +  + 128*mt**2*ge**4*gv**2*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 64*mt**2*ge**4*gv**2*mz**2*k_tp**(-2)*ep_tm*em_k
     +  + 64*mt**2*ge**4*gv**2*mz**2*k_tp**(-2)*em_tm*ep_k
     +  + 256*mt**2*ge**4*gv**2*tm_tp*k_tp**(-1)*k_tm**(-1)*em_ep**2

      me2fzfg = me2fzfg - 128*mt**2*ge**4*gv**2*ep_tp*k_tp**(-2)*em_tm*
     + em_ep
     +  - 128*mt**2*ge**4*gv**2*ep_tp*em_tm*k_tm**(-2)*em_ep
     +  - 128*mt**2*ge**4*gv**2*ep_tp*k_tm**(-2)*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tp*k_tp**(-2)*ep_tm*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tp*ep_tm*k_tm**(-2)*em_ep
     +  - 128*mt**2*ge**4*gv**2*em_tp*k_tm**(-2)*ep_k*em_ep
     +  - 256*mt**2*ge**4*gv**2*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*k_tp**(-2)*ep_tm*em_k*em_ep
     +  - 128*mt**2*ge**4*gv**2*k_tp**(-2)*em_tm*ep_k*em_ep
     +  - 128*ge**4*gv**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k
     +  - 128*ge**4*gv**2*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**2*mz**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k
     +  - 64*ge**4*gv**2*mz**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k

      me2fzfg = me2fzfg + 128*ge**4*gv**2*mz**2*ep_tp*em_tp*k_tp**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp*k_tp**(-1)*em_tm
     +  - 64*ge**4*gv**2*mz**2*ep_tp*em_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*ep_tp*k_tm**(-1)*em_k
     +  - 64*ge**4*gv**2*mz**2*em_tp*k_tp**(-1)*ep_tm
     +  - 64*ge**4*gv**2*mz**2*em_tp*ep_tm*k_tm**(-1)
     +  - 64*ge**4*gv**2*mz**2*em_tp*k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**2*mz**2*k_tp**(-1)*ep_tm*em_k
     +  - 64*ge**4*gv**2*mz**2*k_tp**(-1)*em_tm*ep_k
     +  + 128*ge**4*gv**2*mz**2*ep_tm*em_tm*k_tm**(-1)
     +  + 256*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*em_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*ep_tp*k_tp**(-1)*k_tm**(-1)*em_k*em_ep
     +  + 256*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*tm_tp*em_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_ep
     +  + 128*ge**4*gv**2*tm_tp*k_tp**(-1)*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 128*ge**4*gv**2*tm_tp*k_tp**(-1)*em_tm*k_tm**(-1)*ep_k*em_ep

      me2fzfg = me2fzfg - 256*ge**4*gv**2*ep_tp*em_tp*k_tp**(-1)*em_ep
     +  + 128*ge**4*gv**2*ep_tp*k_tp**(-1)*em_tm*em_ep
     +  + 128*ge**4*gv**2*ep_tp*em_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  + 128*ge**4*gv**2*em_tp*k_tp**(-1)*ep_tm*em_ep
     +  + 128*ge**4*gv**2*em_tp*ep_tm*k_tm**(-1)*em_ep
     +  + 128*ge**4*gv**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  + 128*ge**4*gv**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  + 128*ge**4*gv**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  - 256*ge**4*gv**2*ep_tm*em_tm*k_tm**(-1)*em_ep


      RETURN
      END

      SUBROUTINE CFZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fzaz =
     +  - 64*mt**2*ge**4*gv**4*Fmt*mz**4*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 64*mt**2*ge**4*gv**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  + 256*mt**2*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep
     +  - 256*mt**2*ge**4*gv**4*Fmt*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**4*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  + 64*mt**2*ge**4*ga**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  - 256*mt**2*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep
     +  + 256*mt**2*ge**4*ga**4*Fmt*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2

      me2fzaz = me2fzaz - 64*ge**4*gv**4*Fmt*mz**4*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*k_tp**(-1)*em_tm*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 64*ge**4*gv**4*Fmt*mz**2*gzfs**2*tm_tp*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k

      me2fzaz = me2fzaz + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*
     + ep_tm*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*gv**4*Fmt*mz**2*gzfs**2*em_tm*k_tm**(-1)*ep_k
     +  + 256*ge**4*gv**4*Fmt*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*ep_tp*k_tp**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  - 128*ge**4*gv**4*Fmt*mz**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  - 256*ge**4*gv**4*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2

      me2fzaz = me2fzaz + 128*ge**4*gv**4*Fmt*ep_tp*k_tp**(-1)*em_k*
     + em_ep**2
     +  + 128*ge**4*gv**4*Fmt*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*em_tp*k_tm**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*gv**4*Fmt*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 128*ge**4*gv**2*ga**2*Fmt*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tp*k_tp**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp**(-1)*ep_tm*em_k

      me2fzaz = me2fzaz + 64*ge**4*gv**2*ga**2*Fmt*mz**4*k_tp**(-1)*
     + em_tm*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*ep_tm*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**4*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*ga**2*Fmt*mz**4*em_ep
     +  - 128*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*ep_tm*k_tm**(-1)*em_k
     +  + 64*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*ga**2*Fmt*mz**2*gzfs**2*em_ep

      me2fzaz = me2fzaz + 512*ge**4*gv**2*ga**2*Fmt*mz**2*tm_tp*
     + k_tp**(-1)*k_tm**(-1)*ep_k*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp*k_tp**(-1)*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  - 256*ge**4*gv**2*ga**2*Fmt*mz**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 512*ge**4*gv**2*ga**2*Fmt*mz**2*em_ep**2
     +  - 512*ge**4*gv**2*ga**2*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*em_tp*k_tp**(-1)*ep_k*em_ep**2

      me2fzaz = me2fzaz + 256*ge**4*gv**2*ga**2*Fmt*em_tp*k_tm**(-1)*
     + ep_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*k_tp**(-1)*ep_tm*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 256*ge**4*gv**2*ga**2*Fmt*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 512*ge**4*gv**2*ga**2*Fmt*em_ep**3
     +  - 64*ge**4*ga**4*Fmt*mz**4*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*k_tp**(-1)*em_tm*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**4*ep_tm*k_tm**(-1)*em_k

      me2fzaz = me2fzaz + 32*ge**4*ga**4*Fmt*mz**4*em_tm*k_tm**(-1)*
     + ep_k
     +  - 128*ge**4*ga**4*Fmt*mz**4*em_ep
     +  - 64*ge**4*ga**4*Fmt*mz**2*gzfs**2*tm_tp*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*k_tp**(-1)*em_tm*ep_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*ga**4*Fmt*mz**2*gzfs**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*ga**4*Fmt*mz**2*gzfs**2*em_ep
     +  + 256*ge**4*ga**4*Fmt*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k*em_ep

      me2fzaz = me2fzaz - 128*ge**4*ga**4*Fmt*mz**2*ep_tp*k_tp**(-1)*
     + em_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  - 128*ge**4*ga**4*Fmt*mz**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 512*ge**4*ga**4*Fmt*mz**2*em_ep**2
     +  - 256*ge**4*ga**4*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep**2
     +  + 128*ge**4*ga**4*Fmt*ep_tp*k_tp**(-1)*em_k*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*ep_tp*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*em_tp*k_tp**(-1)*ep_k*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*em_tp*k_tm**(-1)*ep_k*em_ep**2

      me2fzaz = me2fzaz + 128*ge**4*ga**4*Fmt*k_tp**(-1)*ep_tm*em_k*
     + em_ep**2
     +  + 128*ge**4*ga**4*Fmt*k_tp**(-1)*em_tm*ep_k*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*ep_tm*k_tm**(-1)*em_k*em_ep**2
     +  + 128*ge**4*ga**4*Fmt*em_tm*k_tm**(-1)*ep_k*em_ep**2
     +  - 512*ge**4*ga**4*Fmt*em_ep**3


      RETURN
      END

      SUBROUTINE CFZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fzag =
     +  + 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tm**(-2)*em_k
     +  + 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tm**(-2)*ep_k
     +  - 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-2)*ep_tm*em_k
     +  - 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-2)*em_tm*ep_k
     +  + 64*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 128*mt**2*ge**4*gv**2*Fmt*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k

      me2fzag = me2fzag + 128*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*em_tp*
     + k_tp**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tm**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tm**(-1)*ep_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*ep_tm*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*em_tm*ep_k
     +  - 128*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tm*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tp**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*em_tp*k_tp**(-1)*ep_k

      me2fzag = me2fzag - 32*ge**4*gv**2*Fmt*mz**2*em_tp*k_tm**(-1)*
     + ep_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*ep_tm*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*em_tm*ep_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tm*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tp*k_tp**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tp*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tp*k_tp**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tp*k_tm**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*k_tp**(-1)*ep_tm*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tm*k_tm**(-1)*ep_k*em_ep


      RETURN
      END

      SUBROUTINE CFGAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fgaz =
     +  - 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tm**(-2)*em_k
     +  - 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tm**(-2)*ep_k
     +  + 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-2)*ep_tm*em_k
     +  + 64*mt**3*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-2)*em_tm*ep_k
     +  + 64*mt**2*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 128*mt**2*ge**4*gv**2*Fmt*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*ep_tp*k_tp**(-1)*
     + k_tm**(-1)*em_k
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*em_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*ep_tm*
     + k_tm**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*tm_tp*k_tp**(-1)*em_tm*
     + k_tm**(-1)*ep_k

      me2fgaz = me2fgaz - 128*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*em_tp*
     + k_tp**(-1)
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tp**(-1)*em_tm
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*em_tm*k_tm**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tp*k_tm**(-1)*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tp**(-1)*ep_tm
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*ep_tm*k_tm**(-1)
     +  + 64*mt*ge**4*gv*ga*Fet*mz*gzfs*em_tp*k_tm**(-1)*ep_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*ep_tm*em_k
     +  - 64*mt*ge**4*gv*ga*Fet*mz*gzfs*k_tp**(-1)*em_tm*ep_k
     +  + 128*mt*ge**4*gv*ga*Fet*mz*gzfs*ep_tm*em_tm*k_tm**(-1)
     +  + 64*ge**4*gv**2*Fmt*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tp**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tp*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*em_tp*k_tp**(-1)*ep_k

      me2fgaz = me2fgaz - 32*ge**4*gv**2*Fmt*mz**2*em_tp*k_tm**(-1)*
     + ep_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*ep_tm*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*k_tp**(-1)*em_tm*ep_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*ep_tm*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt*mz**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tp*k_tp**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tp*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tp*k_tp**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tp*k_tm**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*k_tp**(-1)*ep_tm*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt*em_tm*k_tm**(-1)*ep_k*em_ep


      RETURN
      END

      SUBROUTINE CFGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2fgag =
     +  - 64*mt**2*ge**4*Fmt*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  - 64*ge**4*Fmt*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k
     +  + 32*ge**4*Fmt*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*Fmt*ep_tp*k_tm**(-1)*em_k
     +  + 32*ge**4*Fmt*em_tp*k_tp**(-1)*ep_k
     +  + 32*ge**4*Fmt*em_tp*k_tm**(-1)*ep_k
     +  + 32*ge**4*Fmt*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*Fmt*k_tp**(-1)*em_tm*ep_k
     +  + 32*ge**4*Fmt*ep_tm*k_tm**(-1)*em_k
     +  + 32*ge**4*Fmt*em_tm*k_tm**(-1)*ep_k


      RETURN
      END

      SUBROUTINE CAZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        me2azag =
     +  + 256*mt**2*ge**4*gv**2*Fet**2*mz**2*tm_tp*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*ep_tp*k_tp**(-1)*em_k
     +  + 128*mt**2*ge**4*gv**2*Fet**2*mz**2*ep_tp*k_tm**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*em_tp*k_tp**(-1)*ep_k
     +  + 128*mt**2*ge**4*gv**2*Fet**2*mz**2*em_tp*k_tm**(-1)*ep_k
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*k_tp*k_tm**(-1)*em_ep
     +  + 128*mt**2*ge**4*gv**2*Fet**2*mz**2*k_tp**(-1)*ep_tm*em_k
     +  + 128*mt**2*ge**4*gv**2*Fet**2*mz**2*k_tp**(-1)*em_tm*ep_k
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*k_tp**(-1)*k_tm*em_ep
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*ep_tm*k_tm**(-1)*em_k
     +  - 128*mt**2*ge**4*gv**2*Fet**2*mz**2*em_tm*k_tm**(-1)*ep_k
     +  - 512*mt**2*ge**4*gv**2*Fet**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k
     + *em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*Fet**2*ep_tp*k_tp**(-1)*em_k*em_ep

      me2azag = me2azag - 256*mt**2*ge**4*gv**2*Fet**2*ep_tp*k_tm**(-1)
     + *em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*Fet**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  - 256*mt**2*ge**4*gv**2*Fet**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*Fet**2*k_tp*k_tm**(-1)*em_ep**2
     +  - 256*mt**2*ge**4*gv**2*Fet**2*k_tp**(-1)*ep_tm*em_k*em_ep
     +  - 256*mt**2*ge**4*gv**2*Fet**2*k_tp**(-1)*em_tm*ep_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*Fet**2*k_tp**(-1)*k_tm*em_ep**2
     +  + 256*mt**2*ge**4*gv**2*Fet**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 256*mt**2*ge**4*gv**2*Fet**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k
     +  - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*
     + em_k
     +  - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp*em_tp*k_tp**(-1)*
     + ep_k

      me2azag = me2azag - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp*
     + ep_tm*k_tm**(-1)*em_k
     +  - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp*em_tm*k_tm**(-1)*
     + ep_k
     +  + 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*tm_tp*em_ep
     +  + 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*ep_tp*em_tp*k_tp**(-1)*
     + k_tm
     +  - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*ep_tp*em_tm
     +  - 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*em_tp*ep_tm
     +  + 64*mt**(-2)*ge**4*gv**2*Fmt**2*mz**2*k_tp*ep_tm*em_tm*
     + k_tm**(-1)
     +  - 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  + 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp*ep_tp*k_tp**(-1)*em_k*
     + em_ep
     +  + 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp*em_tp*k_tp**(-1)*ep_k*
     + em_ep

      me2azag = me2azag + 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp*ep_tm*
     + k_tm**(-1)*em_k*em_ep
     +  + 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp*em_tm*k_tm**(-1)*ep_k*
     + em_ep
     +  - 128*mt**(-2)*ge**4*gv**2*Fmt**2*tm_tp*em_ep**2
     +  - 128*mt**(-2)*ge**4*gv**2*Fmt**2*ep_tp*em_tp*k_tp**(-1)*k_tm*
     + em_ep
     +  + 128*mt**(-2)*ge**4*gv**2*Fmt**2*ep_tp*em_tm*em_ep
     +  + 128*mt**(-2)*ge**4*gv**2*Fmt**2*em_tp*ep_tm*em_ep
     +  - 128*mt**(-2)*ge**4*gv**2*Fmt**2*k_tp*ep_tm*em_tm*k_tm**(-1)*
     + em_ep
     +  + 64*ge**4*gv**2*Fmt**2*mz**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*
     + em_k
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*ep_tp*k_tp**(-1)*em_k
     +  + 32*ge**4*gv**2*Fmt**2*mz**2*ep_tp*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*em_tp*k_tp**(-1)*ep_k

      me2azag = me2azag + 32*ge**4*gv**2*Fmt**2*mz**2*em_tp*k_tm**(-1)*
     + ep_k
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*k_tp*k_tm**(-1)*em_ep
     +  + 32*ge**4*gv**2*Fmt**2*mz**2*k_tp**(-1)*ep_tm*em_k
     +  + 32*ge**4*gv**2*Fmt**2*mz**2*k_tp**(-1)*em_tm*ep_k
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*k_tp**(-1)*k_tm*em_ep
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*ep_tm*k_tm**(-1)*em_k
     +  - 32*ge**4*gv**2*Fmt**2*mz**2*em_tm*k_tm**(-1)*ep_k
     +  - 128*ge**4*gv**2*Fmt**2*tm_tp*k_tp**(-1)*k_tm**(-1)*ep_k*em_k*
     + em_ep
     +  + 64*ge**4*gv**2*Fmt**2*ep_tp*k_tp**(-1)*em_k*em_ep
     +  - 64*ge**4*gv**2*Fmt**2*ep_tp*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt**2*em_tp*k_tp**(-1)*ep_k*em_ep
     +  - 64*ge**4*gv**2*Fmt**2*em_tp*k_tm**(-1)*ep_k*em_ep
     +  + 64*ge**4*gv**2*Fmt**2*k_tp*k_tm**(-1)*em_ep**2
     +  - 64*ge**4*gv**2*Fmt**2*k_tp**(-1)*ep_tm*em_k*em_ep

      me2azag = me2azag - 64*ge**4*gv**2*Fmt**2*k_tp**(-1)*em_tm*ep_k*
     + em_ep
     +  + 64*ge**4*gv**2*Fmt**2*k_tp**(-1)*k_tm*em_ep**2
     +  + 64*ge**4*gv**2*Fmt**2*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 64*ge**4*gv**2*Fmt**2*em_tm*k_tm**(-1)*ep_k*em_ep
     +  + 256*ge**4*gv**2*Fet**2*mz**2*tm_tp**2*k_tp**(-1)*k_tm**(-1)*
     + ep_k*em_k
     +  - 256*ge**4*gv**2*Fet**2*mz**2*tm_tp*ep_tp*k_tp**(-1)*em_k
     +  - 256*ge**4*gv**2*Fet**2*mz**2*tm_tp*em_tp*k_tp**(-1)*ep_k
     +  - 256*ge**4*gv**2*Fet**2*mz**2*tm_tp*ep_tm*k_tm**(-1)*em_k
     +  - 256*ge**4*gv**2*Fet**2*mz**2*tm_tp*em_tm*k_tm**(-1)*ep_k
     +  + 256*ge**4*gv**2*Fet**2*mz**2*tm_tp*em_ep
     +  + 256*ge**4*gv**2*Fet**2*mz**2*ep_tp*em_tp*k_tp**(-1)*k_tm
     +  - 256*ge**4*gv**2*Fet**2*mz**2*ep_tp*em_tm
     +  - 256*ge**4*gv**2*Fet**2*mz**2*em_tp*ep_tm
     +  + 256*ge**4*gv**2*Fet**2*mz**2*k_tp*ep_tm*em_tm*k_tm**(-1)

      me2azag = me2azag - 512*ge**4*gv**2*Fet**2*tm_tp**2*k_tp**(-1)*
     + k_tm**(-1)*ep_k*em_k*em_ep
     +  + 512*ge**4*gv**2*Fet**2*tm_tp*ep_tp*k_tp**(-1)*em_k*em_ep
     +  + 512*ge**4*gv**2*Fet**2*tm_tp*em_tp*k_tp**(-1)*ep_k*em_ep
     +  + 512*ge**4*gv**2*Fet**2*tm_tp*ep_tm*k_tm**(-1)*em_k*em_ep
     +  + 512*ge**4*gv**2*Fet**2*tm_tp*em_tm*k_tm**(-1)*ep_k*em_ep
     +  - 512*ge**4*gv**2*Fet**2*tm_tp*em_ep**2
     +  - 512*ge**4*gv**2*Fet**2*ep_tp*em_tp*k_tp**(-1)*k_tm*em_ep
     +  + 512*ge**4*gv**2*Fet**2*ep_tp*em_tm*em_ep
     +  + 512*ge**4*gv**2*Fet**2*em_tp*ep_tm*em_ep
     +  - 512*ge**4*gv**2*Fet**2*k_tp*ep_tm*em_tm*k_tm**(-1)*em_ep
     +  - 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*gzfs*tm_tp*
     + k_tp**(-1)*k_tm**(-1)*ep_k
     +  + 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*gzfs*tm_tp*
     + k_tp**(-1)*k_tm**(-1)*em_k
     +  + 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*gzfs*ep_tp*
     + k_tp**(-1)

      me2azag = me2azag - 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*
     + gzfs*em_tp*k_tp**(-1)
     +  + 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*gzfs*ep_tm*
     + k_tm**(-1)
     +  - 128*e_(tp,tm,ep,em)*ge**4*gv*ga*Fet**2*mz*gzfs*em_tm*
     + k_tm**(-1)


      RETURN
      END

      SUBROUTINE ICIZIZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2iziz = 0

      RETURN
      END

      SUBROUTINE ICIGIG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2igig = 0

      RETURN
      END

      SUBROUTINE ICFZFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fzfz = 0

      RETURN
      END

      SUBROUTINE ICFGFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fgfg = 0

      RETURN
      END

      SUBROUTINE ICAZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2azaz = 0

      RETURN
      END

      SUBROUTINE ICAGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2agag = 0

      RETURN
      END

      SUBROUTINE ICIZIG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2izig = 0

      RETURN
      END

      SUBROUTINE ICIZFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2izfz = 0

      RETURN
      END

      SUBROUTINE ICIZFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2izfg = 0

      RETURN
      END

      SUBROUTINE ICIZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2izaz =
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep

      ime2izaz = ime2izaz - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 56*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k**2

      ime2izaz = ime2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 56*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + ep_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep

      ime2izaz = ime2izaz - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzfs*ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + ep_tp*k_tp**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + em_tp*k_tp**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzfs*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm
     +  + 16*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzfs*k_tp**(-1)*em_tm
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs
     + *k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 112*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs
     + *k_tp**(-1)*ep_k
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs
     + *k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 112*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs
     + *k_tp**(-1)*em_k

      ime2izaz = ime2izaz + 96*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzfs*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 256*e_(tp,tm,ep,em)*mt**3*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs
     + *k_tp**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 56*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*ep_k
     +  + 24*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 64*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 56*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt**3*ge**4*ga**4*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_ep
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzis*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*em_tp*k_tp**(-1)
     + *ep_k*em_k**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*em_tp*k_tp**(-1)
     + *em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzis*em_tp*k_tp**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*em_tm
     + *ep_k*em_k**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k

      ime2izaz = ime2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzis*k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*em_tm
     + *em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + em_tm
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*ep_k
     + *em_k**(-1)*em_ep
     +  + 28*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*ep_k
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2

      ime2izaz = ime2izaz + 28*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzis*k_tp**(-1)*em_k
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzis*k_tp**(-1)*
     + em_ep
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)

      ime2izaz = ime2izaz - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzfs*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*em_tp*k_tp**(-1)
     + *em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_ep

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzfs*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_tm
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*em_tm
     + *ep_k*em_k**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*em_tm
     + *em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**2*em_k**(-1)

      ime2izaz = ime2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*
     + gzfs*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 28*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*ep_k
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 28*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*em_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep

      ime2izaz = ime2izaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzis*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2

      ime2izaz = ime2izaz - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzis*em_tp*k_tp**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm*ep_k*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep

      ime2izaz = ime2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2

      ime2izaz = ime2izaz - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_ep**2
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*tm_tp*ep_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k

      ime2izaz = ime2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*tm_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*tm_tp*k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2

      ime2izaz = ime2izaz - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**4*Fet*
     + mz**(-1)*gzfs*tm_tp*k_tp**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k

      ime2izaz = ime2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzis*em_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*em_tp*
     + k_tp**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_tm

      ime2izaz = ime2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*
     + mz*gzis*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_tm
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_k

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzis*k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzis*
     + k_tp**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzfs*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)

      ime2izaz = ime2izaz - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzfs*k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_tm
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*k_tp**(-1)
     + *em_tm*ep_k*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzfs*k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*em_tm
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2

      ime2izaz = ime2izaz - 56*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz*gzfs*k_tp**(-1)*em_k
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz*gzfs*
     + k_tp**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + ep_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + ep_tp*k_tp**(-1)*em_ep

      ime2izaz = ime2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz**(-1)*gzis*em_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + em_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + em_tp*k_tp**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2

      ime2izaz = ime2izaz + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz**(-1)*gzis*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k*em_k**(-1)*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzis*k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  - 112*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  + 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  - 112*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k*em_ep
     +  - 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k**(-1)*em_ep**3
     +  + 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_ep**2

      ime2izaz = ime2izaz + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz**(-1)*gzfs*tm_tp*ep_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*ep_tp*k_tp**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*em_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*em_tp*k_tp**(-1)*ep_k**(-1)*em_ep

      ime2izaz = ime2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz**(-1)*gzfs*tm_tp*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*em_tp*k_tp**(-1)
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_tm
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet
     + *mz**(-1)*gzfs*tm_tp*k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_tm
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 112*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_k**(-1)*em_k**2

      ime2izaz = ime2izaz - 128*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*
     + Fet*mz**(-1)*gzfs*tm_tp*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 112*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_k
     +  + 96*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 256*e_(tp,tm,ep,em)*mt*ge**4*gv**2*ga**2*Fet*mz**(-1)*gzfs*
     + tm_tp*k_tp**(-1)*em_ep
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_ep

      ime2izaz = ime2izaz - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzis*ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*ep_tp*
     + k_tp**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*em_tp*k_tp**(-1)
     + *ep_k*em_k**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*em_tp*k_tp**(-1)
     + *em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*em_tp*
     + k_tp**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)

      ime2izaz = ime2izaz + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzis*k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_tm
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*em_tm
     + *ep_k*em_k**(-1)
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*em_tm
     + *em_k**(-1)*em_ep

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzis*k_tp**(-1)*em_tm
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**2*em_k**(-1)
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*ep_k
     + *em_k**(-1)*em_ep
     +  + 28*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*ep_k
     +  + 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_k**2
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_k*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  + 28*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*em_k
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzis*k_tp**(-1)*
     + em_k**(-1)*em_ep**2

      ime2izaz = ime2izaz - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzis*k_tp**(-1)*em_ep
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*ep_tp*k_tp**(-1)
     + *ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*ep_tp*
     + k_tp**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*em_tp*k_tp**(-1)
     + *ep_k*em_k**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzfs*em_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*em_tp*k_tp**(-1)
     + *em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*em_tp*
     + k_tp**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_tm*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*ep_tm
     + *ep_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_tm*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_tm

      ime2izaz = ime2izaz - 4*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzfs*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm*ep_k**(-1)*em_ep
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*em_tm
     + *em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_tm
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**2*em_k**(-1)
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*ep_k
     + *em_k**(-1)*em_ep
     +  - 28*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*ep_k
     +  - 12*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_k**2

      ime2izaz = ime2izaz + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*
     + gzfs*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + ep_k**(-1)*em_ep**2
     +  - 28*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*em_k
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_k**(-1)*em_ep**2
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz*gzfs*k_tp**(-1)*
     + em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2

      ime2izaz = ime2izaz - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzis*ep_tp*k_tp**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k*em_ep

      ime2izaz = ime2izaz + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep**2
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_tm*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm*ep_k*em_k**(-1)*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k*em_ep
     +  + 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep**2
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_tm*em_ep

      ime2izaz = ime2izaz - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*ep_k**2*em_k**(-1)*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep**2
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k*em_ep
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k**2*em_ep
     +  + 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*ep_k**(-1)*em_ep**3
     +  - 56*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k*em_ep
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzis*
     + k_tp**(-1)*em_k**(-1)*em_ep**3

      ime2izaz = ime2izaz + 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*em_ep**2
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + ep_tp*k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*ep_k**(-1)*em_k

      ime2izaz = ime2izaz - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*tm_tp*em_tp*k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + em_tp*k_tp**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_k
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*ep_k**(-1)*em_ep
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_tm

      ime2izaz = ime2izaz + 8*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*tm_tp*k_tp**(-1)*em_tm*ep_k*em_k**(-1)
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 32*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*ep_k**(-1)*em_ep
     +  - 16*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm*em_k**(-1)*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_tm
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**2*em_k**(-1)
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k

      ime2izaz = ime2izaz + 24*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*
     + mz**(-1)*gzfs*tm_tp*k_tp**(-1)*ep_k**(-1)*em_k**2
     +  - 64*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 56*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_k
     +  + 48*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 128*e_(tp,tm,ep,em)*mt*ge**4*ga**4*Fet*mz**(-1)*gzfs*tm_tp*
     + k_tp**(-1)*em_ep


      RETURN
      END

      SUBROUTINE ICIZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2izag =
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*ep_tp*
     + k_tp**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*em_tp*
     + k_tp**(-1)

      ime2izag = ime2izag - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*ep_tm*ep_k*em_k**(-1)
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_tm*em_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_tm
     +  + 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm*ep_k*em_k**(-1)
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm*ep_k**(-1)*em_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_tm

      ime2izag = ime2izag - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*ep_k**2*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_k*em_k**(-1)*em_ep
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_k
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_k**(-1)*em_k**2
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_k**(-1)*em_k*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *ep_k**(-1)*em_ep**2
     +  - 4*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzis*k_tp**(-1)
     + *em_k**(-1)*em_ep**2

      ime2izag = ime2izag + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzis*k_tp**(-1)*em_ep


      RETURN
      END

      SUBROUTINE ICIGFZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2igfz = 0

      RETURN
      END

      SUBROUTINE ICIGFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2igfg = 0

      RETURN
      END

      SUBROUTINE ICIGAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2igaz =
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*ep_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)*ep_k**(-1)*em_ep

      ime2igaz = ime2igaz - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzfs*em_tp*k_tp**(-1)*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*em_tp*
     + k_tp**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_tm*ep_k*em_k**(-1)
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_tm*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_tm*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *em_tm*ep_k*em_k**(-1)

      ime2igaz = ime2igaz + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*em_tm*ep_k**(-1)*em_k
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *em_tm*ep_k**(-1)*em_ep
     +  - 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *em_tm*em_k**(-1)*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_tm
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_k**2*em_k**(-1)
     +  - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k*em_k**(-1)*em_ep
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k
     +  + 8*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*k_tp**(-1)
     + *ep_k**(-1)*em_k**2

      ime2igaz = ime2igaz - 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*
     + mz**(-1)*gzfs*k_tp**(-1)*ep_k**(-1)*em_k*em_ep
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*ep_k**(-1)*em_ep**2
     +  + 24*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k
     +  + 16*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_k**(-1)*em_ep**2
     +  - 48*e_(tp,tm,ep,em)*mt*ge**4*gv**2*Fet*mz**(-1)*gzfs*
     + k_tp**(-1)*em_ep


      RETURN
      END

      SUBROUTINE ICIGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2igag = 0

      RETURN
      END

      SUBROUTINE ICFZFG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fzfg = 0

      RETURN
      END

      SUBROUTINE ICFZAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fzaz = 0

      RETURN
      END

      SUBROUTINE ICFZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fzag = 0

      RETURN
      END

      SUBROUTINE ICFGAZ
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fgaz = 0

      RETURN
      END

      SUBROUTINE ICFGAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2fgag = 0

      RETURN
      END

      SUBROUTINE ICAZAG
      IMPLICIT NONE
C--
C--  Common for things required to make the
C--  | matrix element |^2
C--  We evaluate it completely with double precision.
C--
      COMMON/TTGPARS/MT,    ! tau mass       (GeV)
     +               ME,    ! electron mass  (GeV)
     +               GV,    ! vector coupling constant of the Z
     +               GA,    ! axial vector coupling constant of the Z
     +               FMT,   ! anomalous magnetic dipole moment of the tau
     +               FET,   ! electric dipole moment of the tau
     +               MZ,    ! mass of the Z0 (GeV)
     +               GZ,    ! width of the Z0 (GeV)
     +               EE,    ! charge of e (assumed equal to that of the tau)
     +               GE,    ! charge of e (assumed equal to that of the tau)
     +               ISR,   ! initial state radiation on and off
     +               FSR,   ! sm final state radiation on and off
     +               ASR,   ! anomalous radiation on and off
     +               ZEX,   ! z exchange on and off
     +               GEX,   ! gamma exchange on and off
     +               ZGI,   ! z-gamma exchange on and off
     +               AFI,   ! interference of anomalous and sm f.s.r.
     +               IFI,   ! interference of isr with SM fsr
     +               IAI,   ! interference of isr with anomalous SM fsr
     +               SFL    ! simple options flag


      DOUBLE PRECISION MT,ME,GV,GA,FMT,FET,MZ,GZ
      DOUBLE PRECISION EE,GE
      DOUBLE PRECISION ISR,FSR,ASR,ZEX,GEX,ZGI,AFI,IFI,IAI
      INTEGER SFL
C--   These are all the dotproducts and fragments of
C--   the matrix-element squared that are passed in
C--   common to subroutines which calculate them.

C--   4-Dot products of the vectors of the external
C--   particles. This is *all* the kinematics passed
C--   to the subroutines for pieces of the matrix element
C--   squared.
      DOUBLE PRECISION tm_tp,ep_tp,em_tp,k_tp
      DOUBLE PRECISION ep_tm,em_tm,k_tm
      DOUBLE PRECISION ep_k,em_k
      DOUBLE PRECISION em_ep
      DOUBLE PRECISION gzis,gzfs

C--   Declare all pieces of the matrix-element-squared
C--   and its imaginary parts
      DOUBLE PRECISION
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag



C--   Double precision 4-vectors for the e+,e-,tau+,tau-
C--   and photon.
      DOUBLE PRECISION EP(4),EM(4),TP(4),TM(4),K(4)


      COMMON/ME2ARGS/
     +       ep,   ! array of 4
     +       em,   ! array of 4
     +       tp,   ! array of 4
     +       tm,   ! array of 4
     +       k,    ! array of 4
     +       tm_tp,
     +       ep_tp,
     +       em_tp,
     +       k_tp,
     +       ep_tm,
     +       em_tm,
     +       k_tm,
     +       ep_k,
     +       em_k,
     +       em_ep,
     +       gzis,
     +       gzfs,
     +       me2iziz,
     +       me2igig,
     +       me2fzfz,
     +       me2fgfg,
     +       me2azaz,
     +       me2agag,
     +       me2izig,
     +       me2izfz,
     +       me2izfg,
     +       me2izaz,
     +       me2izag,
     +       me2igfz,
     +       me2igfg,
     +       me2igaz,
     +       me2igag,
     +       me2fzfg,
     +       me2fzaz,
     +       me2fzag,
     +       me2fgaz,
     +       me2fgag,
     +       me2azag,
     +       ime2iziz,
     +       ime2igig,
     +       ime2fzfz,
     +       ime2fgfg,
     +       ime2azaz,
     +       ime2agag,
     +       ime2izig,
     +       ime2izfz,
     +       ime2izfg,
     +       ime2izaz,
     +       ime2izag,
     +       ime2igfz,
     +       ime2igfg,
     +       ime2igaz,
     +       ime2igag,
     +       ime2fzfg,
     +       ime2fzaz,
     +       ime2fzag,
     +       ime2fgaz,
     +       ime2fgag,
     +       ime2azag


C--   Declare the levi-civita tensor contracted with 4 vectors
      DOUBLE PRECISION e_
      EXTERNAL e_

        ime2azag = 0

      RETURN
      END

