 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*               CROSS-SECTIONS AND HELICITY AMPLITUDES               **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION DSIGMA(CSCAT)
C***************************
C  RETURNS THE DIFFERENTIAL CROSS SECTION IN GEV
C      FOR POLARIZED INCOMING BEAMS
C
C  CSCAT = COSINE OF THE SCATTERING ANGLE
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 CSCAT,DSIGMA
      COMMON/EXTPRM/S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN
      REAL*8 S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN
      REAL*8 XSECL,XSECR
      CALL XSGEV(S,CSCAT,T3E,QE,T3F,QF,NCOLF,MF,MINT,XSECL,XSECR)
C================================================================
      DSIGMA=0.25D0*((1.D0+POLN)**2*XSECL+(1.D0-POLN)**2*XSECR)
      RETURN
      END
 
      SUBROUTINE XSGEV(S,CSCAT,T3E,QE,T3F,QF,NCOLF,MF,MINT,
     .                 XSECL,XSECR)
C**********************************************************
C  CALCULATES DIFFERENTIAL CROSS-SECTIONS FOR INITIAL HELICITIES
C
C  S       = MANDELSTAM VARIABLE S
C  CSCAT   = COSINE OF SCATTERING ANGLE
C  T3E, QE = WEAK T3 AND CHARGE OF INCOMING FERMION
C  T3F, QF = WEAK T3 AND CHARGE OF OUTGOING FERMION
C  NCOLF   = NUMBER OF COLOURS IN THE FINAL STATE (NO FINAL STATE QCD)
C  MF      = THE MASS OF THE OUTGOING FERMION
C  MINT    = MASS OF THE ISOSPIN PARTNER OF THE FINAL STATE FERMION
C
C  XSECL   = DIFFERENTIAL CROSS-SECTION FOR INCOMING LEFT HELCITY BEAMS
C  XSECR   = DIFFERENTIAL CROSS-SECTION FOR INCOMING RIGHT HELCITY BEAMS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 S,CSCAT,T3E,QE,T3F,QF,NCOLF,MF,MINT,XSECL,XSECR
      REAL*8 BETA,BETASQ,PI,T
      COMPLEX*16 HAMPLL,HAMPLR,HAMPRL,HAMPRR
      PI=4.D0*DATAN(1.D0)
      BETASQ=1.D0-4.D0*MF**2/S
      IF(BETASQ.LE.0.D0)THEN
        STOP 'FINAL STATE PARTICLE CANNOT BE PRODUCED AT THIS ENERGY'
      END IF
      BETA=DSQRT(BETASQ)
C  KEEP MF DEPENDENCE IN PHASE SPACE ONLY BUT NOT IN LOOPS
C     T=MF**2-0.5D0*S*(1.D0-BETA*CSCAT)
      T=-0.5D0*S*(1.D0-CSCAT)
      CALL HELAMP(DCMPLX(S),DCMPLX(T),T3E,QE,T3F,QF,DCMPLX(MINT**2),
     .            HAMPLL,HAMPLR,HAMPRL,HAMPRR)
C===================================================================
C  FORM CROSS-SECTIONS FOR THE LEFT AND RIGHT HELICITY STATES
      XSECL=CDABS(HAMPLL)**2*(1.D0+BETA*CSCAT)**2
     .     +CDABS(HAMPLR)**2*(1.D0-BETA*CSCAT)**2
     .     +2.D0*DREAL(DCONJG(HAMPLL)*HAMPLR)*(1.D0-BETASQ)
      XSECR=CDABS(HAMPRL)**2*(1.D0-BETA*CSCAT)**2
     .     +CDABS(HAMPRR)**2*(1.D0+BETA*CSCAT)**2
     .     +2.D0*DREAL(DCONJG(HAMPRL)*HAMPRR)*(1.D0-BETASQ)
C  NORMALIZE CROSS-SECTIONS TO GEV
      XSECL=XSECL*NCOLF*BETA*S/(64.D0*PI**2)
      XSECR=XSECR*NCOLF*BETA*S/(64.D0*PI**2)
      RETURN
      END
 
      SUBROUTINE HELAMP(S,T,T3E,QE,T3F,QF,MINTSQ,
     .                  HAMPLL,HAMPLR,HAMPRL,HAMPRR)
C***************************************************
C  HELICITY AMPLITUDES WITHOUT ANGULAR FACTORS AND
C              WITH AN OVERALL FACTOR -I*E**2 REMOVED
C
C  S       = MANDELSTAM VARIABLE S
C  T       = MANDELSTAM VARIABLE T
C  T3E, QE = WEAK T3 AND CHARGE OF INCOMING FERMION
C  T3F, QF = WEAK T3 AND CHARGE OF OUTGOING FERMION
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3E,QE,T3F,QF
      COMPLEX*16 S,T,MINTSQ,HAMPLL,HAMPLR,HAMPRL,HAMPRR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ESQ,PI
      COMPLEX*16 BKGDLL,BKGDLR,BKGDRL,BKGDRR,RESLL,RESLR,RESRL,RESRR
      PI=4.D0*DATAN(1.D0)
      ESQ=4.D0*PI**2*ALFPI
C  CALCULATE RESIDUES AND BACKGROUNDS
      CALL RESID(S,T3E,QE,T3F,QF,MINTSQ,RESLL,RESLR,RESRL,RESRR)
C===============================================================
      CALL BACKGR(S,T,T3E,QE,T3F,QF,MINTSQ,BKGDLL,BKGDLR,BKGDRL,BKGDRR)
C======================================================================
      HAMPLL=-ESQ*(RESLL/(S-SPOLE)+BKGDLL)
      HAMPLR=-ESQ*(RESLR/(S-SPOLE)+BKGDLR)
      HAMPRL=-ESQ*(RESRL/(S-SPOLE)+BKGDRL)
      HAMPRR=-ESQ*(RESRR/(S-SPOLE)+BKGDRR)
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*            INPUT, INITIALIZATION AND OUTPUT SUBPROGRAMS            **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      SUBROUTINE GSWINT(NSCHEM,XSCALE,NCOLOR,MHIGGS,MTOP,MZEDOS)
C***************************************************************
C  INITIALIZES THE PARAMETERS IN THE COMMON BLOCK GSWROB AND
C    CALCULATES ALFPI, MW AND MZ WHICH ARE THE RENORMALIZED
C    PARAMETERS IN THE RENORMALIZATION SCHEME SELECTED USING NSCHEM
C
C  NSCHEM = SELECTS WHICH RENORMALIZATION SCHEME IS TO BE USED
C              NSCHEM = 1, ON-SHELL SCHEME
C                     = 2, MS-BAR SCHEME
C  XSCALE = FOR NSCHEM = 1 THIS IS AN ARBITRARY SCALE,
C               PHYSICAL RESULTS INDEPENDENT OF XSCALE
C           FOR NSCHEM = 2 THIS IS THE 'T HOOFT MASS,
C               PHYSICAL RESULTS WEAKLY DEPENDENT ON XSCALE
C  NCOLOR = NUMBER OF QUARK COLOURS
C  MHIGGS = HIGGS MASS
C  MTOP   = TOP QUARK MASS
C  MZEDOS = Z0 BOSON RENORMALIZED MASS IN THE ON-SHELL SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      INTEGER NSCHEM
      REAL*8 XSCALE,NCOLOR,MHIGGS,MTOP,MZEDOS
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      COMMON/EXPERIM/ALFEXP,GMUEXP,MZEXP
      REAL*8 ALFEXP,GMUEXP,MZEXP
      COMMON/SPEEDY/BCALL1,RCALL1
      LOGICAL BCALL1,RCALL1
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      INTEGER N,MAXF,IPRT,INFO
      REAL*8 A0SQ,PI,
     .       FTOL,XTOL,REN(3),RES(3),WORKSP(18)
      REAL*8 DELTAR
      COMPLEX*16 POLPOS
      EXTERNAL RENCND
      QCDLIB=.TRUE.
      PI=4.D0*DATAN(1.D0)
C  EXPERIMENTAL INPUT
      ALFEXP=1.D0/137.035989561D0
      GMUEXP=1.166372D-5
      MZEXP=MZEDOS
C  PARAMETERS IN COMMON GSWROB THAT ARE NOT VARIED
      NCOL=NCOLOR
      MH=MHIGGS
      SCALE=DCMPLX(XSCALE)
      SCHEME=NSCHEM
      CALL FMASET(MTOP)
C======================
C  INITIAL ESTIMATE FOR RENORMALIZED PARAMETERS
      A0SQ=ALFEXP*PI/(DSQRT(2.D0)*GMUEXP)
      REN(1)=ALFEXP/PI
      REN(2)=DSQRT(MZEXP**2*(1.D0+DSQRT(1.D0-4.D0*A0SQ/MZEXP**2))/2.D0)
      REN(3)=MZEXP
C  SOLVE FOR RENORMALIZED PARAMETERS USING SUBROUTINE DSNLEQ IN CERNLIB
      N=3
      FTOL=1.D-13
      XTOL=1.D-13
      MAXF=300
      IPRT=0
      CALL DSNLEQ(N,REN,RES,FTOL,XTOL,MAXF,IPRT,INFO,RENCND,WORKSP)
C==================================================================
      ALFPI=REN(1)
      MW=REN(2)
      MZ=REN(3)
      SWSQ=1.D0-MW**2/MZ**2
C  CALCULATE POLE POSITION
      SPOLE=POLPOS(0.D0)
      RCALL1=.TRUE.
      BCALL1=.TRUE.
      RETURN
      END
 
      SUBROUTINE RENCND(N,X,F,K)
C*******************************
C  CALCULATES THE DIFFERENCE BETWEEN THE EXPERIMENTALLY MEASURED
C    ELECTROWEAK QUANTITIES AND THOSE CALCULATED FROM THE CURRENT
C    VALUES OF THE RENORMALIZED PARAMETERS. THIS SUBROUTINE IS CALLED
C    BY THE CERNLIB SUBROUTINE, DSNLEQ, AND USED TO SOLVE FOR THE
C    VALUES OF THE RENORMALIZED PARAMETERS IN A PARTICULAR SCHEME.
C
C  RENORMALIZATION PARAMETERS SOLVED FOR
C    X(1)  =  ALPHA/PI
C    X(2)  =  MW
C    X(3)  =  MZ
C
C  RENORMALIZATION CONDITIONS IMPOSED
C    F(1)  =  QED COUPLING CONSTANT, E, TAKES ITS EXPERIMENTAL VALUE
C    F(2)  =  MUON DECAY CONSTANT, G_MU, TAKES ITS EXPERIMENTAL VALUE
C    F(3)  =  ON-SHELL Z0 BOSON MASS TAKES ITS INPUT VALUE
C
      IMPLICIT LOGICAL (A-Z)
      INTEGER N,K
      REAL*8 X(3),F(3)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/EXPERIM/ALFEXP,GMUEXP,MZEXP
      REAL*8 ALFEXP,GMUEXP,MZEXP
      REAL*8 CW,CWSQ,DELTOT,MT,PI,SW,XF
      COMPLEX*16 DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA
      REAL*8 DELTAR
      COMPLEX*16 DOPIAA,ROPIZA,ROPIZZ
      PI=4.D0*DATAN(1.D0)
      ALFPI=X(1)
      MW=X(2)
      MZ=X(3)
      SWSQ=1.D0-MW**2/MZ**2
      SW=DSQRT(SWSQ)
      CWSQ=1.D0-SWSQ
      CW=DSQRT(CWSQ)
      CALL CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C=======================================================
      IF(K.EQ.1)THEN
C  REQUIRE THAT QED COUPLING, E, TAKES ITS MEASURED VALUE
          F(1)=DSQRT(4.D0*PI**2*ALFPI)
     .        /(1.D0+0.5D0*DOPIAA((0.D0,0.D0),SCALE)
     .              +(SW/CW)*ROPIZA((0.D0,0.D0),SCALE)/MZ**2
     .              -(SWSQ*DGG+CWSQ*DGPGP))
     .        -DSQRT(4.D0*PI*ALFEXP)
      ELSE IF(K.EQ.2)THEN
C  REQUIRE THAT MUON DECAY CONSTANT, G_MU, TAKES ITS MEASURED VALUE
          DELTOT=DELTAR(0.D0)
          MT=XQRK(5,1)
          XF=ALFPI/(16.D0*SWSQ)*(MT/MW)**2
          F(2)=ALFPI*PI**2/(DSQRT(2.D0)*MW**2*SWSQ)
     .        *(1.D0/(1.D0-DELTOT)
     .         +2.D0*((CWSQ/SWSQ)*NCOL*XF)**2
     .         +(CWSQ/SWSQ)*NCOL*(2.D0*PI**2-19.D0)*XF**2)
     .        -GMUEXP
      ELSE IF(K.EQ.3)THEN
C  REQUIRE THAT ON-SHELL Z0 BOSON MASS TAKES ON ITS INPUT VALUE
          F(3)=MZ**2+DMZSQ-DREAL(ROPIZZ(DCMPLX(MZ**2),SCALE))
     .        -MZEXP**2
          END IF
      RETURN
      END
 
      SUBROUTINE EXTINT(S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN)
C**********************************************************
C  INITIALIZES THE PARAMETERS OF THE EXTERNAL FERMIONS
C    INTO THE COMMON BLOCK EXTPRM
C
C  S       = MANDELSTAM S FOR THE ANNIHILATION
C  T3E, QE = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE INCOMING FERMION
C  T3F, QF = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE OUTGOING FERMION
C  NCOLF   = NUMBER OF COLOURS IN THE FINAL STATE (NO FINAL STATE QCD)
C  MF      = MASS OF THE FINAL STATE FERMION
C  MINT    = MASS OF THE ISOSPIN PARTNER OF THE FINAL STATE FERMION
C  POLN    = POLARIZATION OF THE INCOMING BEAMS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 S,T3E,QE,T3F,QF,NCOLF,MF,MINT,POLN
      COMMON/EXTPRM/SEXT,T3EEXT,QEEXT,T3FEXT,QFEXT,
     .              NCEXT,MFEXT,MIEXT,POLEXT
      COMMON/SPEEDY/BCALL1,RCALL1
      LOGICAL BCALL1,RCALL1
      REAL*8 SEXT,T3EEXT,QEEXT,T3FEXT,QFEXT,NCEXT,MFEXT,MIEXT,POLEXT
      SEXT=S
      T3EEXT=T3E
      QEEXT=QE
      T3FEXT=T3F
      QFEXT=QF
      NCEXT=NCOLF
      MIEXT=MINT
      MFEXT=MF
      POLEXT=POLN
      RCALL1=.TRUE.
      BCALL1=.TRUE.
      RETURN
      END
 
      SUBROUTINE FMASET(MTOP)
C****************************
C  INITIALIZES THE FERMION MASSES, MTOP = TOP QUARK MASS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 MTOP
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 QUP,QDOWN
      NLPT=6
      NQRK=6
      QUP=2.D0/3.D0
      QDOWN=-1.D0/3.D0
C  ELECTRON NEUTRINO MASS AND CHARGE
      XLPT(1,1) = 0.D0
      XLPT(1,2) = 0.D0
C  ELECTRON MASS AND CHARGE
      XLPT(2,1) = 0.511D-03
      XLPT(2,2) = -1.D0
C  MUON NEUTRINO MASS AND CHARGE
      XLPT(3,1) = 0.D0
      XLPT(3,2) = 0.D0
C  MUON MASS AND CHARGE
      XLPT(4,1) = 0.105D0
      XLPT(4,2) = -1.D0
C  TAU NEUTRINO MASS AND CHARGE
      XLPT(5,1) = 0.D0
      XLPT(5,2) = 0.D0
C  TAU MASS AND CHARGE
      XLPT(6,1) = 1.78D0
      XLPT(6,2) = -1.D0
C  UP QUARK MASS AND CHARGE
      XQRK(1,1) = 0.3D0
      XQRK(1,2) = QUP
C  DOWN QUARK MASS AND CHARGE
      XQRK(2,1) = 0.3D0
      XQRK(2,2) = QDOWN
C  CHARMED QUARK MASS AND CHARGE
      XQRK(3,1) = 1.5D0
      XQRK(3,2) = QUP
C  STRANGE QUARK MASS AND CHARGE
      XQRK(4,1) = 0.7D0
      XQRK(4,2) = QDOWN
C  TOP QUARK MASS AND CHARGE
      XQRK(5,1) = MTOP
      XQRK(5,2) = QUP
C  BOTTOM QUARK MASS AND CHARGE
      XQRK(6,1) = 4.5D0
      XQRK(6,2) = QDOWN
      RETURN
      END
 
      SUBROUTINE PRMZ0P(ALPHA,SIN2W,WMASS,ZMASS,POLE)
C****************************************************
C  EXTRACTS RENORMALIZED PARAMETERS AND POLE POSITION FROM COMMON GSWROB
C    CAN BE USED ONLY AFTER THE SUBROUTINE GSWINT HAS BEEN CALLED
C
C  ALPHA  = THE RENORMALIZED ELECTROMAGNETIC COUPLING
C  SIN2W  = SIN SQUARED OF THE WEAK MIXING ANGLE
C  WMASS  = W BOSON MASS
C  ZMASS  = Z0 BOSON MASS
C  POLE   = POSITION OF THE POLE ON THE COMPLEX S PLANE
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 ALPHA,SIN2W,WMASS,ZMASS
      COMPLEX*16 POLE
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 PI
      PI=4.D0*DATAN(1.D0)
      ALPHA=ALFPI*PI
      SIN2W=SWSQ
      WMASS=MW
      ZMASS=MZ
      POLE=SPOLE
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*       FINITE GAUGE-INVARIANT CORRECTIONS FOR MATRIX ELEMENTS       **
C*                                                                    **
C*                             REFERENCES:                            **
C*                     A. SIRLIN, PRD22(1980)971                      **
C*                   R. G. STUART, PLB262(1991)113                    **
C*                   R. G. STUART, PLB272(1991)353                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION POLPOS(DUMMY)
C***************************
C  THE POSITION OF THE POLE ON THE COMPLEX S-PLANE
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 DUMMY
      COMPLEX*16 POLPOS
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 ALFSPI,BFL,BFR,CW,CWSQ,PI,Q,SW,T3
      COMPLEX*16 DMWSQ,DMZSQ,DGG,DGPGP,DZA,DZW,DZZ,DZZA,
     .           DPIZMZ,PIZAMZ,PIZZMZ,VFZLMZ,VFZRMZ,
     .           BFLCR,BFRCR,MSQ,MPSQ,
     .           LOPIZZ,IMPOLA,POL0,QOPIZZ,REPOLA,
     .           MWSQ,MZSQ
      COMPLEX*16 DOPIZZ,ROPIZA,ROPIZZ
      PI=4.D0*DATAN(1.D0)
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
C  EVALUATE SELF-ENERGY CONTRIBUTIONS
      PIZZMZ=ROPIZZ(MZSQ,SCALE)
      PIZAMZ=ROPIZA(MZSQ,SCALE)
      DPIZMZ=DOPIZZ(MZSQ,SCALE)
C  EVALUATE THE COUNTERTERMS
      CALL CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C=======================================================
C  POLE POSITION TO ONE-LOOP WITH QCD CORRECTIONS
      POL0=MZSQ-PIZZMZ+DMZSQ
C  CALCULATE THE O(ALPHA**2) CORRECTIONS TO THE REAL PART OF POLE
      REPOLA=-DIMAG(PIZZMZ)*DIMAG(DPIZMZ)
C-----------------------------------------------------------------------
C  IN THE ON-SHELL SCHEME THE REAL PART THIS IS CORRECT TO O(ALPHA**2)
C  IN THE MS-BAR SCHEME ADDITIONAL O(ALPHA**2) CORRECTIONS NEED TO BE
C    ADDED TO THE MASS COUNTER-TERM DMZSQ
C  IN BOTH CASES THE REAL(SPOLE) IS EFFECTIVELY OBTAINED FROM THE
C    ON-SHELL CALCULATION SO THAT CLOSE AGREEMENT SHOULD NOT BE TAKEN
C    AS AN ESTIMATE FOR THE SIZE OF SCHEME DEPENDENCE.
C-----------------------------------------------------------------------
C  CALCULATE THE O(ALPHA**2) CORRECTIONS TO THE IMAGINARY PART OF POLE
C    ASSUMING THE Z0 ONLY DECAYS INTO MASSLESS FERMION PAIRS
C
C  CONTRIBUTIONS FROM DECAYS TO LEPTON PAIRS
      LOPIZZ=(0.D0,0.D0)
      DO 10 I=1,NLPT
          MSQ=DCMPLX(XLPT(I,1)**2)
          MPSQ=DCMPLX(XLPT(I+2*MOD(I,2)-1,1)**2)
          IF(DREAL(MZSQ).GT.4.D0*DREAL(MSQ))THEN
              T3=DBLE(MOD(I,2))-0.5D0
              Q=XLPT(I,2)
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              CALL VERTZ1(MZSQ,T3,Q,MPSQ,SCALE,VFZLMZ,VFZRMZ)
C============================================================
              BFLCR=-0.5D0*DPIZMZ*BFL
     .              +VFZLMZ
     .              -(PIZAMZ/MZSQ)*Q
     .              +(DGG*CWSQ+DGPGP*SWSQ)*BFL
     .              +2.D0*SW*CW*(DGG-DGPGP)*Q
     .              +0.5D0*(-PIZZMZ+DMZSQ)*BFL/MZSQ
     .              +3.D0/8.D0*ALFPI*Q**2*BFL
              BFRCR=-0.5D0*DPIZMZ*BFR
     .              +VFZRMZ
     .              -(PIZAMZ/MZSQ)*Q
     .              +(DGG*CWSQ+DGPGP*SWSQ)*BFR
     .              +2.D0*SW*CW*(DGG-DGPGP)*Q
     .              +0.5D0*(-PIZZMZ+DMZSQ)*BFR/MZSQ
     .              +3.D0/8.D0*ALFPI*Q**2*BFR
              LOPIZZ=LOPIZZ
     .              +BFL*DREAL(BFLCR)+BFR*DREAL(BFRCR)
              END IF
10        CONTINUE
C  CONTRIBUTIONS FROM DECAYS TO QUARK PAIRS
      QOPIZZ=(0.D0,0.D0)
      DO 20 I=1,NQRK
          MSQ=DCMPLX(XQRK(I,1)**2)
          MPSQ=DCMPLX(XQRK(I+2*MOD(I,2)-1,1)**2)
          IF(DREAL(MZSQ).GT.4.D0*DREAL(MSQ))THEN
              T3=DBLE(MOD(I,2))-0.5D0
              Q=XQRK(I,2)
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              CALL VERTZ1(MZSQ,T3,Q,MPSQ,SCALE,VFZLMZ,VFZRMZ)
C============================================================
              BFLCR=-0.5D0*DPIZMZ*BFL
     .              +VFZLMZ
     .              -(PIZAMZ/MZSQ)*Q
     .              +(DGG*CWSQ+DGPGP*SWSQ)*BFL
     .              +2.D0*SW*CW*(DGG-DGPGP)*Q
     .              +0.5D0*(-PIZZMZ+DMZSQ)*BFL/MZSQ
     .              +3.D0/8.D0*ALFPI*Q**2*BFL
              BFRCR=-0.5D0*DPIZMZ*BFR
     .              +VFZRMZ
     .              -(PIZAMZ/MZSQ)*Q
     .              +(DGG*CWSQ+DGPGP*SWSQ)*BFR
     .              +2.D0*SW*CW*(DGG-DGPGP)*Q
     .              +0.5D0*(-PIZZMZ+DMZSQ)*BFR/MZSQ
     .              +3.D0/8.D0*ALFPI*Q**2*BFR
              QOPIZZ=QOPIZZ
     .              +BFL*DREAL(BFLCR)+BFR*DREAL(BFRCR)
              END IF
20        CONTINUE
      IMPOLA=-(0.D0,1.D0)*ALFPI*PI*MZSQ/3.D0*(LOPIZZ+NCOL*QOPIZZ)
C  POLE POSITION TO CORRECT TO O(ALPHA**2)
      POLPOS=POL0+REPOLA+IMPOLA
      RETURN
      END
 
      SUBROUTINE RESID(S,T3E,QE,T3F,QF,MINTSQ,
     .                 RESLL,RESLR,RESRL,RESRR)
C**********************************************
C  RESIDUES AT THE POLE WITH AN OVERALL FACTOR OF -E**2 REMOVED
C
C  S = MANDELSTAM VARIABLE S
C  T3E, QE = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE INCOMING FERMION
C  T3F, QF = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE OUTGOING FERMION
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C
C  RESLL = RESIDUE FOR INCOMING LEFT HELICITY
C                  AND OUTGOING LEFT HELICITY FERMIONS
C  RESLR = RESIDUE FOR INCOMING LEFT HELICITY
C                  AND OUTGOING RIGHT HELICITY FERMIONS
C  RESRL = RESIDUE FOR INCOMING RIGHT HELICITY
C                  AND OUTGOING LEFT HELICITY FERMIONS
C  RESRR = RESIDUE FOR INCOMING RIGHT HELICITY
C                  AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3E,QE,T3F,QF
      COMPLEX*16 S,MINTSQ,RESLL,RESLR,RESRL,RESRR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      COMMON/SPEEDY/BCALL1,RCALL1
      LOGICAL BCALL1,RCALL1
      REAL*8 BEL,BER,BFL,BFR,CW,CWSQ,SW
      COMPLEX*16 DMWSQ,DMZSQ,DGG,DGPGP,DZA,DZW,DZZ,DZZA,
     .           DPIZMZ,PIZAMZ,
     .           VEZLMZ,VEZRMZ,VFZLMZ,VFZRMZ,
     .           RESLL0,RESLR0,RESRL0,RESRR0,
     .           RESLLA,RESLRA,RESRLA,RESRRA,
     .           MZSQ
      COMPLEX*16 DOPIZZ,ROPIZA
C  DO NOT RECALCULATE RESIDUE IF PARAMETERS HAVE NOT BEEN CHANGED
      IF(.NOT.RCALL1)GO TO 10
      MZSQ=DCMPLX(MZ*MZ)
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
C  LEFT- AND RIGHT-HANDED COUPLING CONSTANTS OF THE Z0
      BEL=(T3E-SWSQ*QE)/(SW*CW)
      BER=-QE*SW/CW
      BFL=(T3F-SWSQ*QF)/(SW*CW)
      BFR=-QF*SW/CW
C  EVALUATE SELF-ENERGY CONTRIBUTIONS
      DPIZMZ=DOPIZZ(MZSQ,SCALE)
      PIZAMZ=ROPIZA(MZSQ,SCALE)
C  CALCULATE VERTEX CONTRIBUTIONS
      CALL VERTZ1(MZSQ,T3E,QE,(0.D0,0.D0),SCALE,VEZLMZ,VEZRMZ)
C=============================================================
      CALL VERTZ1(MZSQ,T3F,QF,MINTSQ,SCALE,VFZLMZ,VFZRMZ)
C========================================================
C  EVALUATE THE COUNTERTERMS
      CALL CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C=======================================================
C  LOWEST ORDER CONTRIBUTIONS TO THE RESIDUES
      RESLL0=DCMPLX(BEL*BFL)
      RESLR0=DCMPLX(BEL*BFR)
      RESRL0=DCMPLX(BER*BFL)
      RESRR0=DCMPLX(BER*BFR)
C  ORDER ALPHA CONTRIBUTIONS TO THE RESIDUES
      RESLLA=-BEL*DPIZMZ*BFL
     .       +VEZLMZ*BFL+BEL*VFZLMZ
     .       -BEL*(PIZAMZ/MZSQ)*QF-QE*(PIZAMZ/MZSQ)*BFL
     .       +BEL*2.D0*(DGG*CWSQ+DGPGP*SWSQ)*BFL
     .       +QE*2.D0*SW*CW*(DGG-DGPGP)*BFL
     .       +BEL*2.D0*SW*CW*(DGG-DGPGP)*QF
      RESLRA=-BEL*DPIZMZ*BFR
     .       +VEZLMZ*BFR+BEL*VFZRMZ
     .       -BEL*(PIZAMZ/MZSQ)*QF-QE*(PIZAMZ/MZSQ)*BFR
     .       +BEL*2.D0*(DGG*CWSQ+DGPGP*SWSQ)*BFR
     .       +QE*2.D0*SW*CW*(DGG-DGPGP)*BFR
     .       +BEL*2.D0*SW*CW*(DGG-DGPGP)*QF
      RESRLA=-BER*DPIZMZ*BFL
     .       +VEZRMZ*BFL+BER*VFZLMZ
     .       -BER*(PIZAMZ/MZSQ)*QF-QE*(PIZAMZ/MZSQ)*BFL
     .       +BER*2.D0*(DGG*CWSQ+DGPGP*SWSQ)*BFL
     .       +QE*2.D0*SW*CW*(DGG-DGPGP)*BFL
     .       +BER*2.D0*SW*CW*(DGG-DGPGP)*QF
      RESRRA=-BER*DPIZMZ*BFR
     .       +VEZRMZ*BFR+BER*VFZRMZ
     .       -BER*(PIZAMZ/MZSQ)*QF-QE*(PIZAMZ/MZSQ)*BFR
     .       +BER*2.D0*(DGG*CWSQ+DGPGP*SWSQ)*BFR
     .       +QE*2.D0*SW*CW*(DGG-DGPGP)*BFR
     .       +BER*2.D0*SW*CW*(DGG-DGPGP)*QF
C  COMPLETE RESIDUES TO ORDER ALPHA
10    RESLL=RESLL0+RESLLA
      RESLR=RESLR0+RESLRA
      RESRL=RESRL0+RESRLA
      RESRR=RESRR0+RESRRA
      RCALL1=.FALSE.
      RETURN
      END
 
      SUBROUTINE BACKGR(S,T,T3E,QE,T3F,QF,MINTSQ,
     .                  BKGDLL,BKGDLR,BKGDRL,BKGDRR)
C***************************************************
C  BACKGROUND CORRECTIONS WITH AN OVERALL FACTOR OF -E**2 REMOVED
C
C  S = MANDELSTAM VARIABLE S
C  T = MANDELSTAM VARIABLE T
C  T3E, QE = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE INCOMING FERMION
C  T3F, QF = WEAK ISOSPIN AND ELECTRIC CHARGE OF THE OUTGOING FERMION
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C
C  BKGDLL = BACKGROUND FOR INCOMING LEFT HELICITY
C                      AND OUTGOING LEFT HELICITY FERMIONS
C  BKGDLR = BACKGROUND FOR INCOMING LEFT HELICITY
C                      AND OUTGOING RIGHT HELICITY FERMIONS
C  BKGDRL = BACKGROUND FOR INCOMING RIGHT HELICITY
C                      AND OUTGOING LEFT HELICITY FERMIONS
C  BKGDRR = BACKGROUND FOR INCOMING RIGHT HELICITY
C                      AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3E,QE,T3F,QF
      COMPLEX*16 S,T,MINTSQ,BKGDLL,BKGDLR,BKGDRL,BKGDRR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      COMMON/SPEEDY/BCALL1,RCALL1
      LOGICAL BCALL1,RCALL1
      REAL*8 BEL,BER,BFL,BFR,CW,CWSQ,EPS,SW
      COMPLEX*16 DMWSQ,DMZSQ,DGG,DGPGP,DZA,DZW,DZZ,DZZA,
     .           DPIAAS,PIZARS,PIZZRS,
     .           BGDLL0,BGDLR0,BGDRL0,BGDRR0,
     .           BGDLLA,BGDLRA,BGDRLA,BGDRRA,
     .           VEALMZ,VEARMZ,VEZLMZ,VEZRMZ,
     .           VFALMZ,VFARMZ,VFZLMZ,VFZRMZ,
     .           VEALS,VEARS,VEZLS,VEZRS,
     .           VFALS,VFARS,VFZLS,VFZRS,
     .           VEZLRS,VEZRRS,VFZLRS,VFZRRS,
     .           BWLL,BWLR,BWRL,BWRR,BZLL,BZLR,BZRL,BZRR,
     .           S1,MZSQ
      COMPLEX*16 DOPIAA,DOPIZZ,ROPIZA,ROPIZZ
C  IF PARAMETERS HAVE NOT BEEN CHANGED THEN ONLY RECALCULATE BOXES
      IF(.NOT.BCALL1)GO TO 10
      MZSQ=DCMPLX(MZ*MZ)
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
C  LEFT- AND RIGHT-HANDED COUPLING CONSTANTS OF THE Z0
      BEL=(T3E-SWSQ*QE)/(SW*CW)
      BER=-QE*SW/CW
      BFL=(T3F-SWSQ*QF)/(SW*CW)
      BFR=-QF*SW/CW
C  STAY AWAY FROM S = MZ**2
      EPS=1.D-8
      IF(CDABS(1.D0-S/MZSQ).LE.EPS)THEN
          S1=MZSQ*(1.D0+EPS)
      ELSE
          S1=S
          END IF
C  EVALUATE SELF-ENERGY CONTRIBUTIONS
      DPIAAS=DOPIAA(S1,SCALE)
      PIZARS=(ROPIZA(S1,SCALE)/S1-ROPIZA(MZSQ,SCALE)/MZSQ)/(S1-MZSQ)
      PIZZRS=ROPIZZ(S1,SCALE)/(S1-MZSQ)**2
     .      -DOPIZZ(S1,SCALE)/(S1-MZSQ)
     .      -ROPIZZ(MZSQ,SCALE)/(S1-MZSQ)**2
C  EVALUATE VERTEX CONTRIBUTIONS
      CALL VERTZ1(S1,T3E,QE,(0.D0,0.D0),SCALE,VEZLS,VEZRS)
C=========================================================
      CALL VERTA1(S1,T3E,QE,(0.D0,0.D0),SCALE,VEALS,VEARS)
C=========================================================
      CALL VERTZ1(S1,T3F,QF,MINTSQ,SCALE,VFZLS,VFZRS)
C====================================================
      CALL VERTA1(S1,T3F,QF,MINTSQ,SCALE,VFALS,VFARS)
C====================================================
      CALL VERTZ1(MZSQ,T3E,QE,(0.D0,0.D0),SCALE,VEZLMZ,VEZRMZ)
C=============================================================
      CALL VERTZ1(MZSQ,T3F,QF,MINTSQ,SCALE,VFZLMZ,VFZRMZ)
C========================================================
      VEZLRS=(VEZLS-VEZLMZ)/(S1-MZSQ)
      VEZRRS=(VEZRS-VEZRMZ)/(S1-MZSQ)
      VFZLRS=(VFZLS-VFZLMZ)/(S1-MZSQ)
      VFZRRS=(VFZRS-VFZRMZ)/(S1-MZSQ)
C  EVALUATE THE COUNTERTERMS
      CALL CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C=======================================================
C  LOWEST ORDER CONTRIBUTIONS TO THE BACKGROUNDS
      BGDLL0=QE*QF/S
      BGDLR0=QE*QF/S
      BGDRL0=QE*QF/S
      BGDRR0=QE*QF/S
C  ORDER ALPHA CONTRIBUTIONS TO THE BACKGROUNDS
      BGDLLA=-QE*(DPIAAS/S)*QF
     .      +VEALS*(1.D0/S)*QF+QE*(1.D0/S)*VFALS
     .      -QE*PIZARS*BFL-BEL*PIZARS*QF
     .      -BEL*PIZZRS*BFL
     .      +VEZLRS*BFL+BEL*VFZLRS
     .      +QE*2.D0*(SWSQ*DGG+CWSQ*DGPGP)/S*QF
      BGDLRA=-QE*(DPIAAS/S)*QF
     .      +VEALS*(1.D0/S)*QF+QE*(1.D0/S)*VFARS
     .      -QE*PIZARS*BFR-BEL*PIZARS*QF
     .      -BEL*PIZZRS*BFR
     .      +VEZLRS*BFR+BEL*VFZRRS
     .      +QE*2.D0*(SWSQ*DGG+CWSQ*DGPGP)/S*QF
      BGDRLA=-QE*(DPIAAS/S)*QF
     .      +VEARS*(1.D0/S)*QF+QE*(1.D0/S)*VFALS
     .      -QE*PIZARS*BFL-BER*PIZARS*QF
     .      -BER*PIZZRS*BFL
     .      +VEZRRS*BFL+BER*VFZLRS
     .      +QE*2.D0*(SWSQ*DGG+CWSQ*DGPGP)/S*QF
      BGDRRA=-QE*(DPIAAS/S)*QF
     .      +VEARS*(1.D0/S)*QF+QE*(1.D0/S)*VFARS
     .      -QE*PIZARS*BFR-BER*PIZARS*QF
     .      -BER*PIZZRS*BFR
     .      +VEZRRS*BFR+BER*VFZRRS
     .      +QE*2.D0*(SWSQ*DGG+CWSQ*DGPGP)/S*QF
C  EVALUATE BOX DIAGRAM CONTRIBUTIONS
10    CALL BOXW(S,T,T3E,QE,T3F,QF,MINTSQ,
     .          BWLL,BWLR,BWRL,BWRR)
C========================================
      CALL BOXZ(S,T,T3E,QE,T3F,QF,(0.D0,0.D0),
     .          BZLL,BZLR,BZRL,BZRR)
C=============================================
C  COMPLETE BACKGROUNDS TO ORDER ALPHA
      BKGDLL=BGDLL0+BGDLLA+BWLL+BZLL
      BKGDLR=BGDLR0+BGDLRA+BWLR+BZLR
      BKGDRL=BGDRL0+BGDRLA+BWRL+BZRL
      BKGDRR=BGDRR0+BGDRRA+BWRR+BZRR
      BCALL1=.FALSE.
      RETURN
      END
 
      FUNCTION DELTAR(DUMMY)
C***************************
C  DELTA R FROM THE RADIATIVE CORRECTIONS TO THE MUON LIFETIME
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 DUMMY,DELTAR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 CCNSTA,CWSQ,VERTS,WSELFE,ZWBOX
      COMPLEX*16 DMWSQ,DMZSQ,DZA,DZW,DZZA,DZZ,DGG,DGPGP,MWSQ,MZSQ
      COMPLEX*16 DOPIAA,ROPIWW,ROPIZZ
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      CWSQ=1.D0-SWSQ
      CCNSTA=ALFPI/(4.D0*SWSQ)
      CALL CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C=======================================================
      WSELFE=ROPIWW((0.D0,0.D0),SCALE)/MWSQ
      VERTS=CCNSTA*(-4.D0*CDLOG(MZSQ/SCALE)+6.D0
     C            +(6.D0*CWSQ/SWSQ-4.D0-CWSQ)*DLOG(CWSQ))
      ZWBOX=CCNSTA/(2.D0*SWSQ)
     C     *(3.D0*SWSQ*SWSQ-5.D0*CWSQ*CWSQ)*DLOG(CWSQ)
      DELTAR=WSELFE+VERTS+ZWBOX-DMWSQ/MWSQ+2.D0*DGG
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*             LIBRARY OF 1PI STANDARD MODEL CORRECTIONS              **
C*                                                                    **
C*                             REFERENCES:                            **
C*               B. GRZADKOWSKI ET AL., NPB281(1987)18                **
C*           B. W. LYNN AND R. G. STUART, PLB252(1990)676             **
C*                                                                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      SUBROUTINE CTSET(DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA)
C*************************************************************
C  CALCULATES THE COUNTERTERMS FOR THE RENORMALIZATION SCHEME
C    SPECIFIED IN THE COMMON BLOCK GSWROB
C
C     NSCHEM = 1    ON-SHELL RENORMALIZATION SCHEME
C            = 2    MS-BAR RENORMALIZATION SCHEME
C
C      SCALE = FOR SCHEME = 1 THIS IS AN ARBITRARY SCALE,
C                  PHYSICAL RESULTS INDEPENDENT OF SCALE
C              FOR NSCHEM = 2 THIS IS THE 'T HOOFT MASS,
C                  PHYSICAL RESULTS WEAKLY DEPENDENT SCALE
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 DMWSQ,DMZSQ,DGG,DGPGP,DZW,DZA,DZZ,DZZA
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 CW,CWSQ,SW
      COMPLEX*16 DELMIN,MWSQ,MZSQ,PIZA0
      COMPLEX*16 DOPIAA,ROPIWW,ROPIZA,ROPIZZ
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      IF(SCHEME.EQ.1)THEN
          DMWSQ=DCMPLX(DREAL(ROPIWW(MWSQ,SCALE)))
          DMZSQ=DCMPLX(DREAL(ROPIZZ(MZSQ,SCALE)))
          DELMIN=(DMWSQ/MWSQ-DMZSQ/MZSQ)/(4.D0*SWSQ)
          PIZA0=DCMPLX(DREAL(ROPIZA((0.D0,0.D0),SCALE)))
          DZA=DCMPLX(DREAL(-DOPIAA((0.D0,0.D0),SCALE)))
          DZZA=DCMPLX(DREAL(-4.D0*SW*CW*DELMIN+2.D0/MZSQ*PIZA0))
          DZW=DCMPLX(DREAL(DZA+CW/SW*DZZA))
          DZZ=DCMPLX(DREAL(DZA-(SWSQ-CWSQ)/(SW*CW)*DZZA))
          DGG=DCMPLX(DREAL(-0.5D0*DZA
     .                     +(SW/CW)*PIZA0/MZSQ+2.D0*CWSQ*DELMIN))
          DGPGP=DCMPLX(DREAL(-0.5D0*DZA
     .                       +(SW/CW)*PIZA0/MZSQ-2.D0*SWSQ*DELMIN))
      ELSE IF(SCHEME.EQ.2)THEN
          DMWSQ=(0.D0,0.D0)
          DMZSQ=(0.D0,0.D0)
          DZA=(0.D0,0.D0)
          DZZA=(0.D0,0.D0)
          DZW=(0.D0,0.D0)
          DZZ=(0.D0,0.D0)
          DGG=(0.D0,0.D0)
          DGPGP=(0.D0,0.D0)
      ELSE
          STOP
     .    'INVALID RENORMALIZATION SCHEME SPECIFIED IN SUBROUTINE CTSET'
          END IF
      RETURN
      END
 
C-----------------------------------------------------------------------
C               CONTRIBUTIONS TO THE W BOSON SELF-ENERGY
C-----------------------------------------------------------------------
 
      FUNCTION ROPIWW(S,XSC)
C***************************
C  REGULARIZED ONE PARTICLE IRREDUCIBLE W BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,ROPIWW
      COMPLEX*16 BPIWW1,LPIWW1,QPIWW1
      ROPIWW=BPIWW1(S,XSC)+LPIWW1(S,XSC)+QPIWW1(S,XSC)
      RETURN
      END
 
      FUNCTION BPIWW1(S,XSC)
C***************************
C  ONE-LOOP BOSONIC CONTRIBUTION TO THE 1PI W BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,BPIWW1
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 CWSQ
      COMPLEX*16 MHSQ,MWSQ,MZSQ
      COMPLEX*16 B0
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      MHSQ=DCMPLX(MH*MH)
      CWSQ=1.D0-SWSQ
      IF(S.EQ.(0.D0,0.D0))THEN
          BPIWW1=((19.D0*CWSQ-2.D0)*MHSQ-(16.D0*CWSQ+1.D0)*MWSQ)*MWSQ
     .             /(4.D0*SWSQ*(MWSQ-MHSQ))*CDLOG(MWSQ/XSC)
     .           +3.D0*MHSQ*MWSQ/(4.D0*(MWSQ-MHSQ))*CDLOG(MHSQ/XSC)
     .           +(8.D0*CWSQ*CWSQ+13.D0*CWSQ-4.D0)*MZSQ/(4.D0*SWSQ)
     .                        *CDLOG(MZSQ/XSC)
     .           +(7.D0*MZSQ-34.D0*MWSQ-MHSQ)/8.D0
      ELSE
          BPIWW1=(((1.D0-40.D0*CWSQ)*S*S
     .           +(-16.D0*CWSQ-54.D0+10.D0/CWSQ)*S*MWSQ
     .           +SWSQ*SWSQ*(8.D0*CWSQ+1.D0)*MZSQ*MZSQ)
     .                      *B0(-S,MWSQ,MZSQ,XSC)
     .           +((S+MWSQ-MHSQ)**2+8.D0*S*MWSQ)*B0(-S,MWSQ,MHSQ,XSC)
     .           +8.D0*SWSQ*(-5.D0*S*S-2.D0*S*MWSQ+MWSQ*MWSQ)
     .                       *B0(-S,MWSQ,(0.D0,0.D0),XSC)
     .         -(20.D0*S-2.D0*MWSQ+MZSQ+MHSQ)*MWSQ*CDLOG(MWSQ/XSC)
     .         -(8.D0*CWSQ+1.D0)*(2.D0*S-SWSQ*MZSQ)*MZSQ*CDLOG(MZSQ/XSC)
     .         -(MWSQ-MHSQ+2.D0*S)*MHSQ*CDLOG(MHSQ/XSC)
     .         -4.D0/3.D0*S*S-SWSQ*(8.D0*CWSQ+SWSQ)*MZSQ*MZSQ
     .         -(MWSQ-MHSQ)**2)/(12.D0*S)
          END IF
      BPIWW1=ALFPI/(4.D0*SWSQ)*BPIWW1
      RETURN
      END
 
      FUNCTION LPIWW1(S,XSC)
C***************************
C  ONE-LOOP LEPTONIC CONTRIBUTION TO THE 1PI W BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,LPIWW1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      COMPLEX*16 B0FN,M1SQ,M2SQ
      COMPLEX*16 B0
      LPIWW1=(0.D0,0.D0)
      DO 10 I=1,NLPT-1,2
          M1SQ=DCMPLX(XLPT(I,1)**2)
          M2SQ=DCMPLX(XLPT(I+1,1)**2)
          IF(S.EQ.(0.D0,0.D0))THEN
              IF(M1SQ.EQ.M2SQ)THEN
                  LPIWW1=LPIWW1+M1SQ*CDLOG(M1SQ/XSC)
              ELSE
                  IF(M1SQ.NE.(0.D0,0.D0))THEN
                      LPIWW1=LPIWW1
     .                      -M1SQ*(M1SQ/(M2SQ-M1SQ)*CDLOG(M1SQ/XSC)
     .                      +0.5D0)/2.D0
                      END IF
                  IF(M2SQ.NE.(0.D0,0.D0))THEN
                      LPIWW1=LPIWW1
     .                      -M2SQ*(M2SQ/(M1SQ-M2SQ)*CDLOG(M2SQ/XSC)
     .                      +0.5D0)/2.D0
                      END IF
                  END IF
          ELSE
              LPIWW1=LPIWW1
     .              +((S-(M1SQ+M2SQ)/2.D0-(M1SQ-M2SQ)**2/(2.D0*S))
     .                             *B0(-S,M1SQ,M2SQ,XSC)
     .              +(M1SQ-M2SQ)**2/(2.D0*S)-S/3.D0)/3.D0
              IF(M1SQ.NE.(0.D0,0.D0))THEN
                  LPIWW1=LPIWW1-(-1.D0+(M1SQ-M2SQ)/(2.D0*S))
     .                          *M1SQ*CDLOG(M1SQ/XSC)/3.D0
                  END IF
              IF(M2SQ.NE.(0.D0,0.D0))THEN
                  LPIWW1=LPIWW1-(-1.D0+(M2SQ-M1SQ)/(2.D0*S))
     .                         *M2SQ*CDLOG(M2SQ/XSC)/3.D0
                  END IF
              END IF
10        CONTINUE
      LPIWW1=ALFPI/(4.D0*SWSQ)*LPIWW1
      RETURN
      END
 
      FUNCTION QPIWW1(S,XSC)
C***************************
C  ONE-LOOP QUARK CONTRIBUTION TO THE 1PI W BOSON SELF-ENERGY
C    WITH QCD CORRECTIONS INCLUDED
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,QPIWW1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      COMPLEX*16 B0FN,M1SQ,M2SQ,PI,QCDCOR
      COMPLEX*16 B0,QCDMSA,QCDMSV
      PI=4.D0*DATAN(1.D0)
      QPIWW1=(0.D0,0.D0)
      DO 10 I=1,NQRK-1,2
          M1SQ=DCMPLX(XQRK(I,1)**2)
          M2SQ=DCMPLX(XQRK(I+1,1)**2)
          IF(S.EQ.(0.D0,0.D0))THEN
              IF(M1SQ.EQ.M2SQ)THEN
                  QPIWW1=QPIWW1+M1SQ*CDLOG(M1SQ/XSC)
              ELSE
                  IF(M1SQ.NE.(0.D0,0.D0))THEN
                      QPIWW1=QPIWW1
     .                      -M1SQ*(M1SQ/(M2SQ-M1SQ)*CDLOG(M1SQ/XSC)
     .                      +0.5D0)/2.D0
                      END IF
                  IF(M2SQ.NE.(0.D0,0.D0))THEN
                      QPIWW1=QPIWW1
     .                      -M2SQ*(M2SQ/(M1SQ-M2SQ)*CDLOG(M2SQ/XSC)
     .                      +0.5D0)/2.D0
                      END IF
                  END IF
          ELSE
              QPIWW1=QPIWW1
     .              +((S-(M1SQ+M2SQ)/2.D0-(M1SQ-M2SQ)**2/(2.D0*S))
     .                             *B0(-S,M1SQ,M2SQ,XSC)
     .              +(M1SQ-M2SQ)**2/(2.D0*S)-S/3.D0)/3.D0
              IF(M1SQ.NE.(0.D0,0.D0))THEN
                  QPIWW1=QPIWW1-(-1.D0+(M1SQ-M2SQ)/(2.D0*S))
     .                          *M1SQ*CDLOG(M1SQ/XSC)/3.D0
                  END IF
              IF(M2SQ.NE.(0.D0,0.D0))THEN
                  QPIWW1=QPIWW1-(-1.D0+(M2SQ-M1SQ)/(2.D0*S))
     .                         *M2SQ*CDLOG(M2SQ/XSC)/3.D0
                  END IF
              END IF
C  CALCULATE THE QCD CORRECTIONS
          QCDCOR=2.D0*PI**2*(QCDMSV(S,M1SQ,M2SQ,XSC)
     .                      +QCDMSA(S,M1SQ,M2SQ,XSC))
          QPIWW1=QPIWW1+QCDCOR
10        CONTINUE
      QPIWW1=ALFPI/(4.D0*SWSQ)*NCOL*QPIWW1
      RETURN
      END
 
C-----------------------------------------------------------------------
C               CONTRIBUTIONS TO THE Z BOSON SELF-ENERGY
C-----------------------------------------------------------------------
 
      FUNCTION ROPIZZ(S,XSC)
C***************************
C  REGULARIZED ONE PARTICLE IRREDUCIBLE Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,ROPIZZ
      COMPLEX*16 BPIZZ1,LPIZZ1,QPIZZ1
      ROPIZZ=BPIZZ1(S,XSC)+LPIZZ1(S,XSC)+QPIZZ1(S,XSC)
      RETURN
      END
 
      FUNCTION BPIZZ1(S,XSC)
C***************************
C  ONE-LOOP BOSONIC CONTRIBUTION TO THE 1PI Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,BPIZZ1
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 CWSQ
      COMPLEX*16 B0FN,MHSQ,MWSQ,MZSQ
      COMPLEX*16 B0
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      MHSQ=DCMPLX(MH*MH)
      CWSQ=1.D0-SWSQ
      IF(S.EQ.(0.D0,0.D0))THEN
          BPIZZ1=(2.D0*(CWSQ-SWSQ)*MWSQ*CDLOG(MWSQ/XSC)
     .         +3.D0*MZSQ*MHSQ/(4.D0*(MZSQ-MHSQ))*CDLOG(MHSQ/XSC)
     .         +(4.D0*MZSQ-MHSQ)*MZSQ/(4.D0*(MHSQ-MZSQ))*CDLOG(MZSQ/XSC)
     .         +(7.D0*MZSQ-MHSQ)/8.D0)/(SWSQ*CWSQ)
      ELSE
          BPIZZ1=((-(9.D0*CWSQ*CWSQ+CWSQ-0.25D0)*S
     .             -(12.D0*CWSQ*CWSQ+8.D0*CWSQ-5.D0)*MWSQ)
     .                       *4.D0*S*B0(-S,MWSQ,MWSQ,XSC)
     .             -(12.D0*CWSQ*CWSQ-4.D0*CWSQ+1.D0)
     .                       *4.D0*S*MWSQ*CDLOG(MWSQ/XSC)
     .          +((S+MZSQ-MHSQ)**2+8.D0*S*MZSQ)*B0(-S,MZSQ,MHSQ,XSC)
     .          -(2.D0*S-MZSQ+MHSQ)*MZSQ*CDLOG(MZSQ/XSC)
     .          -(2.D0*S-MHSQ+MZSQ)*MHSQ*CDLOG(MHSQ/XSC)
     .          -4.D0/3.D0*S*S*(CWSQ-SWSQ)-(MZSQ-MHSQ)**2)
     .          /(12.D0*S*SWSQ*CWSQ)
          END IF
      BPIZZ1=ALFPI/4.D0*BPIZZ1
      RETURN
      END
 
      FUNCTION LPIZZ1(S,XSC)
C***************************
C  ONE-LOOP LEPTONIC CONTRIBUTION TO THE 1PI Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,LPIZZ1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,Q,SW,T3
      COMPLEX*16 B0FN,MSQ
      COMPLEX*16 B0
      SW=DSQRT(SWSQ)
      CWSQ=1.D0-SWSQ
      CW=DSQRT(CWSQ)
      LPIZZ1=(0.D0,0.D0)
      DO 10 I=1,NLPT
          MSQ=DCMPLX(XLPT(I,1)**2)
          IF(S.NE.(0.D0,0.D0).OR.MSQ.NE.(0.D0,0.D0))THEN
              T3=DBLE(MOD(I,2))-0.5D0
              Q=XLPT(I,2)
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              B0FN=B0(-S,MSQ,MSQ,XSC)
              LPIZZ1=LPIZZ1
     .              +(BFL**2+BFR**2)*2.D0/3.D0*
     .                ((S+2.D0*MSQ)*B0FN-S/3.D0)
     .              -2.D0*MSQ*(BFL-BFR)**2*B0FN
              IF(MSQ.NE.(0.D0,0.D0))THEN
                  LPIZZ1=LPIZZ1+(BFL**2+BFR**2)*4.D0/3.D0*MSQ
     .                          *CDLOG(MSQ/XSC)
                  END IF
              END IF
10        CONTINUE
      LPIZZ1=ALFPI/4.D0*LPIZZ1
      RETURN
      END
 
      FUNCTION QPIZZ1(S,XSC)
C***************************
C  ONE-LOOP QUARK CONTRIBUTION TO THE 1PI Z BOSON SELF-ENERGY
C    WITH QCD CORRECTIONS INCLUDED
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,QPIZZ1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,PI,Q,SW,T3
      COMPLEX*16 B0FN,MSQ,QCDCOR
      COMPLEX*16 B0,QCDMSA,QCDMSV
      PI=4.D0*DATAN(1.D0)
      SW=DSQRT(SWSQ)
      CWSQ=1.D0-SWSQ
      CW=DSQRT(CWSQ)
      QPIZZ1=(0.D0,0.D0)
      DO 10 I=1,NQRK
          MSQ=DCMPLX(XQRK(I,1)**2)
          IF(S.NE.(0.D0,0.D0).OR.MSQ.NE.(0.D0,0.D0))THEN
              T3=DBLE(MOD(I,2))-0.5D0
              Q=XQRK(I,2)
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              B0FN=B0(-S,MSQ,MSQ,XSC)
              QPIZZ1=QPIZZ1
     .              +(BFL**2+BFR**2)*2.D0/3.D0*
     .                ((S+2.D0*MSQ)*B0FN-S/3.D0)
     .              -2.D0*MSQ*(BFL-BFR)**2*B0FN
              IF(MSQ.NE.(0.D0,0.D0))THEN
                  QPIZZ1=QPIZZ1+(BFL**2+BFR**2)*4.D0/3.D0*MSQ
     .                          *CDLOG(MSQ/XSC)
                  END IF
              END IF
C  CALCULATE THE QCD CORRECTIONS
          QCDCOR=16.D0*PI**2
     .                *(((BFL+BFR)/2.D0)**2*QCDMSV(S,MSQ,MSQ,XSC)
     .                 +((BFL-BFR)/2.D0)**2*QCDMSA(S,MSQ,MSQ,XSC))
          QPIZZ1=QPIZZ1+QCDCOR
10        CONTINUE
      QPIZZ1=ALFPI/4.D0*NCOL*QPIZZ1
      RETURN
      END
 
C-----------------------------------------------------------------------
C                 CONTRIBUTIONS TO THE Z-PHOTON MIXING
C-----------------------------------------------------------------------
 
      FUNCTION ROPIZA(S,XSC)
C***************************
C  REGULARIZED ONE PARTICLE IRREDUCIBLE Z-PHOTON MIXING
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,ROPIZA
      COMPLEX*16 BPIZA1,LPIZA1,QPIZA1
      ROPIZA=BPIZA1(S,XSC)+LPIZA1(S,XSC)+QPIZA1(S,XSC)
      RETURN
      END
 
      FUNCTION BPIZA1(S,XSC)
C***************************
C  ONE-LOOP BOSONIC CONTRIBUTION TO THE 1PI Z-PHOTON MIXING
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,BPIZA1
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 CW,CWSQ,SW
      COMPLEX*16 MWSQ
      COMPLEX*16 B0
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      MWSQ=DCMPLX(MW*MW)
      BPIZA1=((-(9.D0*CWSQ+0.5D0)*S
     .          -4.D0*(3.D0*CWSQ+1.D0)*MWSQ)*B0(-S,MWSQ,MWSQ,XSC)
     .          -2.D0*(6.D0*CWSQ-1.D0)*MWSQ*CDLOG(MWSQ/XSC)
     .          -S/3.D0)/(3.D0*SW*CW)
      BPIZA1=ALFPI/4.D0*BPIZA1
      RETURN
      END
 
 
      FUNCTION LPIZA1(S,XSC)
C***************************
C  ONE-LOOP LEPTONIC CONTRIBUTION TO THE 1PI Z-PHOTON MIXING
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,LPIZA1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,Q,SW,T3
      COMPLEX*16 MSQ
      COMPLEX*16 B0
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      LPIZA1=(0.D0,0.D0)
      DO 10 I=1,NLPT
          T3=DBLE(MOD(I,2))-0.5D0
          Q=XLPT(I,2)
          MSQ=DCMPLX(XLPT(I,1)**2)
          IF(Q.NE.0.D0)THEN
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              LPIZA1=LPIZA1
     .          +(BFL+BFR)*Q*2.D0/3.D0*((S+2.D0*MSQ)*B0(-S,MSQ,MSQ,XSC)
     .                                 +2.D0*MSQ*CDLOG(MSQ/XSC)-S/3.D0)
              END IF
10        CONTINUE
      LPIZA1=ALFPI/4.D0*LPIZA1
      RETURN
      END
 
      FUNCTION QPIZA1(S,XSC)
C***************************
C  ONE-LOOP QUARK CONTRIBUTION TO THE 1PI Z-PHOTON MIXING
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,QPIZA1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,PI,Q,SW,T3
      COMPLEX*16 MSQ,QCDCOR
      COMPLEX*16 B0,QCDMSA,QCDMSV
      PI=4.D0*DATAN(1.D0)
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      QPIZA1=(0.D0,0.D0)
      DO 10 I=1,NQRK
          T3=DBLE(MOD(I,2))-0.5D0
          Q=XQRK(I,2)
          MSQ=DCMPLX(XQRK(I,1)**2)
          IF(Q.NE.0.D0)THEN
              BFL=(T3-SWSQ*Q)/(SW*CW)
              BFR=-SWSQ*Q/(SW*CW)
              QPIZA1=QPIZA1
     .          +(BFL+BFR)*Q*2.D0/3.D0*((S+2.D0*MSQ)*B0(-S,MSQ,MSQ,XSC)
     .                                 +2.D0*MSQ*CDLOG(MSQ/XSC)-S/3.D0)
C  CALCULATE THE QCD CORRECTIONS
              QCDCOR=16.D0*PI**2*Q*(BFL+BFR)/2.D0*QCDMSV(S,MSQ,MSQ,XSC)
              QPIZA1=QPIZA1+QCDCOR
              END IF
10        CONTINUE
      QPIZA1=ALFPI/4.D0*NCOL*QPIZA1
      RETURN
      END
 
C-----------------------------------------------------------------------
C            CONTRIBUTIONS TO THE PHOTON SELF-ENERGY OVER S
C-----------------------------------------------------------------------
 
      FUNCTION DOPIAA(S,XSC)
C***************************
C  REGULARIZED ONE PARTICLE IRREDUCIBLE PHOTON SELF-ENERGY OVER S
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,DOPIAA
      COMPLEX*16 BDPIA1,LDPIA1,QDPIA1
      DOPIAA=BDPIA1(S,XSC)+LDPIA1(S,XSC)+QDPIA1(S,XSC)
      RETURN
      END
 
      FUNCTION BDPIA1(S,XSC)
C***************************
C  ONE-LOOP BOSONIC CONTRIBUTION TO THE 1PI PHOTON SELF-ENERGY OVER S
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,BDPIA1
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      COMPLEX*16 MWSQ
      COMPLEX*16 B0
      MWSQ=DCMPLX(MW*MW)
      IF(S.EQ.0.D0)THEN
          BDPIA1=3.D0*CDLOG(MWSQ/XSC)-2.D0/3.D0
      ELSE
          BDPIA1=-(3.D0+4.D0*MWSQ/S)*B0(-S,MWSQ,MWSQ,XSC)
     .            -4.D0*MWSQ/S*CDLOG(MWSQ/XSC)
          END IF
      BDPIA1=ALFPI/4.D0*BDPIA1
      RETURN
      END
 
      FUNCTION LDPIA1(S,XSC)
C***************************
C  ONE-LOOP LEPTONIC CONTRIBUTION TO THE 1PI PHOTON SELF-ENERGY OVER S
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,LDPIA1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 QSQ,T3
      INTEGER I
      COMPLEX*16 MSQ
      COMPLEX*16 B0
      LDPIA1=(0.D0,0.D0)
      DO 10 I=1,NLPT
          QSQ=XLPT(I,2)**2
          MSQ=DCMPLX(XLPT(I,1)**2)
          IF(QSQ.NE.0.D0)THEN
              IF(S.EQ.0.D0)THEN
                  LDPIA1=LDPIA1-QSQ*4.D0/3.D0*CDLOG(MSQ/XSC)
              ELSE
                  LDPIA1=LDPIA1
     .              +QSQ*4.D0/3.D0*((1.D0+2.D0*MSQ/S)*B0(-S,MSQ,MSQ,XSC)
     .              +2.D0*MSQ/S*CDLOG(MSQ/XSC)-1.D0/3.D0)
                  END IF
              END IF
10        CONTINUE
      LDPIA1=ALFPI/4.D0*LDPIA1
      RETURN
      END
 
      FUNCTION QDPIA1(S,XSC)
C***************************
C  ONE-LOOP QUARK CONTRIBUTION TO THE 1PI PHOTON SELF-ENERGY OVER S
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,QDPIA1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 PSQ,QSQ,SNPERT,T3
      INTEGER I
      COMPLEX*16 MSQ
      COMPLEX*16 B0,EXPPIA,FREQRK
C-----------------------------------------------------------------------
C  SNPERT IS THE SQUARED MOMENTUM VALUE BELOW WHICH THE PERTURBATIVE
C    CALCULATION BECOMES UNRELIABLE.
C
C  IF ABS(SNPERT) >= ABS(S) AND S NOT EQUAL 0 THEN
C      AN ERROR MESSAGE IS ISSUED
C  ELSE IF S = 0 THEN
C      DOPIAA FOR LIGHT QUARKS IS CALCULATED FROM DISPERSION RELATIONS
C
C  PSQ IS AN ARBITRARY SCALE USED IN THE DISPERION RELATION CALCULATION
C    RESULT SHOULD BE INSENSITIVE TO PSQ
C  DISPERSION RELATION TREATMENT IS TURNED OFF FOR SNPERT=0
C-----------------------------------------------------------------------
      SNPERT=10.D0**2
      PSQ=91.176D0**2
      QDPIA1=(0.D0,0.D0)
C-----------------------------------------------------------------------
C  TOP QUARK (I=5) AND EXTRA GENERATIONS (I>6) CONTRIBUTIONS ARE ALWAYS
C  CALCULATED PERTURBATIVELY. THE LIGHT U,D,C,S& B QUARKS ARE CALCULATED
C  USING DISPERSION RELATIONS IF ABS(S)<ABS(SNPRT)
C-----------------------------------------------------------------------
      IF(CDABS(S).GE.ABS(SNPERT))THEN
          DO 10 I=1,NQRK
10            QDPIA1=QDPIA1+FREQRK(I,S,XSC)
      ELSE IF(S.EQ.(0.D0,0.D0))THEN
          DO 20 I=1,NQRK
              IF(I.EQ.5.OR.I.GT.6)THEN
                  QDPIA1=QDPIA1+FREQRK(I,S,XSC)
              ELSE
                  QDPIA1=QDPIA1+DREAL(FREQRK(I,DCMPLX(PSQ),XSC))
                  END IF
20            CONTINUE
          QDPIA1=QDPIA1-EXPPIA(DCMPLX(PSQ))
      ELSE
          STOP 'FUNCTION QDPIA1 CALLED IN NON-PERTURBATIVE REGION'
          END IF
      RETURN
      END
 
      FUNCTION FREQRK(I,S,XSC)
C*****************************
C  CONTRIBUTION TO PHOTON VACUUM POLARIZATION FROM INTERNAL LOOPS
C  CONTAINING THE I-TH QUARK, CALCULATED PERTURBATIVELY
C
      IMPLICIT LOGICAL (A-Z)
      INTEGER I
      COMPLEX*16 S,XSC,FREQRK
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 PI,QSQ
      COMPLEX*16 MSQ,QCDCOR
      COMPLEX*16 B0,QCDMSA,QCDMSV,QCDPIP
      PI=4.D0*DATAN(1.D0)
      QSQ=XQRK(I,2)**2
      MSQ=DCMPLX(XQRK(I,1)**2)
      IF(QSQ.EQ.(0.D0,0.D0))THEN
          FREQRK=(0.D0,0.D0)
          RETURN
          END IF
      QCDCOR=(0.D0,0.D0)
      IF(S.NE.(0.D0,0.D0))THEN
          FREQRK=QSQ*4.D0/3.D0*((1.D0+2.D0*MSQ/S)*B0(-S,MSQ,MSQ,XSC)
     .          +2.D0*MSQ/S*CDLOG(MSQ/XSC)-1.D0/3.D0)
C  CALCULATE THE QCD CORRECTIONS
          QCDCOR=16.D0*PI**2*QSQ/S*QCDMSV(S,MSQ,MSQ,XSC)
      ELSE
          FREQRK=-QSQ*4.D0/3.D0*CDLOG(MSQ/XSC)
C  CALCULATE THE QCD CORRECTIONS
          QCDCOR=16.D0*PI**2*QSQ*QCDPIP(S,MSQ,XSC)
      END IF
      FREQRK=(ALFPI/4.D0)*NCOL*(FREQRK+QCDCOR)
      RETURN
      END
 
      FUNCTION EXPPIA(S)
C***********************
C  QUARK CONTRIBUTION TO REAL PART OF THE PHOTON SELF ENERGY
C    SUBTRACTED AT S = 0, OBTAINED USING DISPERSION RELATIONS
C    APPLIED TO THE EXPERMENTAL DATA FOR E+E- -> HADRONS.
C  SEE H. BURKHARDT, TASSO NOTE NO. 192, (1981),
C      H. BURKHARDT, F. JEGERLEHNER, G. PENSO AND C. VERZEGNASSI
C        IN CERN YELLOW REPORT ON "POLARIZATION AT LEP" 1988,
C      H. BURKHARDT, CERN/ALEPH, AUGUST 1988,
C      LATEST UPDATE BY F. JEGERLEHNER, IN PROC. OF 1990 TASI IN
C        ELEMENTARY PARTICLE PHYSICS
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S
      COMPLEX*16 EXPPIA
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      REAL*8 A1,A2,A3,A4,B1,B2,B3,B4,C1,C2,C3,C4,ABSS
      DATA A1,B1,C1/ 0.0D0    ,   0.00835D0,  1.0D0   /
      DATA A2,B2,C2/ 0.0D0    ,   0.00238D0,  3.927D0 /
      DATA A3,B3,C3/ 0.00165D0,   0.003D0  ,  1.0D0   /
      DATA A4,B4,C4/ 0.00221D0,   0.00293D0,  1.0D0   /
      ABSS=CDABS(S)
C  CHECK THAT S IS REAL
      IF(DIMAG(S).NE.0.D0)THEN
          WRITE(6,1000)DREAL(S)
          STOP
          END IF
C  CHECK THAT S IS NOT IN THE HADRONIC RESONANCE REGION
C    THE VALUE 0.072 (GEV)**2 IS THE 2 PION THRESHOLD
      IF(0.072D0.LT.DREAL(S).AND.DREAL(S).LT.79.D0)WRITE(6,1001)DREAL(S)
C  CHECK THAT 0 OR 5 QUARK FLAVOURS ARE SPECIFIED
      IF(NQRK.LT.5.AND.NQRK.NE.0)THEN
          WRITE(6,1002)NQRK
          STOP
          END IF
      IF(NQRK.EQ.0.OR.S.EQ.(0.D0,0.D0))THEN
          EXPPIA=(0.D0,0.D0)
          RETURN
          END IF
      IF(ABSS.LT.0.3D0**2)THEN
          EXPPIA=-A1-B1*DLOG(1.D0+C1*ABSS)
      ELSE IF(ABSS.LT.3.D0**2)THEN
          EXPPIA=-A2-B2*DLOG(1.D0+C2*ABSS)
      ELSE IF(ABSS.LT.100.D0**2)THEN
C          EXPPIA=-A3-B3*DLOG(1.D0+C3*ABSS)
C JEGERLEHNER'S UPDATE:
          EXPPIA=-0.0282D0-0.002980D0*(DLOG(ABSS/91.176D0**2)
     .           +0.005696D0*(ABSS/91.176D0**2-1.D0))
      ELSE
          EXPPIA=-A4-B4*DLOG(1.D0+C4*ABSS)
          END IF
      RETURN
1000  FORMAT(' FUNCTION EXPPIA CALLED WITH S COMPLEX. PARAMETERIZATION
     .NOT AVIALABLE, S = (',E10.2,',',E10.2,') (GEV)**2')
1001  FORMAT(' FUNCTION EXPPIA CALLED IN HADRONIC RESONANCE REGION. PARA
     .METERIZATION IS A POOR APPROXIMATION, S =  ',
     .  F10.2,' (GEV)**2')
1002  FORMAT(' FUNCTION EXPPIA CALLED FOR ',I2,' QUARK FLAVOURS. PARAMET
     .IZATION ONLY AVAILABLE FOR 0 OR 5 LIGHT QUARKS')
      END
 
C-----------------------------------------------------------------------
C  CONTRIBUTIONS TO THE DERIVATIVE WRT S OF THE Z BOSON SELF-ENERGY
C-----------------------------------------------------------------------
 
      FUNCTION DOPIZZ(S,XSC)
C***************************
C  THE DERIVATIVE WRT S OF ONE PARTICLE IRREDUCIBLE Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,DOPIZZ
      COMPLEX*16 BDPIZ1,LDPIZ1,QDPIZ1
      DOPIZZ=BDPIZ1(S,XSC)+LDPIZ1(S,XSC)+QDPIZ1(S,XSC)
      RETURN
      END
 
      FUNCTION BDPIZ1(S,XSC)
C***************************
C  ONE-LOOP BOSONIC CONTRIBUTION TO THE DERIVATIVE WRT S
C      OF THE 1PI Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,BDPIZ1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 CWSQ,PI,Q,T3
      COMPLEX*16 BOZZ1,BOZZ2,MHSQ,MWSQ,MZSQ
      COMPLEX*16 B0
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      MHSQ=DCMPLX(MH*MH)
      CWSQ=1.D0-SWSQ
C  CHECK FOR THRESHOLDS
      IF(S.EQ.DCMPLX(MH+MZ)**2)THEN
          STOP 'FUNCTION BDPIZ1 DOESNOT EXIST AT Z0-HIGGS THRESHOLD'
          END IF
      IF(S.EQ.4.D0*MWSQ)THEN
          STOP 'FUNCTION BDPIZ1 DOESNOT EXIST AT W-W THRESHOLD'
          END IF
      BOZZ1=((S*(S-2.D0*MWSQ)*(1.D0-4.D0*CWSQ-36.D0*CWSQ**2)
     .       +8.D0*MWSQ**2*(5.D0-8.D0*CWSQ-12.D0*CWSQ**2))
     .            *B0(-S,MWSQ,MWSQ,XSC)
     .       +2.D0*MWSQ*(S*(1.D0-4.D0*CWSQ-36.D0*CWSQ**2)
     .                  +4.D0*MWSQ*(5.D0-8.D0*CWSQ-12.D0*CWSQ**2))
     .            *CDLOG(MWSQ/XSC)
     .       +S/3.D0*(S*(108.D0*CWSQ**2+4.D0*CWSQ+1.D0)
     .                  +MWSQ*(144.D0*CWSQ**2+128.D0*CWSQ-76.D0)))
     .       /((12.D0*S*SWSQ*CWSQ)*(S-4.D0*MWSQ))
      BOZZ2=((S**3*(S-MHSQ-MZSQ)
     .       -S**2*(3.D0*MHSQ**2-10.D0*MHSQ*MZSQ-9.D0*MZSQ**2)
     .       +S*(5.D0*MHSQ**3-17.D0*MHSQ**2*MZSQ
     .          +19.D0*MHSQ*MZSQ**2-7.D0*MZSQ**3)
     .       -2.D0*(MHSQ-MZSQ)**4)
     .            *B0(-S,MHSQ,MZSQ,XSC)
     .      +MZSQ*(S**3+8.D0*S**2*MZSQ
     .            -S*(3.D0*MHSQ**2-10.D0*MHSQ*MZSQ+7.D0*MZSQ**2)
     .            +2.D0*(MHSQ-MZSQ)**3)
     .           *CDLOG(MZSQ/XSC)
     .      +MHSQ*(S**3+4*S**2*(3.D0*MZSQ-MHSQ)
     .            +S*(5.D0*MHSQ**2-14.D0*MHSQ*MZSQ+9.D0*MZSQ**2)
     .            -2.D0*(MHSQ-MZSQ)**3)
     .           *CDLOG(MHSQ/XSC)
     .      -(S**4-S**3*(2.D0*MHSQ-10.D0*MZSQ)
     .       -S**2*(MHSQ**2-2.D0*MHSQ*MZSQ+MZSQ**2)
     .       +S*(4.D0*MHSQ**3-16.D0*MHSQ**2*MZSQ
     .          +20.D0*MHSQ*MZSQ**2-8.D0*MZSQ**3)
     .       -2.D0*(MHSQ-MZSQ)**4))
     .       /((12.D0*S**2*SWSQ*CWSQ)
     .        *(S**2+MHSQ**2+MZSQ**2
     .         -2.D0*S*MHSQ-2.D0*S*MZSQ-2.D0*MHSQ*MZSQ))
      BDPIZ1=ALFPI/4.D0*(BOZZ1+BOZZ2)
      RETURN
      END
 
      FUNCTION LDPIZ1(S,XSC)
C***************************
C  ONE-LOOP LEPTONIC CONTRIBUTION TO THE DERIVATIVE WRT S
C      OF THE 1PI Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,LDPIZ1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,Q,SW,T3
      COMPLEX*16 B0FN,MSQ
      COMPLEX*16 B0
      SW=DSQRT(SWSQ)
      CWSQ=1.D0-SWSQ
      CW=DSQRT(CWSQ)
C  CHECK FOR THRESHOLDS
      DO 10 I=1,NLPT
          IF(S.EQ.DCMPLX(4.D0*XLPT(I,1)**2))THEN
              STOP 'FUNCTION LDPIZ1 DOESNOT EXIST AT LEPTON PAIR THRESHO
     .LD'
              END IF
10        CONTINUE
      LDPIZ1=(0.D0,0.D0)
      DO 20 I=1,NLPT
          MSQ=DCMPLX(XLPT(I,1)**2)
          T3=DBLE(MOD(I,2))-0.5D0
          Q=XLPT(I,2)
          BFL=(T3-SWSQ*Q)/(SW*CW)
          BFR=-SWSQ*Q/(SW*CW)
          B0FN=B0(-S,MSQ,MSQ,XSC)
          LDPIZ1=LDPIZ1
     .        +(2.D0*((BFL**2+BFR**2)*(S**2-2.D0*S*MSQ-2.D0*MSQ**2)
     .         +12.D0*BFL*BFR*MSQ**2)*B0FN
     .        -2.D0/3.D0*S*((4.D0*S-7.D0*MSQ)*(BFL**2+BFR**2)
     .                      +18.D0*BFL*BFR*MSQ))
     .             /(3.D0*S*(S-4.D0*MSQ))
          IF(MSQ.NE.0.D0)THEN
              LDPIZ1=LDPIZ1
     .              +4.D0*MSQ*((BFL**2+BFR**2)*(S-MSQ)
     .                        +6.D0*BFL*BFR*MSQ)*CDLOG(MSQ/XSC)
     .                   /(3.D0*S*(S-4.D0*MSQ))
              END IF
20        CONTINUE
      LDPIZ1=ALFPI/4.D0*LDPIZ1
      RETURN
      END
 
      FUNCTION QDPIZ1(S,XSC)
C***************************
C  ONE-LOOP QUARK CONTRIBUTION TO THE DERIVATIVE WRT S
C      OF THE 1PI Z BOSON SELF-ENERGY
C
C  S = SQUARE OF EXTERNAL MOMEMTUM
C  XSC = ARBITRARY SCALE IN THE ON-SHELL RENORMALIZATION SCHEME
C      = 'T HOOFT MASS IN THE MS-BAR RENORMALIZATION SCHEME
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,XSC,QDPIZ1
      COMMON/FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      INTEGER I
      REAL*8 BFL,BFR,CW,CWSQ,PI,Q,SW,T3
      COMPLEX*16 B0FN,MSQ,QCDCOR
      COMPLEX*16 B0,QCDDEA,QCDDEV
      PI=4.D0*DATAN(1.D0)
      SW=DSQRT(SWSQ)
      CWSQ=1.D0-SWSQ
      CW=DSQRT(CWSQ)
C  CHECK FOR THRESHOLDS
      DO 10 I=1,NQRK
          IF(S.EQ.DCMPLX(4.D0*XQRK(I,1)**2))THEN
              STOP 'FUNCTION QDPIZ1 DOESNOT EXIST AT QUARK PAIR THRESHOL
     .D'
              END IF
10        CONTINUE
      QDPIZ1=(0.D0,0.D0)
      DO 20 I=1,NQRK
          MSQ=DCMPLX(XQRK(I,1)**2)
          T3=DBLE(MOD(I,2))-0.5D0
          Q=XQRK(I,2)
          BFL=(T3-SWSQ*Q)/(SW*CW)
          BFR=-SWSQ*Q/(SW*CW)
          B0FN=B0(-S,MSQ,MSQ,XSC)
          QDPIZ1=QDPIZ1
     .        +(2.D0*((BFL**2+BFR**2)*(S**2-2.D0*S*MSQ-2.D0*MSQ**2)
     .         +12.D0*BFL*BFR*MSQ**2)*B0FN
     .        -2.D0/3.D0*S*((4.D0*S-7.D0*MSQ)*(BFL**2+BFR**2)
     .                      +18.D0*BFL*BFR*MSQ))
     .             /(3.D0*S*(S-4.D0*MSQ))
          IF(MSQ.NE.0.D0)THEN
              QDPIZ1=QDPIZ1
     .              +4.D0*MSQ*((BFL**2+BFR**2)*(S-MSQ)
     .                        +6.D0*BFL*BFR*MSQ)*CDLOG(MSQ/XSC)
     .                   /(3.D0*S*(S-4.D0*MSQ))
              END IF
C  CALCULATE THE QCD CORRECTIONS
          QCDCOR=16.D0*PI**2
     .                *(((BFL+BFR)/2.D0)**2*QCDDEV(S,MSQ,XSC)
     .                 +((BFL-BFR)/2.D0)**2*QCDDEA(S,MSQ,XSC))
          QDPIZ1=QDPIZ1+QCDCOR
20        CONTINUE
      QDPIZ1=ALFPI/4.D0*NCOL*QDPIZ1
      RETURN
      END
 
C-----------------------------------------------------------------------
 
      SUBROUTINE VERTZ1(S,T3,Q,MINTSQ,XSC,VZ1L,VZ1R)
C***************************************************
C  CALCULATES THE ONE-LOOP CORRECTIONS TO THE Z0-FERMION-ANTIFERMION
C  VERTEX WITH AN OVERALL FACTOR I*E REMOVED.
C  EXTERNAL FERMIONS ARE ASSUMED TO BE MASSLESS AND ON-SHELL
C
C     S    = THE 4-MOMENTUM SQUARED (MINKOWSKIAN METRIC) OF THE Z
C     T3   = THE T3 OF THE WEAK ISOSPIN OF THE FERMION
C     Q    = THE ELECTRIC CHARGE OF THE FERMION
C     MINTSQ = THE MASS SQUARED OF THE ISOSPIN PARTNER OF THE FERMION
C                 (I.E. MT FOR THE B-QUARK, OTHERWISE 0)
C     VZ1L = THE ONE-LOOP VERTEX CORRECTION FOR A LEFT HELICITY FERMION
C     VZ1R = THE ONE-LOOP VERTEX CORRECTION FOR A RIGHT HELICITY FERMION
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3,Q
      COMPLEX*16 S,MINTSQ,XSC,VZ1L,VZ1R
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 BL,BLP,BR,CW,CWSQ,SW,SWCB
      COMPLEX*16 MWSQ,MZSQ,RHOMW,RHOMZ,XIMW,XLMW
      COMPLEX*16 RHO,XLMDA,XI
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      SWCB=SW**3
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      BL=(T3-SWSQ*Q)/(SW*CW)
      BR=-Q*SW/CW
      BLP=BL-2.D0*T3*CW/SW
      RHOMZ=RHO(S,MZSQ,(0.D0,0.D0))
      RHOMW=RHO(S,MWSQ,MINTSQ)
      XLMW=XLMDA(S,MWSQ,MINTSQ)
      XIMW=XI(S,MWSQ,MINTSQ)
      VZ1L=(ALFPI/4.D0)*(RHOMZ*BL**3
     .                  +RHOMW*BLP/(2.D0*SWSQ)
     .                  -XLMW*T3*CW/SWCB
     .                  -XIMW*T3/(CW*SWCB)
     .                  -CDLOG(MWSQ/XSC)*2.D0*T3*CW/SWCB)
      VZ1R=(ALFPI/4.D0)*RHOMZ*BR**3
      RETURN
      END
 
      SUBROUTINE VERTA1(S,T3,Q,MINTSQ,XSC,VA1L,VA1R)
C***************************************************
C  CALCULATES THE ONE-LOOP CORRECTIONS TO PHOTON-FERMION-ANTIFERMION
C  VERTEX WITH AN OVERALL FACTOR I*E REMOVED.
C  EXTERNAL FERMIONS ARE ASSUMED TO BE MASSLESS AND ON-SHELL
C
C     S      = THE 4-MOMENTUM SQUARED (MINKOWSKIAN METRIC) OF THE PHOTON
C     T3     = THE T3 OF THE WEAK ISOSPIN OF THE FERMION
C     Q      = THE ELECTRIC CHARGE OF THE FERMION
C     MINTSQ = THE MASS SQUARED OF THE ISOSPIN PARTNER OF THE FERMION
C                 (I.E. MT FOR THE B-QUARK, OTHERWISE 0)
C     VA1L = THE ONE-LOOP VERTEX CORRECTION FOR A LEFT HELICITY FERMION
C     VA1R = THE ONE-LOOP VERTEX CORRECTION FOR A RIGHT HELICITY FERMION
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3,Q
      COMPLEX*16 S,MINTSQ,XSC,VA1L,VA1R
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 BL,BR,CW,CWSQ,QP,SW
      COMPLEX*16 MWSQ,MZSQ,RHOMW,RHOMZ,XLMW
      COMPLEX*16 RHO,XLMDA
      CWSQ=1.D0-SWSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(CWSQ)
      BL=(T3-SWSQ*Q)/(SW*CW)
      BR=-Q*SW/CW
      QP=Q-2.D0*T3
      MWSQ=DCMPLX(MW*MW)
      MZSQ=DCMPLX(MZ*MZ)
      RHOMZ=RHO(S,MZSQ,(0.D0,0.D0))
      RHOMW=RHO(S,MWSQ,MINTSQ)
      XLMW=XLMDA(S,MWSQ,MINTSQ)
      VA1L=(ALFPI/4.D0)*(RHOMZ*Q*BL**2
     .                  +RHOMW*QP/(2.D0*SWSQ)
     .                  -XLMW*T3/SWSQ
     .                  -CDLOG(MWSQ/XSC)*2.D0*T3/SWSQ)
      VA1R=(ALFPI/4.D0)*RHOMZ*Q*BR**2
      RETURN
      END
 
      SUBROUTINE BOXW(S,T,T3E,QE,T3F,QF,MINTSQ,BWLL,BWLR,BWRL,BWRR)
C******************************************************************
C  CALCULATES THE HELICITY FORM FACTORS FOR THE TWO Z-Z BOX DIAGRAMS
C    WITH AN OVERALL FACTOR OF -E**2 REMOVED
C
C  S = MANDELSTAM VARIABLE S
C  T = MANDELSTAM VARIABLE T
C  T3E = T3 OF THE WEAK ISOSPIN OF THE INITIAL STATE FERMION
C  QE  = ELECTRIC CHARGE OF THE INITIAL STATE FERMION
C  T3F = T3 OF THE WEAK ISOSPIN OF THE FINAL STATE FERMION
C  QF  = ELECTRIC CHARGE OF THE FINAL STATE FERMION
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C           (VALUE RETURNED IS CURRENTLY INDEPENDENT OF THIS PARAMETER)
C
C  BWLL = AMPLITUDE FOR INCOMING LEFT HELICITY
C                   AND OUTGOING LEFT HELICITY FERMIONS
C  BWLR = AMPLITUDE FOR INCOMING LEFT HELICITY
C                   AND OUTGOING RIGHT HELICITY FERMIONS
C  BWRL = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                   AND OUTGOING LEFT HELICITY FERMIONS
C  BWRR = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                   AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3E,QE,T3F,QF
      COMPLEX*16 S,T,MINTSQ,BWLL,BWLR,BWRL,BWRR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 WCPSQ
      COMPLEX*16 BOXWLL,BOXWLR,BOXWRL,BOXWRR,MWSQ
      WCPSQ=ALFPI/(4.D0*SWSQ**2)
      MWSQ=DCMPLX(MW*MW)
C  W BOX DIAGRAM DEPENDS ON WHETHER ISOSPINS OF INCOMING AND OUTGOING
C    FERMIONS ARE THE SAME OR DIFFERENT
      IF(T3E.EQ.T3F)THEN
          CALL CRSBOX(S,T,MWSQ,MINTSQ,BOXWLL,BOXWLR,BOXWRL,BOXWRR)
C=================================================================
      ELSE
          CALL DIRBOX(S,T,MWSQ,MINTSQ,BOXWLL,BOXWLR,BOXWRL,BOXWRR)
C=================================================================
          END IF
      BWLL=WCPSQ*BOXWLL
      BWLR=(0.D0,0.D0)
      BWRL=(0.D0,0.D0)
      BWRR=(0.D0,0.D0)
      RETURN
      END
 
      SUBROUTINE BOXZ(S,T,T3E,QE,T3F,QF,MINTSQ,BZLL,BZLR,BZRL,BZRR)
C******************************************************************
C  CALCULATES THE HELICITY FORM FACTORS FOR THE TWO Z-Z BOX DIAGRAMS
C    WITH AN OVERALL FACTOR OF -E**2 REMOVED
C
C  S = MANDELSTAM VARIABLE S
C  T = MANDELSTAM VARIABLE T
C  T3E = T3 OF THE WEAK ISOSPIN OF THE INITIAL STATE FERMION
C  QE  = ELECTRIC CHARGE OF THE INITIAL STATE FERMION
C  T3F = T3 OF THE WEAK ISOSPIN OF THE FINAL STATE FERMION
C  QF  = ELECTRIC CHARGE OF THE FINAL STATE FERMION
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C           (VALUE RETURNED IS CURRENTLY INDEPENDENT OF THIS PARAMETER)
C
C  BZLL = AMPLITUDE FOR INCOMING LEFT HELICITY
C                   AND OUTGOING LEFT HELICITY FERMIONS
C  BZLR = AMPLITUDE FOR INCOMING LEFT HELICITY
C                   AND OUTGOING RIGHT HELICITY FERMIONS
C  BZRL = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                   AND OUTGOING LEFT HELICITY FERMIONS
C  BZRR = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                   AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 T3E,QE,T3F,QF
      COMPLEX*16 S,T,MINTSQ,BZLL,BZLR,BZRL,BZRR
      COMMON/GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 BEL,BFL,BER,BFR,CW,SW
      COMPLEX*16 CBXZLL,CBXZLR,CBXZRL,CBXZRR,
     .           DBXZLL,DBXZLR,DBXZRL,DBXZRR,MZSQ
      SW=DSQRT(SWSQ)
      CW=DSQRT(1.D0-SWSQ)
      MZSQ=DCMPLX(MZ*MZ)
C  CALCULATE LEFT AND RIGHT COUPLINGS TO Z FOR INCOMING PARTICLE
      BEL=(T3E-SWSQ*QE)/(SW*CW)
      BER=-QE*SW/CW
C  CALCULATE LEFT AND RIGHT COUPLINGS TO Z FOR OUTGOING PARTICLE
      BFL=(T3F-SWSQ*QF)/(SW*CW)
      BFR=-QF*SW/CW
      CALL CRSBOX(S,T,MZSQ,MINTSQ,CBXZLL,CBXZLR,CBXZRL,CBXZRR)
C=============================================================
      CALL DIRBOX(S,T,MZSQ,MINTSQ,DBXZLL,DBXZLR,DBXZRL,DBXZRR)
C=============================================================
      BZLL=ALFPI*BEL*BEL*BFL*BFL*(CBXZLL+DBXZLL)
      BZLR=ALFPI*BEL*BEL*BFR*BFR*(CBXZLR+DBXZLR)
      BZRL=ALFPI*BER*BER*BFL*BFL*(CBXZRL+DBXZRL)
      BZRR=ALFPI*BER*BER*BFR*BFR*(CBXZRR+DBXZRR)
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*            LIBRARY OF GENERAL ONE-LOOP FEYNMAN DIAGRAMS            **
C*                                                                    **
C*                            REFERENCES:                             **
C*           B. W. LYNN AND R. G. STUART, PLB252(1990)676             **
C*                                                                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION RHO(S,MSQ,MISQ)
C*****************************
C  RENORMALIZED MASSLESS FERMION VERTEX FORM FACTOR ASSOCIATED WITH
C  THE VERTEX GRAPH CONTAINING A SINGLE VIRTUAL HEAVY BOSON
C
C  S = SQUARE OF EXTERNAL BOSON MOMENTUM
C  MSQ = MASS SQUARED OF INTERNAL VIRTUAL BOSON
C  MISQ = MASS SQUARED OF THE ISO SPIN PARTNER OF THE EXTERNAL FERMION
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,MISQ,RHO
      REAL*8 PI,PISQ6
      COMPLEX*16 BETA,CLG
      COMPLEX*16 CDLN,SP,B0,C0
      PI=4.D0*DATAN(1.D0)
      PISQ6=PI**2/6.D0
      IF(CDABS(S).GE.100.D0*CDABS(MISQ).OR.
     .   CDABS(MSQ).GE.100.D0*CDABS(MISQ))THEN
C  THIS IS CASE OF MASSLESS INTERNAL FERMIONS
          BETA=-MSQ/S
          CLG=CDLN(BETA,(1.D0,0.D0))
          RHO=CLG+0.5D0
     .       +(2.D0*BETA-4.D0)*(CLG+1.D0)
     .       -2.D0*(1.D0-BETA)**2*(PISQ6-SP(1.D0-1.D0/BETA,(1.D0,0.D0)))
      ELSE
          RHO=(2.D0*S**2*MSQ+4.D0*S*MSQ**2-2.D0*S*MSQ*MISQ+S*MISQ**2
     .        +2.D0*MSQ**3-3.D0*MSQ**2*MISQ+MISQ**3)/(S*MSQ)
     .       *C0((0.D0,0.D0),-S,(0.D0,0.D0),MSQ,MISQ,MISQ)
     .       +(-6.D0*S*MSQ**2+5.D0*S*MSQ*MISQ-2.D0*S*MISQ**2
     .         -4.D0*MSQ**3+6.D0*MSQ**2*MISQ-2*MISQ**3)
     .         /(2.D0*S*(MISQ-MSQ)**2)
     .       *B0(-S,MISQ,MISQ,MSQ)
     .       -(-8.D0*S*MSQ**2+6.D0*S*MSQ*MISQ-S*MISQ**2
     .         -4.D0*MSQ**3+6.D0*MSQ**2*MISQ-2.D0*MISQ**3)
     .        *(MISQ/MSQ)/(2.D0*S*(MISQ-MSQ)**2)
     .       *B0(-S,MISQ,MISQ,MISQ)
     .       -(-10.D0*S*MSQ**2+9.D0*S*MSQ*MISQ-5.D0*S*MISQ**2
     .         -8.D0*MSQ**3+12.D0*MSQ**2*MISQ-4.D0*MISQ**3)
     .         /(4.D0*S*MSQ*(MSQ-MISQ))
          END IF
      RETURN
      END
 
      FUNCTION XLMDA(S,MSQ,MISQ)
C*******************************
C  RENORMALIZED MASSLESS FERMION VERTEX FORM FACTOR ASSOCIATED WITH THE
C  VERTEX GRAPH CONTAINING 2 IDENTICAL HEAVY BOSONS
C
C  S = SQUARE OF EXTERNAL BOSON MOMENTUM
C  MSQ = MASS SQUARED OF INTERNAL VIRTUAL BOSONS
C  MISQ = MASS SQUARED OF THE ISO SPIN PARTNER OF THE EXTERNAL FERMION
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,MISQ,XLMDA
      COMPLEX*16 BETA,CLG,X1,X2
      COMPLEX*16 CDLN,SP,B0,C0
      IF(CDABS(S).GE.100.D0*CDABS(MISQ).OR.
     .   CDABS(MSQ).GE.100.D0*CDABS(MISQ))THEN
C  THIS IS CASE OF MASSLESS INTERNAL FERMIONS
          BETA=-MSQ/S
          CALL QDRT(S,-S,MSQ,X1,X2)
C==================================
          CLG=CDLN(-X2,(1.D0,0.D0))-CDLN(X1,(1.D0,0.D0))
          XLMDA=(1.D0-2.D0*BETA)*(X1-X2)*CLG
     .         +(4.D0-2.D0*BETA)*BETA*CLG**2
     .         -2.D0*BETA-2.5D0
      ELSE
          XLMDA=-(-4.D0*S*MSQ**2+2.D0*S*MSQ*MISQ-S*MISQ**2
     .            -2.D0*MSQ**3+3.D0*MSQ**2*MISQ-MISQ**3)/(S*MSQ)
     .         *C0(-S,(0.D0,0.D0),(0.D0,0.D0),MSQ,MSQ,MISQ)
     .          -(-2.D0*S*MSQ**2-3.D0*S*MSQ*MISQ+2.D0*S*MISQ**2
     .            -4.D0*MSQ**3+6.D0*MSQ**2*MISQ-2.D0*MISQ**3)
     .           /(2.D0*S*(MISQ-MSQ)**2)
     .         *B0(-S,MSQ,MSQ,MSQ)
     .          +(-8.D0*S*MSQ**2+6.D0*S*MSQ*MISQ-S*MISQ**2
     .            -4.D0*MSQ**3+6.D0*MSQ**2*MISQ-2.D0*MISQ**3)
     .          *(MISQ/MSQ)/(2.D0*S*(MISQ-MSQ)**2)
     .         *B0(-S,MSQ,MSQ,MISQ)
     .          +(-18.D0*S*MSQ**2+13.D0*S*MSQ*MISQ-S*MISQ**2
     .            -8.D0*MSQ**3+12.D0*MSQ**2*MISQ-4.D0*MISQ**3)
     .           /(4.D0*S*MSQ*(MSQ-MISQ))
          END IF
      RETURN
      END
 
      FUNCTION XI(S,MSQ,MISQ)
C****************************
C  RENORMALIZED MASSLESS FERMION VERTEX FORM FACTOR
C  THIS FUNCTION CONTAINS THE LEADING MISQ DEPENDENCE
C    AND VANISHES FOR MISQ=0
C
C  S = SQUARE OF EXTERNAL BOSON MOMENTUM
C  MSQ = MASS SQUARED OF INTERNAL VIRTUAL BOSONS
C  MISQ = MASS SQUARED OF THE ISO SPIN PARTNER OF THE EXTERNAL FERMION
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,MISQ,XI
      COMPLEX*16 B0,C0
      IF(CDABS(S).GE.100.D0*CDABS(MISQ).OR.
     .   CDABS(MSQ).GE.100.D0*CDABS(MISQ))THEN
          XI=(0.D0,0.D0)
      ELSE
          XI=-(-4.D0*S*MSQ+S*MISQ+MSQ**2-2.D0*MSQ*MISQ+MISQ**2)
     .        *(MISQ/(2.D0*S*MSQ))
     .      *C0(-S,(0.D0,0.D0),(0.D0,0.D0),MSQ,MSQ,MISQ)
     .       +(-2.D0*S*MSQ-MSQ**2+2.D0*MISQ*MSQ-MISQ**2)
     .        *(MISQ/(2.D0*S*MSQ))
     .      *C0((0.D0,0.D0),-S,(0.D0,0.D0),MSQ,MISQ,MISQ)
     .       -(-S+2.D0*MSQ-2.D0*MISQ)
     .        *(MISQ/(4.D0*S*MSQ))
     .      *(B0(-S,MSQ,MSQ,MSQ)-B0(-S,MISQ,MISQ,MSQ))
     .       +MISQ/(2.D0*MSQ)
          END IF
      RETURN
      END
 
      SUBROUTINE DIRBOX(S,T,MSQ,MINTSQ,BOXLL,BOXLR,BOXRL,BOXRR)
C**************************************************************
C  CALCULATES THE HELICITY FORM FACTORS IN THE DIRECT BOX DIAGRAM
C    FOR MASSLESS EXTERNAL FERMIONS
C
C  S = MANDELSTAM VARIABLE S
C  T = MANDELSTAM VARIABLE T
C  MSQ = SQUARE OF MASS OF INTERNAL VIRTUAL BOSONS
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C           (VALUE RETURNED IS CURRENTLY INDEPENDENT OF THIS PARAMETER)
C
C  BOXLL = AMPLITUDE FOR INCOMING LEFT HELICITY
C                    AND OUTGOING LEFT HELICITY FERMIONS
C  BOXLR = AMPLITUDE FOR INCOMING LEFT HELICITY
C                    AND OUTGOING RIGHT HELICITY FERMIONS
C  BOXRL = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                    AND OUTGOING LEFT HELICITY FERMIONS
C  BOXRR = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                    AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,T,MSQ,MINTSQ,BOXLL,BOXLR,BOXRL,BOXRR
      REAL*8 PI,PISQ6
      COMPLEX*16 B13B24,C123,C124,CLG,D1234,U,X1,X2,XSC
      COMPLEX*16 B0,CDLN,D0,SP
      PI=4.D0*DATAN(1.D0)
      PISQ6=PI**2/6.D0
      U=-S-T
      XSC=MSQ
      CALL QDRT(S,-S,MSQ,X1,X2)
C==============================
      CLG=CDLN(-X2,(1.D0,0.D0))-CDLN(X1,(1.D0,0.D0))
      C123=-CLG*CLG/S
      B13B24=CDLN(-U/XSC,(-1.D0,0.D0))-2.D0
     .      +B0(-S,MSQ,MSQ,XSC)
      C124=(SP(1.D0+(U/MSQ),(1.D0,0.D0))-PISQ6)/U
      D1234=D0(S,U,MSQ)
      BOXLL=(C123+0.5D0*U*D1234)
      BOXRR=BOXLL
      BOXLR=(0.5D0/T*B13B24
     .      +0.5D0*(T**2+U**2-2.D0*S*MSQ)/T**2*C123
     .      +0.5D0*U*(T-U-2.D0*MSQ)/T**2*C124
     .      +0.25D0*(U*(T**2+U**2)+4.D0*U**2*MSQ
     .                            +2.D0*(U-S)*MSQ**2)*D1234/T**2)
      BOXRL=BOXLR
      RETURN
      END
 
      SUBROUTINE CRSBOX(S,T,MSQ,MINTSQ,BOXLL,BOXLR,BOXRL,BOXRR)
C**************************************************************
C  CALCULATES THE HELICITY FORM FACTORS IN THE CROSSED BOX DIAGRAM
C    FOR MASSLESS EXTERNAL FERMIONS
C
C  S = MANDELSTAM VARIABLE S
C  T = MANDELSTAM VARIABLE T
C  MSQ = SQUARE OF MASS OF INTERNAL VIRTUAL BOSONS
C  MINTSQ = SQUARE OF MASS OF THE INTERNAL FINAL STATE FERMION
C           (VALUE RETURNED IS CURRENTLY INDEPENDENT OF THIS PARAMETER)
C
C  BOXLL = AMPLITUDE FOR INCOMING LEFT HELICITY
C                    AND OUTGOING LEFT HELICITY FERMIONS
C  BOXLR = AMPLITUDE FOR INCOMING LEFT HELICITY
C                    AND OUTGOING RIGHT HELICITY FERMIONS
C  BOXRL = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                    AND OUTGOING LEFT HELICITY FERMIONS
C  BOXRR = AMPLITUDE FOR INCOMING RIGHT HELICITY
C                    AND OUTGOING RIGHT HELICITY FERMIONS
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,T,MSQ,MINTSQ,BOXLL,BOXLR,BOXRL,BOXRR
      REAL*8 PI,PISQ6
      COMPLEX*16 B13B24,C123,C124,CLG,D1234,U,X1,X2,XSC
      COMPLEX*16 B0,CDLN,D0,SP
      PI=4.D0*DATAN(1.D0)
      PISQ6=PI**2/6.D0
      U=-S-T
      XSC=MSQ
      CALL QDRT(S,-S,MSQ,X1,X2)
C==============================
      CLG=CDLN(-X2,(1.D0,0.D0))-CDLN(X1,(1.D0,0.D0))
      C123=-CLG*CLG/S
      B13B24=CDLN(-T/XSC,(-1.D0,0.D0))-2.D0
     .      +B0(-S,MSQ,MSQ,XSC)
      C124=(SP(1.D0+(T/MSQ),(1.D0,0.D0))-PISQ6)/T
      D1234=D0(S,T,MSQ)
      BOXLL=-(0.5D0/U*B13B24
     .       +0.5D0*(T**2+U**2-2.D0*S*MSQ)/U**2*C123
     .       +0.5D0*T*(U-T-2.D0*MSQ)/U**2*C124
     .       +0.25D0*(T*(T**2+U**2)+4.D0*T**2*MSQ
     .                             +2.D0*(T-S)*MSQ**2)*D1234/U**2)
      BOXRR=BOXLL
      BOXLR=-(C123+0.5D0*T*D1234)
      BOXRL=BOXLR
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*              LIBRARY  OF  ONE-LOOP  SCALAR  INTEGRALS              **
C*                                                                    **
C*                            REFERENCES:                             **
C*            G. 'T HOOFT AND M. VELTMAN, NPB153(1979)365.            **
C*           G. PASSARINO AND M. VELTMAN, NPB160(1979)151.            **
C*                                                                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION B0(PSQ,M1SQ,M2SQ,XSC)
C***********************************
C  THIS IS THE GENERAL B0/(I*PI**2) AS DEFINED BY
C    'T HOOFT AND VELTMAN, NPB153 (1979) 365.
C  THIS IS CONSISTENT WITH THE DEFINITION OF
C    PASSARINO AND VELTMAN, NPB160 (1979) 151.
C  TIME-LIKE MOMENTA SATISFY P**2<0
C  B0 IS DIVERGENT AND THE VALUE RETURNED IS B0 - DELTA
C    WHERE DELTA IS A DIVERGENT CONSTANT FROM DIMENSIONAL REGULARIZATION
C  XSC IS AN ARBITRARY SCALE OFTEN DENOTED MU
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 PSQ,M1SQ,M2SQ,XSC,B0
      COMPLEX*16 R1,R2
      COMPLEX*16 CDLN,F1PLN
      IF(PSQ.NE.(0.D0,0.D0))THEN
          CALL QDRT(-PSQ,PSQ+M2SQ-M1SQ,M1SQ,R1,R2)
C=================================================
          B0=-CDLN(-PSQ/XSC,(-1.D0,0.D0))-F1PLN(R1,-PSQ)-F1PLN(R2,PSQ)
      ELSE IF(PSQ.EQ.(0.D0,0.D0).AND.M1SQ.NE.M2SQ)THEN
          B0=(1.D0,0.D0)
          IF(M1SQ.NE.(0.D0,0.D0))B0=B0+M1SQ/(M2SQ-M1SQ)*CDLOG(M1SQ/XSC)
          IF(M2SQ.NE.(0.D0,0.D0))B0=B0+M2SQ/(M1SQ-M2SQ)*CDLOG(M2SQ/XSC)
      ELSE IF(PSQ.EQ.(0.D0,0.D0).AND.M1SQ.EQ.M2SQ)THEN
          B0=-CDLOG(M1SQ/XSC)
          END IF
C  SET IMAG(B0) EXPLICITLY TO ZERO IF BELOW THRESHOLD
      IF(DIMAG(PSQ).EQ.0.D0.AND.
     .   DIMAG(M1SQ).EQ.0.D0.AND.
     .   DIMAG(M2SQ).EQ.0.D0.AND.
     .   DREAL(PSQ).GE.-DREAL(CDSQRT(M1SQ)+CDSQRT(M2SQ))**2)B0=DREAL(B0)
      RETURN
      END
 
      FUNCTION C0(P1SQ,P2SQ,P5SQ,M1SQ,M2SQ,M3SQ)
C***********************************************
C THIS IS THE GENERAL C0/(I*PI**2) AS DEFINED BY
C   'T HOOFT AND VELTMAN, NPB153 (1979) 365.
C THIS IS CONSISTENT WITH THE DEFINITION OF
C   PASSARINO AND VELTMAN, NPB160 (1979) 151.
C TIME-LIKE MOMENTA SATISFY P**2<0
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 P1SQ,P2SQ,P5SQ,M1SQ,M2SQ,M3SQ,C0
      COMPLEX*16 A,B,C,D,E,F,ALPHA,BETA,C2AB,Y0,DET,P1P2,
     .           PP1SQ,PP2SQ,PP5SQ,MM1SQ,MM2SQ,MM3SQ,SAVE
      COMPLEX*16 S3
      P1P2=0.5D0*(P5SQ-P1SQ-P2SQ)
      DET=P1P2*P1P2-P1SQ*P2SQ
C CHECK THAT THE C0 FORM FACTOR CANNOT BE EXPRESSED AS B0'S
      IF(DET.EQ.(0.D0,0.D0))
     .  STOP 'C0 FORM FACTOR IS REDUCIBLE TO TWO POINT FUNCTIONS (B0)'
C ARRANGE MOMENTA IN DECREASING ORDER OF MAGNITUDE USING
C  SYMMETRY PROPERTIES OF THE C0.
      PP1SQ=P1SQ
      PP2SQ=P2SQ
      PP5SQ=P5SQ
      MM1SQ=M1SQ
      MM2SQ=M2SQ
      MM3SQ=M3SQ
      IF(CDABS(PP2SQ).LT.CDABS(PP5SQ))THEN
          SAVE=PP2SQ
          PP2SQ=PP5SQ
          PP5SQ=SAVE
          SAVE=MM1SQ
          MM1SQ=MM2SQ
          MM2SQ=SAVE
          END IF
      IF(CDABS(PP1SQ).LT.CDABS(PP2SQ))THEN
          SAVE=PP1SQ
          PP1SQ=PP2SQ
          PP2SQ=SAVE
          SAVE=MM1SQ
          MM1SQ=MM3SQ
          MM3SQ=SAVE
          END IF
      IF(CDABS(PP2SQ).LT.CDABS(PP5SQ))THEN
          SAVE=PP2SQ
          PP2SQ=PP5SQ
          PP5SQ=SAVE
          SAVE=MM1SQ
          MM1SQ=MM2SQ
          MM2SQ=SAVE
          END IF
      A=-PP2SQ
      B=-PP1SQ
      C=PP1SQ+PP2SQ-PP5SQ
      D=MM2SQ-MM3SQ+PP2SQ
      E=MM1SQ-MM2SQ+PP1SQ-C
      F=MM3SQ
      CALL QDRT(B,C,A,BETA,ALPHA)
C  ALPHA AND BETA WILL BE REAL FOR DET>=0 WHICH IS TRUE UNLESS
C    THE MOMENTA COMPLEX OR ALL ARE STRICTLY SPACE-LIKE.
C  THIS IS THE ONLY CONDITION UNDER WHICH THE TERMS
C    OF THE FORM ETA()*CDLN() IN FUNCTION S3 CAN CONTRIBUTE.
      C2AB=C+2.D0*ALPHA*B
C C2AB CANNOT VANISH SINCE C2AB=0 IMPLIES DET=0
      Y0=-(D+ALPHA*E)/C2AB
      C0=S3(B,C+E,A+D+F,Y0-(2.D0*A+ALPHA*C)/C2AB,(-1.D0,0.D0))/C2AB
      IF(ALPHA.NE.(1.D0,0.D0))THEN
          C0=C0-S3(A+B+C,D+E,F,Y0/(1.D0-ALPHA),(-1.D0,0.D0))/C2AB
          END IF
      IF(ALPHA.NE.(0.D0,0.D0))THEN
          C0=C0+S3(A,D,F,-Y0/ALPHA,(-1.D0,0.D0))/C2AB
          END IF
C  IF ALL THE MOMENTA AND MASSES ARE REAL AND
C     ALL CUTS ARE BELOW THRESHOLD THEN DISCARD THE IMAGINARY PART
      IF(DIMAG(PP1SQ).EQ.0.D0
     .  .AND.DIMAG(PP2SQ).EQ.0.D0
     .  .AND.DIMAG(PP5SQ).EQ.0.D0
     .  .AND.DIMAG(MM1SQ).EQ.0.D0
     .  .AND.DIMAG(MM2SQ).EQ.0.D0
     .  .AND.DIMAG(MM3SQ).EQ.0.D0
     .  .AND.DREAL(PP1SQ).GE.-DREAL(CDSQRT(MM1SQ)+CDSQRT(MM2SQ))**2
     .  .AND.DREAL(PP2SQ).GE.-DREAL(CDSQRT(MM2SQ)+CDSQRT(MM3SQ))**2
     .  .AND.DREAL(PP5SQ).GE.-DREAL(CDSQRT(MM1SQ)+CDSQRT(MM3SQ))**2)
     .   C0=DREAL(C0)
      RETURN
      END
 
      FUNCTION D0(S,T,MSQ)
C*************************
C  D0 FORM-FACTOR (ASSOCIATED WITH BOX DIAGRAMS) VALID BELOW THRESHOLD
C
C  S, T = USUAL MANDELSTAM VARIABLES
C  MSQ = SQUARED MASS OF INTERNAL HEAVY BOSONS
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,T,MSQ
      COMPLEX*16 D0
      COMPLEX*16 THRES,X,Y
      COMPLEX*16 SP
      THRES=4.D0*MSQ/S
      X=CDSQRT(1.D0-THRES)
      Y=CDSQRT(1.D0-THRES*(1.D0+MSQ/T))
      D0=SP((Y+1.D0)/(Y-X),(1.D0,0.D0))
     .  -SP((Y-1.D0)/(Y-X),(1.D0,0.D0))
     .  +SP((Y+1.D0)/(Y+X),(1.D0,0.D0))
     .  -SP((Y-1.D0)/(Y+X),(1.D0,0.D0))
      D0=-2.D0*D0/(S*T*Y)
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*                  LIBRARY  OF  ANALYTIC  FUNCTIONS                  **
C*                                                                    **
C*                                                                    **
C*                            REFERENCES:                             **
C*            G. 'T HOOFT AND M. VELTMAN, NPB153(1979)365.            **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION F1PLN(X,A)
C************************
C  THIS FUNCTION EVALUATES THE INTEGRAL OF  LOG(T-X)
C    WITH RESPECT TO T IN THE INTERVAL (0,1).
C    A IS THE INFINITESSIMAL IMAGINARY PART OF X, I.E. X->X+I*A
C  SPECIAL CARE IS TAKEN OF STRONG NUMERICAL CANCELLATIONS NEAR X=1.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 X,A,F1PLN
      COMPLEX*16 PI
      COMPLEX*16 CDLN
      PI=DCMPLX(4.D0*DATAN(1.D0))
      IF(CDABS(X).GT.4.D0)THEN
          F1PLN=CDLN(1.D0-X,-A)-X*CDLN(1.D0-1.D0/X,A)-1.D0
          RETURN
          END IF
      IF(X.NE.(0.D0,0.D0).AND.X.NE.(1.D0,0.D0))THEN
          F1PLN=(1.D0-X)*CDLN(1.D0-X,-A)+X*CDLN(-X,-A)-1.D0
          RETURN
          END IF
      IF(X.EQ.(1.D0,0.D0))
     .    F1PLN=(0.D0,-1.D0)*PI*DSIGN(1.D0,DREAL(A))-1.D0
      IF(X.EQ.(0.D0,0.D0))F1PLN=(-1.D0,0.D0)
      RETURN
      END
 
      FUNCTION SPXX(Y,E)
C*********************
C  SPENCE FUNCTION OF Y+I*REAL(E) WHERE E IS AN INFINITESIMAL
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 Y,E,SP
      INTEGER I1,I2,J
      REAL*8 B(9),FACT,PI,PISQ6
      COMPLEX*16 A,CLN,PROD,TERM,X,Z,ZSQ
      COMPLEX*16 CDLN
      SPXX=.FALSE.
      B(1)=1.D0/6.D0
      B(2)=-1.D0/30.D0
      B(3)=1.D0/42.D0
      B(4)=B(2)
      B(5)=5.D0/66.D0
      B(6)=-691.D0/2730.D0
      B(7)=7.D0/6.D0
      B(8)=-3617.D0/510.D0
      B(9)=43867.D0/798.D0
      PI=4.D0*DATAN(1.D0)
      PISQ6=PI**2/6.D0
      I1=0
      I2=0
      X=Y
      A=E
      IF(X.EQ.(0.D0,0.D0))THEN
          SP=(0.D0,0.D0)
          RETURN
          END IF
      IF(X.EQ.(1.D0,0.D0))THEN
          SP=PISQ6
          RETURN
          END IF
C  IF X LIES OUTSIDE THE UNIT CIRCLE THEN EVALUATE SP(1/X)
      IF(CDABS(X).GT.1.D0)THEN
          X=1.D0/X
          A=-A
          I1=1
          END IF
C  IF REAL(X)>1/2 THEN EVALUATE SP(1-X)
      IF(DREAL(X).GT.0.5D0)THEN
          X=1.D0-X
          A=-A
          I2=1
          END IF
C  EVALUATE SERIES FOR SP(X)
      Z=-CDLN(1.D0-X,-A)
      ZSQ=Z*Z
      SP=Z-ZSQ/4.D0
      PROD=Z
      FACT=1.D0
      DO 10 J=2,18,2
          FACT=FACT*DCMPLX(DBLE((J+1)*J))
          PROD=PROD*ZSQ
          TERM=B(J/2)/FACT*PROD
          SP=SP+TERM
          IF(CDABS(TERM/SP).LT.1.D-20)GO TO 20
10        CONTINUE
C  ADD APPROPRIATE LOGS TO OBTAIN SPENCE FUNCTION OF ORIGINAL ARGUMENT
20    IF(I2.EQ.1)THEN
          SP=-SP+PISQ6-CDLN(X,A)*CDLN(1.D0-X,-A)
          X=1.D0-X
          A=-A
          END IF
      IF(I1.EQ.1)THEN
          CLN=CDLN(-X,-A)
          SP=-SP-PISQ6-CLN*CLN/2.D0
          END IF
      RETURN
      END
 
      FUNCTION CDLNXX(X,A)
C***********************
C  COMPLEX LOGARITHM OF X+I*REAL(A) WHERE A IS AN INFINITESIMAL
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 A,X,CDLN
      REAL*8 PI
      CDLNXX=.FALSE.
      PI=4.D0*DATAN(1.D0)
      IF(DIMAG(X).EQ.0.D0.AND.DREAL(X).LE.0.D0)THEN
          CDLN=CDLOG(-X)+(0.D0,1.D0)*PI*DSIGN(1.D0,DREAL(A))
      ELSE
          CDLN=CDLOG(X)
          END IF
      IF(DIMAG(CDLN).GT.PI)CDLN=CDLN-(0.D0,1.D0)*PI
      IF(DIMAG(CDLN).LT.-PI)CDLN=CDLN+(0.D0,1.D0)*PI
      RETURN
      END
 
        FUNCTION ETA(A,EA,B,EB,EAB)
C**********************************
C THIS FUNCTION IS DEFINED AS ETA=LOG(AB)-LOG(A)-LOG(B)=0,(+/-)2*PI
C   EA, EB AND EAB ARE INFINITESSIMAL IMAGINARY PARTS OF A, B AND A*B
C   I.E. A->A+I*EA   B->B+I*EB   A*B->A*B+I*EAB
C   THE QUANTITY EAB NEEDS ONLY BE CONSIDERED WHEN A*B IS REAL NEGATIVE
C   I.E. WHEN REAL(A)=-REAL(B) AND IMAG(A)=IMAG(B)
C
        IMPLICIT LOGICAL (A-Z)
        COMPLEX*16 A,EA,B,EB,EAB,ETA
        REAL*8 EPS,PI,XIM
        COMPLEX*16 AB
        COMPLEX*16 CDLN
        EPS=1.D-13
        PI=4.D0*DATAN(1.D0)
C???????????????????????????????????????????????????????????????????????
C  THE FOLLOWING "IF" STATEMENT SHOULD READ
C       IF(A.EQ.(0.D0,0.D0).OR.B.EQ.(0.D0,0.D0))THEN
C  HOWEVER SOME COMPILERS MAY FAIL TO RESPOND TO THIS DUE PRESUMABLY
C  TO SMALL COMPONENTS THAT ARE PRESENT YET ARE OUTPUT AS 0
C???????????????????????????????????????????????????????????????????????
        IF(CDABS(A).LT.EPS.OR.CDABS(B).LT.EPS)THEN
            ETA=(0.D0,0.D0)
            RETURN
            END IF
        AB=A*B
        XIM=DIMAG(CDLN(A,EA)+CDLN(B,EB))
C  TEST TO SEE IF THE PRODUCT A*B IS REAL NEGATIVE
        IF(DREAL(AB).LT.0.D0.AND.
     .      DABS(DIMAG(AB)/DREAL(AB)).LT.EPS)THEN
C  THIS IS THE CASE WHEN A*B IS REAL NEGATIVE
            IF(XIM.GT.0.D0.AND.DREAL(EAB).LT.0.D0)THEN
                ETA=(0.D0,-2.D0)*PI
                RETURN
                END IF
            IF(XIM.LT.0.D0.AND.DREAL(EAB).GT.0.D0)THEN
                ETA=(0.D0,2.D0)*PI
                RETURN
                END IF
        ELSE
C  THIS IS THE CASE WHEN A*B IS NOT REAL NEGATIVE
            IF(XIM.GT.PI+EPS)THEN
                ETA=(0.D0,-2.D0)*PI
                RETURN
                END IF
            IF(XIM.LT.-PI-EPS)THEN
               ETA=(0.D0,2.D0)*PI
               RETURN
               END IF
            END IF
        ETA=(0.D0,0.D0)
        RETURN
        END
 
       FUNCTION S3(A,B,C,D,E)
C****************************
C  THIS FUNCTION EVALUATES IN THE INTERVAL (0,1) THE INTEGRAL
C    OF (LOG(A*X**2+B*X+C)-LOG(A*D**2+B*D+C))/(X-D)
C    E IS THE INFINITESSIMAL IMAGINARY PART OF C, I.E. C->C+I*E
C  THE ARGUMENT, A, MUST HAVE ZERO IMAGINARY PART
C  THE IMAGINARY PART OF A*X**2+B*X+C MUST NOT CHANGE SIGN
C    FOR X IN THE INTERVAL (0,1)
C  (IN CASES OF PHYSICAL INTEREST A*D**2+B*D+C IS REAL FOR REAL MASSES)
C  THE PROGRAM WILL FAIL IF A*D**2+B*D+C=0 I.E. IF D=R1 OR D=R2
C
       IMPLICIT LOGICAL (A-Z)
       COMPLEX*16 A,B,C,D,E,S3
       REAL*8 ZEROIM
       COMPLEX*16 AR1R2,DELTA,EPSILN,R1,R2
       COMPLEX*16 CDLN,ETA,SP
C  CHECK THAT A IS REAL
       IF(DIMAG(A).NE.0.D0)
     .     STOP 'FIRST ARGUMENT OF FUNCTION S3 MUST BE REAL'
C  CHECK THAT THE IMAGINARY PART OF A*X**2+B*X+C DOES NOT CHANGE SIGN
       IF(DIMAG(B).NE.0.D0)THEN
           ZEROIM=-DIMAG(C)/DIMAG(B)
           IF(ZEROIM.GT.0.D0.AND.ZEROIM.LT.1.D0)STOP
     .       'IMAGINARY PART OF A*X**2+B*X+C IN S3 MUST NOT CHANGE SIGN'
           END IF
C  CALCULATE SPECIFIC CASES OF S3
       IF(A.EQ.(0.D0,0.D0).AND.B.EQ.(0.D0,0.D0))THEN
           S3=(0.D0,0.D0)
           RETURN
           END IF
       IF(A.EQ.(0.D0,0.D0))THEN
           S3=SP(D/(D+C/B),-(E/B)*D/(D+C/B)**2)
     .       -SP((D-1.D0)/(D+C/B),-(E/B)*(D-1.D0)/(D+C/B)**2)
C-----------------------------------------------------------------------
C  IF D IS REAL THE ETA FUNCTIONS VANISH
C     BECAUSE IMAG(1/(D+C/B))=-IMAG(1+C/B)=-IMAG(C/B)
C  NOTE HOWEVER THAT THIS ASSUMES THAT THE EXISTENCE OF COMPLEX MASSES
C     IN THE PROBLEM GUARANTEES THAT D IS COMPLEX.
C     IF D IS REAL AND THE ARGUMENTS OF THE LOGS ARE COMPLEX
C     THEN THE ETAS CAN STILL CONTRIBUTE.
C-----------------------------------------------------------------------
CCC        IF(DIMAG(D).NE.0.D0)THEN
               S3=S3
     .           +ETA(1.D0/(D+C/B),-(E/B)/(D+C/B)**2,
     .                C/B,E/B,(E/B)*D/(D+C/B)**2)
     .                *CDLN(D/(D+C/B),-(E/B)*D/(D+C/B)**2)
     .           -ETA(1.D0/(D+C/B),-(E/B)/(D+C/B)**2,
     .                1.D0+C/B,E/B,(E/B)*(D-1.D0)/(D+C/B)**2)
     .                *CDLN((D-1.D0)/(D+C/B),-(E/B)*(D-1.D0)/(D+C/B)**2)
     .          +(ETA(B,(1.D0,0.D0),C/B,E/B,E)
     .           -ETA(B,(1.D0,0.D0),D+C/B,E/B,E))
     .                 *CDLN((D-1.D0)/D,(1.D0,0.D0))
CCC            END IF
               RETURN
           END IF
       CALL QDRT(A,B,C,R1,R2)
C  THE CASE OF EQUAL ROOTS NEEDS SPECIAL TREATMENT
       IF(R1.NE.R2)THEN
           AR1R2=A*(R1-R2)
       ELSE
           AR1R2=(1.D0,0.D0)
           END IF
       S3=SP(D/(D-R1),-(E/AR1R2)*D/(D-R1)**2)
     .   -SP((D-1.D0)/(D-R1),-(E/AR1R2)*(D-1.D0)/(D-R1)**2)
     .   +SP(D/(D-R2),(E/AR1R2)*D/(D-R2)**2)
     .   -SP((D-1.D0)/(D-R2),(E/AR1R2)*(D-1.D0)/(D-R2)**2)
C-----------------------------------------------------------------------
C  IF D IS REAL THE ETA FUNCTIONS VANISH
C     BECAUSE IMAG(1/(D-R1))=-IMAG(1-R1)=-IMAG(-R1) ETC.
C  NOTE THE SIGN OF THE LAST TERM IS WRONG
C    IN EQ.(B2) OF 'T HOOFT AND VELTMAN, NPB153 (1979) 365.
C-----------------------------------------------------------------------
CCC    IF(DIMAG(D).NE.0.D0)THEN
C-----------------------------------------------------------------------
C  EPSILN AND DELTA HAVE THE OPPOSITE SIGN TO THE IMAGINARY
C    PART OF THE ARGUMENTS OF THE LOGS IN THE INTEGRAND
C-----------------------------------------------------------------------
             IF(DIMAG(B*0.5D0+C).NE.0.D0)THEN
                 EPSILN=-DIMAG(B*0.5D0+C)
             ELSE
                 EPSILN=-E
                 END IF
             IF(DIMAG(A*D**2+B*D+C).NE.0.D0)THEN
                 DELTA=-DIMAG(A*D**2+B*D+C)
             ELSE
                 DELTA=-E
                 END IF
           S3=S3
     .     +ETA(1.D0/(D-R1),-(E/AR1R2)/(D-R1)**2,
     .          -R1,(E/AR1R2),(E/AR1R2)*D/(D-R1)**2)
     .              *CDLN(D/(D-R1),-(E/AR1R2)*D/(D-R1)**2)
     .     -ETA(1.D0/(D-R1),-(E/AR1R2)/(D-R1)**2,
     .          1.D0-R1,(E/AR1R2),(E/AR1R2)*(D-1.D0)/(D-R1)**2)
     .              *CDLN((D-1.D0)/(D-R1),-(E/AR1R2)*(D-1.D0)/(D-R1)**2)
     .     +ETA(1.D0/(D-R2),(E/AR1R2)/(D-R2)**2,
     .          -R2,-(E/AR1R2),-(E/AR1R2)*D/(D-R2)**2)
     .              *CDLN(D/(D-R2),(E/AR1R2)*D/(D-R2)**2)
     .     -ETA(1.D0/(D-R2),(E/AR1R2)/(D-R2)**2,
     .          1.D0-R2,-(E/AR1R2),-(E/AR1R2)*(D-1.D0)/(D-R2)**2)
     .              *CDLN((D-1.D0)/(D-R2),(E/AR1R2)*(D-1.D0)/(D-R2)**2)
          IF(D.NE.(0.D0,0.D0))
     .      S3=S3+(ETA(-R1,E/AR1R2,-R2,-E/AR1R2,E/A)
     .           -ETA(D-R1,E/AR1R2,D-R2,-E/AR1R2,E/A)
     .           -ETA(A,-EPSILN,1.D0/A,DELTA,(1.D0,0.D0)))
     .              *CDLOG((D-1.D0)/D)
CCC       END IF
       RETURN
       END
 
      SUBROUTINE QDRT(A,B,C,R1,R2)
C*********************************
C  ROOTS OF THE QUADRATIC A*X**2+B*X+C=0 WITH REAL(R1)>REAL(R2)
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 A,B,C,R1,R2
      COMPLEX*16 CNT,DIS
      CNT=-B/(2.D0*A)
      DIS=CDSQRT(CNT*CNT-C/A)
      DIS=DIS*DCMPLX(DSIGN(1.D0,DREAL(DIS)))
      R1=CNT+DIS
      R2=CNT-DIS
      RETURN
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*                LIBRARY OF QCD INTERFACE SUBPROGRAMS                **
C*                         BY BERND A. KNIEHL                         **
C*                                                                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION QCDMSV(S,M1SQ,M2SQ,XSC)
C     ********************************
C QCDMSV = Pi_V(s, m_1, m_2, mu) / N in O(alpha*alpha_s),
C S      = s,
C M1SQ   = m_1^2,
C M2SQ   = m_2^2,
C XSC    = mu^2, where mu is the 't Hooft mass,
C N refers to the colour gauge group SU(N).
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,M1SQ,M2SQ,XSC,QCDMSV
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON /GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ALPHAS,ASZPI,MTSQ,PI,QCDALF
      COMPLEX*16 CDLN,CF1,CLI3,CLN,CV1,L,R,X,ZETA2,ZETA3
      PI=4.D0*DATAN(1.D0)
      ZETA2=DCMPLX(PI**2/6.D0)
      ZETA3=(1.20205690315959428540D0,0.D0)
C All quarks are treated as light.
      IF (.NOT.QCDLIB) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSV=(0.D0,0.D0)
          ELSE
              QCDMSV=(NCOL**2-1.D0)/(8.D0*NCOL)
     .               *QCDALF(MZ**2)/PI * S/(4.D0*PI**2)
     .               *(-CDLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3
     .               +55.D0/12.D0)
              END IF
          RETURN
          END IF
      MTSQ=XQRK(5,1)**2
C Both quarks are light.
      IF (DREAL(M1SQ).LT.16.D0.AND.DREAL(M2SQ).LT.16.D0) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSV=(0.D0,0.D0)
          ELSE
              ASZPI=ALPHAS(MZ**2)/PI
              IF (NCOL.EQ.3.D0) THEN
                  QCDMSV=ASZPI * S/(4.D0*PI**2)
     .                   *(-CLN(-S/XSC,(-1.D0,0.D0))
     .                   *(1.D0+1.4092D0*ASZPI-12.7673D0*ASZPI**2)
     .                   -4.D0*ZETA3+55.D0/12.D0)
              ELSE
                  QCDMSV=ASZPI * S/(4.D0*PI**2)
     .                   *(-CLN(-S/XSC,(-1.D0,0.D0))
     .                   -4.D0*ZETA3+55.D0/12.D0)
                  END IF
              END IF
C Both quarks are beauties.
      ELSE IF (DREAL(M1SQ).LT.36.D0.AND.DREAL(M2SQ).LT.36.D0) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSV=(0.D0,0.D0)
          ELSE
              QCDMSV=ALPHAS(MTSQ)/PI * S/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+55.D0/12.D0)
              END IF
C One quark is the top and the other one is the beauty.
      ELSE IF (DREAL(M2SQ).LT.36.D0) THEN
          X=S/MTSQ
          L=CDLOG(MTSQ/XSC)
          QCDMSV=ALPHAS(MTSQ)/PI * MTSQ/PI**2
     .           *((X*(-L-4.D0*ZETA3+55.D0/12.D0)
     .           +3.D0*L**2-11.D0/2.D0*L+6.D0*ZETA3+3.D0*ZETA2
     .           -11.D0/8.D0)/4.D0+CF1(X))
C Both quarks are tops.
      ELSE
          R=S/(4.D0*MTSQ)
          QCDMSV=ALPHAS(MTSQ)/PI * MTSQ/PI**2
     .           *(R*(-CDLOG(MTSQ/XSC)-4.D0*ZETA3+55.D0/12.D0)+CV1(R))
          END IF
C Correct for colour.
      QCDMSV=(NCOL**2-1.D0)/(8.D0*NCOL)*QCDMSV
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION QCDMSA(S,M1SQ,M2SQ,XSC)
C     ********************************
C QCDMSA = Pi_A(s, m_1, m_2, mu) / N in O(alpha*alpha_s),
C S      = s,
C M1SQ   = m_1^2,
C M2SQ   = m_2^2,
C XSC    = mu^2, where mu is the 't Hooft mass,
C N refers to the colour gauge group SU(N).
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,M1SQ,M2SQ,XSC,QCDMSA
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON /GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ALPHAS,ASZPI,MTSQ,PI,QCDALF
      COMPLEX*16 CA1,CDLN,CF1,CLI3,CLN,L,R,X,ZETA2,ZETA3
      PI=4.D0*DATAN(1.D0)
      ZETA2=DCMPLX(PI**2/6.D0)
      ZETA3=(1.20205690315959428540D0,0.D0)
C All quarks are treated as light.
      IF (.NOT.QCDLIB) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSA=(0.D0,0.D0)
          ELSE
              QCDMSA=(NCOL**2-1.D0)/(8.D0*NCOL)
     .               *QCDALF(MZ**2)/PI * S/(4.D0*PI**2)
     .               *(-CDLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3
     .               +55.D0/12.D0)
              END IF
          RETURN
          END IF
      MTSQ=XQRK(5,1)**2
C Both quarks are light.
      IF (DREAL(M1SQ).LT.16.D0.AND.DREAL(M2SQ).LT.16.D0) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSA=(0.D0,0.D0)
          ELSE
              ASZPI=ALPHAS(MZ**2)/PI
              IF (NCOL.EQ.3.D0) THEN
                  QCDMSA=ASZPI * S/(4.D0*PI**2)
     .                   *(-CLN(-S/XSC,(-1.D0,0.D0))
     .                   *(1.D0+1.4092D0*ASZPI-12.7673D0*ASZPI**2)
     .                   -4.D0*ZETA3+55.D0/12.D0)
              ELSE
                  QCDMSA=ASZPI * S/(4.D0*PI**2)
     .                   *(-CLN(-S/XSC,(-1.D0,0.D0))
     .                   -4.D0*ZETA3+55.D0/12.D0)
                  END IF
              END IF
C Both quarks are beauties.
      ELSE IF (DREAL(M1SQ).LT.36.D0.AND.DREAL(M2SQ).LT.36.D0) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDMSA=(0.D0,0.D0)
          ELSE
              QCDMSA=ALPHAS(MTSQ)/PI * S/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+55.D0/12.D0)
              END IF
C One quark is the top and the other one is the beauty.
      ELSE IF (DREAL(M2SQ).LT.36.D0) THEN
          X=S/MTSQ
          L=CDLOG(MTSQ/XSC)
          QCDMSA=ALPHAS(MTSQ)/PI * MTSQ/PI**2
     .           *((X*(-L-4.D0*ZETA3+55.D0/12.D0)
     .           +3.D0*L**2-11.D0/2.D0*L+6.D0*ZETA3+3.D0*ZETA2
     .           -11.D0/8.D0)/4.D0+CF1(X))
C Both quarks are tops.
      ELSE
          R=S/(4.D0*MTSQ)
          L=CDLOG(MTSQ/XSC)
          QCDMSA=ALPHAS(MTSQ)/PI * MTSQ/PI**2
     .           *(R*(-L-4.D0*ZETA3+55.D0/12.D0)
     .           +3.D0*L**2-11.D0/2.D0*L+6.D0*ZETA3+3.D0*ZETA2
     .           -11.D0/8.D0+CA1(R))
          END IF
C Correct for colour.
      QCDMSA=(NCOL**2-1.D0)/(8.D0*NCOL)*QCDMSA
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION QCDPIP(S,MSQ,XSC)
C     **************************
C QCDPIP = Pi_V(s, m, m, mu) / (N*s) in O(alpha*alpha_s),
C S      = s,
C MSQ    = m^2,
C XSC    = mu^2, where mu is the 't Hooft mass,
C N refers to the colour gauge group SU(N).
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,XSC,QCDPIP
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON /GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ALPHAS,ASZPI,MTSQ,PI,QCDALF
      COMPLEX*16 CDLN,CLI3,CLN,ZETA3
      PI=4.D0*DATAN(1.D0)
      ZETA3=(1.20205690315959428540D0,0.D0)
C All quarks are treated as light; s is non-zero.
      IF (.NOT.QCDLIB) THEN
          IF (S.EQ.(0.D0,0.D0)) THEN
              QCDPIP=(0.D0,0.D0)
          ELSE
              QCDPIP=(NCOL**2-1.D0)/(8.D0*NCOL)
     .               *QCDALF(MZ**2)/PI * 1.D0/(4.D0*PI**2)
     .               *(-CDLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3
     .               +55.D0/12.D0)
              END IF
          RETURN
          END IF
      MTSQ=XQRK(5,1)**2
C Both quarks are light and s is non-zero.
      IF (DREAL(MSQ).LT.16.D0) THEN
          ASZPI=ALPHAS(MZ**2)/PI
          IF (NCOL.EQ.3.D0) THEN
              QCDPIP=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))
     .               *(1.D0+1.4092D0*ASZPI-12.7673D0*ASZPI**2)
     .               -4.D0*ZETA3+55.D0/12.D0)
          ELSE
              QCDPIP=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+55.D0/12.D0)
              END IF
C Both quarks are beauties and s is non-zero.
      ELSE IF (DREAL(MSQ).LT.36.D0) THEN
          QCDPIP=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+55.D0/12.D0)
C Both quarks are tops and s = 0.
      ELSE
          QCDPIP=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CDLOG(MTSQ/XSC)+15.D0/4.D0)
          END IF
C Correct for colour.
      QCDPIP=(NCOL**2-1.D0)/(8.D0*NCOL)*QCDPIP
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION QCDDEV(S,MSQ,XSC)
C     **************************
C QCDDEV = dPi_V(s, m, m, mu)/ds / N in O(alpha*alpha_s),
C S      = s,
C MSQ    = m^2,
C XSC    = mu^2, where mu is the 't Hooft mass,
C N refers to the colour gauge group SU(N).
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,XSC,QCDDEV
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON /GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ALPHAS,ASZPI,MTSQ,PI,QCDALF
      COMPLEX*16 CDLN,CDV1,CLI3,CLN,R,ZETA3
      PI=4.D0*DATAN(1.D0)
      ZETA3=(1.20205690315959428540D0,0.D0)
C All quarks are treated as light; s is non-zero.
      IF (.NOT.QCDLIB) THEN
          QCDDEV=(NCOL**2-1.D0)/(8.D0*NCOL)
     .           *QCDALF(MZ**2)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CDLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
          RETURN
          END IF
      MTSQ=XQRK(5,1)**2
C Both quarks are light and s is non-zero.
      IF (DREAL(MSQ).LT.16.D0) THEN
          ASZPI=ALPHAS(MZ**2)/PI
          IF (NCOL.EQ.3.D0) THEN
              QCDDEV=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-(CLN(-S/XSC,(-1.D0,0.D0))+1.D0)
     .               *(1.D0+1.4092D0*ASZPI-12.7673D0*ASZPI**2)
     .               -4.D0*ZETA3+55.D0/12.D0)
          ELSE
              QCDDEV=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
              END IF
C Both quarks are beauties and s is non-zero.
      ELSE IF (DREAL(MSQ).LT.36.D0) THEN
          QCDDEV=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
C Both quarks are tops.
      ELSE
          R=S/(4.D0*MTSQ)
          QCDDEV=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CDLOG(MTSQ/XSC)-4.D0*ZETA3+55.D0/12.D0+CDV1(R))
          END IF
C Correct for colour.
      QCDDEV=(NCOL**2-1.D0)/(8.D0*NCOL)*QCDDEV
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION QCDDEA(S,MSQ,XSC)
C     **************************
C QCDDEA = dPi_A(s, m, m, mu)/ds / N in O(alpha*alpha_s),
C S      = s,
C MSQ    = m^2,
C XSC    = mu^2, where mu is the 't Hooft mass,
C N refers to the colour gauge group SU(N).
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 S,MSQ,XSC,QCDDEA
      COMMON /HOCOM/QCDLIB
      LOGICAL QCDLIB
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      COMMON /GSWROB/ALFPI,SWSQ,NCOL,MH,MW,MZ,SPOLE,SCALE,SCHEME
      INTEGER SCHEME
      REAL*8 ALFPI,SWSQ,NCOL,MH,MW,MZ
      COMPLEX*16 SPOLE,SCALE
      REAL*8 ALPHAS,ASZPI,MTSQ,PI,QCDALF
      COMPLEX*16 CDA1,CDLN,CLI3,CLN,R,ZETA3
      PI=4.D0*DATAN(1.D0)
      ZETA3=(1.20205690315959428540D0,0.D0)
C All quarks are treated as light; s is non-zero.
      IF (.NOT.QCDLIB) THEN
          QCDDEA=(NCOL**2-1.D0)/(8.D0*NCOL)
     .           *QCDALF(MZ**2)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CDLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
          RETURN
          END IF
      MTSQ=XQRK(5,1)**2
C Both quarks are light and s is non-zero.
      IF (DREAL(MSQ).LT.16.D0) THEN
          ASZPI=ALPHAS(MZ**2)/PI
          IF (NCOL.EQ.3.D0) THEN
              QCDDEA=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-(CLN(-S/XSC,(-1.D0,0.D0))+1.D0)
     .               *(1.D0+1.4092D0*ASZPI-12.7673D0*ASZPI**2)
     .               -4.D0*ZETA3+55.D0/12.D0)
          ELSE
              QCDDEA=ASZPI * 1.D0/(4.D0*PI**2)
     .               *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
              END IF
C Both quarks are beauties and s is non-zero.
      ELSE IF (DREAL(MSQ).LT.36.D0) THEN
          QCDDEA=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CLN(-S/XSC,(-1.D0,0.D0))-4.D0*ZETA3+43.D0/12.D0)
C Both quarks are tops.
      ELSE
          R=S/(4.D0*MTSQ)
          QCDDEA=ALPHAS(MTSQ)/PI * 1.D0/(4.D0*PI**2)
     .           *(-CDLOG(MTSQ/XSC)-4.D0*ZETA3+55.D0/12.D0+CDA1(R))
          END IF
C Correct for colour.
      QCDDEA=(NCOL**2-1.D0)/(8.D0*NCOL)*QCDDEA
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION QCDALF(S)
C     ******************
C Strong coupling constant, alpha_s(s), to lowest order.
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 S,QCDALF
      REAL*8 LAMBDA,NF,PI
      PI=4.D0*DATAN(1.D0)
C Lambda(4) = (185 +/- 50) MeV according to
C A.D. Martin, R.G. Roberts, and W.J. Stirling,
C Phys. Rev. D43 (1991) 3648.
      LAMBDA=.185D0
      NF=4.D0
      QCDALF=12.D0*PI/((33.D0-2.D0*NF)*DLOG(S/LAMBDA**2))
      RETURN
C-----------------------------------------------------------------------
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*             LIBRARY OF HIGHER ORDER QCD CORRECTIONS                **
C*                                                                    **
C*                             REFERENCES:                            **
C*                   W. J. Marciano, PRD29(1984)580                   **
C*           A. Djouadi and C. Verzegnassi, PLB195(1987)265           **
C*                    A. Djouadi, NC100A(1988)357                     **
C*                    B. A. Kniehl, NPB347(1990)86                    **
C*            F. Halzen and B. A. Kniehl, NPB353(1991)567             **
C* L. R. Surguladze and M. A. Samuel, PRL66(1991)560, (E)66(1991)2416 **
C*   S. G. Gorishny, A. L. Kataev, and S. A. Larin, PLB259(1991)144   **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION ALPHAS(S)
C     ******************
C Parametrization of the strong coupling constant according to
C W.J. Marciano, Phys. Rev. D29 (1984) 580.
C
      IMPLICIT LOGICAL (A-Z)
      REAL*8 S,ALPHAS
      COMMON /FERMAS/NLPT,NQRK,XLPT,XQRK
      INTEGER NLPT,NQRK
      REAL*8 XLPT(40,2),XQRK(40,2)
      REAL*8 A,B,B0,B1,B2,C,LAMBDA,NF,PI
      PI=4.D0*DATAN(1.D0)
C Lambda(4) = (185 +/- 50) MeV according to
C A.D. Martin, R.G. Roberts, and W.J. Stirling,
C Phys. Rev. D43 (1991) 3648.
      LAMBDA=.185D0
      IF (S.LE.XQRK(3,1)**2) THEN
          NF=3.D0
          LAMBDA=LAMBDA*(XQRK(3,1)/LAMBDA)**(2.D0/27.D0)
     .           *DLOG(XQRK(3,1)**2/LAMBDA**2)**(107.D0/2025.D0)
      ELSE IF (S.LE.XQRK(6,1)**2) THEN
          NF=4.D0
      ELSE IF (S.LE.XQRK(5,1)**2) THEN
          NF=5.D0
          LAMBDA=LAMBDA*(LAMBDA/XQRK(6,1))**(2.D0/23.D0)
     .           *DLOG(XQRK(6,1)**2/LAMBDA**2)**(-963.D0/13225.D0)
      ELSE
          NF=6.D0
          LAMBDA=LAMBDA*(LAMBDA/XQRK(6,1))**(2.D0/23.D0)
     .           *DLOG(XQRK(6,1)**2/LAMBDA**2)**(-963.D0/13225.D0)
          LAMBDA=LAMBDA*(LAMBDA/XQRK(5,1))**(2.D0/21.D0)
     .           *DLOG(XQRK(5,1)**2/LAMBDA**2)**(-107.D0/1127.D0)
          END IF
      B0=11.D0-2.D0/3.D0*NF
      B1=102.D0-38.D0/3.D0*NF
      B2=(2857.D0-5033.D0/9.D0*NF+325.D0/27.D0*NF**2)/2.D0
      A=DLOG(S/LAMBDA**2)
      B=DLOG(A)
      C=B1/(B0**2*A)
      ALPHAS=4.D0*PI/(B0*A)
     .       *(1.D0-C*B+C**2*((B-1.D0/2.D0)**2+B2*B0/B1**2-5.D0/4.D0))
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CV1(CR)
C     ****************
C V_1(r) defined by Eq. (10) in
C B.A. Kniehl, Nucl. Phys. B347 (1990) 86.
C r = (s + i*epsilon)/(4*m^2), where s may be complex.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CR,CV1
      REAL*8 PI
      COMPLEX*16 C1,C2,CD,CF,CFS,CG,CH,CLI2,CLI3,CM,CMQ,CMS,CONE,CP,CRT,
     .           CS,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=CLI3(CONE,CONE)
      IF (CR.EQ.CZERO) THEN
          CV1=CZERO
      ELSE IF (CR.EQ.CONE) THEN
          CV1=DCMPLX(0.D0,PI**3)
          WRITE (6,*)
     .        'Warning: Coulomb singularity of V_1(r) at r = 1 !'
      ELSE
          C1=CRT(CONE-CR,-CONE)
          C2=CRT(-CR,-CONE)
          CS=C1/C2
          CP=C1+C2
          CM=C1-C2
          CMS=CM**2
          CMQ=CMS**2
          CF=CDLOG(CP)
          CFS=CF**2
          CG=CDLOG(CP-CM)
          CH=CDLOG(CP+CM)
          CD=CLI2(CMS,CONE)-CLI2(CMQ,CONE)
          CV1=4.D0*(CR-1.D0/(4.D0*CR))*(2.D0*CLI3(CMS,CONE)
     .        -CLI3(CMQ,CONE)+8.D0/3.D0*CF*CD
     .        +4.D0*CFS*(-CF+CG/3.D0+2.D0/3.D0*CH))
     .        +CS*(8.D0/3.D0*(CR+1.D0/2.D0)*(CD
     .        +CF*(-3.D0*CF+2.D0*CG+4.D0*CH))-2.D0*(CR+3.D0/2.D0)*CF)
     .        -8.D0*(CR-1.D0/6.D0-7.D0/(48.D0*CR))*CFS+13.D0/6.D0
     .        +CZETA3/CR
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CDV1(CR)
C     *****************
C dV_1(r)/dr, where V_1(r) is defined by Eq. (10) in
C B.A. Kniehl, Nucl. Phys. B347 (1990) 86.
C r = (s + i*epsilon)/(4*m^2), where s may be complex.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CR,CDV1
      REAL*8 PI
      COMPLEX*16 C1,C2,CD,CF,CFS,CG,CH,CLI2,CLI3,CM,CMQ,CMS,CONE,CP,CRS,
     .           CRT,CS,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=CLI3(CONE,CONE)
      IF (CR.EQ.CZERO) THEN
          CDV1=4.D0*CZETA3-5.D0/6.D0
      ELSE IF (CR.EQ.CONE) THEN
          CDV1=CZERO
          WRITE (6,*)
     .        'Warning: Coulomb singularity of dV_1(r)/dr at r = 1 !'
      ELSE
          CRS=CR**2
          C1=CRT(CONE-CR,-CONE)
          C2=CRT(-CR,-CONE)
          CS=C1/C2
          CP=C1+C2
          CM=C1-C2
          CMS=CM**2
          CMQ=CMS**2
          CF=CDLOG(CP)
          CFS=CF**2
          CG=CDLOG(CP-CM)
          CH=CDLOG(CP+CM)
          CD=CLI2(CMS,CONE)-CLI2(CMQ,CONE)
          CDV1=4.D0*(1.D0+1.D0/(4.D0*CRS))*(2.D0*CLI3(CMS,CONE)
     .         -CLI3(CMQ,CONE)+8.D0/3.D0*CF*CD
     .         +4.D0*CFS*(-CF+CG/3.D0+2.D0/3.D0*CH))
     .         +2.D0*CS*(2.D0/(3.D0*CR)*(-CD
     .         -CF*(-3.D0*CF+2.D0*CG+4.D0*CH))-(1.D0-5.D0/(6.D0*CR))*CF)
     .         +(4.D0/(3.D0*CR)-11.D0/(6.D0*CRS)-4.D0/(1.D0-CR))*CFS
     .         -1.D0-3.D0/(2.D0*CR)-CZETA3/CRS
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CA1(CR)
C     ****************
C A_1(r) defined by Eq. (11) in
C B.A. Kniehl, Nucl. Phys. B347 (1990) 86.
C r = (s + i*epsilon)/(4*m^2), where s may be complex.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CR,CA1
      REAL*8 PI
      COMPLEX*16 C1,C2,CD,CF,CFS,CG,CH,CLI2,CLI3,CM,CMQ,CMS,CONE,CP,CRT,
     .           CS,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=CLI3(CONE,CONE)
      IF (CR.EQ.CZERO) THEN
          CA1=3.D0*(-2.D0*CZETA3-CZETA2+7.D0/4.D0)
      ELSE IF (CR.EQ.CONE) THEN
          CA1=-2.D0*CZETA3-3.D0/8.D0*CZETA2+29.D0/12.D0
      ELSE
          C1=CRT(CONE-CR,-CONE)
          C2=CRT(-CR,-CONE)
          CS=C1/C2
          CP=C1+C2
          CM=C1-C2
          CMS=CM**2
          CMQ=CMS**2
          CF=CDLOG(CP)
          CFS=CF**2
          CG=CDLOG(CP-CM)
          CH=CDLOG(CP+CM)
          CD=CLI2(CMS,CONE)-CLI2(CMQ,CONE)
          CA1=4.D0*(CR-3.D0/2.D0+1.D0/(2.D0*CR))*(2.D0*CLI3(CMS,CONE)
     .        -CLI3(CMQ,CONE)+8.D0/3.D0*CF*CD
     .        +4.D0*CFS*(-CF+CG/3.D0+2.D0/3.D0*CH))
     .        +CS*(8.D0/3.D0*(CR-1.D0)*(CD
     .        +CF*(-3.D0*CF+2.D0*CG+4.D0*CH))
     .        -2.D0*(CR-3.D0+1.D0/(4.D0*CR))*CF)
     .        -8.D0*(CR-11.D0/12.D0+5.D0/(48.D0*CR)
     .        +1.D0/(32.D0*CR**2))*CFS
     .        -3.D0*CZETA2+13.D0/6.D0+(-2.D0*CZETA3+1.D0/4.D0)/CR
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CDA1(CR)
C     *****************
C dA_1(r)/dr, where A_1(r) is defined by Eq. (11) in
C B.A. Kniehl, Nucl. Phys. B347 (1990) 86.
C r = (s + i*epsilon)/(4*m^2), where s may be complex.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CR,CDA1
      REAL*8 PI
      COMPLEX*16 C1,C2,CD,CF,CFS,CG,CH,CLI2,CLI3,CM,CMQ,CMS,CONE,CP,CRS,
     .           CRT,CS,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=CLI3(CONE,CONE)
      IF (CR.EQ.CZERO) THEN
          CDA1=4.D0*CZETA3-49.D0/18.D0
      ELSE IF (CR.EQ.CONE) THEN
          CDA1=DCMPLX(0.D0,2.D0/3.D0*PI**3)
          WRITE (6,*)
     .        'Warning: Coulomb singularity of dA_1(r)/dr at r = 1 !'
      ELSE
          CRS=CR**2
          C1=CRT(CONE-CR,-CONE)
          C2=CRT(-CR,-CONE)
          CS=C1/C2
          CP=C1+C2
          CM=C1-C2
          CMS=CM**2
          CMQ=CMS**2
          CF=CDLOG(CP)
          CFS=CF**2
          CG=CDLOG(CP-CM)
          CH=CDLOG(CP+CM)
          CD=CLI2(CMS,CONE)-CLI2(CMQ,CONE)
          CDA1=4.D0*(1.D0-1.D0/(2.D0*CRS))*(2.D0*CLI3(CMS,CONE)
     .         -CLI3(CMQ,CONE)+8.D0/3.D0*CF*CD
     .         +4.D0*CFS*(-CF+CG/3.D0+2.D0/3.D0*CH))
     .         +2.D0*CS*(4.D0/(3.D0*CR)*(CD
     .         +CF*(-3.D0*CF+2.D0*CG+4.D0*CH))
     .         -(1.D0+13.D0/(6.D0*CR)-1.D0/(2.D0*CRS))*CF)
     .         +(-20.D0/3.D0+13.D0/(6.D0*CR)+1.D0/(2.D0*CRS))/CR*CFS
     .         -1.D0+3.D0/CR+(2.D0*CZETA3-1.D0/2.D0)/CRS
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CF1(CX)
C     ****************
C F_1(x) defined by Eq. (12) in
C B.A. Kniehl, Nucl. Phys. B347 (1990) 86.
C x = (s + i*epsilon)/m^2, where s may be complex.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CX,CF1
      REAL*8 PI
      COMPLEX*16 CA,CB,CBS,CD,CLI2,CLI3,CLN,CXS,CONE,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=CLI3(CONE,CONE)
      IF (CX.EQ.CZERO) THEN
          CF1=-3.D0/2.D0*CZETA3-CZETA2/2.D0+23.D0/16.D0
      ELSE IF (CX.EQ.CONE) THEN
          CF1=-CZETA3/2.D0-CZETA2/12.D0+7.D0/8.D0
      ELSE
          CXS=CX**2
          CA=CLN(-CX,-CONE)
          CB=CLN(CONE-CX,-CONE)
          CBS=CB**2
          CD=CLI2(CONE/(CONE-CX),CONE)
          CF1=(CX-3.D0/2.D0+1.D0/(2.D0*CXS))*(CLI3(CONE/(CONE-CX),CONE)
     .        +2.D0/3.D0*CB*CD-CBS/6.D0*(CA-CB))
     .        +(CX+1.D0/2.D0-1.D0/(2.D0*CX))/3.D0*(CD-CA*CB)
     .        +(CX-1.D0/8.D0-1.D0/CX+5.D0/(8.D0*CXS))/3.D0*CBS
     .        -(CX-5.D0/2.D0+2.D0/(3.D0*CX)+5.D0/(6.D0*CXS))/4.D0*CB
     .        -3.D0/4.D0*CZETA2+13.D0/12.D0-5.D0/(24.D0*CX)
     .        -CZETA3/(2.D0*CXS)
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
C***********************************************************************
C***********************************************************************
C*                                                                    **
C*                                                                    **
C*         LIBRARY OF ANALYTIC FUNCTIONS USED IN THE QCD PART         **
C*                                                                    **
C*                                                                    **
C***********************************************************************
C***********************************************************************
 
      FUNCTION CLI2(CY,CE)
C     ********************
C Li_2(y + i*epsilon*sign(Re(e)))
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CY,CE,CLI2
      INTEGER I,IFLAG1,IFLAG2,N
      REAL*8 B(0:60),EPS,PI
      COMPLEX*16 CH,CLN,COLD,CONE,CX,CZ,CZERO,CZETA2
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
C N = # of terms to be evaluated in the expansion, N <= 61.
      N=30
C Relative precision.
      EPS=1.D-20
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      DO 1 I=3,59,2
1         B(I)=0.D0
      B(0)=1.D0
      B(1)=-.5D0
      B(2)=1.D0/6.D0
      B(4)=-1.D0/30.D0
      B(6)=1.D0/42.D0
      B(8)=B(4)
      B(10)=5.D0/66.D0
      B(12)=-691.D0/2730.D0
      B(14)=7.D0/6.D0
      B(16)=-3617.D0/510.D0
      B(18)=43867.D0/798.D0
      B(20)=-174611.D0/330.D0
      B(22)=854513.D0/138.D0
      B(24)=-236364091.D0/2730.D0
      B(26)=8553103.D0/6.D0
      B(28)=-23749461029.D0/870.D0
      B(30)=8615841276005.D0/14322.D0
      B(32)=-7709321041217.D0/510.D0
      B(34)=2577687858367.D0/6.D0
      B(36)=-26315271553053477373.D0/1919190.D0
      B(38)=2929993913841559.D0/6.D0
      B(40)=-261082718496449122051.D0/13530.D0
      B(42)=1520097643918070802691.D0/1806.D0
      B(44)=-27833269579301024235023.D0/690.D0
      B(46)=596451111593912163277961.D0/282.D0
      B(48)=-5609403368997817686249127547.D0/46410.D0
      B(50)=495057205241079648212477525.D0/66.D0
      B(52)=-801165718135489957347924991853.D0/1590.D0
      B(54)=29149963634884862421418123812691.D0/798.D0
      B(56)=-2479392929313226753685415739663229.D0/870.D0
      B(58)=84483613348880041862046775994036021.D0/354.D0
C      B(60)=-1215233140483755572040304994079820246041491.D0/56786730.D0
      CX=CY
      IF (CX.EQ.CZERO) THEN
          CLI2=CZERO
      ELSE IF (CX.EQ.CONE) THEN
          CLI2=CZETA2
      ELSE
          IFLAG1=0
          IFLAG2=0
          IF (CDABS(CX).GT.1.D0) THEN
              CX=CONE/CX
              IFLAG1=1
              END IF
          IF (DREAL(CX).GT..5D0) THEN
              CX=CONE-CX
              IFLAG2=1
              END IF
          CZ=-CDLOG(CONE-CX)
          CH=CONE
          CLI2=CZERO
          DO 2 I=1,N
              COLD=CLI2
              CH=CH*CZ/DCMPLX(DFLOAT(I))
              CLI2=CLI2+DCMPLX(B(I-1))*CH
2             IF (B(I-1).NE.0.D0.AND.CDABS(COLD/CLI2-CONE).LT.EPS)
     .            GOTO 3
3         IF (IFLAG2.EQ.1) THEN
              CLI2=-CLI2+CZETA2-CDLOG(CX)*CDLOG(CONE-CX)
              END IF
          IF (IFLAG1.EQ.1) THEN
              CLI2=-CLI2-CZETA2-DCMPLX(.5D0)*CLN(-CY,-CE)**2
              END IF
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CLI3(CY,CE)
C     ********************
C Li_3(y + i*epsilon*sign(Re(e)))
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CY,CE,CLI3
      INTEGER IFLAG
      REAL*8 PI
      COMPLEX*16 CH,CLI3H,CLN,CONE,CX,CZERO,CZETA2,CZETA3
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
      PI=4.D0*DATAN(1.D0)
      CZETA2=DCMPLX(PI**2/6.D0)
      CZETA3=DCMPLX(1.20205690315959428540D0)
      CX=CY
      IF (CX.EQ.CZERO) THEN
          CLI3=CZERO
      ELSE IF (CX.EQ.CONE) THEN
          CLI3=CZETA3
      ELSE
          IFLAG=0
          IF (CDABS(CX).GT.1.D0) THEN
              CX=CONE/CX
              IFLAG=1
              END IF
          IF (DREAL(CX).GT..5D0) THEN
              CH=CDLOG(CX)
              CLI3=-CLI3H(CONE-CONE/CX)-CLI3H(CONE-CX)+CZETA3+CZETA2*CH
     .             +CH**3/DCMPLX(6.D0)-DCMPLX(.5D0)*CH**2*CDLOG(CONE-CX)
          ELSE
              CLI3=CLI3H(CX)
              END IF
          IF (IFLAG.EQ.1) THEN
              CH=CLN(-CY,-CE)
              CLI3=CLI3-CZETA2*CH-CH**3/DCMPLX(6.D0)
              END IF
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CLI3H(CX)
C     ******************
C Li_3(x) for 0 < |x| <= 1, Re(x) <= 1/2.
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CX,CLI3H
      INTEGER I,J,N
C N = # of terms to be evaluated in the expansion, N <= 61.
      PARAMETER (N=30)
      REAL*8 B(0:60),EPS,F(0:N)
      COMPLEX*16 CC,COLD,CONE,CP,CZ,CZERO
      DATA CZERO,CONE/(0.D0,0.D0),(1.D0,0.D0)/
C Relative precision.
      EPS=1.D-20
      DO 1 I=3,59,2
1         B(I)=0.D0
      B(0)=1.D0
      B(1)=-.5D0
      B(2)=1.D0/6.D0
      B(4)=-1.D0/30.D0
      B(6)=1.D0/42.D0
      B(8)=B(4)
      B(10)=5.D0/66.D0
      B(12)=-691.D0/2730.D0
      B(14)=7.D0/6.D0
      B(16)=-3617.D0/510.D0
      B(18)=43867.D0/798.D0
      B(20)=-174611.D0/330.D0
      B(22)=854513.D0/138.D0
      B(24)=-236364091.D0/2730.D0
      B(26)=8553103.D0/6.D0
      B(28)=-23749461029.D0/870.D0
      B(30)=8615841276005.D0/14322.D0
      B(32)=-7709321041217.D0/510.D0
      B(34)=2577687858367.D0/6.D0
      B(36)=-26315271553053477373.D0/1919190.D0
      B(38)=2929993913841559.D0/6.D0
      B(40)=-261082718496449122051.D0/13530.D0
      B(42)=1520097643918070802691.D0/1806.D0
      B(44)=-27833269579301024235023.D0/690.D0
      B(46)=596451111593912163277961.D0/282.D0
      B(48)=-5609403368997817686249127547.D0/46410.D0
      B(50)=495057205241079648212477525.D0/66.D0
      B(52)=-801165718135489957347924991853.D0/1590.D0
      B(54)=29149963634884862421418123812691.D0/798.D0
      B(56)=-2479392929313226753685415739663229.D0/870.D0
      B(58)=84483613348880041862046775994036021.D0/354.D0
C      B(60)=-1215233140483755572040304994079820246041491.D0/56786730.D0
      F(0)=1.D0
      DO 2 I=1,N
2         F(I)=F(I-1)*DFLOAT(I)
      CZ=-CDLOG(CONE-CX)
      CP=CONE
      CLI3H=CZERO
      DO 3 I=1,N
          COLD=CLI3H
          CP=CP*CZ
          CC=CZERO
          DO 4 J=1,I
4             CC=CC+DCMPLX(B(J-1)*B(I-J)/(F(J)*F(I-J)))
          CLI3H=CLI3H+CP*CC/DCMPLX(DFLOAT(I))
3     IF (CDABS(COLD/CLI3H-CONE).LT.EPS) GOTO 5
5     RETURN
C-----------------------------------------------------------------------
      END
 
      FUNCTION CRT(CX,CE)
C     *******************
C sqrt(x + i*epsilon*sign(Re(e)))
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CX,CE,CRT
      IF (DREAL(CX).LT.0.D0.AND.DIMAG(CX).EQ.0.D0) THEN
          CRT=DSIGN(1.D0,DREAL(CE))*(0.D0,1.D0)*CDSQRT(-CX)
      ELSE
          CRT=CDSQRT(CX)
          END IF
      RETURN
C-----------------------------------------------------------------------
      END

      FUNCTION CLN(CX,CE)
C     *******************
C ln(x + i*epsilon*sign(Re(e)))
C
      IMPLICIT LOGICAL (A-Z)
      COMPLEX*16 CX,CE,CLN
      REAL*8 PI
      PI=4.D0*DATAN(1.D0)
      IF (DREAL(CX).LT.0.D0.AND.DIMAG(CX).EQ.0.D0) THEN
          CLN=CDLOG(-CX)+DSIGN(1.D0,DREAL(CE))*(0.D0,1.D0)*PI
      ELSE
          CLN=CDLOG(CX)
          END IF
      RETURN
C-----------------------------------------------------------------------
      END
      DOUBLE PRECISION FUNCTION DGAUSS(F,A,B,EPS)
      DOUBLE PRECISION W(12),X(12),A,B,EPS,DELTA,CONST,AA,BB,Y,C1,C2,S8,
     1                 S16,U,F
      DATA CONST /1.0D-25/
      DATA W
     1       / 0.10122 85362 90376 25915 25313 543D0,
     2         0.22238 10344 53374 47054 43559 944D0,
     3         0.31370 66458 77887 28733 79622 020D0,
     4         0.36268 37833 78361 98296 51504 493D0,
     5         0.02715 24594 11754 09485 17805 725D0,
     6         0.06225 35239 38647 89286 28438 370D0,
     7         0.09515 85116 82492 78480 99251 076D0,
     8         0.12462 89712 55533 87205 24762 822D0,
     9         0.14959 59888 16576 73208 15017 305D0,
     A         0.16915 65193 95002 53818 93120 790D0,
     B         0.18260 34150 44923 58886 67636 680D0,
     C         0.18945 06104 55068 49628 53967 232D0 /
      DATA X
     1       / 0.96028 98564 97536 23168 35608 686D0,
     2         0.79666 64774 13626 73959 15539 365D0,
     3         0.52553 24099 16328 98581 77390 492D0,
     4         0.18343 46424 95649 80493 94761 424D0,
     5         0.98940 09349 91649 93259 61541 735D0,
     6         0.94457 50230 73232 57607 79884 155D0,
     7         0.86563 12023 87831 74388 04678 977D0,
     8         0.75540 44083 55003 03389 51011 948D0,
     9         0.61787 62444 02643 74844 66717 640D0,
     A         0.45801 67776 57227 38634 24194 430D0,
     B         0.28160 35507 79258 91323 04605 015D0,
     C         0.09501 25098 37637 44018 53193 354D0 /
      DELTA=CONST*DABS(A-B)
      DGAUSS=0.
      AA=A
    5 Y=B-AA
      IF(DABS(Y) .LE. DELTA) RETURN
    2 BB=AA+Y
      C1=0.5D0*(AA+BB)
      C2=C1-AA
      S8=0.
      S16=0.
      DO 1 I = 1,4
      U=X(I)*C2
    1 S8=S8+W(I)*(F(C1+U)+F(C1-U))
      DO 3 I = 5,12
      U=X(I)*C2
    3 S16=S16+W(I)*(F(C1+U)+F(C1-U))
      S8=S8*C2
      S16=S16*C2
      IF(DABS(S16-S8) .GT. EPS*(1.0D0+DABS(S16))) GO TO 4
      DGAUSS=DGAUSS+S16
      AA=BB
      GO TO 5
    4 Y=0.5D0*Y
      IF(DABS(Y) .GT. DELTA) GO TO 2
      PRINT 7
      DGAUSS=0.
      RETURN
    7 FORMAT(1X,37HDGAUSS ... TOO HIGH ACCURACY REQUIRED)
      END
      SUBROUTINE DSNLEQ(N,X,F,FTOL,XTOL,MAXF,IPRT,INFO,SUB,W)
 
C     Based on   J.J. More  and  M.Y. Cosnard
C
C       ALGORITHM 554 BRENTM, A Fortran Subroutine for the
C       Numerical Solution of Systems of Nonlinear Equations [C5]
C
C     ACM Trans. Math. Software 6 (1980) 240-251.
 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C**** EPS = SQRT(SMALLEST FP.NUMBER)
C     EPS = 1 / SQRT( 16D0**13 )
      PARAMETER (EPS =  0.37252 90298 46191 40625D-08)
      DIMENSION X(N),F(N),W(N,*),MPT(288)
      LOGICAL LCV
      EXTERNAL SUB
 
      PARAMETER (Z1 = 1, SCALE = 10, P05 = 5*Z1/100)
      DATA (MPT(I),I=1,288)
     1/1* 1,1* 2,3* 3,3* 4,4* 5,4* 6,4* 7,4* 8,5* 9,5*10,5*11,5*12,
     2 5*13,5*14,6*15,6*16,5*17,6*18,6*19,6*20,7*21,6*22,6*23,7*24,
     3 6*25,7*26,6*27,7*28,7*29,7*30,7*31,7*32,7*33,7*34,7*35,7*36,
     4 8*37,7*38,7*39,8*40,7*41,8*42,7*43,8*44,8*45,7*46,8*47,8*48/
 
      INFO=0
      IF(N .LE. 0 .OR. FTOL .LE. 0 .OR. XTOL .LE. 0) RETURN
C
C     Find optimal MOPT for iterative refinement
C
      IF(N .LE. 288) THEN
       MOPT=MPT(N)
      ELSE
       H=0
       DO 1 I = 49,N
       TEMP=LOG(I+Z1)/(N+2*I+1)
       IF(TEMP .LT. H) THEN
        MOPT=I-1
        GO TO 2
       ENDIF
    1  H=TEMP
      ENDIF
 
    2 IFLAG=0
      NUMF=0
      NFCALL=0
 
      NIER6=-1
      NIER7=-1
      NIER8=0
      FNORM=0
      DIFIT=0
      XNORM=0
      DO 10 I = 1,N
   10 XNORM=MAX(XNORM,ABS(X(I)))
      DELTA=SCALE*XNORM
      IF(XNORM .EQ. 0) DELTA=SCALE
 
   20 IF(IPRT .NE. 0) WRITE(6,'(1X,I5,D25.14)') (I,X(I),I=1,N)
 
      NSING=N
      FNORM1=FNORM
      DIFIT1=DIFIT
      FNORM=0
C
C     Compute step H for the divided difference which approximates
C     the K-th row of the Jacobian matrix
C
      H=EPS*XNORM
      IF(H .EQ. 0) H=EPS
      DO 40 J = 1,N
      DO 30 I = 1,N
   30 W(I,J+3)=0
      W(J,J+3)=H
   40 W(J,2)=X(J)
C
C     Enter a subiteration
C
      DO 150 K = 1,N
      IFLAG=K
      CALL SUB(N,W(1,2),F,IFLAG)
      FKY=F(K)
      NFCALL=NFCALL+1
      NUMF=NFCALL/N
      IF(IFLAG .LT. 0) GO TO 230
      FNORM=MAX(FNORM,ABS(FKY))
C
C     Compute the K-th row of the Jacobian matrix
C
      DO 60 J = K,N
      DO 50 I = 1,N
   50 W(I,3)=W(I,2)+W(I,J+3)
      CALL SUB(N,W(1,3),F,IFLAG)
      FKZ=F(K)
      NFCALL=NFCALL+1
      NUMF=NFCALL/N
      IF(IFLAG .LT. 0) GO TO 230
   60 W(J,1)=FKZ-FKY
      F(K)=FKY
C
C     Compute the Householder transformation to reduce the K-th row
C     of the Jacobian matrix to a multiple of the K-th unit vector
C
      ETA=0
      DO 70 I = K,N
   70 ETA=MAX(ETA,ABS(W(I,1)))
      IF(ETA .EQ. 0) GO TO 150
      NSING=NSING-1
      SKNORM=0
      DO 80 I = K,N
      W(I,1)=W(I,1)/ETA
   80 SKNORM=SKNORM+W(I,1)**2
      SKNORM=SQRT(SKNORM)
      IF(W(K,1) .LT. 0) SKNORM=-SKNORM
      W(K,1)=W(K,1)+SKNORM
C
C     Apply the transformation
C
      DO 90 I = 1,N
   90 W(I,3)=0
      DO 100 J = K,N
      DO 100 I = 1,N
  100 W(I,3)=W(I,3)+W(J,1)*W(I,J+3)
      DO 120 J = K,N
      TEMP=W(J,1)/(SKNORM*W(K,1))
      DO 120 I = 1,N
  120 W(I,J+3)=W(I,J+3)-TEMP*W(I,3)
C
C     Compute the subiterate
C
      W(K,1)=SKNORM*ETA
      TEMP=FKY/W(K,1)
      IF(H*ABS(TEMP) .GT. DELTA) TEMP=SIGN(DELTA/H,TEMP)
      DO 140 I = 1,N
  140 W(I,2)=W(I,2)+TEMP*W(I,K+3)
  150 CONTINUE
C
C     Compute the norms of the iterate and correction vector
C
      XNORM=0
      DIFIT=0
      DO 160 I = 1,N
      XNORM=MAX(XNORM,ABS(W(I,2)))
      DIFIT=MAX(DIFIT,ABS(X(I)-W(I,2)))
  160 X(I)=W(I,2)
C
C     Update the bound on the correction vector
C
      DELTA=MAX(DELTA,SCALE*XNORM)
C
C     Determine the progress of the iteration
C
      LCV=FNORM .LT. FNORM1 .AND. DIFIT .LT. DIFIT1 .AND. NSING .EQ. 0
      NIER6=NIER6+1
      NIER7=NIER7+1
      NIER8=NIER8+1
      IF(LCV) NIER6=0
      IF(FNORM .LT. FNORM1 .OR. DIFIT .LT. DIFIT1) NIER7=0
      IF(DIFIT .GT. EPS*XNORM) NIER8=0
C
C     Tests for convergence
C
      IF(FNORM .LE. FTOL) INFO=1
      IF(DIFIT .LE. XTOL*XNORM .AND. LCV) INFO=2
      IF(FNORM .LE. FTOL .AND. INFO .EQ. 2) INFO=3
      IF(INFO .NE. 0) GO TO 230
C
C     Tests for termination
C
      IF(NUMF .GE. MAXF) INFO=4
      IF(NSING .EQ. N) INFO=5
      IF(NIER6 .EQ. 5) INFO=6
      IF(NIER7 .EQ. 3) INFO=7
      IF(NIER8 .EQ. 4) INFO=8
      IF(INFO .NE. 0) GO TO 230
      IF(.NOT.LCV .OR. DIFIT .GT. P05*XNORM) GO TO 20
C
C     Iterative refinement  (if the iteration is converging)
C
      DO 210 M = 2,MOPT
      FNORM1=FNORM
      FNORM=0
      DO 190 K = 1,N
      IFLAG=K
      CALL SUB(N,W(1,2),F,IFLAG)
      FKY=F(K)
      NFCALL=NFCALL+1
      NUMF=NFCALL/N
      IF(IFLAG .LT. 0) GO TO 230
      FNORM=MAX(FNORM,ABS(FKY))
C
C     Iterative refinement is terminated if it does not give a
C     reduction on residuals
C
      IF(FNORM .GE. FNORM1) THEN
       FNORM=FNORM1
       GO TO 20
      ENDIF
      TEMP=FKY/W(K,1)
      DO 180 I = 1,N
  180 W(I,2)=W(I,2)+TEMP*W(I,K+3)
  190 CONTINUE
C
C     Compute the norms of the iterate and correction vector
C
      XNORM=0
      DIFIT=0
      DO 200 I = 1,N
      XNORM=MAX(XNORM,ABS(W(I,2)))
      DIFIT=MAX(DIFIT,ABS(X(I)-W(I,2)))
  200 X(I)=W(I,2)
C
C     Stopping criteria for iterative refinement
C
      IF(FNORM .LE. FTOL) INFO=1
      IF(DIFIT .LE. XTOL*XNORM) INFO=2
      IF(FNORM .LE. FTOL .AND. INFO .EQ. 2) INFO=3
      IF(NUMF .GE. MAXF .AND. INFO .EQ. 0) INFO=4
      IF(INFO .NE. 0) GO TO 230
  210 CONTINUE
      GO TO 20
  230 IF(IFLAG .LT. 0) INFO=IFLAG
      RETURN
      END






